; ROM FOR OUR MP3 PLAYER
	
	processor   6502

;   --------------------------   ZERO PAGE   -------------------
; 	seg.u   data		
; 	org     $00

TMPZP			equ $00		;  Temporary zeropage addresses
METASTORAGE		equ $80         ;  $80-$81 pointer to song metadata storage
					;  ($0240)
METAEND			equ $82		;  $82-$83 = end of metadata storage
					;  ($0240 + x*$20)
NUMSONGS		equ $84         ;  number of songs
BUFF32			equ $85		;  $85-$88 32 bit buffer
BCD			equ $89		;  $89-$8c 32 bit BCD number
EXP			equ $8d		;  Used by BCD routine
STATE			equ $8e		;  00 = STOP, 01 = PLAY, 02 = PAUSE
CURRENT_SONG		equ $8f		;  $8f-$90 = pointer to current song
CLOCK_STATE		equ $91		;  $91,$92,$93 = hrs, min, sec
;   --------------------------  NORMAL RAM   -------------------

	;   STACK
; 	org     $0100

;  ------------------------------ PORT MAPPING ------------------
;  Free ports: $8006-$8009
; 
LCD_UART_CTRL_IN	equ $8000	;  Bit 7 = txempty
LCD_UART_CTRL_OUT	equ $8001	;  Bit 7 = send
LCD_UART_DATA_OUT	equ $8002	;  8 bit character
FLASH_CTRL_IN		equ $8003       ;  Bit 7 = stopped play
FLASH_CTRL_OUT		equ $8004       ;  Bit 7 = erase chip - do not drive this high by mistake
			                ;  Bit 6 = NOT USED
                                        ;  Bit 5 = play from flash
                                        ;  Bit 4 = stop play from flash
					;  Bit 3 = pause play from flash
FLASH_DATA_OUT		equ $8005       ;  8 bit to write flash
CLOCK_CTRL_OUT		equ $8006       ;  Bit 7 = reset clock
					;  Bit 6 = hold (pause) clock
					;  Bit 5 = read from clock (update out signals)
CLOCK_HOURS		equ $8007       ;  4+4 bits for reading hours
CLOCK_MINUTES		equ $8008       ;  4+4 bits for reading minutes
CLOCK_SECONDS		equ $8009       ;  4+4 bits for feading seconds
FLASH_PLAY_ADDRESS	equ $800a       ;  $800a-$800d play address
                                        ;  $800a = MSB
FLASH_STOP_ADDRESS	equ $800e       ;  $800e-$8011 stop address
                                        ;  $800e = MSB
BUTTONS			equ $8012	;  Bit 7 = play
					;  Bit 6 = stop
					;  Bit 5 = prev
					;  Bit 4 = next
PC_UART_CTRL_IN		equ $8013	;  Bit 7 = byte available
PC_UART_CTRL_OUT	equ $8014	;  Bit 7 = ack
PC_UART_DATA_IN		equ $8015	;  Byte from host computer
DAC_CTRL_IN		equ $8017	;  Bit 7 = "txempty"/request DAC_DATA
DAC_CTRL_OUT		equ $8018	;  Bit 7 = send DAC_DATA 
DAC_DATA		equ $8019	;  $8019-$801a 2 bytes for DAC
FLASH_WRITE_ADDRESS	equ $8020	;  $8020-$8023 address for flash write
;   --------------------------     CODE      -------------------
	seg     code
	org     $F000
PROMBEG equ     .
 
RESET	subroutine

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This is where the BOOT sequence starts.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
BOOT
	ldx #$ff
	txs
	jsr INIT
	jsr INIT_DAC
	jsr CHECK_PC_CONNECTION
	lda $0400
	sta NUMSONGS
	jsr INIT_METADATA
STOP_IN_STOP_MODE
	jsr PRINT_STATUS

	; Poll the keys
POLL_PLAY
	lda BUTTONS
	bpl POLL_STOP
	; Play was pressed
RLS_PLAY
	lda BUTTONS
	bmi RLS_PLAY
	lda STATE
	cmp #1			; Are we already playing?
	bne RLSP1
	jsr PAUSE_SONG
	lda #2
	sta STATE
	jsr PRINTSTATE
	jmp POLL_STOP
RLSP1
	cmp #2			; Are we pausing?
	bne RLSP2
	jsr UNPAUSE_SONG
	lda #1
	sta STATE
	jsr PRINTSTATE
	jmp POLL_STOP
RLSP2
	lda #1
	sta STATE		; PLAY state
	jsr PLAY_SONG
	jsr PRINT_TITLE
	; Also do something here eventually
	
POLL_STOP	
	lda BUTTONS
	and #$40
	beq POLL_PREV_NEXT
	; Stop was pressed
RLS_STOP
	lda BUTTONS
	and #$40
	bne RLS_STOP
	lda STATE
	beq STOP_IN_STOP_MODE
	jsr STOP_SONG		; Stop it
	lda #0
	sta STATE		; STOP state
	jsr PRINTSTATE

POLL_PREV_NEXT
	lda BUTTONS
	and #$30
	beq POLL_FLASH
	pha
	; Prev or next was pressed
RLS_PREV
	lda BUTTONS
	and #$30
	bne RLS_PREV
	pla
	jsr ADVANCE
POLL_FLASH
	lda FLASH_CTRL_IN	; Check if flash is stopped
	bpl LOOPS
	lda STATE		; And if we are in play mode
	cmp #1
	bne LOOPS
	lda #$10		; Emulate press on "next" button
	jsr ADVANCE
LOOPS
	lda STATE
	cmp #1
	bne LOP1
	jsr PRINT_CLOCK		; Whenever you have time, Mr.
LOP1
	jmp POLL_PLAY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Initialization of environment
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
INIT
	lda #0
	sta STATE
	sta CURRENT_SONG
	sta CURRENT_SONG+1
	sta CLOCK_STATE
	sta CLOCK_STATE+1
	sta CLOCK_STATE+2
	rts
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Initialzation of DAC (copy bytes from memory to DAC)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
DAC_INIT_SEQUENCE
        dc.b	%00000000,%10010111; mute and off input left
	dc.b	%00000010,%10010111; mute and off input right
        dc.b	%00000100,%11111001; maximize headphone left
	dc.b	%00000110,%11111001; maximize headphone right
        dc.b	%00001000,%00011010; DAC on etc
	dc.b	%00001010,%00000100; 44.1 kHz
        dc.b	%00001100,%00000111; Power control
	dc.b	%00001110,%00000010; Data format
        dc.b	%00010000,%10100011; Sample rate
	dc.b	%00010010,%00000001; Activate digital interface
        dc.b	%00010100,%00000000; N/A
	dc.b	%00010110,%00000000; N/A
        dc.b	%00011000,%00000000; N/A
	dc.b	%00011010,%00000000; N/A
        dc.b	%00011100,%00000000; N/A
	dc.b	%00011100,%00000000; And reset!
INIT_DAC
	ldx #0
LOOPD
	lda DAC_CTRL_IN
	bpl LOOPD
	lda DAC_INIT_SEQUENCE,x
	sta DAC_DATA
	inx
	lda DAC_INIT_SEQUENCE,x
	sta DAC_DATA+1
	inx
	lda #128
	sta DAC_CTRL_OUT
	cpx #32
	bne LOOPD
INIT_DAC_RET
	rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  See if the PC wants to upload something
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
CHECK_PC_CONNECTION
	lda BUTTONS
	and #$c0		; Both stop and play pressed at startup
	cmp #$c0
	beq FW_UPLOAD
	rts
FW_UPLOAD
	ldx #0
FW1
	lda LOADER,x		; Copy a loader of max 256 bytes
	sta $0200,x		; to $0200 and start it.
	inx
	bne FW1
	jmp $0200

	;; This is copied to $0200 i.e. not executed here.
	;; Sequence <header> <bytes> ...... <header> <bytes> <eofmark>

	;; BNF
	;; <transfer> ::= <chunk>+ <eofmark>
	;; <chunk>    ::= <header> <bytes>
	;; <header>   ::= $aaaaaaaa $llllllll  a = 32-bit address, l = 32-bit length
	;; <eofmark>  ::= $ffffffff $ffffffff

LOADER
	lda #$80
	sta FLASH_CTRL_OUT	; Erase entire chip!
	ldy #0
L1
	ldx PC_UART_CTRL_IN	; Retrieve start address 4 bytes
	bpl L1			; and length, 4 bytes
	lda PC_UART_DATA_IN
	stx PC_UART_CTRL_OUT	; Ack
	sta TMPZP,y
	iny
	cpy #8
	bne L1
	lda TMPZP		; If address $FFFFFFFF is recieved
	and TMPZP+1
	and TMPZP+2
	and TMPZP+3
	cmp #$ff
	bne L9
	jmp ($FFFC)		; Jump to reset vector
L9
	lda TMPZP
	sta FLASH_WRITE_ADDRESS
	lda TMPZP+1
	sta FLASH_WRITE_ADDRESS+1
	lda TMPZP+2
	sta FLASH_WRITE_ADDRESS+2
	lda TMPZP+3
	sta FLASH_WRITE_ADDRESS+3
L10
	lda PC_UART_CTRL_IN	; Byte available?
	bpl L10
	ldx PC_UART_DATA_IN
	sta PC_UART_CTRL_OUT	; Ack
	stx FLASH_DATA_OUT	; Byte to write
	inc TMPZP+3		; Increase memory address
	bne L11
	inc TMPZP+2
	bne L11
	inc TMPZP+1
	bne L11
	inc TMPZP
L11	dec TMPZP+7		; Count down
	bne L12
	dec TMPZP+6
	bne L12
	dec TMPZP+5
	bne L12
	dec TMPZP+4
L12	lda TMPZP+4		; Until we reach zer0
	ora TMPZP+5
	ora TMPZP+6
	ora TMPZP+7
	bne L9

	beq LOADER		; Get next chunk

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Play control current song
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PLAY_SONG
	ldy #7
	ldx #3
PLA1
	lda (CURRENT_SONG),y
	sta FLASH_PLAY_ADDRESS,x
	sta BUFF32,x
	dey
	dex
	bpl PLA1
	ldy #3
	clc
PLA2
	lda (CURRENT_SONG),y
	adc BUFF32,y
	sta FLASH_STOP_ADDRESS,y
	dey
	bpl PLA2

	; Start the playing
	lda #$20
	sta FLASH_CTRL_OUT
	; Reset the clock
	lda #$80
	sta CLOCK_CTRL_OUT
	jmp PRINT_CLOCK_ERASE
	
PAUSE_SONG			;  Pause it
UNPAUSE_SONG			;  ... or Unpause it
	lda #$20		; send a second "play" command to flash controller
	sta FLASH_CTRL_OUT
	lda #$40		; send a pause/unpause signal to the clock
	sta CLOCK_CTRL_OUT
	rts

STOP_SONG
	lda #$10
	sta FLASH_CTRL_OUT
	rts
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Advance to previous or next
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ADVANCE
	and #$20		; Prev
	beq NEXT_PRESSED
	lda CURRENT_SONG	; In the beginning of the list
	cmp METASTORAGE
	bne NOR1
	lda CURRENT_SONG+1
	cmp METASTORAGE+1
	bne NOR1
	lda METAEND		; Loop around to first song
	sec
	sbc #$20
	sta CURRENT_SONG
	lda METAEND+1
	sbc #0
	sta CURRENT_SONG+1
	jmp PRINT_AND_PLAY
NOR1
	lda CURRENT_SONG
	sec
	sbc #$20
	sta CURRENT_SONG
	lda CURRENT_SONG+1
	sbc #0
	sta CURRENT_SONG+1
	jmp PRINT_AND_PLAY
NEXT_PRESSED
	lda CURRENT_SONG	; Increase pointer
	clc
	adc #$20
	sta CURRENT_SONG
	lda CURRENT_SONG+1
	adc #0
	sta CURRENT_SONG+1
	lda CURRENT_SONG	; Loop around to start if needed
	cmp METAEND
	bne NEXTP1
	lda CURRENT_SONG+1
	cmp METAEND+1
	bne NEXTP1
	ldx METASTORAGE
	ldy METASTORAGE+1
	stx CURRENT_SONG
	sty CURRENT_SONG+1
NEXTP1
	; Fall through
PRINT_AND_PLAY
	jsr PRINT_TITLE
	lda STATE
	cmp #1
	bne NOTPLAY
	jsr STOP_SONG
	jmp PLAY_SONG
NOTPLAY
	rts
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Waits the number of seconds stored in accumulator
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
WAITSECS			; Please implement this. Timer?
	rts
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Clears the screen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
CLRSCR
	lda #12
	jmp LCD_SEND_CHAR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Prints the current time on top
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRINT_CLOCK_ERASE
	; Erase first line and print clock
	lda #16			; Go to first line
	jsr LCD_SEND_CHAR
	lda #64+7		; Clock on top
	jsr LCD_SEND_CHAR
	ldx #8			; Erase including any hour figure.
PRC3
	lda #$20
	jsr LCD_SEND_CHAR
	dex
	bpl PRC3
	jmp PRC2
PRINT_CLOCK
	lda #$20
	sta CLOCK_CTRL_OUT	; Copy state to registers
	lda CLOCK_HOURS
	cmp CLOCK_STATE
	bne PRC2
	lda CLOCK_MINUTES
	cmp CLOCK_STATE+1
	bne PRC2
	lda CLOCK_SECONDS
	cmp CLOCK_STATE+2
	bne PRC2
	rts
PRC2
	lda #16
	jsr LCD_SEND_CHAR
	lda #64+7		; Clock on top
	jsr LCD_SEND_CHAR
	lda CLOCK_HOURS
	beq PRC1
	jsr PRINTBCD8
	lda #58			; ":"
	jsr LCD_SEND_CHAR

PRC1	lda CLOCK_MINUTES
	jsr PRINTBCD8
	lda #58			; ":"
	jsr LCD_SEND_CHAR
	lda CLOCK_SECONDS
	jsr PRINTBCD8
	lda CLOCK_HOURS
	sta CLOCK_STATE
	lda CLOCK_MINUTES
	sta CLOCK_STATE+1
	lda CLOCK_SECONDS
	sta CLOCK_STATE+2
	rts
PRINTBCD8			; Prints a BCD coded clock 2-digit
	pha
	lsr
	lsr
	lsr
	lsr
	clc
	adc #$30
	jsr LCD_SEND_CHAR
	pla
	and #$0f
	clc
	adc #$30
	jmp LCD_SEND_CHAR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Prints the current status: number of songs, memory usage and
;;  play state.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRINT_STATUS
	jsr CLRSCR		; Clear screen
	jsr PRINTSONGS
	lda NUMSONGS
	jsr PRINTDEC
	jsr PRINTMEMUSAGE
	jmp PRINTSTATE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Print title of current song
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRINT_TITLE
	lda #16
	jsr LCD_SEND_CHAR
	lda #64+20		; Go to second row
	jsr LCD_SEND_CHAR
	lda #21
	ldy #8
TITL1
	lda (CURRENT_SONG),y
	beq TITL2
	jsr LCD_SEND_CHAR
	iny
	cpy #28			;  Max on display, actually $20
	bne TITL1
TITL2
	jmp PRINTSTATE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Prints the welcome message
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRINTWELCOME
	ldx #$00
LOOPW
	lda $0201,x
	beq PRINTRET		; Check for zero-termination
	jsr LCD_SEND_CHAR
	inx
	cpx #$1f
	bne LOOPW
PRINTRET
	rts
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Prints machine state
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRINTSTATE
	lda STATE
	bne SPLAY
	ldx #<STOPSTRING
	ldy #>STOPSTRING
	jmp PRINTSTR
SPLAY
	cmp #1
	bne SPAUSE
	ldx #<PLAYSTRING
	ldy #>PLAYSTRING
	jmp PRINTSTR
SPAUSE
	ldx #<PAUSESTRING
	ldy #>PAUSESTRING
	jmp PRINTSTR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Prints "Songs: "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRINTSONGS
	ldx #<SONGSSTRING
	ldy #>SONGSSTRING
	jmp PRINTSTR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  This routine prints the amount of memory used in KB
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRINTMEMUSAGE
	ldx #<STORAGESTRING
	ldy #>STORAGESTRING
	jsr PRINTSTR

	lda METAEND		; Calculate memory usage
	sec			; Back end pointer to last song
	sbc #$20
	sta TMPZP		; TMPZP points to file length
	lda METAEND+1
	sbc #0
	sta TMPZP+1

	lda TMPZP		; Frsk inte vara smart!
	clc
	adc #4
	sta TMPZP+2
	lda TMPZP+1
	adc #0
	sta TMPZP+3

	; Add last offset + length 
	; and subtract metadata length to get total size.
	; total size is stored in TMPZP+4 ... TMPZP+7
	
	ldy #3			; Add offset and song length
	clc
MEM1
	lda (TMPZP),y
	adc (TMPZP+2),y
	sta BUFF32,y
	dey
	bpl MEM1

	lda BUFF32+1		; Subtract metadata size
	sec			
	sbc #1
	sta BUFF32+1
	lda BUFF32
	sbc #0
	sta BUFF32

	; The four bytes at TMPZP+4 ... TMPZP+7 now
	; contain the number of used bytes.

	ldx #9			; Divide by 1024 to get KBytes
MEM2
	lsr BUFF32
	ror BUFF32+1
	ror BUFF32+2
	ror BUFF32+3
	dex
	bpl MEM2
	
	jsr PRINTDEC32
	lda #$20
	jsr LCD_SEND_CHAR
	lda #75
	jsr LCD_SEND_CHAR
	lda #66
	jmp LCD_SEND_CHAR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Printstr routine, takes the string at $yyxx and prints it.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRINTSTR
	stx TMPZP
	sty TMPZP+1
	ldy #$00
PRIN1
	lda (TMPZP),y
	beq PRIN2		; Check for zero-termination
	jsr LCD_SEND_CHAR
	iny
	bne PRIN1
PRIN2
	rts
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Printdec routine, takes the byte in the accumulator and
;;  prints it as a decimal figure.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRINTDEC
	ldx #0
	ldy #0
DEC1
	cmp #100		; greater than 100?
	bcc DEC2
	inx
	sec
	sbc #100
	jmp DEC1
DEC2
	cmp #10			; greater than 10?
	bcc DEC3
	iny
	sec
	sbc #10
	jmp DEC2
DEC3
	pha
	txa			; Skip zeroes
	beq DEC4
	clc
	adc #$30
	jsr LCD_SEND_CHAR
DEC4
	tya
	cpx #0			; Write a "0" if hundreds were printed
	bne DEC6
	tya			; Otherwise skip zeroes
	beq DEC5
DEC6
	clc			; 
	adc #$30
	jsr LCD_SEND_CHAR
DEC5
	pla
	clc
	adc #$30
	jmp LCD_SEND_CHAR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  PRINTDEC32 routine, takes the 4 byte unsigned at $yyxx and
;;  prints it as a decimal figure.
;;  PRINTBCD may be used individually, if desired.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRINTDEC32
	jsr LE2BE		; Convert to big endian
	jsr BIN_BCD32		; Convert to BCD
PRINTBCD
	lda #0
	sta TMPZP		; We did not write anything != 0 yet
	ldy #3
BC1
	lda BCD,y
	lsr
	lsr
	lsr
	lsr
	bne BC4
	ldx TMPZP		; Did we write something != 0 before this?
	beq BC2
BC4
	inc TMPZP		; Now we wrote something != 0
	clc
	adc #$30
	jsr LCD_SEND_CHAR
BC2	
	lda BCD,y
	and #$0f
	bne BC5
	ldx TMPZP		; Did we write something != 0 before this?
	beq BC3
BC5
	inc TMPZP		; Now we wrote something != 0
	clc
	adc #$30
	jsr LCD_SEND_CHAR
BC3
	dey
	bpl BC1
	
	rts

;This program takes a 4 byte binary value from locations BUFF32 to 
;BUFF32+3, converts it to packed BCD and stores it in locations BCD 
;to BCD+3 and puts the dp value in location EXP.

BIN_BCD32
	lda	#0
	sta	BCD
	sta	BCD+1
	sta	BCD+2
	sta	BCD+3
	
	lda     BUFF32+3	;Get MSBY
	and     #$f0		;Filter out low nibble
        lsr			;Move hi nibble right (dp)
        lsr
        lsr
        lsr
        sta     EXP		;store dp
        lda     BUFF32+3
        and     #$0f		;Filter out high nibble
        sta     BUFF32+3
BCD_DP
	ldy     #$00     ;Clear table pointer
NXTDIG
	ldx     #$00     ;Clear digit count
SUB_MEM
	lda     BUFF32   ;Get LSBY of binary value
        sec
        sbc     SUBTBL,y ;Subtract LSBY + y of table value
        sta     BUFF32   ;Return result
        lda     BUFF32+1 ;Get next byte of binary value
        iny
        sbc     SUBTBL,y ;Subtract next byte of table value
        sta     BUFF32+1
        lda     BUFF32+2 ;Get next byte
        iny
        sbc     SUBTBL,y ;Subtract next byte of table
        sta     BUFF32+2
        lda     BUFF32+3 ;Get MSBY of binary value
        iny
        sbc     SUBTBL,y ;Subtract MSBY of table
        bcc     ADBACK   ;If result is neg go add back
        sta     BUFF32+3 ;Store MSBY then point back to LSBY of table
        dey
        dey
        dey
        inx
        jmp     SUB_MEM  ;Go subtract again
ADBACK
	dey			;Point back to LSBY of table
        dey
        dey
        lda     BUFF32   ;Get LSBY of binary value and add LSBY
        adc     SUBTBL,y ;of table value
        sta     BUFF32
        lda     BUFF32+1 ;Get next byte
        iny
        adc     SUBTBL,y ;Add next byte of table
        sta     BUFF32+1
        lda     BUFF32+2 ;Next byte
        iny
        adc     SUBTBL,y ;Add next byte of table
        sta     BUFF32+2
        txa              ;Put dec count in acc
        jsr     BCDREG   ;Put in BCD reg
        iny
        iny
        cpy     #$20     ;End of table?
        bcc     NXTDIG   ;No? go back with next dec weight
        lda     BUFF32   ;Yes? put remainder in acc and put in BCD reg
BCDREG
	asl
        asl
        asl
        asl
        ldx     #$04
	
SHFT_L
	asl
        rol     BCD
        rol     BCD+1
        rol     BCD+2
        rol     BCD+3
        dex
        bne     SHFT_L
        rts

SUBTBL
	dc.b	$00,$e1,$f5,$05
        dc.b	$80,$96,$98,$00
        dc.b	$40,$42,$0f,$00
        dc.b	$a0,$86,$01,$00
        dc.b	$10,$27,$00,$00
        dc.b	$e8,$03,$00,$00
        dc.b	$64,$00,$00,$00
        dc.b	$0a,$00,$00,$00
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Hexdump routine $yyxx a bytes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
HEXDUMP
	stx TMPZP
	sty TMPZP+1
	sta TMPZP+2
	lda #16			; Go to first line
	jsr LCD_SEND_CHAR
	lda #64
	jsr LCD_SEND_CHAR
	ldy #0
HEX1
	lda (TMPZP),y
	lsr
	lsr
	lsr
	lsr
	tax
	lda HEXTBL,x
	jsr LCD_SEND_CHAR
	lda (TMPZP),y
	and #$0f
	tax
	lda HEXTBL,x
	jsr LCD_SEND_CHAR
	iny
	cpy TMPZP+2
	bne HEX1
	rts
HEXTBL
	dc.t "0123456789ABCDEF"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  LE2BE Converts 32bit buffer from little-endian to big-endian
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
LE2BE
	ldx #3
LE3
	lda BUFF32,x
	pha
	dex
	bpl LE3

	ldx #3
LE4	
	pla
	sta BUFF32,x
	dex
	bpl LE4
	rts
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Set up pointers into flash memory
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
INIT_METADATA
	lda NUMSONGS
	cmp #$00		; 0 songs?
	bne META1
	rts
META1
	ldx #$40		; Metadata is here...
	ldy #$04
	stx METASTORAGE		; Set up the pointer in memory
	sty METASTORAGE+1
	stx CURRENT_SONG
	sty CURRENT_SONG+1
	stx METAEND
	sty METAEND+1
				; Calculate end of metadata
	tax
META2
	lda METAEND
	clc
	adc #$20
	sta METAEND
	bcc META3
	inc METAEND+1
META3
	dex
	bne META2

	rts
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Send a char to the LCD lda #char, jsr LCD_SEND_CHAR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
LCD_SEND_CHAR
	pha
LCD_WAIT
	lda LCD_UART_CTRL_IN	;  Wait for empty queue
	beq LCD_WAIT
	pla
	sta LCD_UART_DATA_OUT   ;  Send char
	lda #$80
	sta LCD_UART_CTRL_OUT   ;  Tell UART to push to LCD
	rts
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  IRQ and NMI service routines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
IRQ_ISR
	rti

NMI_ISR
	rti

	; HUKEBUCK
	
IRQ	jmp     IRQ_ISR
NMI	jmp     NMI_ISR

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Hardcoded data
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
STOPSTRING
	dc.b	16, 75+64
	dc.t	"stop "
	dc.b	0
PLAYSTRING
	dc.b	16, 75+64
	dc.t	"play "
	dc.b	0
PAUSESTRING
	dc.b	16, 75+64
	dc.t	"pause"
	dc.b	0
SONGSSTRING
	dc.b	13
	dc.t	"Songs: "
	dc.b	0
STORAGESTRING
	dc.b	13
	dc.t	"Storage: "
	dc.b	0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  JUMP VECTORS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
	seg     vector
	org     $FFFA
	dc.w    NMI
	dc.w    RESET
	dc.w    IRQ

PROMEND     equ     .
 



