------------------------------------------------------------------------------
--
--              ModelVersion : 2.1.4
--
--              Release Date : Dec 03, 1996
--
------------------------------------------------------------------------------
LIBRARY IEEE;                   USE IEEE.STD_LOGIC_1164.ALL;
                                USE IEEE.STD_LOGIC_ARITH.ALL;
                                USE IEEE.VITAL_TIMING.ALL;

use IEEE.std_logic_textio.all;
use IEEE.VITAL_primitives.all;

LIBRARY STD;
use STD.TEXTIO.all;

LIBRARY STVLIB;
use STVLIB.ST_Vss.all;

ARCHITECTURE VIT of SPS2_1024x8m8 IS

--synopsys synthesis_off

    ATTRIBUTE VITAL_LEVEL1 OF VIT : ARCHITECTURE IS FALSE;

    SIGNAL D_ipd : std_logic_vector(7 DOWNTO 0);
    SIGNAL Dint : std_logic_vector(7 DOWNTO 0);
    SIGNAL A_ipd : std_logic_vector(9 DOWNTO 0);
    SIGNAL Aint : std_logic_vector(9 DOWNTO 0);
    SIGNAL CK_ipd : std_logic;
    SIGNAL CKint : std_logic;
    SIGNAL CSN_ipd : std_logic;
    SIGNAL CSNint : std_logic;
    SIGNAL WEN_ipd : std_logic;
    SIGNAL WENint : std_logic;
    SIGNAL OEN_ipd : std_logic;
    SIGNAL OENint : std_logic;
    SIGNAL Dreg : std_logic_vector(7 DOWNTO 0);
    SIGNAL Areg : std_logic_vector(9 DOWNTO 0);
    SIGNAL CKreg : std_logic;
    SIGNAL CSNreg : std_logic;
    SIGNAL WENreg : std_logic;
    SIGNAL OENreg : std_logic;
    SIGNAL Qint : std_logic_vector(7 DOWNTO 0);
    SIGNAL TimingViol : bit_vector(10 DOWNTO 0) := (OTHERS => '0');

    FUNCTION AddrFilter(Input : std_logic_vector) RETURN std_logic_vector IS
        VARIABLE UFound : Boolean := FALSE;
        VARIABLE Output : std_logic_vector(Input'RANGE);
    BEGIN
        FOR i IN Input'RANGE LOOP
            IF (Input(i) = 'U') THEN
                UFound := TRUE;
                EXIT;
            END IF;
        END LOOP;
        IF (UFound) THEN
            Output := (OTHERS => 'U');
        ELSIF (is_x(Input)) THEN
            Output := (OTHERS => 'X');
        ELSE
            Output := to_x01(Input);
        END IF;
        RETURN Output;
    END AddrFilter;

--synopsys synthesis_on

BEGIN

--synopsys synthesis_off

---------------------------------Wire Delay Block------------------------
    WireDelay : BLOCK
        BEGIN
            GenD : FOR i IN D'RANGE GENERATE
                VitalWireDelay(OutSig => D_ipd(i),
                               InSig => D(i),
                               twire => tipd_D(i));
            END GENERATE;
            GenA : FOR i IN A'RANGE GENERATE
                VitalWireDelay(OutSig => A_ipd(i),
                               InSig => A(i),
                               twire => tipd_A(i));
            END GENERATE;
            VitalWireDelay(OutSig => CK_ipd,
                           InSig => CK,
                           twire => tipd_CK);
            VitalWireDelay(OutSig => CSN_ipd,
                           InSig => CSN,
                           twire => tipd_CSN);
            VitalWireDelay(OutSig => WEN_ipd,
                           InSig => WEN,
                           twire => tipd_WEN);
            VitalWireDelay(OutSig => OEN_ipd,
                           InSig => OEN,
                           twire => tipd_OEN);
        END BLOCK;
--------------------------------- Filter Block ---------------------------
    Filter : BLOCK
        BEGIN
            Dint <= to_ux01(D_ipd);
            CKint <= to_ux01(CK_ipd);
            CSNint <= to_ux01(CSN_ipd);
            WENint <= to_ux01(WEN_ipd);
            OENint <= to_ux01(OEN_ipd);
            Aint <= AddrFilter(A_ipd);
        END BLOCK;
--------------------------------Timing Checks Process--------------------
    TimingChecks : PROCESS(
                           Dint,
                           Aint,
                           CKint,
                           CSNint,
                           WENint
                          )
        VARIABLE TM_E01 : VitalTimingDataType := VitalTimingDataInit;
        VARIABLE TM_E02 : VitalTimingDataType := VitalTimingDataInit;
        VARIABLE TM_E03 : VitalTimingDataType := VitalTimingDataInit;
        VARIABLE TM_E04 : VitalTimingDataType := VitalTimingDataInit;
        VARIABLE TM_E05 : VitalTimingDataType := VitalTimingDataInit;
        VARIABLE TM_E06 : VitalTimingDataType := VitalTimingDataInit;
        VARIABLE TM_E07 : VitalTimingDataType := VitalTimingDataInit;
        VARIABLE TM_E08 : VitalTimingDataType := VitalTimingDataInit;
        VARIABLE PPW_CK_E09 : VitalPeriodDataType;
        VARIABLE PPW_CK_E10 : VitalPeriodDataType;
        VARIABLE PPW_CK_E11 : VitalPeriodDataType;
        VARIABLE Notifiers : std_logic_vector(10 DOWNTO 0) := (OTHERS => '0');
    BEGIN
        IF TimingChecksOn THEN
-------------------------------- Setup Checks ----------------------

            VitalSetupHoldCheck(
                TestSignal      => Aint,
                TestSignalName  => "A",
                TestDelay       => 0 ns,
                RefSignal       => CKint,
                RefSignalName   => "CK",
                RefDelay        => 0 ns,
                SetupHigh       => TSETUP_A_CK_NOEDGE_posedge(0),
                SetupLow        => TSETUP_A_CK_NOEDGE_posedge(0),
                HoldHigh        => 0 ns,
                HoldLow         => 0 ns,
                CheckEnabled    => CKint'EVENT  AND (CSNint /= '1'),
                RefTransition   => '/',
                MsgOn           => MsgOn,
                XOn             => XOn,
                HeaderMsg       => InstancePath,
                TimingData      => TM_E01,
                MsgSeverity     => WARNING,
                Violation       => Notifiers(0));

            VitalSetupHoldCheck(
                TestSignal      => Dint,
                TestSignalName  => "D",
                TestDelay       => 0 ns,
                RefSignal       => CKint,
                RefSignalName   => "CK",
                RefDelay        => 0 ns,
                SetupHigh       => TSETUP_D_CK_NOEDGE_posedge(0),
                SetupLow        => TSETUP_D_CK_NOEDGE_posedge(0),
                HoldHigh        => 0 ns,
                HoldLow         => 0 ns,
                CheckEnabled    => CKint'EVENT  AND (CSNint /= '1') AND WENint /= '1',
                RefTransition   => '/',
                MsgOn           => MsgOn,
                XOn             => XOn,
                HeaderMsg       => InstancePath,
                TimingData      => TM_E03,
                MsgSeverity     => WARNING,
                Violation       => Notifiers(2));

            VitalSetupHoldCheck(
                TestSignal      => CSNint,
                TestSignalName  => "CSN",
                TestDelay       => 0 ns,
                RefSignal       => CKint,
                RefSignalName   => "CK",
                RefDelay        => 0 ns,
                SetupHigh       => TSETUP_CSN_CK_NOEDGE_posedge,
                SetupLow        => TSETUP_CSN_CK_NOEDGE_posedge,
                HoldHigh        => 0 ns,
                HoldLow         => 0 ns,
                CheckEnabled    => CKint'EVENT  AND (TRUE),
                RefTransition   => '/',
                MsgOn           => MsgOn,
                XOn             => XOn,
                HeaderMsg       => InstancePath,
                TimingData      => TM_E05,
                MsgSeverity     => WARNING,
                Violation       => Notifiers(4));

            VitalSetupHoldCheck(
                TestSignal      => WENint,
                TestSignalName  => "WEN",
                TestDelay       => 0 ns,
                RefSignal       => CKint,
                RefSignalName   => "CK",
                RefDelay        => 0 ns,
                SetupHigh       => TSETUP_WEN_CK_NOEDGE_posedge,
                SetupLow        => TSETUP_WEN_CK_NOEDGE_posedge,
                HoldHigh        => 0 ns,
                HoldLow         => 0 ns,
                CheckEnabled    => CKint'EVENT  AND (CSNint /= '1'),
                RefTransition   => '/',
                MsgOn           => MsgOn,
                XOn             => XOn,
                HeaderMsg       => InstancePath,
                TimingData      => TM_E07,
                MsgSeverity     => WARNING,
                Violation       => Notifiers(6));

-------------------------------- Hold Checks ----------------------

            VitalSetupHoldCheck(
                TestSignal      => Aint,
                TestSignalName  => "A",
                TestDelay       => 0 ns,
                RefSignal       => CKint,
                RefSignalName   => "CK",
                RefDelay        => 0 ns,
                SetupHigh       => 0 ns,
                SetupLow        => 0 ns,
                HoldHigh        => THOLD_A_CK_NOEDGE_posedge(0),
                HoldLow         => THOLD_A_CK_NOEDGE_posedge(0),
                CheckEnabled    => Aint'EVENT  AND (CSNreg /= '1'),
                RefTransition   => '/',
                MsgOn           => MsgOn,
                XOn             => XOn,
                HeaderMsg       => InstancePath,
                TimingData      => TM_E02,
                MsgSeverity     => WARNING,
                Violation       => Notifiers(1));

            VitalSetupHoldCheck(
                TestSignal      => Dint,
                TestSignalName  => "D",
                TestDelay       => 0 ns,
                RefSignal       => CKint,
                RefSignalName   => "CK",
                RefDelay        => 0 ns,
                SetupHigh       => 0 ns,
                SetupLow        => 0 ns,
                HoldHigh        => THOLD_D_CK_NOEDGE_posedge(0),
                HoldLow         => THOLD_D_CK_NOEDGE_posedge(0),
                CheckEnabled    => Dint'EVENT  AND (CSNreg /= '1') AND WENreg /= '1',
                RefTransition   => '/',
                MsgOn           => MsgOn,
                XOn             => XOn,
                HeaderMsg       => InstancePath,
                TimingData      => TM_E04,
                MsgSeverity     => WARNING,
                Violation       => Notifiers(3));

            VitalSetupHoldCheck(
                TestSignal      => CSNint,
                TestSignalName  => "CSN",
                TestDelay       => 0 ns,
                RefSignal       => CKint,
                RefSignalName   => "CK",
                RefDelay        => 0 ns,
                SetupHigh       => 0 ns,
                SetupLow        => 0 ns,
                HoldHigh        => THOLD_CSN_CK_NOEDGE_posedge,
                HoldLow         => THOLD_CSN_CK_NOEDGE_posedge,
                CheckEnabled    => CSNint'EVENT  AND (TRUE),
                RefTransition   => '/',
                MsgOn           => MsgOn,
                XOn             => XOn,
                HeaderMsg       => InstancePath,
                TimingData      => TM_E06,
                MsgSeverity     => WARNING,
                Violation       => Notifiers(5));

            VitalSetupHoldCheck(
                TestSignal      => WENint,
                TestSignalName  => "WEN",
                TestDelay       => 0 ns,
                RefSignal       => CKint,
                RefSignalName   => "CK",
                RefDelay        => 0 ns,
                SetupHigh       => 0 ns,
                SetupLow        => 0 ns,
                HoldHigh        => THOLD_WEN_CK_NOEDGE_posedge,
                HoldLow         => THOLD_WEN_CK_NOEDGE_posedge,
                CheckEnabled    => WENint'EVENT  AND (CSNreg /= '1'),
                RefTransition   => '/',
                MsgOn           => MsgOn,
                XOn             => XOn,
                HeaderMsg       => InstancePath,
                TimingData      => TM_E08,
                MsgSeverity     => WARNING,
                Violation       => Notifiers(7));

------------------PulseWidth and Period Checks ----------------------

            VitalPeriodPulseCheck(
                TestSignal      => CKint,
                TestSignalName  => "CK",
                TestDelay       => 0 ns,
                Period          => 0 ns,
                PulseWidthHigh  => TPW_CK_posedge,
                PulseWidthLow   => 0 ns,
                CheckEnabled    => (CKint'EVENT) AND CKint = '0' AND CSNreg /= '1',
                PeriodData      => PPW_CK_E09,
                MsgOn           => MsgOn,
                XOn             => XOn,
                HeaderMsg       => InstancePath,
                MsgSeverity     => WARNING,
                Violation       => Notifiers(8));


            VitalPeriodPulseCheck(
                TestSignal      => CKint,
                TestSignalName  => "CK",
                TestDelay       => 0 ns,
                Period          => 0 ns,
                PulseWidthHigh  => 0 ns,
                PulseWidthLow   => TPW_CK_negedge,
                CheckEnabled    => (CKint'EVENT) AND CKint = '1' AND CSNint /= '1',
                PeriodData      => PPW_CK_E10,
                MsgOn           => MsgOn,
                XOn             => XOn,
                HeaderMsg       => InstancePath,
                MsgSeverity     => WARNING,
                Violation       => Notifiers(9));


            VitalPeriodPulseCheck(
                TestSignal      => CKint,
                TestSignalName  => "CK",
                TestDelay       => 0 ns,
                Period          => TPERIOD_CK,
                PulseWidthHigh  => 0 ns,
                PulseWidthLow   => 0 ns,
                CheckEnabled    => (CKint'EVENT) AND CKint = '1' AND CSNint /= '1',
                PeriodData      => PPW_CK_E11,
                MsgOn           => MsgOn,
                XOn             => XOn,
                HeaderMsg       => InstancePath,
                MsgSeverity     => WARNING,
                Violation       => Notifiers(10));

        END IF;


        IF (CKint'EVENT AND CKint /= '0') THEN
            Areg <= Aint;
            Dreg <= Dint;
            CSNreg <= CSNint;
            WENreg <= WENint;
        END IF;

        CKreg <= CKint;

        FOR i IN Notifiers'RANGE LOOP
            IF Notifiers(i) = 'X' THEN
                TimingViol(i) <= NOT TimingViol(i);
            END IF;
        END LOOP;

    END PROCESS TimingChecks;

    ReadWrite : PROCESS (CKreg, TimingViol, WENint, CSNint)

        TYPE MemType IS ARRAY (integer RANGE <>) OF std_logic_vector(D'RANGE);
        VARIABLE Mem : MemType(1023 DOWNTO 0);
        CONSTANT WordX : std_logic_vector(D'RANGE) := (OTHERS => 'X');
        CONSTANT WordU : std_logic_vector(D'RANGE) := (OTHERS => 'U');

----------------- READ, WRITE AND MEMORY CORRUPTION PROCEDURES -------

        PROCEDURE WriteOut(CONSTANT tCK_Q_HZ, tCK_Q_R: IN Time;
            CONSTANT C_DATA: IN std_logic_vector;
            CONSTANT RefTime: IN Time;
            SIGNAL S_Q: OUT std_logic_vector) IS

            VARIABLE V_delta : Time := 0 ns;
            VARIABLE V_allx  : Boolean := TRUE;
            CONSTANT C_wordx : std_logic_vector(C_DATA'RANGE) := (OTHERS => 'X');

        BEGIN
            V_allx := (C_DATA = C_wordx);

            IF V_allx THEN
                V_DELTA := tCK_Q_HZ - RefTime;
            ELSE
                V_DELTA := tCK_Q_R - RefTime;
            END IF;

            IF (V_DELTA < 0 ns) THEN
                S_Q <= TRANSPORT C_DATA;
            ELSE
                S_Q <= TRANSPORT C_DATA AFTER V_DELTA;
            END IF;
        END WriteOut;

        PROCEDURE ReadCycle (CONSTANT tCK_Q_HZ,tCK_Q_R : IN Time;
                             CONSTANT Address : IN std_logic_vector;
                             CONSTANT RefTime : IN Time;
                             SIGNAL S_Q : OUT std_logic_vector(D'RANGE)) IS

            VARIABLE V_Location : integer;

        BEGIN

            WriteOut(tCK_Q_HZ => tCK_Q_HZ,
                     tCK_Q_R  => tCK_Q_R,
                     C_Data   => WordX,
                     RefTime  => RefTime,
                     S_Q      => S_Q);

            CASE Address(0) IS
                WHEN 'U' =>
                    WriteOut(tCK_Q_HZ => tCK_Q_HZ,
                             tCK_Q_R  => tCK_Q_R,
                             C_Data   => WordU,
                             RefTime  => RefTime,
                             S_Q      => S_Q);

                WHEN 'X' =>
                    ASSERT FALSE
                        REPORT "(" & InstancePath & "-1)" &
                               " Illegal Address bit during Read cycle"
                        SEVERITY WARNING;

                WHEN OTHERS =>
                    V_Location := conv_integer(unsigned(Address));
                    IF (V_Location >= 1024) THEN
                        ASSERT FALSE
                            REPORT  "(" & InstancePath & "-2)" &
                                    " Address out of range during Read cycle"
                            SEVERITY WARNING;
                    ELSE
                        WriteOut(tCK_Q_HZ => tCK_Q_HZ,
                                 tCK_Q_R  => tCK_Q_R,
                                 C_Data   => (MEM(V_Location)),
                                 RefTime  => RefTime,
                                 S_Q      => S_Q);
                    END IF;

            END CASE;

        END ReadCycle;

        PROCEDURE WriteCycle (CONSTANT tCK_Q_HZ,tCK_Q_R : IN Time;
                              CONSTANT Address : IN std_logic_vector;
                              CONSTANT Data : IN std_logic_vector(D'RANGE);
                              CONSTANT RefTime : IN Time;
                              SIGNAL S_Q : OUT std_logic_vector) IS

            VARIABLE V_Location : integer;

        BEGIN

            WriteOut(tCK_Q_HZ => tCK_Q_HZ,
                     tCK_Q_R  => tCK_Q_R,
                     C_Data   => WordX,
                     RefTime  => RefTime,
                     S_Q      => S_Q);

            WriteOut(tCK_Q_HZ => tCK_Q_HZ,
                     tCK_Q_R  => tCK_Q_R,
                     C_Data   => Data,
                     RefTime  => RefTime,
                     S_Q      => S_Q);

            CASE Address(0) IS
                WHEN 'U' =>
                    MEM := (OTHERS => (OTHERS=>'U'));

                WHEN 'X' =>
                    ASSERT FALSE
                        REPORT "(" & InstancePath & "-3) " &
                               " Illegal Address bit during Write cycle"
                        SEVERITY WARNING;
                    MEM := (OTHERS => (OTHERS=>'X'));

                WHEN OTHERS =>
                    V_Location := conv_integer(unsigned(Address));
                    IF (V_Location >= 1024) THEN
                        ASSERT FALSE
                            REPORT  "(" & InstancePath & "-4) " &
                                    " Address out of range during Write cycle"
                            SEVERITY WARNING;
                    ELSE
                        MEM(V_Location) := Data;
                    END IF;
            END CASE;

        END WriteCycle;

        PROCEDURE CorruptMem(CONSTANT Address : IN std_logic_vector(A'RANGE);
                             CONSTANT Data : IN std_logic_vector(D'RANGE)) IS

            VARIABLE V_Location : integer;

        BEGIN

            CASE Address(0) IS
                WHEN 'U' =>
                    MEM := (OTHERS => (OTHERS=>'U'));

                WHEN 'X' =>
                    MEM := (OTHERS => (OTHERS=>'X'));

                WHEN OTHERS =>
                    V_Location := conv_integer(unsigned(Address));
                    IF (V_Location < 1024) THEN
                        MEM(V_Location) := Data;
                    END IF;
            END CASE;

        END CorruptMem;

-----------------------------------------------------------

    BEGIN

        IF (CKreg'EVENT AND CSNreg /= '1') THEN
            IF rising_edge(CKreg) THEN
                IF (WENreg = '0' AND CSNreg = '0') THEN
                    WriteCycle(tCK_Q_HZ => TPD_CK_Q_NT_WEN(0)(tr0z),
                               tCK_Q_R  => TPD_CK_Q_NT_WEN(0)(tr01),
                               Address  => Areg,
                               Data     => Dreg,
                               RefTime  => CKreg'LAST_EVENT,
                               S_Q      => Qint);
                ELSIF (WENreg = '1' AND CSNreg = '0') THEN
                    ReadCycle(tCK_Q_HZ => TPD_CK_Q(0)(tr0z),
                              tCK_Q_R  => TPD_CK_Q(0)(tr01),
                              Address  => Areg,
                              RefTime  => CKreg'LAST_EVENT,
                              S_Q      => Qint);
                ELSIF ((CSNreg = 'U' AND WENreg /= '1') OR
                       (WENreg = 'U' AND CSNreg /= '1')) THEN
                    WriteOut(tCK_Q_HZ => TPD_CK_Q_NT_WEN(0)(tr0z),
                             tCK_Q_R  => TPD_CK_Q_NT_WEN(0)(tr0z),
                             C_Data   => WordU,
                             RefTime  => CKreg'LAST_EVENT,
                             S_Q      => Qint);
                    CorruptMem(Areg, (OTHERS => 'U'));

                ELSIF (WENreg = '1') THEN
                    IF (CSNreg = 'U') THEN
                        WriteOut(tCK_Q_HZ => TPD_CK_Q(0)(tr0z),
                                 tCK_Q_R  => TPD_CK_Q(0)(tr0z),
                                 C_Data   => WordU,
                                 RefTime  => CKreg'LAST_EVENT,
                                 S_Q      => Qint);
                    ELSIF (CSNreg = 'X') THEN
                        WriteOut(tCK_Q_HZ => TPD_CK_Q(0)(tr0z),
                                 tCK_Q_R  => TPD_CK_Q(0)(tr01),
                                 C_Data   => WordX,
                                 RefTime  => CKreg'LAST_EVENT,
                                 S_Q      => Qint);
                    END IF;
                ELSIF ((WENreg = 'X' AND CSNreg /= '1') OR
                       (WENreg /= '1' AND CSNreg = 'X')) THEN
                    WriteOut(tCK_Q_HZ => TPD_CK_Q_NT_WEN(0)(tr0z),
                             tCK_Q_R  => TPD_CK_Q_NT_WEN(0)(tr01),
                             C_Data   => WordX,
                             RefTime  => CKreg'LAST_EVENT,
                             S_Q      => Qint);
                    CorruptMem(Areg, (OTHERS => 'X'));
                END IF;
            ELSIF is_x(CKreg) THEN
                IF (CKreg = 'X') THEN
                    WriteOut(tCK_Q_HZ => TPD_CK_Q_NT_WEN(0)(tr0z),
                             tCK_Q_R  => TPD_CK_Q_NT_WEN(0)(tr01),
                             C_Data   => WordX,
                             RefTime  => CKreg'LAST_EVENT,
                             S_Q      => Qint);
                ELSE
                    WriteOut(tCK_Q_HZ => TPD_CK_Q_NT_WEN(0)(tr0z),
                             tCK_Q_R  => TPD_CK_Q_NT_WEN(0)(tr0z),
                             C_Data   => WordU,
                             RefTime  => CKreg'LAST_EVENT,
                             S_Q      => Qint);
                END IF;
                IF (WENreg /= '1') THEN
                    CorruptMem((OTHERS => CKreg), (OTHERS => CKreg));
                END IF;
            END IF;
        END IF;

        IF (WENint'EVENT) THEN
            IF (WENint = '0') THEN
                IF ((CSNint /= '1') AND is_x(CKint)) THEN
                    CorruptMem((OTHERS => CKint), (OTHERS => CKint));
                END IF;
            END IF;
        END IF;

        IF (CSNint'EVENT) THEN
            IF ((CSNint = '0') AND is_x(CKint)) THEN
                Qint <= TRANSPORT (OTHERS => CKint) AFTER TPD_CK_Q(0)(tr0z);
                IF (WENint /= '1') THEN
                    CorruptMem((OTHERS => CKint), (OTHERS => CKint));
                END IF;
            END IF;
        END IF;

        IF (TimingViol'EVENT) THEN
            IF (TimingViol(4)'EVENT OR TimingViol(5)'EVENT OR TimingViol(8)'EVENT OR
                TimingViol(9)'EVENT OR TimingViol(10)'EVENT) THEN
                WriteOut(tCK_Q_HZ => TPD_CK_Q_NT_WEN(0)(tr0z),
                         tCK_Q_R  => TPD_CK_Q_NT_WEN(0)(tr01),
                         C_Data   => WordX,
                         RefTime  => CKreg'LAST_EVENT,
                         S_Q      => Qint);
                IF (WENreg /= '1') THEN
                    CorruptMem((OTHERS => 'X'), (OTHERS => 'X'));
                END IF;
            ELSIF (TimingViol(0)'EVENT OR TimingViol(1)'EVENT) THEN
                IF (WENreg = '1') THEN
                    WriteOut(tCK_Q_HZ => TPD_CK_Q_NT_WEN(0)(tr0z),
                             tCK_Q_R  => TPD_CK_Q_NT_WEN(0)(tr01),
                             C_Data   => WordX,
                             RefTime  => CKreg'LAST_EVENT,
                             S_Q      => Qint);
                ELSE
                    CorruptMem((OTHERS => 'X'), (OTHERS => 'X'));
                END IF;
            ELSIF (TimingViol(2)'EVENT OR TimingViol(3)'EVENT) THEN
                IF (WENreg /= '1') THEN
                    WriteOut(tCK_Q_HZ => TPD_CK_Q_NT_WEN(0)(tr0z),
                             tCK_Q_R  => TPD_CK_Q_NT_WEN(0)(tr01),
                             C_Data   => WordX,
                             RefTime  => CKreg'LAST_EVENT,
                             S_Q      => Qint);
                    CorruptMem(Areg, (OTHERS => 'X'));
                END IF;
            ELSIF (TimingViol(6)'EVENT OR TimingViol(7)'EVENT) THEN
                WriteOut(tCK_Q_HZ => TPD_CK_Q_NT_WEN(0)(tr0z),
                         tCK_Q_R  => TPD_CK_Q_NT_WEN(0)(tr01),
                         C_Data   => WordX,
                         RefTime  => CKreg'LAST_EVENT,
                         S_Q      => Qint);
                CorruptMem(Areg, (OTHERS => 'X'));
            END IF;
        END IF;

    END PROCESS ReadWrite;

---------- Functionality due to the Output Enable OEN -----------
    OENregProc : PROCESS (OENint)
    BEGIN
        IF (OENint = '0') THEN
            OENreg <= '0' AFTER TPD_OEN_Q(0)(tr01);
        ELSE
            OENreg <= OENint AFTER TPD_OEN_Q(0)(tr0z);
        END IF;
    END PROCESS OENregProc;

    QProc : PROCESS (OENreg, Qint)
    BEGIN
        IF (OENreg = '1') THEN
            Q <= (OTHERS => 'Z');
        ELSIF (OENreg = '0') THEN
            Q <= Qint;
        ELSIF (OENreg = 'U') THEN
            Q <= (OTHERS => 'U');
        ELSE
            Q <= (OTHERS => 'X');
        END IF;

    END PROCESS QProc;

----------------------------------------------------------------------
--
--                      WARNING MESSAGES LIST
--                      =====================
--
-- (InstancePath-1)  Illegal Address bit during Read cycle
--
-- (InstancePath-2)  Address out of range during Read cycle
--
-- (InstancePath-3)  Illegal Address bit during Write cycle
--
-- (InstancePath-4)  Address out of range during Write cycle
--
----------------------------------------------------------------------

--synopsys synthesis_on

END VIT;
