/*********************************************************************
--
--              ModelVersion : 2.0.5
--
--              Release Date : Oct 31, 1996
--
*********************************************************************/

`celldefine
`suppress_faults
`enable_portfaults
`ifdef functional
    `timescale 1ns / 1ns
    `delay_mode_distributed
    `delay_mode_unit
`else
    `timescale 1ns / 10ps
    `delay_mode_path
`endif

module SPS2_1024x8m8 (Q, CK, CSN, WEN, OEN, A, D);

    parameter
        Words = 1024,
        Bits = 8,
        Addr = 10;

    parameter
        WordX = 8'bx,
        AddrX = 10'bx,
        X = 1'bx;


    output [Bits-1 : 0] Q;
    input [Bits-1 : 0] D;
    input [Addr-1 : 0] A;
    input CK;
    input CSN;
    input WEN;
    input OEN;

    wire [Bits-1 : 0] Qint;
    wire [Bits-1 : 0] Dint;
    wire [Addr-1 : 0] Aint;
    wire CKint;
    wire CSNint;
    wire WENint;

    reg [Bits-1 : 0] OutReg; // Output register
    reg [Bits-1 : 0] Dreg;
    reg [Addr-1 : 0] Areg;
    reg lastCK, CKreg;
    reg CSNreg;
    reg WENreg;

    reg TimingViol0, TimingViol1, TimingViol2, TimingViol3;
    reg TimingViol4, TimingViol5, TimingViol6, TimingViol7;
    reg TimingViol8, TimingViol9, TimingViol10;

    reg [Bits-1 : 0] Mem [Words-1 : 0]; // RAM array

    reg Unk;
    reg ValidAddress;
    reg StopOutput;

    wire UnkCK = ((CKreg === X && CSNint !== 1) || (CSNint === 0 && CKint === X));
    wire Flash = (WENint === 0 && CKint === X && CSNint !== 1);
    integer i;

    bufif0 #1 (Q[0], Qint[0], OEN);
    bufif0 #1 (Q[1], Qint[1], OEN);
    bufif0 #1 (Q[2], Qint[2], OEN);
    bufif0 #1 (Q[3], Qint[3], OEN);
    bufif0 #1 (Q[4], Qint[4], OEN);
    bufif0 #1 (Q[5], Qint[5], OEN);
    bufif0 #1 (Q[6], Qint[6], OEN);
    bufif0 #1 (Q[7], Qint[7], OEN);

    buf (Dint[0], D[0]);
    buf (Dint[1], D[1]);
    buf (Dint[2], D[2]);
    buf (Dint[3], D[3]);
    buf (Dint[4], D[4]);
    buf (Dint[5], D[5]);
    buf (Dint[6], D[6]);
    buf (Dint[7], D[7]);

    buf (Aint[0], A[0]);
    buf (Aint[1], A[1]);
    buf (Aint[2], A[2]);
    buf (Aint[3], A[3]);
    buf (Aint[4], A[4]);
    buf (Aint[5], A[5]);
    buf (Aint[6], A[6]);
    buf (Aint[7], A[7]);
    buf (Aint[8], A[8]);
    buf (Aint[9], A[9]);

    buf (CKint, CK);
    buf (CSNint, CSN);
    buf (WENint, WEN);


`ifdef functional
`else

    not (CS, CSN);
    not (WE, WEN);
    and (CSWE, CS, WE);

    specify

        specparam
            tcycle = 8.540,
            tckh = 1.156,
            tckl = 2.768,
            tps = 5.234,
            tph = 2.700,
            tws = 1.971,
            twh = 0.010,
            tas = 3.005,
            tah = 0.010,
            tds = 4.893,
            tdh = 0.010,
            taa = 3.888,
            taaw = 2.733,
            tlz = 0.799,
            thz = 0.974,
            th = 1.182;

        /*-------------------- Propagation Delays ------------------*/

        if (WENreg) (posedge CK *> (Q[0] : D[0])) = (taa, taa, th, taa, th, taa);
        if (WENreg) (posedge CK *> (Q[1] : D[1])) = (taa, taa, th, taa, th, taa);
        if (WENreg) (posedge CK *> (Q[2] : D[2])) = (taa, taa, th, taa, th, taa);
        if (WENreg) (posedge CK *> (Q[3] : D[3])) = (taa, taa, th, taa, th, taa);
        if (WENreg) (posedge CK *> (Q[4] : D[4])) = (taa, taa, th, taa, th, taa);
        if (WENreg) (posedge CK *> (Q[5] : D[5])) = (taa, taa, th, taa, th, taa);
        if (WENreg) (posedge CK *> (Q[6] : D[6])) = (taa, taa, th, taa, th, taa);
        if (WENreg) (posedge CK *> (Q[7] : D[7])) = (taa, taa, th, taa, th, taa);

        if (!WENreg) (posedge CK *> (Q[0] : D[0])) = (taaw, taaw, th, taaw, th, taaw);
        if (!WENreg) (posedge CK *> (Q[1] : D[1])) = (taaw, taaw, th, taaw, th, taaw);
        if (!WENreg) (posedge CK *> (Q[2] : D[2])) = (taaw, taaw, th, taaw, th, taaw);
        if (!WENreg) (posedge CK *> (Q[3] : D[3])) = (taaw, taaw, th, taaw, th, taaw);
        if (!WENreg) (posedge CK *> (Q[4] : D[4])) = (taaw, taaw, th, taaw, th, taaw);
        if (!WENreg) (posedge CK *> (Q[5] : D[5])) = (taaw, taaw, th, taaw, th, taaw);
        if (!WENreg) (posedge CK *> (Q[6] : D[6])) = (taaw, taaw, th, taaw, th, taaw);
        if (!WENreg) (posedge CK *> (Q[7] : D[7])) = (taaw, taaw, th, taaw, th, taaw);

        if (!CSN) (negedge CSN *> (Q[0] : D[0])) = (th, th);
        if (!CSN) (negedge CSN *> (Q[1] : D[1])) = (th, th);
        if (!CSN) (negedge CSN *> (Q[2] : D[2])) = (th, th);
        if (!CSN) (negedge CSN *> (Q[3] : D[3])) = (th, th);
        if (!CSN) (negedge CSN *> (Q[4] : D[4])) = (th, th);
        if (!CSN) (negedge CSN *> (Q[5] : D[5])) = (th, th);
        if (!CSN) (negedge CSN *> (Q[6] : D[6])) = (th, th);
        if (!CSN) (negedge CSN *> (Q[7] : D[7])) = (th, th);

        (OEN *> Q[0]) = (tlz, tlz, thz, tlz, thz, tlz);
        (OEN *> Q[1]) = (tlz, tlz, thz, tlz, thz, tlz);
        (OEN *> Q[2]) = (tlz, tlz, thz, tlz, thz, tlz);
        (OEN *> Q[3]) = (tlz, tlz, thz, tlz, thz, tlz);
        (OEN *> Q[4]) = (tlz, tlz, thz, tlz, thz, tlz);
        (OEN *> Q[5]) = (tlz, tlz, thz, tlz, thz, tlz);
        (OEN *> Q[6]) = (tlz, tlz, thz, tlz, thz, tlz);
        (OEN *> Q[7]) = (tlz, tlz, thz, tlz, thz, tlz);

        /*---------------------- Timing Checks ---------------------*/

        $setup(A[0], posedge CK &&& CS, tas, TimingViol0);
        $setup(A[1], posedge CK &&& CS, tas, TimingViol0);
        $setup(A[2], posedge CK &&& CS, tas, TimingViol0);
        $setup(A[3], posedge CK &&& CS, tas, TimingViol0);
        $setup(A[4], posedge CK &&& CS, tas, TimingViol0);
        $setup(A[5], posedge CK &&& CS, tas, TimingViol0);
        $setup(A[6], posedge CK &&& CS, tas, TimingViol0);
        $setup(A[7], posedge CK &&& CS, tas, TimingViol0);
        $setup(A[8], posedge CK &&& CS, tas, TimingViol0);
        $setup(A[9], posedge CK &&& CS, tas, TimingViol0);
        $hold(posedge CK &&& CS, A[0], tah, TimingViol1);
        $hold(posedge CK &&& CS, A[1], tah, TimingViol1);
        $hold(posedge CK &&& CS, A[2], tah, TimingViol1);
        $hold(posedge CK &&& CS, A[3], tah, TimingViol1);
        $hold(posedge CK &&& CS, A[4], tah, TimingViol1);
        $hold(posedge CK &&& CS, A[5], tah, TimingViol1);
        $hold(posedge CK &&& CS, A[6], tah, TimingViol1);
        $hold(posedge CK &&& CS, A[7], tah, TimingViol1);
        $hold(posedge CK &&& CS, A[8], tah, TimingViol1);
        $hold(posedge CK &&& CS, A[9], tah, TimingViol1);

        $setup(D[0], posedge CK &&& CSWE, tds, TimingViol2);
        $setup(D[1], posedge CK &&& CSWE, tds, TimingViol2);
        $setup(D[2], posedge CK &&& CSWE, tds, TimingViol2);
        $setup(D[3], posedge CK &&& CSWE, tds, TimingViol2);
        $setup(D[4], posedge CK &&& CSWE, tds, TimingViol2);
        $setup(D[5], posedge CK &&& CSWE, tds, TimingViol2);
        $setup(D[6], posedge CK &&& CSWE, tds, TimingViol2);
        $setup(D[7], posedge CK &&& CSWE, tds, TimingViol2);
        $hold(posedge CK &&& CSWE, D[0], tdh, TimingViol3);
        $hold(posedge CK &&& CSWE, D[1], tdh, TimingViol3);
        $hold(posedge CK &&& CSWE, D[2], tdh, TimingViol3);
        $hold(posedge CK &&& CSWE, D[3], tdh, TimingViol3);
        $hold(posedge CK &&& CSWE, D[4], tdh, TimingViol3);
        $hold(posedge CK &&& CSWE, D[5], tdh, TimingViol3);
        $hold(posedge CK &&& CSWE, D[6], tdh, TimingViol3);
        $hold(posedge CK &&& CSWE, D[7], tdh, TimingViol3);

        $setup(CSN, posedge CK, tps, TimingViol4);
        $hold(posedge CK, CSN, tph, TimingViol5);

        $setup(WEN, posedge CK &&& CS, tws, TimingViol6);
        $hold(posedge CK &&& CS, WEN, twh, TimingViol7);

        $width(posedge CK, tckh, 0, TimingViol8);
        $width(negedge CK, tckl, 0, TimingViol9);
        $period(posedge CK &&& CS, tcycle, TimingViol10);

    endspecify
`endif

    task WriteMemX;
        begin
            for (i = 0; i < Words; i = i + 1)
                Mem[i] = WordX;
        end
    endtask

    task WriteLocX;
        input [Addr-1 : 0] Address;
        begin
            if (^Address !== X)
                Mem[Address] = WordX;
            else
                WriteMemX;
        end
    endtask

    task WriteOutX;
        begin
            OutReg = WordX;
        end
    endtask

    task WriteCycle;
        input [Addr-1 : 0] Address;
        begin
            // WRITE CYCLE

`ifdef functional
`else
            // Schedule the X-transient event
            if (^Q === X)
                Unk = 1;
            else
                Unk = 0;
            OutReg = WordX;
`endif

            // Overwrite memory with new data (if address is in range)
            if (^Address !== X)
                begin
                    if (ValidAddress)
                        Mem[Address] = Dreg;
                    else
                        $display("%m - %t WARNING: Address out of range during Write cycle", $realtime);
                end
            else
                begin
                    $display("%m - %t WARNING: Illegal Address bit during Write cycle", $realtime);
                    WriteMemX;
                end


            // Schedule the output event
            if (Unk === 0 && OEN === 0)
                @(Q or OEN) OutReg = Dreg;
            else
                OutReg = Dreg;
        end
    endtask

    task ReadCycle;
        input [Addr-1 : 0] Address;
        begin
            // READ cycle

`ifdef functional
`else
            // Schedule the X-transient event
            if (^Q === X)
                Unk = 1;
            else
                Unk = 0;
            OutReg = WordX;
`endif

            // Schedule output event (if address is in range)
            if (ValidAddress)
                begin
                    if (Unk === 0 && OEN === 0)
                        begin
                            @(Q or OEN)
                                if (StopOutput === 0)
                                    OutReg = Mem[Address];
                                else
                                    StopOutput = 0;
                        end
                    else
                        begin
                            if (StopOutput === 0)
                                OutReg = Mem[Address];
                            else
                                StopOutput = 0;
                        end
                end
             else if(ValidAddress === X)
                 $display("%m - %t  WARNING: Illegal Address bit during Read cycle", $realtime);
             else
                 $display("%m - %t  WARNING: Address out of range during Read cycle", $realtime);

        end
    endtask

        // BEHAVIOURAL MODULE DESCRIPTION

        // Constantly assign the values of OutReg onto Qint
        assign
            Qint = OutReg;

        // Define format for timing values
        initial
            $timeformat (-9, 2, " ns", 0);


    /*---------- Corruption due to Timing Violations ---------------*/

        always @(TimingViol4 or TimingViol5 or TimingViol8 or TimingViol9 or TimingViol10)
            // tps - tph - tckh - tckl - tcycle
            begin
                Dreg = WordX;
                StopOutput = 1;
                WriteOutX;
                if (WENint !== 1)
                    WriteMemX;
            end

        always @(TimingViol0 or TimingViol1)
            // tas - tah
            begin
                if (WENint === 1)
                    begin
                        StopOutput = 1;
                        WriteOutX;
                    end
                else
                    begin
                        Areg = AddrX;
                        WriteMemX;
                    end
            end

        always @(TimingViol2 or TimingViol3 or TimingViol6 or TimingViol7)
            // tds - tdh - tws -twh
            begin
                Dreg = WordX;
                WriteLocX(Aint);
                WriteOutX;
            end

    /*---------------------- Latching signals ----------------------*/

        always @(CKint)
            begin
                lastCK = CKreg;
                CKreg = CKint;
            end

        always @(posedge CKint)
            begin
                StopOutput = 0;
                Dreg = Dint;
                Areg = Aint;
                if (^Areg === X)
                    ValidAddress = X;
                else if (Areg < Words)
                    ValidAddress = 1;
                else
                    ValidAddress = 0;
                CSNreg = CSNint;
                WENreg = WENint;
            end

    /*---------------------- Normal Read and Write -----------------*/

        always @(CKreg)
            begin
                if ((CSNreg !== 1) && (CKreg === 1 && lastCK === 0))
                    begin
                        if (WENreg === 1 && CSNreg === 0)
                            ReadCycle(Areg);
                        else if (WENreg === 0 && CSNreg === 0)
                            WriteCycle(Areg);

    /*---------- Corruption due to faulty values on signals --------*/

                        else if (WENreg === X || (WENreg === 0 && CSNreg === X))
                            begin
                                // Uncertain write cycle
                                WriteLocX(Areg);
                                WriteOutX;
                            end
                        else if (WENreg === 1 && CSNreg === X)
                            // Uncertain read cycle
                            WriteOutX;
                    end
        end

        always @(posedge UnkCK)
            begin
                WriteOutX;
                if (WENint !== 1)
                    WriteMemX;
            end

        always @(posedge Flash)
            WriteMemX;


specify
  specparam adsPowerDown$OEN=0;
  specparam adsPowerDown$CSN=1;
endspecify
endmodule

`disable_portfaults
`nosuppress_faults
`endcelldefine
