----------------------------------------------------------------------------
-- RAM CAPSULE
-- (L) Copyleft 2003 Linus Walleij and Hkan Kvist
----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- The mp3ram_sim is not used anymore. May be
-- removed eventually.
package ram is
  component mp3ram_sim
    generic (BYTES : integer);
    port (addr	:in std_logic_vector (9 downto 0);
	  clk :in std_logic;
	  we :in std_logic;
	  data_in :in std_logic_vector (7 downto 0);
          data_out :out std_logic_vector (7 downto 0)
         );
  end component;

  component mp3ram_syn
    generic (BYTES : integer);
    port (addr	:in std_logic_vector (9 downto 0);
	  clk :in std_logic;
	  we :in std_logic;
	  data_in :in std_logic_vector (7 downto 0);
          data_out :out std_logic_vector (7 downto 0)
         );
  end component;
end ram;

-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
--use work.test_suite.all;

entity mp3ram_sim is
    generic (BYTES : integer);
    port (addr	:in std_logic_vector (9 downto 0);
	  clk :in std_logic;
	  we :in std_logic;
	  data_in :in std_logic_vector (7 downto 0);
          data_out :out std_logic_vector (7 downto 0)
         );
end mp3ram_sim;

architecture SIMUL of mp3ram_sim is
    -- Generate 1KB RAM, hope the compiler gets this ...
    type ram_type is array (0 to (BYTES-1)) of std_logic_vector(7 downto 0);
    signal ram_array : ram_type; -- Clear := (others=>(others=>'0'));
begin
  -- Write process
  process(clk, addr, we)
  begin
      if (rising_edge(clk)) then
	  if (we = '1') then
	      ram_array(conv_integer(addr)) <= data_in;
	  end if;
      end if;
  end process;

  -- Read process
  process(clk, addr)
  begin
      if (rising_edge(clk)) then
	  data_out <= ram_array(conv_integer(addr));
      end if;
  end process;

end SIMUL;

-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
--use work.test_suite.all;
-- Genererat RAM
LIBRARY STVLIB;                       -- uncomment these lines if you are using ASIC memories
USE STVLIB.ST_TIMING.all;
USE STVLIB.ST_VSS.all;
LIBRARY LIB_GENERATORS;
USE LIB_GENERATORS.PCK_GENERATORS.all;

entity mp3ram_syn is
    generic (BYTES : integer);
    port (addr	:in std_logic_vector (9 downto 0);
	  clk :in std_logic;
	  we :in std_logic;
	  data_in :in std_logic_vector (7 downto 0);
          data_out :out std_logic_vector (7 downto 0)
         );
end mp3ram_syn;

architecture SYNT of mp3ram_syn is
    -- Generate 1KB RAM
    signal gnd : std_logic;
    signal ram_we_inv : std_logic;
begin
  -- Write process
  mr_ram: SPS2_1024x8m8
      port map (Q => data_out,
		D => data_in,
		A => addr,
		CK => clk, 
                CSN => gnd,
		WEN => ram_we_inv,
		OEN => gnd );
  ram_we_inv <= not we;
  gnd <= '0';
end SYNT;

