---------------------------------------------------
--
-- Simple happy clockdevice
--
---------------------------------------------------

library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use std.textio.all;

package mr_fnord_clock is
    component clock
	generic ( MASTER_CLOCK : integer
	    );
	port( clk   : in  std_logic;
	      rst   : in  std_logic;
	      zero  : in  std_logic;
	      hold  : in  std_logic;
	      read  : in  std_logic;
	      seconds : out std_logic_vector(7 downto 0);
	      minutes : out std_logic_vector(7 downto 0);   
	      hours : out std_logic_vector(7 downto 0)
	  );
    end component;
end;

library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use work.mr_fnord_clock.all;
use std.textio.all;

entity clock is
--  generic ( MASTER_CLOCK : integer := 12_000_000
--	    );
    generic ( MASTER_CLOCK : integer := 1_000
	    );
  port ( clk     : in  std_logic;
         rst     : in  std_logic;
	 zero    : in  std_logic;
         read    : in  std_logic;
	 hold	 : in  std_logic;
         seconds : out std_logic_vector(7 downto 0);
	 minutes : out std_logic_vector(7 downto 0);
	 hours   : out std_logic_vector(7 downto 0)
       );
end;


-------------------------------------------------------------------------------
-- HAPPY ARCHITECTURE OF THIS CLOCK
-------------------------------------------------------------------------------

architecture mrfnord of clock is

-------------------------------------------------------------------------------
-- mr printhelper
--     procedure print(s : string) is
-- 	variable L1 : line;
--     begin
-- 	L1:= new string'(s);
-- 	writeline(output,L1);
--     end;
-------------------------------------------------------------------------------
  
    constant COUNTALOTFACTOR : std_logic_vector(6*4-1 downto 0) := CONV_STD_LOGIC_VECTOR( MASTER_CLOCK , 6*4);
    constant COUNTALOTZEROS : std_logic_vector(6*4-1 downto 0)  := CONV_STD_LOGIC_VECTOR( 0 , 6*4);
    
    type state_type is ( IDLE, SENDTIME, HOLDTIME1, HOLDTIME2, WASSENDTIME );

    type reg_type is record
	     state : state_type;
	     mr_countingfactor : std_logic_vector(6*4-1 downto 0);
	     seconds : std_logic_vector(7 downto 0);
	     minutes : std_logic_vector(7 downto 0);
	     hours   : std_logic_vector(7 downto 0);
	     out_seconds : std_logic_vector(7 downto 0);
	     out_minutes : std_logic_vector(7 downto 0);
	     out_hours   : std_logic_vector(7 downto 0);
    end record;

    signal mr_reg : reg_type;
    signal mr_reg2 : reg_type;
    
begin

    mr_statemachine_clock : process(clk, mr_reg2)
	variable states : reg_type;
    begin
	if( rising_edge(clk) ) then
	    states := mr_reg2;
	    if (states.state /= HOLDTIME2) then
		if (states.mr_countingfactor = "000000000000000000000000") then
		    states.mr_countingfactor := COUNTALOTFACTOR;
		    states.seconds := states.seconds + 1;
		    if (states.seconds(3 downto 0) = "1010") then
			states.seconds := states.seconds + 6;
			if (states.seconds(7 downto 4) = "0110") then
			    states.seconds := "00000000";
			    states.minutes := states.minutes + 1;
			end if;
		    end if;
		    if (states.minutes(3 downto 0) = "1010") then
			states.minutes := states.minutes + 6;
			if (states.minutes(7 downto 4) = "0110") then
			    states.minutes := "00000000";
			    states.hours := states.hours + 1;
			end if;
		    end if;
		    if (states.hours(3 downto 0) = "1010") then
			states.hours := states.hours + 6;
			if (states.hours(7 downto 4) = "0110") then
			    states.hours := "00000000";
			end if;
		    end if;
		else
		    states.mr_countingfactor := states.mr_countingfactor - 1;
		end if;
	    end if;
	    
	    mr_reg <= states;
	end if;
    end process;

    --only sensitive to rst, mr_reg!!!
    --mr_clock_clock : process(rst, read, hold, mr_reg)
    mr_clock_clock : process(rst, mr_reg, read, hold, zero)
	variable states : reg_type;
    begin
	states := mr_reg;
	
	case states.state is
	    when IDLE => 
		if (read = '1') then
		    states.state := SENDTIME;
		elsif (hold = '1') then
		    states.state := HOLDTIME1;
		end if;
		-- We should wait for a 0  (don't hold signal) until we hold the signal
		-- once more.
	    when SENDTIME =>
		states.state := WASSENDTIME;
		states.out_seconds := states.seconds;
		states.out_minutes := states.minutes;
		states.out_hours   := states.hours;
	    when WASSENDTIME =>
		states.state := IDLE;
	    when HOLDTIME1 =>
		if (hold = '0') then
		    states.state := HOLDTIME2;
		end if;
	    when HOLDTIME2 =>
		if (hold = '1') then
		    states.state := IDLE;
		end if;
	end case;

	if (rst = '0' or zero ='1') then
	   states.state := IDLE;
	   states.mr_countingfactor := COUNTALOTFACTOR;
	   states.seconds := "00000000";
	   states.minutes := "00000000";
	   states.hours   := "00000000";
	   states.out_seconds := "00000000";
	   states.out_minutes := "00000000";
	   states.out_hours   := "00000000";
	end if;
	
	seconds <= states.out_seconds;
	minutes <= states.out_minutes;
	hours <= states.out_hours;
	mr_reg2 <= states;
	
    end process;
end;


