library IEEE;
use IEEE.std_logic_1164.all;
use work.controller_types.all;

entity controller_top is
  port( clk         : in  std_logic;                                      -- system clock 12MHz
        rst         : in  std_logic;                                      -- system reset
        flash_addr  : out std_logic_vector(FLASH_ADDR_BITS-1 downto 0);   -- address of the external FLASH
	-- flash_wdata : out std_logic_vector(FLASH_DATA_BITS-1 downto 0);	  -- data to the external FLASH
	-- flash_en    : out std_logic;
	-- flash_we    : out std_logic;
	-- flash_oe    : out std_logic;
        flash_data  : in  std_logic_vector(FLASH_DATA_BITS-1 downto 0);   -- data from the external FLASH
        button_play : in  std_logic;                                      -- play
        button_stop : in  std_logic;                                      -- stop
        button_prev : in  std_logic;                                      -- previous song
        button_next : in  std_logic;                                      -- next song
        mp3_rst     : out std_logic;                                      -- to MP3 chip
        mp3_data    : out std_logic;                                      -- to MP3 chip
        mp3_valid   : out std_logic;                                      -- to MP3 chip
        mp3_req     : in  std_logic;                                      -- from MP3 chip
        spi_cs      : out std_logic;                                      -- D/A control (data)
        spi_sclk    : out std_logic;                                      -- D/A control (clk)
        spi_din     : out std_logic;                                      -- D/A control (clk)
        rxd         : in  std_logic;                                      -- serial input (from PC)
        txd         : out std_logic                                       -- serial output (display)
      );

end;

architecture structural of controller_top is
  
  signal flasho  : flash_in_type;
  signal flashi  : flash_out_type;
  
  signal spi     : spi_type;

  signal mp3o    : mp3_in_type;
  signal mp3i    : mp3_out_type; 

  signal buttoni : button_type;

  component controller
    port( clk     : in  std_logic; 
          rst     : in  std_logic;
          flashi  : in  flash_out_type;
          flasho  : out flash_in_type;
          buttoni : in  button_type;
          mp3i    : in  mp3_out_type;
          mp3o    : out mp3_in_type;
          spi     : out spi_type;
          rxd     : in  std_logic;
          txd     : out std_logic
         );
  end component;

begin

  ctrl: controller
    port map( clk     => clk, 
              rst     => rst,
              flashi  => flashi,
              flasho  => flasho,                                
              buttoni => buttoni,
              mp3i    => mp3i,
              mp3o    => mp3o,
              spi     => spi,
              rxd     => rxd,
              txd     => txd
            );  

  flashi.rdata <= flash_data;
  flash_addr   <= flasho.addr;
  -- We removed the ability to flash the flash
  -- flash_wdata  <= flasho.wdata;
  -- flash_en     <= flasho.en;
  -- flash_we     <= flasho.we;
  -- flash_oe     <= flasho.oe;
                       
  buttoni.play <= button_play;
  buttoni.stop <= button_stop;
  buttoni.prev <= button_prev;
  buttoni.nxt  <= button_next;

  mp3_rst   <= mp3o.rst;
  mp3_data  <= mp3o.data;
  mp3_valid <= mp3o.valid;
  mp3i.req  <= mp3_req;

  spi_cs    <= spi.cs;
  spi_sclk  <= spi.sclk;
  spi_din   <= spi.din;

end;


