---------------------------------------
-- This package contains constants and 
-- types used by the controller.
---------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

package controller_types is

  -- target settings --------------------------------------------------------------------------
                       
  type target_type is ( SIM, FPGA, ASIC );

  constant TARGET_TECH : target_type := SIM;                -- select the target here

  -- constant declarations --------------------------------------------------------------------

  constant FLASH_ADDR_BITS      : integer := 20;            -- flash address bits
  constant FLASH_DATA_BITS      : integer := 8;             -- flash data bits

  constant SYSTEM_CLOCK         : integer := 12_000_000;	-- system clock frequency

  -- Bd rejt ogrinal = 9600
  constant BAUD_RATE_LCD        : integer := 9_600;		-- communication speed with LCD
  constant BAUD_RATE_PC         : integer := 115200;		-- communication speed with PC

  -- type declarations -------------------------------------------------------------------------

  type mp3_in_type is record
    rst   : std_logic;         -- reset MP3 chip
    data  : std_logic;         -- serial data
    valid : std_logic;         -- valid data
  end record;

  type mp3_out_type is record
    req : std_logic;           -- request from MP3 chip
  end record;

  type button_type is record
    play : std_logic;          -- play button pressed
    stop : std_logic;          -- stop button pressed
    prev : std_logic;          -- prev button pressed
    nxt  : std_logic;          -- next button pressed
  end record;

  type spi_type is record
    cs   : std_logic;          -- chip select
    sclk : std_logic;          -- serial clock
    din  : std_logic;          -- data input
  end record;

  -- memory types --------------------------------------------------------------------

  type flash_in_type is record
    wdata : std_logic_vector(FLASH_DATA_BITS-1 downto 0);
    addr  : std_logic_vector(FLASH_ADDR_BITS-1 downto 0);
    we    : std_logic;
    oe    : std_logic;
    en    : std_logic;
  end record;

  type flash_out_type is record
    rdata : std_logic_vector(FLASH_DATA_BITS-1 downto 0);
  end record;

  -- sound output types --------------------------------------------------------------------

  type sound_out_type is record
    clk  : std_logic;	-- I2S clock
    data : std_logic;	-- I2S data
    sel  : std_logic;	-- I2S channel select
  end record;

end;
