---------------------------------------------------
--
-- Memory Management Unit (MMU)
--
-- Copyleft 2003 Hkan Kvist & Linus Walleij
--
---------------------------------------------------

library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
library work;
use work.uart_pck.all;
use work.controller_types.all;

package mmu_pck is
  component mmu
  port ( clk			: in std_logic;
	 halt		        : out std_logic;
	 rst			: in std_logic;
	 -- UART ports
         uarti			: out uart_ctrl_in_type;
         uarto			: in uart_ctrl_out_type;
	 -- CPU ports
	 cpu_addr		: in std_logic_vector (15 downto 0);
         cpu_in			: out std_logic_vector (7 downto 0);
         cpu_out		: in std_logic_vector (7 downto 0);
         cpu_we			: in std_logic;
         cpu_re			: in std_logic;
	 -- RAM ports
         ram_data_in		: out std_logic_vector (7 downto 0);
         ram_data_out		: in std_logic_vector (7 downto 0);
	 ram_we			: out std_logic;
	 -- FLASH ports
	 flashi			: in flash_out_type;
	 flasho                 : out flash_in_type;
	 -- Buttons
	 button_play		: in std_logic;
	 button_stop		: in std_logic;
	 button_prev		: in std_logic;
	 button_next		: in std_logic;
	 -- Clock unit
	 clock_rst              : out std_logic;
	 clock_hold             : out std_logic;
	 clock_read             : out std_logic;
	 clock_seconds          : in std_logic_vector(7 downto 0);
	 clock_minutes          : in std_logic_vector(7 downto 0);
	 clock_hours            : in std_logic_vector(7 downto 0);
	 -- Interface to MP3 chip
         mp3i			: in  mp3_out_type;
	 mp3o			: out mp3_in_type;
	 -- to dacuart
	 dac_data_byteh         : out std_logic_vector(7 downto 0);
	 dac_data_bytel         : out std_logic_vector(7 downto 0);
	 dac_send_data          : out std_logic;
	 dac_data_req           : in std_logic
       );
  end component;
end;

-------------------------------------------------------------------------------

library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
library work;
use work.mmu_pck.all;
use work.mp3uart_pck.all;
use work.uart_pck.all;
use work.controller_types.all;
use std.textio.all;
use work.dacuart_pck.all;

entity mmu is
  port ( clk			: in std_logic;
	 halt                   : out std_logic;
	 rst			: in std_logic;
	 -- UART ports
         uarti			: out uart_ctrl_in_type;
         uarto			: in uart_ctrl_out_type;
	 -- CPU ports
	 cpu_addr		: in std_logic_vector (15 downto 0);
         cpu_in			: out std_logic_vector (7 downto 0);
         cpu_out		: in std_logic_vector (7 downto 0);
         cpu_we			: in std_logic;
         cpu_re			: in std_logic;
	 -- RAM ports
         ram_data_in		: out std_logic_vector (7 downto 0);
         ram_data_out		: in std_logic_vector (7 downto 0);
	 ram_we			: out std_logic;
	 -- FLASH ports
	 flashi			: in flash_out_type;
	 flasho                 : out flash_in_type;
	 -- Buttons
	 button_play		: in std_logic;
	 button_stop		: in std_logic;
	 button_prev		: in std_logic;
	 button_next		: in std_logic;
	 -- Clock unit
	 clock_rst              : out std_logic;
	 clock_hold             : out std_logic;
	 clock_read             : out std_logic;
	 clock_seconds          : in std_logic_vector(7 downto 0);
	 clock_minutes          : in std_logic_vector(7 downto 0);
	 clock_hours            : in std_logic_vector(7 downto 0);
	 -- Interface to MP3 chip
         mp3i			: in  mp3_out_type;
	 mp3o			: out mp3_in_type;
	 -- to dacuart
	 dac_data_byteh         : out std_logic_vector(7 downto 0);
	 dac_data_bytel         : out std_logic_vector(7 downto 0);
	 dac_send_data          : out std_logic;
	 dac_data_req           : in std_logic
	 );
end;


-------------------------------------------------------------------------------
-- HAPPY ARCHITECTURE OF THIS MMU
-------------------------------------------------------------------------------

architecture FNORD of mmu is

--     procedure print(s : string) is
--  	variable L1 : line;
--       begin
-- 	  L1:= new string'(s);
--  	writeline(output,L1);
--      end;

--     procedure printbyte(b : std_logic_vector(7 downto 0)) is
-- 	variable L1 : line;
--     begin
--         write(L1, to_bitvector(b));
-- 	writeline(output,L1);
--     end;
	  
--     procedure print16address(a : std_logic_vector(15 downto 0)) is
-- 	variable L1 : line;
--     begin
--         write(L1, to_bitvector(a));
-- 	writeline(output,L1);
--     end;
	  
--     procedure print20address(a : std_logic_vector(19 downto 0)) is
-- 	variable L1 : line;
--     begin
--         write(L1, to_bitvector(a));
-- 	writeline(output,L1);
--     end;

    constant DELAY_CYCLES_WRITE : std_logic_vector(3 downto 0) := "0010";
    type state_type is (IDLE, RAM_READOUT, WAIT_ROM, WAIT_ROM2, WAIT_ROM3, ROM_READOUT,
			ERASE_ROM_1, ERASE_ROM_2, ERASE_ROM_3, ERASE_ROM_4, ERASE_ROM_5, ERASE_ROM_6,
			WRITE_ROM_1, WRITE_ROM_2, WRITE_ROM_3, WRITE_ROM_4,
			WRITE_COMMIT, WRITE_CONFIRM_1, WRITE_CONFIRM_2);
    type mp3_dma_state_type is (IDLE, WAIT_MP3, WAIT_MP32, MP3_READOUT, MP3_DELAY);
    type playstate_type is (STOPMODE, PLAYMODE, PAUSEMODE);
    
    type output_buffer_type is record
	     -- CPU strangler
	     halt : std_logic;
	     state : state_type;
	     mp3_dma_state : mp3_dma_state_type;
	     -- Buffered outputs
	     lcd_uart_send : std_logic;
	     lcd_uart_data : std_logic_vector (7 downto 0);
	     pc_uart_ack_rcv : std_logic;
	     flasho : flash_in_type;
	     flash_addr : std_logic_vector (FLASH_ADDR_BITS-1 downto 0);
	     flash_play_addr : std_logic_vector (31 downto 0);  -- Setup values
	     flash_stop_addr : std_logic_vector (31 downto 0);
	     flash_write_addr : std_logic_vector (31 downto 0);
	     flash_write_confirm_bit : std_logic;
	     -- Playing
	     playstate : playstate_type;
	     play_addr : std_logic_vector (31 downto 0);  -- The one that
							     -- increases by
							     -- one while playing
	     flash_play : std_logic;
	     flash_stop : std_logic;
	     clock_rst : std_logic;
	     clock_hold : std_logic;
	     clock_read : std_logic;
	     -- Buffered outputs
	     stopped : std_logic;
	     -- To MP3 chip
	     mp3byte : std_logic_vector (7 downto 0);
	     takethismp3 : std_logic;
	     stopit : std_logic;
	     -- RAM access
	     cpu_in : std_logic_vector(7 downto 0);
	     ram_data_in : std_logic_vector(7 downto 0);
	     ram_we : std_logic;
	     -- MR dac
	     dac_data_byteh : std_logic_vector(7 downto 0);
	     dac_data_bytel : std_logic_vector(7 downto 0);
	     dac_send_data : std_logic;
    end record;
	  
    -- The buffer pair
    signal buff : output_buffer_type;
    signal buff_in : output_buffer_type;

    -- Ports to the mp3 feed process
    signal mp3byte : std_logic_vector(7 downto 0);
    signal takethismp3 : std_logic;
    signal requestmp3 : std_logic;
    signal stopit : std_logic;

begin

    mr_mp3uart : mp3uart
	port map ( clk => clk,
		   rst => rst,
		   mp3byte => mp3byte,
		   takethismp3 => takethismp3,
		   requestmp3 => requestmp3,
		   stopit => stopit,
		   mp3i => mp3i,
		   mp3o => mp3o);

    
    -- These two talk to the memory blocks
    mr_buffer : process(clk, buff_in)
    begin
	if (rising_edge(clk)) then
	    -- Copy outputs to ports
	    flasho <= buff_in.flasho;
	    ram_data_in <= buff_in.ram_data_in;
	    ram_we <= buff_in.ram_we;
	    uarti.send <= buff_in.lcd_uart_send;
	    uarti.data <= buff_in.lcd_uart_data;
	    uarti.ack_rcv <= buff_in.pc_uart_ack_rcv;
	    clock_rst <= buff_in.clock_rst;
	    clock_hold <= buff_in.clock_hold;
	    clock_read <= buff_in.clock_read;
	    mp3byte <= buff_in.mp3byte;
	    takethismp3 <= buff_in.takethismp3;
	    stopit <= buff_in.stopit;
	    dac_data_byteh <= buff_in.dac_data_byteh;
	    dac_data_bytel <= buff_in.dac_data_bytel;
	    dac_send_data <= buff_in.dac_send_data;
	    -- Update states
	    buff <= buff_in;
	    -- This is to automatically deassert requests after
	    -- one clock cycle.
	    buff.lcd_uart_send <= '0';
	    buff.pc_uart_ack_rcv <= '0';
	    buff.clock_rst <= '0';
	    buff.clock_hold <= '0';
	    buff.clock_read <= '0';
	    -- buff.ram_we <= '0';
	    buff.takethismp3 <= '0';
	    buff.stopit <= '0';
	    buff.dac_send_data <= '0';
	end if;
    end process;
    
    ---------------------------------------------------------------------------
    -- THIS PROCESS SWITCH SIGNALS BETWEEN MEMORY BANKS AND
    -- I/O PORTS
    ---------------------------------------------------------------------------

    process(rst, buff, requestmp3, cpu_re, cpu_we, cpu_addr, cpu_out, uarto,
	    clock_hours, clock_minutes, clock_seconds, button_play,
	    button_stop, button_next, button_prev, dac_data_req, flashi, ram_data_out)
	variable b : output_buffer_type;
    begin
      b := buff;
      case b.state is
	  when IDLE =>
	      if (cpu_re = '1' or cpu_we = '1') then
		  case cpu_addr(15 downto 10) is
		      -- First, ignore addresses that are retrieved within
		      -- a single cycle. (They are handled above.) If it is
		      -- a read from the Flash PROM, then enter the wait
		      -- state and inhibit the clock.
		      when "000000" =>
			  if (cpu_we = '1') then
			      b.ram_data_in := cpu_out;
			      b.ram_we := '1';
			      b.state := RAM_READOUT;
			  elsif (cpu_re = '1') then
			      b.state := RAM_READOUT;
			  end if;
			  b.halt := '1';
		      when "100000" =>
			  -- The ports are totally asynchronous so they
			  -- may be written/read in one clock cycle only.
			  if (cpu_re = '1') then
			      -- print("8000 page, ports");
			      case cpu_addr is
				  -- $8000 LCD_UART_CTRL IN
				  when "1000000000000000" =>
				      -- print("Read from LCD_UART_CTRL IN");
				      b.cpu_in(7) := uarto.txempty;
				      b.cpu_in(6 downto 0) := "0000000";
				  -- $8003 FLASH_CTRL_IN
				  when "1000000000000011" =>
				      b.cpu_in(7) := b.stopped;
				      b.cpu_in(6 downto 0) := "0000000";
				  -- $8007 - $8009 HAPPY TIME IN
				  when "1000000000000111" =>
				      b.cpu_in := clock_hours;
				  when "1000000000001000" =>
				      b.cpu_in := clock_minutes;
				  when "1000000000001001" =>
				      b.cpu_in := clock_seconds;
				  -- $8012 BUTTONS IN
				  when "1000000000010010" =>
				      b.cpu_in(7) := button_play;
				      b.cpu_in(6) := button_stop;
				      b.cpu_in(5) := button_prev;
				      b.cpu_in(4) := button_next;
				      b.cpu_in(3 downto 0) := "0000";
				  -- $8013 PC UART CTRL IN
				  when "1000000000010011" =>
				      b.cpu_in(7) := uarto.rxready;
				      b.cpu_in(6 downto 0) := (others => '0');
				  -- $8015 PC UART DATA IN
				  when "1000000000010101" =>
				      b.cpu_in := uarto.data;
				  -- $8017 DAC CTRL IN
				  when "1000000000010111" =>
				      b.cpu_in(7) := dac_data_req;
				      b.cpu_in(6 downto 0) := (others => '0');
				  when others =>
				      b.cpu_in := "00000000";
			      end case;
			  elsif (cpu_we = '1') then
			      -- Alla portar skrives hr!
			      case cpu_addr is
				  -- $8001 LCD_UART_CTRL OUT
				  when "1000000000000001" =>
				      -- print("Wrote to LCD_UART_CTRL OUT");
				      b.lcd_uart_send := cpu_out(7);
				  -- $8002 LCD_UART_DATA OUT
				  when "1000000000000010" =>
				      -- print("Wrote to LCD_UART_DATA OUT");   
				      b.lcd_uart_data := cpu_out;
				  -- $8004 FLASH_CTRL OUT
				  when "1000000000000100" =>
				      -- print("Wrote to FLASH_CTRL OUT");
				      case b.playstate is
					  when STOPMODE =>
					      if (cpu_out(7) = '1') then
						  -- Erase the
						  -- chip! (Don't do this
						  -- by mistake.)
						  b.halt := '1';
						  b.state := ERASE_ROM_1;
					      end if;
					      if (cpu_out(5) = '1') then
						  b.playstate := PLAYMODE;
						  b.play_addr := b.flash_play_addr;
-- 						  print("Vi stller oss i playmode frn stopmode (port)");
						  b.playstate := PLAYMODE;
						  b.stopped := '0';
					      end if;
					  when PLAYMODE =>
					      if (cpu_out(4) = '1') then
						  --print("Vi stller oss i stopmode frn playmode (port)");
						  b.playstate := STOPMODE;
						  b.stopped := '1';
						  b.stopit := '1';
					      elsif (cpu_out(5) = '1') then
--						  print("Vi stller oss i pausemode frn playmode (port)");
						  b.playstate := PAUSEMODE;
						  b.stopit := '1';
					      end if;
					  when PAUSEMODE =>
					      if (cpu_out(4) = '1') then
--						  print("Vi stller oss i stopmode frn pausemode (port)");
						  b.playstate := STOPMODE;
						  b.stopped := '1';
					      elsif (cpu_out(5) = '1') then
--						  print("Vi stller oss i playmode frn pausemode (port)");
						  b.playstate := PLAYMODE;
					      end if;
				      end case;
				  -- $8005 WRITE_DATA
				  when "1000000000000101" =>
				      -- Particularly nasty port - when
				      -- written to it will nuke the
				      -- processor and start a write cycle
				      -- to flash memory.
				      b.halt := '1';
				      b.state := WRITE_ROM_1;
				  -- $8006 CLOCK_CTRL_OUT
				  when "1000000000000110" =>
				      b.clock_rst := cpu_out(7);
				      b.clock_hold := cpu_out(6);
				      b.clock_read := cpu_out(5);
				  -- $800a-$800d FLASH_PLAY_ADDRESS
				  when "1000000000001010" =>
				      b.flash_play_addr(31 downto 24) := cpu_out;
				  when "1000000000001011" =>
				      b.flash_play_addr(23 downto 16) := cpu_out;
				  when "1000000000001100" =>
				      b.flash_play_addr(15 downto 8) := cpu_out;
				  when "1000000000001101" =>
				      b.flash_play_addr(7 downto 0) := cpu_out;
				  -- $800e-$8011 FLASH_STOP_ADDRESS
				  when "1000000000001110" =>
				      b.flash_stop_addr(31 downto 24) := cpu_out;
				  when "1000000000001111" =>
				      b.flash_stop_addr(23 downto 16) := cpu_out;
				  when "1000000000010000" =>
				      b.flash_stop_addr(15 downto 8) := cpu_out;
				  when "1000000000010001" =>
				      b.flash_stop_addr(7 downto 0) := cpu_out;
				  -- $8014 PC UART CTRL OUT
				  when "1000000000010100" =>
				      b.pc_uart_ack_rcv := cpu_out(7);
				  -- $8018 send DAC_DATA, $8019, $8020, DAC_DATA
				  when "1000000000011000" =>
				      b.dac_send_data := cpu_out(7);
				  when "1000000000011001" =>
				      b.dac_data_byteh := cpu_out;
				  when "1000000000011010" =>
				      b.dac_data_bytel := cpu_out;
				  -- $8020-$8023 FLASH_WRITE_ADDRESS
				  when "1000000000100000" =>
				      b.flash_write_addr(31 downto 24) := cpu_out;
				  when "1000000000100001" =>
				      b.flash_write_addr(23 downto 16) := cpu_out;
				  when "1000000000100010" =>
				      b.flash_write_addr(15 downto 8) := cpu_out;
				  when "1000000000100011" =>
				      b.flash_write_addr(7 downto 0) := cpu_out;
				  when others =>
			      end case;
			  end if;
		      when others =>
			  if (cpu_re = '1') then
			      b.halt := '1';
			      -- FIXME: Ta hnsyn till om MP3 chip redan lser
			      -- frn Flash
			      if (b.mp3_dma_state = IDLE) then
				  b.flasho.addr(15 downto 0) := cpu_addr;
				  b.flasho.addr(FLASH_ADDR_BITS-1 downto 16) := (others => '0');
				  -- print("Requested address:");
				  -- print20address(b.flash_addr);
				  b.state := WAIT_ROM;
				  -- elsif (cpu_we = '1') then
				  -- insert Flash write code here (NO)
			      end if;
			  end if;
		  end case; -- End address case
	      end if;

	  when RAM_READOUT =>
	      -- If we requested a read from RAM/ports on last cycle, then
	      -- read it in now.
	      if (cpu_re = '1') then
		  -- Now we have read the requested byte.
		  case cpu_addr(15 downto 10) is
		      -- RAM
		      when "000000" =>
			  -- If we inhibit the processor 1 clock cycle
			  -- here, things will work.
			  -- set address, wait, serve
			  b.cpu_in := ram_data_out;
		      when others =>
		  end case;
	      end if;
	      b.ram_we := '0';
	      b.halt := '0';
	      b.state := IDLE;
	  when WAIT_ROM =>
	      b.state := WAIT_ROM2;
	  when WAIT_ROM2 =>
	      b.state := WAIT_ROM3;
	  when WAIT_ROM3 =>
	      b.state := ROM_READOUT;
	  when ROM_READOUT =>
	      b.cpu_in := flashi.rdata;
	      -- print("Read byte:");
	      -- printbyte(flashi.rdata);
	      b.halt := '0';
	      b.state := IDLE;
	  when ERASE_ROM_1 =>
	      b.flasho.we := '0';
	      b.flasho.oe := '1';
	      b.flasho.addr(11 downto 0) := "010101010101";  -- 555
	      b.flasho.wdata := "10101010";  -- AA
	      b.state := ERASE_ROM_2;
	  when ERASE_ROM_2 =>
	      b.flasho.addr(11 downto 0) := "001010101010";  -- 2AA
	      b.flasho.wdata := "01010101";  -- 55
	      b.state := ERASE_ROM_3;
	  when ERASE_ROM_3 =>
	      b.flasho.addr(11 downto 0) := "010101010101";  -- 555
	      b.flasho.wdata := "10000000";  -- 80
	      b.state := ERASE_ROM_4;
	  when ERASE_ROM_4 =>
	      b.flasho.addr(11 downto 0) := "010101010101";  -- 555
	      b.flasho.wdata := "10101010";  -- AA
	      b.state := ERASE_ROM_5;
	  when ERASE_ROM_5 =>
	      b.flasho.addr(11 downto 0) := "001010101010";  -- 2AA
	      b.flasho.wdata := "01010101";  -- 55
	      b.state := ERASE_ROM_6;
	  when ERASE_ROM_6 =>
	      b.flasho.addr(11 downto 0) := "010101010101";  -- 555
	      b.flasho.wdata := "00010000";  -- 10
	      b.state := WRITE_COMMIT;
	  when WRITE_ROM_1 =>
	      b.flasho.we := '0';
	      b.flasho.oe := '1';
	      b.flasho.addr(11 downto 0) := "010101010101";  -- 555
	      b.flasho.wdata := "10101010";  -- AA
	      b.state := WRITE_ROM_2;
	  when WRITE_ROM_2 =>
	      b.flasho.addr(11 downto 0) := "001010101010";  -- 2AA
	      b.flasho.wdata := "01010101";  -- 55
	      b.state := WRITE_ROM_3;
	  when WRITE_ROM_3 =>
	      b.flasho.addr(11 downto 0) := "010101010101";  -- 555
	      b.flasho.wdata := "10100000";  -- A0
	      b.state := WRITE_ROM_4;
	  when WRITE_ROM_4 =>
	      b.flasho.addr(FLASH_ADDR_BITS-1 downto 0) := b.flash_write_addr(FLASH_ADDR_BITS-1 downto 0);
	      b.flasho.wdata := cpu_out;
	      b.state := WRITE_COMMIT;
	  when WRITE_COMMIT =>
	      b.flasho.we := '1';
	      b.flasho.oe := '0';
	      b.state := WRITE_CONFIRM_1;
	  when WRITE_CONFIRM_1 =>
	      b.flash_write_confirm_bit := flashi.rdata(6);
	      b.state := WRITE_CONFIRM_2;
	  when WRITE_CONFIRM_2 =>
	      -- Wait for bit6 to stop toggling
	      if (flashi.rdata(6) = b.flash_write_confirm_bit) then
		  b.halt := '0';
		  b.state := IDLE;
	      end if;
	      b.flash_write_confirm_bit := flashi.rdata(6);
      end case;

      case b.mp3_dma_state is
	  when IDLE =>
	      -- If the mp3 chip requests a new byte, and we are in
	      -- playmode, await any CPU activity to finish and then fetch
	      -- a byte from Flash memory to the mp3 chip.
	      if (b.playstate = PLAYMODE and requestmp3 = '1' and b.state = IDLE) then
		  b.flasho.addr := b.play_addr(FLASH_ADDR_BITS-1 downto 0);
		  -- print("MP3 Requested address:");
		  -- print20address(b.flash_addr);
		  b.mp3_dma_state := WAIT_MP3;
	      end if;
	  when WAIT_MP3 =>
	      b.mp3_dma_state := WAIT_MP32;
	  when WAIT_MP32 =>
	      b.mp3_dma_state := MP3_READOUT;
	  when MP3_READOUT =>
	      b.play_addr := b.play_addr + 1;
	      if (b.play_addr = b.flash_stop_addr) then
		  -- print("Vi stller oss i stopmode frn playmode (end of song)");
		  b.playstate := STOPMODE;
		  b.stopped := '1';
		  b.stopit := '1';
	      else
		  b.mp3byte := flashi.rdata;
		  -- printbyte(b.mp3byte);
		  b.takethismp3 := '1';
	      end if;
	      b.mp3_dma_state := MP3_DELAY;
	  when MP3_DELAY =>
	      b.mp3_dma_state := IDLE;
      end case;

      -- Active low reset signal
      if (rst = '0') then
	  -- Zero some states
	  b.state := IDLE;
	  b.mp3_dma_state := IDLE;
	  b.playstate := STOPMODE;
	  -- CPU values
	  b.cpu_in(7 downto 0) := (others => '0');
	  -- MP3 DMA status
	  b.stopped := '0';
	  -- UART control
	  b.lcd_uart_send := '0';
	  b.lcd_uart_data := "00000000";
	  b.pc_uart_ack_rcv := '0';
	  -- Clock control (works in post-synt)
	  b.clock_rst := '1';
	  b.clock_hold := '0';
	  b.clock_read := '0';
	  -- Flash memory control
	  b.flasho.wdata(FLASH_DATA_BITS-1 downto 0) := (others => '0');
	  b.flasho.addr(FLASH_ADDR_BITS-1 downto 0) := (others => '0');
	  b.flasho.we := '1';
	  b.flasho.oe := '0';
	  b.flasho.en := '0'; -- May be switched off for power save.
	  b.halt := '0';
	  -- RAM control
	  b.ram_we := '0';
	  -- DAC
	  b.dac_data_byteh := "00000000";
	  b.dac_data_bytel := "00000000";
	  b.dac_send_data := '0';
      end if;
      
      -- Drive some output signals
      halt <= b.halt;
      cpu_in <= b.cpu_in;
      
      buff_in <= b;
    end process;
end;
