---------------------------------------------------
--
-- MP3 Uart for stupid device
--
-- Copyleft 2003 Hkan Kvist & Linus Walleij
--
---------------------------------------------------

library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
library work;
use work.uart_pck.all;
use work.controller_types.all;

package mp3uart_pck is
  component mp3uart
  port ( clk			: in std_logic;
	 rst			: in std_logic;
	 mp3byte		: in std_logic_vector(7 downto 0);  -- Byte to
								    -- serve
	 takethismp3		: in std_logic;  -- Take the byte above and
						 -- send it
	 requestmp3		: out std_logic;  -- Gimme something
	 stopit			: in std_logic;  -- Reset yourself
	 mp3i			: in  mp3_out_type;
	 mp3o			: out mp3_in_type
       );
  end component;
end;


library IEEE;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
library work;
use work.uart_pck.all;
use work.controller_types.all;
use std.textio.all;

entity mp3uart is
  port ( clk			: in std_logic;
	 rst			: in std_logic;
	 mp3byte		: in std_logic_vector(7 downto 0);
	 takethismp3		: in std_logic;
	 requestmp3		: out std_logic;
	 stopit			: in std_logic;
         mp3i			: in  mp3_out_type;
	 mp3o			: out mp3_in_type
       );
end;


architecture FOOBAR of mp3uart is

--     procedure printbyte(b : std_logic_vector(7 downto 0)) is
-- 	variable L1 : line;
--     begin
--         write(L1, to_bitvector(b));
-- 	writeline(output,L1);
--     end;

    type feed_state_type is ( IDLE, SENDING );
        
    type feed_type is record
	     state : feed_state_type;
	     reset : std_logic;
	     stopit : std_logic;
	     bitcount : std_logic_vector(3 downto 0);
	     mp3byte : std_logic_vector(7 downto 0);
	     requestmp3 : std_logic;
    end record;

    signal feed : feed_type;
    signal feed_in : feed_type;

begin
    
    -- These two throw bytes at the MP3 decoder
    mr_feedbuffer : process(clk, feed_in)
    begin
	if (rising_edge(clk)) then
	    requestmp3 <= feed_in.requestmp3;
	    if (feed_in.reset = '1') then
		mp3o.rst <= '0';
		mp3o.data <= '0';
		mp3o.valid <= '0';
	    elsif (feed_in.bitcount /= "0000" and mp3i.req = '1') then
		mp3o.data <= feed_in.mp3byte(7);
		mp3o.valid <= '1';
		mp3o.rst <= '1';
	    else
		mp3o.rst <= '1';
		mp3o.valid <= '0';
	    end if;
	    -- Iterate state
	    feed <= feed_in;
	    if (feed_in.bitcount /= "0000" and mp3i.req = '1') then
		feed.mp3byte <= shl(feed_in.mp3byte, "1");  -- Shift
		feed.bitcount <= feed_in.bitcount - 1;
	    end if;
	    -- Strobed signals
	    feed.reset <= '0';
	end if;
    end process;
    
    mr_feed : process(rst, feed, mp3i, mp3byte, stopit, takethismp3)
	variable f : feed_type;
    begin
	f := feed;

	if (stopit = '1') then
	    f.stopit := '1';
	end if;

	-- Wait until all bits in current byte are sent before stopping.
	if (f.stopit = '1' and f.bitcount = "0000") then
	    f.reset := '1';
	    f.state := IDLE;
	    f.stopit := '0';
	end if;

	case f.state is
	    when IDLE =>
		if (mp3i.req = '1') then
		    f.requestmp3 := '1';
		end if;
		if (takethismp3 = '1' and f.requestmp3 = '1') then
		    f.mp3byte := mp3byte;
		    -- printbyte(f.mp3byte);
		    f.bitcount := "1000";
		    f.requestmp3 := '0';
		    f.state := SENDING;
		end if;
	    when SENDING =>
		if (f.bitcount = "0000") then
		    f.requestmp3 := '1';
		    f.state := IDLE;
		end if;
	end case;

	if (rst = '0') then
	    f.state := IDLE;
	    f.stopit := '0';
	    -- Reset MP3 chip
	    f.reset := '1';
	    -- Signals
	    f.requestmp3 := '0';
	    -- Variables
	    f.mp3byte := "00000000";
	    f.bitcount := "0000";
	    -- Throw bytes at mp3chip if read out
	end if;

	feed_in <= f;

    end process;

end;
