library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_signed.all;
use IEEE.std_logic_arith.all;
use std.textio.all;

entity flash is
  generic( ADDR_BITS : integer := 2;
           DATA_BITS : integer := 8;
           ROM_FILE  : string
         );
  port( CLK  : in  std_logic;
        ADDR : in  std_logic_vector(ADDR_BITS-1 downto 0);
        DO   : out std_logic_vector(DATA_BITS-1 downto 0)
      );
end;

architecture behavioral of flash is
begin
  
  process
    type rom_array is array(0 to 2**ADDR_BITS-1) of std_logic_vector(DATA_BITS-1 downto 0);
    variable rom_data : rom_array;
    file load_file : text is in ROM_FILE;
    variable index : integer;
    variable input_line : line;
    variable value : bit_vector(DATA_BITS-1 downto 0); 
  begin
    index := 0;
    while (not endfile(load_file)) and (index < 2**ADDR_BITS) loop
      readline(load_file, input_line);
      read(input_line, value);
      rom_data(index) := TO_STDLOGICVECTOR(value); -- CONV_STD_LOGIC_VECTOR(value, DATA_BITS);
      index := index + 1; 
    end loop;
  
    loop
      wait until CLK='1';
      if (not is_x(ADDR)) then
        DO <= rom_data(CONV_INTEGER('0' & ADDR));		-- avoid signed index value
      else                                                                         
        DO <= (others => 'U');
      end if;
    end loop;
  end process;
  
end;
