library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;
use STD.textio.all;

entity sram is
  generic( ADDR_BITS : integer := 4;
           DATA_BITS : integer := 4
           );
    
  port( CLK  : in  std_logic;					-- clock
        ADDR : in  std_logic_vector(ADDR_BITS-1 downto 0);	-- read address
        D    : in  std_logic_vector(DATA_BITS-1 downto 0);	-- data input
        Q    : out std_logic_vector(DATA_BITS-1 downto 0);	-- data output
        nWE  : in  std_logic;					-- write enable
        nEN  : in  std_logic;					-- enable
        nOE  : in  std_logic					-- output enable
      );
end;

architecture behavioral of sram is
begin

  process
    type memory_array is array(0 to 2**ADDR_BITS-1) of std_logic_vector(DATA_BITS-1 downto 0);
    variable data : memory_array;
  begin    
    loop
      wait until CLK = '1';
      Q <= (others => 'U');
     
      if nEN = '0' then				-- if enabled
        if nWE='0' then		-- write operation and nOE='1' 
          if not is_x(ADDR) then
            data(CONV_INTEGER(ADDR)) := D;
          end if;
        end if;
        if nOE='0' then		-- read operation nWE='1' and 
          if not is_x(ADDR) then
            Q <= data(CONV_INTEGER(ADDR)) after 5 ns;
          end if;
        end if;
      end if;
    end loop;
  end process;

end;

