library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_signed.all;
use IEEE.std_logic_arith.all;
use std.textio.all;
use work.tbcomp.all;
use work.dacuart_pck.all;
use work.controller_types.all;

entity tb_dacuart is
end;

architecture behavioral of tb_dacuart is

  -- Not needed in first TB.
  -- constant DAC_FILENAME  : string := "debug/dac.dat";   --data that will be sent to the DAC

  procedure print(s : string) is
    variable L1 : line;
  begin
    L1:= new string'(s);
    writeline(output,L1);
  end;
  
  -- The usual stuff
  signal clk, rst, gnd, vdd                     : std_logic;

  -- Interface to DAC chip
  signal daco : spi_type;
  
  -- Ports to the DAC feed process
  signal dacbytes : std_logic_vector(15 downto 0);
  signal takethisdac : std_logic;
  signal requestdac : std_logic;

  -- fnord to hell
  signal mr_counter : std_logic_vector(7 downto 0);
  
begin

  mr_dacuart : dacuart
      port map ( clk => clk,
		 rst => rst,
		 dacbytes => dacbytes,
		 takethisdac => takethisdac,
		 requestdac => requestdac,
		 daco => daco);
  
  ----------------------------
  -- Clock and Reset
  ----------------------------

  clkgen: clock_generator
    generic map( 83 ns )
    port map ( CLK => CLK );

  rstgen: reset_generator
    generic map( 450 ns )
    port map ( RST => RST );

  gnd <= '0';
  vdd <= '1';

  bar : process(clk, rst)
      variable text_line : line;
      variable indata : integer;
      variable aFnord : std_logic;
  begin
      if (rst = '0') then
	  mr_counter <= "00000000";
	  takethisdac <= '0';
      elsif rising_edge(clk) then
	  case mr_counter is
	      when "00000000" =>
		  if (requestdac = '1') then
		      mr_counter <= mr_counter + 1;
                  end if;
	      when "00000001" =>
		  if (requestdac = '1') then
		      dacbytes <= "1010101010101010";
		      takethisdac <= '1';		      
		  else
		      takethisdac <= '0';
		      mr_counter <= mr_counter + 1;
		  end if;
	      when "00000010" =>
		  if (requestdac = '1') then
		      mr_counter <= mr_counter + 1;
		  end if;
	      when "00000011" =>
		  if (requestdac = '1') then
		      dacbytes <= "1111111100000000";
		      takethisdac <= '1';		      
		  else
		      takethisdac <= '0';
		      mr_counter <= mr_counter + 1;
		  end if;
	      when others =>
		  if ( requestdac = '1') then
		      dacbytes <= (others => '0');
		      dacbytes(7 downto 0) <=  mr_counter;
		      takethisdac <= '1';
		      aFnord := '1';
		  else
		      if aFnord = '1' then
			  mr_counter <= mr_counter + 1;	
			  aFnord := '0';
		      end if;
		      takethisdac <= '0';
		  end if;
	  end case;
      end if;
  end process;

end;

