library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_signed.all;
use IEEE.std_logic_arith.all;
use std.textio.all;
use work.tbcomp.all;
use work.mp3uart_pck.all;
use work.controller_types.all;

entity tb_mp3uart is
end;

architecture behavioral of tb_mp3uart is

  constant MP3_FILENAME      : string := "debug/song.dat";                 -- data sent to the MP3 chip

  procedure print(s : string) is
    variable L1 : line;
  begin
    L1:= new string'(s);
    writeline(output,L1);
  end;
  
  signal clk, cpu_clk, rst, gnd, vdd                     : std_logic;

  -- Interface to MP3 chip
  signal mp3i : mp3_out_type;
  signal mp3o : mp3_in_type;
  
  -- MP3 Chip simulator
  signal mp3_rst, mp3_data, mp3_valid, mp3_req  : std_logic;

  -- Ports to the mp3 feed process
  signal mp3byte : std_logic_vector(7 downto 0);
  signal takethismp3 : std_logic;
  signal requestmp3 : std_logic;
  signal stopit : std_logic;

  signal mr_counter : std_logic_vector(7 downto 0);
  
begin

  mp3_rst <= mp3o.rst;
  mp3_data <= mp3o.data;
  mp3_valid <= mp3o.valid;
  mp3i.req <= mp3_req;

  mr_mp3uart : mp3uart
      port map ( clk => clk,
		 rst => rst,
		 mp3byte => mp3byte,
		 takethismp3 => takethismp3,
		 requestmp3 => requestmp3,
		 stopit => stopit,
		 mp3i => mp3i,
		 mp3o => mp3o);
  
  ----------------------------
  -- Clock and Reset
  ----------------------------

  clkgen: clock_generator
    generic map( 83 ns )
    port map ( CLK => CLK );

  rstgen: reset_generator
    generic map( 450 ns )
    port map ( RST => RST );

  gnd <= '0';
  vdd <= '1';

  bar : process(clk, rst)
      variable text_line : line;
      variable indata : integer;
  begin
      if (rst = '0') then
	  mr_counter <= "00000000";
	  takethismp3 <= '0';
      elsif rising_edge(clk) then
	  mr_counter <= mr_counter + 1;
	  case mr_counter is
	      when "00010000" =>
		  mp3byte <= "10101010";
		  takethismp3 <= '1';
	      when "00010001" =>
		  takethismp3 <= '0';
	      when "00001111" =>
	      when others =>
	  end case;
      end if;
  end process;


  ----------------------------
  -- MP3 chip emulator
  ----------------------------

  process
    variable sample     : std_logic_vector(FLASH_DATA_BITS-1 downto 0);
    variable counter, delay    : integer;
    file     mp3_chip_out : TEXT OPEN WRITE_MODE IS MP3_FILENAME;
    variable value      : line;
  begin
    mp3_req <= '0';
    counter := FLASH_DATA_BITS - 1;

    print("MP3 chip is waiting for reset pulse");
    wait until mp3_rst = '1';
    print("MP3 chip ready, asserting request");
                          
    loop
      case counter is
        when 0 => delay := 2; 
        when 1 => delay := 0; 
        when 2 => delay := 1; 
        when 3 => delay := 3; 
        when others => delay := 2;
      end case;

      if (delay > 0) then
        for i in 1 to delay loop
          mp3_req <= '0';                     -- stop request
          wait until clk = '1';
        end loop;
      end if;

      mp3_req <= '1';                         -- request data
      wait until clk = '1';                   -- wait for clock
      if (mp3_valid = '0') then
        wait until mp3_valid = '1';           -- wait for valid data
      end if;
      wait until clk = '0';                   -- wait for clock
      sample(counter) := mp3_data;            -- store sample
                                              
      if (counter = 0) then                   -- received one word
        write(value, to_bitvector(sample));   -- write to file
        writeline(mp3_chip_out, value);               
        counter := FLASH_DATA_BITS;   
      end if;
      counter := counter - 1;
    end loop;
  end process;

end;

