library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_signed.all;
use IEEE.std_logic_arith.all;
use std.textio.all;
use work.tbcomp.all;
use work.ram.all;

entity tb_ram is
end;

architecture behavioral of tb_ram is

  -- Component was here

  procedure print(s : string) is
    variable L1 : line;
  begin
    L1:= new string'(s);
    writeline(output,L1);
  end;
  
  signal clk, rst, gnd, vdd                     : std_logic;
  -- CPU signals
  signal cpu_addr : std_logic_vector (15 downto 0) := "0000000000000000";
  signal cpu_in   : std_logic_vector (7 downto 0) := "00000000";
  signal cpu_out  : std_logic_vector (7 downto 0) := "00000000";
  signal cpu_we   : std_logic := '0';
  signal cpu_re   : std_logic := '0';

  -- Used to count cycles
  signal mr_counter : std_logic_vector(7 downto 0);

  
begin

  ----------------------------
  -- UART
  ----------------------------

  mr_ram : mp3ram
      generic map ( BYTES => 1024 )
      port map (clk => clk,
		addr => cpu_addr(9 downto 0),
                data_out => cpu_in,
                data_in => cpu_out,
                we => cpu_we
		);
  
  ----------------------------
  -- Clock and Reset
  ----------------------------

  clkgen: clock_generator
    generic map( 83 ns )
    port map ( CLK => CLK );

  rstgen: reset_generator
    generic map( 450 ns )
    port map ( RST => RST );

  gnd <= '0';
  vdd <= '1';

  bar : process(clk, rst)
      variable text_line : line;
      variable indata : integer;
  begin
      if (rst = '0') then
	  cpu_addr <= "0000000000000000";
	  cpu_re <= '0';
	  mr_counter <= "00000000";
      elsif rising_edge(clk) then
	  mr_counter <= mr_counter + 1;
	  -- Test to read from rom address $FFFD
	  -- cpu_addr <= "1111111111111101";
	  -- cpu_re <= '1';
	  case mr_counter is
	      when "00000001" =>
		  -- Test to write to adress $0001
		  cpu_addr <= "0000000000000001";
		  cpu_out <= "10101010";
		  cpu_we <= '1';
	      when "00000010" =>
		  cpu_we <= '0';
	      when "00000011" =>
		  cpu_addr <= "0000000000000000";
	      when "00000100" =>
		  -- Test to read from adress $0001
		  cpu_addr <= "0000000000000001";
	      when "00000101" =>
		  cpu_addr <= "0000000000000000";
	      when others =>
	  end case;
      end if;
  end process;

end;
