library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_signed.all;
use IEEE.std_logic_arith.all;
use std.textio.all;
use work.tbcomp.all;
-- use work.memcomp.all;
-- use work.controller_types.all;
use work.uart_pck.all;

entity tb_uart is
end;

architecture behavioral of tb_uart is

  constant UART_IN_FILENAME      : string := "debug/uart_in.txt";  -- UART jox
  constant UART_OUT_FILENAME      : string := "debug/uart_out.txt";  -- UART jox

--   component uart
    
--     port( clk   : in  std_logic;
-- 	  rst   : in  std_logic;
-- 	  rxd   : in  std_logic;
-- 	  txd   : out std_logic;
-- 	  uarti : in  uart_ctrl_in_type;
-- 	  uarto : out uart_ctrl_out_type
-- 	  );
--   end component;

  procedure print(s : string) is
    variable L1 : line;
  begin
    L1:= new string'(s);
    writeline(output,L1);
  end;
  
  signal clk, rst, gnd, vdd                     : std_logic;
  signal rxd, txd                               : std_logic;
  signal uarti					: uart_ctrl_in_type;
  signal uarto					: uart_ctrl_out_type;

begin

  ----------------------------
  -- UART
  ----------------------------

  uarten: uart
    -- Test loopback with maste clock @ 12 MHz and Baud rate @ 115.2 kbps
    generic map ( MASTER_CLOCK => 12_000_000, BAUD_RATE_OUT => 115_200, BAUD_RATE_IN => 115_200 )
    port map ( clk         => clk,
               rst         => rst,
               rxd         => rxd,
               txd         => txd,
	       uarti       => uarti,
	       uarto       => uarto
             );
               
  ----------------------------
  -- Clock and Reset
  ----------------------------

  clkgen: clock_generator
    generic map( 83 ns )
    port map ( CLK => CLK );

  rstgen: reset_generator
    generic map( 450 ns )
    port map ( RST => RST );

  gnd <= '0';
  vdd <= '1';
  -- Loop-back
  rxd <= txd;

  foo : process(clk, rst)
      file uart_input : TEXT IS IN UART_IN_FILENAME;
      variable text_line : line;
      variable indata : integer;
  begin
      if (rst = '0') then
	 uarti.send <= '0';
      elsif rising_edge(clk) then
	  if (uarto.txempty = '1') then
	      if (not endfile(uart_input)) then
		  readline(uart_input, text_line);
		  read(text_line, indata);
		  uarti.data <= conv_std_logic_vector(indata,uarti.data'length);
		  uarti.send <= '1';
	      else
		  uarti.send <= '0';
	      end if;
	  end if;
      end if;
  end process;

  bar: process(clk, rst)
      file uart_output : TEXT OPEN WRITE_MODE IS UART_OUT_FILENAME;
      variable utdata : line;
  begin
      if (rst = '0') then
	  uarti.ack_rcv <= '0';
      elsif rising_edge(clk) then
	  if (uarto.rxready = '1') and (uarti.ack_rcv = '0') then
	      write(utdata, to_bitvector(uarto.data));  -- write to file
	      -- print(utdata);
	      writeline(uart_output, utdata);
	      uarti.ack_rcv <= '1';
	  elsif (uarti.ack_rcv = '1') then
	      uarti.ack_rcv <= '0';
	  end if;
      end if;
  end process;
end;
