----------------------------------------------------------------------------
----------------------------------------------------------------------------
--  The Free IP Project
--  VHDL DES Core
--  (c) 1999, The Free IP Project and David Kessner
--
--
--  FREE IP GENERAL PUBLIC LICENSE
--  TERMS AND CONDITIONS FOR USE, COPYING, DISTRIBUTION, AND MODIFICATION
--
--  1.  You may copy and distribute verbatim copies of this core, as long
--      as this file, and the other associated files, remain intact and
--      unmodified.  Modifications are outlined below.  Also, see the
--      import/export warning above for further restrictions on
--      distribution.
--  2.  You may use this core in any way, be it academic, commercial, or
--      military.  Modified or not.
--  3.  Distribution of this core must be free of charge.  Charging is
--      allowed only for value added services.  Value added services
--      would include copying fees, modifications, customizations, and
--      inclusion in other products.
--  4.  If a modified source code is distributed, the original unmodified
--      source code must also be included (or a link to the Free IP web
--      site).  In the modified source code there must be clear
--      identification of the modified version.
--  5.  Visit the Free IP web site for additional information.
--      http://www.free-ip.com
--
----------------------------------------------------------------------------
----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

package rom is
  component mp3rom
    port (addr	:in std_logic_vector (11 downto 0);
          data  :out std_logic_vector (7 downto 0)
         );
  end component;
end rom;


------------------------------------------------------------------------------
------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.test_suite.all;

entity mp3rom is
    port (addr	:in std_logic_vector (11 downto 0);
          data  :out std_logic_vector (7 downto 0)
         );
end mp3rom;

architecture JESUS of mp3rom is
begin
  process (addr)
  begin
    case addr(11 downto 0) is
      --addrbits 12
      --databits 8
      --varname data
      --little-endian
      --start_of_rom      -- rom2vhdl output.  Tue Dec  7 11:33:07 2004
      when "000000000000" =>  data <= "10100010";  -- 0 = A2
      when "000000000001" =>  data <= "11111111";  -- 1 = FF
      when "000000000010" =>  data <= "10011010";  -- 2 = 9A
      when "000000000011" =>  data <= "00100000";  -- 3 = 20
      when "000000000100" =>  data <= "10011010";  -- 4 = 9A
      when "000000000101" =>  data <= "11110000";  -- 5 = F0
      when "000000000110" =>  data <= "00100000";  -- 6 = 20
      when "000000000111" =>  data <= "11001001";  -- 7 = C9
      when "000000001000" =>  data <= "11110000";  -- 8 = F0
      when "000000001001" =>  data <= "00100000";  -- 9 = 20
      when "000000001010" =>  data <= "11101000";  -- A = E8
      when "000000001011" =>  data <= "11110000";  -- B = F0
      when "000000001100" =>  data <= "10101101";  -- C = AD
      when "000000001101" =>  data <= "00000000";  -- D = 0
      when "000000001110" =>  data <= "00000100";  -- E = 4
      when "000000001111" =>  data <= "10000101";  -- F = 85
      when "000000010000" =>  data <= "10000100";  -- 10 = 84
      when "000000010001" =>  data <= "00100000";  -- 11 = 20
      when "000000010010" =>  data <= "11000010";  -- 12 = C2
      when "000000010011" =>  data <= "11110100";  -- 13 = F4
      when "000000010100" =>  data <= "00100000";  -- 14 = 20
      when "000000010101" =>  data <= "10010011";  -- 15 = 93
      when "000000010110" =>  data <= "11110010";  -- 16 = F2
      when "000000010111" =>  data <= "10101101";  -- 17 = AD
      when "000000011000" =>  data <= "00010010";  -- 18 = 12
      when "000000011001" =>  data <= "10000000";  -- 19 = 80
      when "000000011010" =>  data <= "00010000";  -- 1A = 10
      when "000000011011" =>  data <= "00110011";  -- 1B = 33
      when "000000011100" =>  data <= "10101101";  -- 1C = AD
      when "000000011101" =>  data <= "00010010";  -- 1D = 12
      when "000000011110" =>  data <= "10000000";  -- 1E = 80
      when "000000011111" =>  data <= "00110000";  -- 1F = 30
      when "000000100000" =>  data <= "11111011";  -- 20 = FB
      when "000000100001" =>  data <= "10100101";  -- 21 = A5
      when "000000100010" =>  data <= "10001110";  -- 22 = 8E
      when "000000100011" =>  data <= "11001001";  -- 23 = C9
      when "000000100100" =>  data <= "00000001";  -- 24 = 1
      when "000000100101" =>  data <= "11010000";  -- 25 = D0
      when "000000100110" =>  data <= "00001101";  -- 26 = D
      when "000000100111" =>  data <= "00100000";  -- 27 = 20
      when "000000101000" =>  data <= "10011101";  -- 28 = 9D
      when "000000101001" =>  data <= "11110001";  -- 29 = F1
      when "000000101010" =>  data <= "10101001";  -- 2A = A9
      when "000000101011" =>  data <= "00000010";  -- 2B = 2
      when "000000101100" =>  data <= "10000101";  -- 2C = 85
      when "000000101101" =>  data <= "10001110";  -- 2D = 8E
      when "000000101110" =>  data <= "00100000";  -- 2E = 20
      when "000000101111" =>  data <= "11010001";  -- 2F = D1
      when "000000110000" =>  data <= "11110010";  -- 30 = F2
      when "000000110001" =>  data <= "01001100";  -- 31 = 4C
      when "000000110010" =>  data <= "01001111";  -- 32 = 4F
      when "000000110011" =>  data <= "11110000";  -- 33 = F0
      when "000000110100" =>  data <= "11001001";  -- 34 = C9
      when "000000110101" =>  data <= "00000010";  -- 35 = 2
      when "000000110110" =>  data <= "11010000";  -- 36 = D0
      when "000000110111" =>  data <= "00001101";  -- 37 = D
      when "000000111000" =>  data <= "00100000";  -- 38 = 20
      when "000000111001" =>  data <= "10011101";  -- 39 = 9D
      when "000000111010" =>  data <= "11110001";  -- 3A = F1
      when "000000111011" =>  data <= "10101001";  -- 3B = A9
      when "000000111100" =>  data <= "00000001";  -- 3C = 1
      when "000000111101" =>  data <= "10000101";  -- 3D = 85
      when "000000111110" =>  data <= "10001110";  -- 3E = 8E
      when "000000111111" =>  data <= "00100000";  -- 3F = 20
      when "000001000000" =>  data <= "11010001";  -- 40 = D1
      when "000001000001" =>  data <= "11110010";  -- 41 = F2
      when "000001000010" =>  data <= "01001100";  -- 42 = 4C
      when "000001000011" =>  data <= "01001111";  -- 43 = 4F
      when "000001000100" =>  data <= "11110000";  -- 44 = F0
      when "000001000101" =>  data <= "10101001";  -- 45 = A9
      when "000001000110" =>  data <= "00000001";  -- 46 = 1
      when "000001000111" =>  data <= "10000101";  -- 47 = 85
      when "000001001000" =>  data <= "10001110";  -- 48 = 8E
      when "000001001001" =>  data <= "00100000";  -- 49 = 20
      when "000001001010" =>  data <= "01110011";  -- 4A = 73
      when "000001001011" =>  data <= "11110001";  -- 4B = F1
      when "000001001100" =>  data <= "00100000";  -- 4C = 20
      when "000001001101" =>  data <= "10100100";  -- 4D = A4
      when "000001001110" =>  data <= "11110010";  -- 4E = F2
      when "000001001111" =>  data <= "10101101";  -- 4F = AD
      when "000001010000" =>  data <= "00010010";  -- 50 = 12
      when "000001010001" =>  data <= "10000000";  -- 51 = 80
      when "000001010010" =>  data <= "00101001";  -- 52 = 29
      when "000001010011" =>  data <= "01000000";  -- 53 = 40
      when "000001010100" =>  data <= "11110000";  -- 54 = F0
      when "000001010101" =>  data <= "00010101";  -- 55 = 15
      when "000001010110" =>  data <= "10101101";  -- 56 = AD
      when "000001010111" =>  data <= "00010010";  -- 57 = 12
      when "000001011000" =>  data <= "10000000";  -- 58 = 80
      when "000001011001" =>  data <= "00101001";  -- 59 = 29
      when "000001011010" =>  data <= "01000000";  -- 5A = 40
      when "000001011011" =>  data <= "11010000";  -- 5B = D0
      when "000001011100" =>  data <= "11111001";  -- 5C = F9
      when "000001011101" =>  data <= "10100101";  -- 5D = A5
      when "000001011110" =>  data <= "10001110";  -- 5E = 8E
      when "000001011111" =>  data <= "11110000";  -- 5F = F0
      when "000001100000" =>  data <= "10110011";  -- 60 = B3
      when "000001100001" =>  data <= "00100000";  -- 61 = 20
      when "000001100010" =>  data <= "10101000";  -- 62 = A8
      when "000001100011" =>  data <= "11110001";  -- 63 = F1
      when "000001100100" =>  data <= "10101001";  -- 64 = A9
      when "000001100101" =>  data <= "00000000";  -- 65 = 0
      when "000001100110" =>  data <= "10000101";  -- 66 = 85
      when "000001100111" =>  data <= "10001110";  -- 67 = 8E
      when "000001101000" =>  data <= "00100000";  -- 68 = 20
      when "000001101001" =>  data <= "11010001";  -- 69 = D1
      when "000001101010" =>  data <= "11110010";  -- 6A = F2
      when "000001101011" =>  data <= "10101101";  -- 6B = AD
      when "000001101100" =>  data <= "00010010";  -- 6C = 12
      when "000001101101" =>  data <= "10000000";  -- 6D = 80
      when "000001101110" =>  data <= "00101001";  -- 6E = 29
      when "000001101111" =>  data <= "00110000";  -- 6F = 30
      when "000001110000" =>  data <= "11110000";  -- 70 = F0
      when "000001110001" =>  data <= "00001100";  -- 71 = C
      when "000001110010" =>  data <= "01001000";  -- 72 = 48
      when "000001110011" =>  data <= "10101101";  -- 73 = AD
      when "000001110100" =>  data <= "00010010";  -- 74 = 12
      when "000001110101" =>  data <= "10000000";  -- 75 = 80
      when "000001110110" =>  data <= "00101001";  -- 76 = 29
      when "000001110111" =>  data <= "00110000";  -- 77 = 30
      when "000001111000" =>  data <= "11010000";  -- 78 = D0
      when "000001111001" =>  data <= "11111001";  -- 79 = F9
      when "000001111010" =>  data <= "01101000";  -- 7A = 68
      when "000001111011" =>  data <= "00100000";  -- 7B = 20
      when "000001111100" =>  data <= "10101110";  -- 7C = AE
      when "000001111101" =>  data <= "11110001";  -- 7D = F1
      when "000001111110" =>  data <= "10101101";  -- 7E = AD
      when "000001111111" =>  data <= "00000011";  -- 7F = 3
      when "000010000000" =>  data <= "10000000";  -- 80 = 80
      when "000010000001" =>  data <= "00010000";  -- 81 = 10
      when "000010000010" =>  data <= "00001011";  -- 82 = B
      when "000010000011" =>  data <= "10100101";  -- 83 = A5
      when "000010000100" =>  data <= "10001110";  -- 84 = 8E
      when "000010000101" =>  data <= "11001001";  -- 85 = C9
      when "000010000110" =>  data <= "00000001";  -- 86 = 1
      when "000010000111" =>  data <= "11010000";  -- 87 = D0
      when "000010001000" =>  data <= "00000101";  -- 88 = 5
      when "000010001001" =>  data <= "10101001";  -- 89 = A9
      when "000010001010" =>  data <= "00010000";  -- 8A = 10
      when "000010001011" =>  data <= "00100000";  -- 8B = 20
      when "000010001100" =>  data <= "10101110";  -- 8C = AE
      when "000010001101" =>  data <= "11110001";  -- 8D = F1
      when "000010001110" =>  data <= "10100101";  -- 8E = A5
      when "000010001111" =>  data <= "10001110";  -- 8F = 8E
      when "000010010000" =>  data <= "11001001";  -- 90 = C9
      when "000010010001" =>  data <= "00000001";  -- 91 = 1
      when "000010010010" =>  data <= "11010000";  -- 92 = D0
      when "000010010011" =>  data <= "00000011";  -- 93 = 3
      when "000010010100" =>  data <= "00100000";  -- 94 = 20
      when "000010010101" =>  data <= "00101100";  -- 95 = 2C
      when "000010010110" =>  data <= "11110010";  -- 96 = F2
      when "000010010111" =>  data <= "01001100";  -- 97 = 4C
      when "000010011000" =>  data <= "00010111";  -- 98 = 17
      when "000010011001" =>  data <= "11110000";  -- 99 = F0
      when "000010011010" =>  data <= "10101001";  -- 9A = A9
      when "000010011011" =>  data <= "00000000";  -- 9B = 0
      when "000010011100" =>  data <= "10000101";  -- 9C = 85
      when "000010011101" =>  data <= "10001110";  -- 9D = 8E
      when "000010011110" =>  data <= "10000101";  -- 9E = 85
      when "000010011111" =>  data <= "10001111";  -- 9F = 8F
      when "000010100000" =>  data <= "10000101";  -- A0 = 85
      when "000010100001" =>  data <= "10010000";  -- A1 = 90
      when "000010100010" =>  data <= "10000101";  -- A2 = 85
      when "000010100011" =>  data <= "10010001";  -- A3 = 91
      when "000010100100" =>  data <= "10000101";  -- A4 = 85
      when "000010100101" =>  data <= "10010010";  -- A5 = 92
      when "000010100110" =>  data <= "10000101";  -- A6 = 85
      when "000010100111" =>  data <= "10010011";  -- A7 = 93
      when "000010101000" =>  data <= "01100000";  -- A8 = 60
      when "000010101001" =>  data <= "00000000";  -- A9 = 0
      when "000010101010" =>  data <= "10010111";  -- AA = 97
      when "000010101011" =>  data <= "00000010";  -- AB = 2
      when "000010101100" =>  data <= "10010111";  -- AC = 97
      when "000010101101" =>  data <= "00000100";  -- AD = 4
      when "000010101110" =>  data <= "11111001";  -- AE = F9
      when "000010101111" =>  data <= "00000110";  -- AF = 6
      when "000010110000" =>  data <= "11111001";  -- B0 = F9
      when "000010110001" =>  data <= "00001000";  -- B1 = 8
      when "000010110010" =>  data <= "00011010";  -- B2 = 1A
      when "000010110011" =>  data <= "00001010";  -- B3 = A
      when "000010110100" =>  data <= "00000100";  -- B4 = 4
      when "000010110101" =>  data <= "00001100";  -- B5 = C
      when "000010110110" =>  data <= "00000111";  -- B6 = 7
      when "000010110111" =>  data <= "00001110";  -- B7 = E
      when "000010111000" =>  data <= "00000010";  -- B8 = 2
      when "000010111001" =>  data <= "00010000";  -- B9 = 10
      when "000010111010" =>  data <= "10100011";  -- BA = A3
      when "000010111011" =>  data <= "00010010";  -- BB = 12
      when "000010111100" =>  data <= "00000001";  -- BC = 1
      when "000010111101" =>  data <= "00010100";  -- BD = 14
      when "000010111110" =>  data <= "00000000";  -- BE = 0
      when "000010111111" =>  data <= "00010110";  -- BF = 16
      when "000011000000" =>  data <= "00000000";  -- C0 = 0
      when "000011000001" =>  data <= "00011000";  -- C1 = 18
      when "000011000010" =>  data <= "00000000";  -- C2 = 0
      when "000011000011" =>  data <= "00011010";  -- C3 = 1A
      when "000011000100" =>  data <= "00000000";  -- C4 = 0
      when "000011000101" =>  data <= "00011100";  -- C5 = 1C
      when "000011000110" =>  data <= "00000000";  -- C6 = 0
      when "000011000111" =>  data <= "00011100";  -- C7 = 1C
      when "000011001000" =>  data <= "00000000";  -- C8 = 0
      when "000011001001" =>  data <= "10100010";  -- C9 = A2
      when "000011001010" =>  data <= "00000000";  -- CA = 0
      when "000011001011" =>  data <= "10101101";  -- CB = AD
      when "000011001100" =>  data <= "00010111";  -- CC = 17
      when "000011001101" =>  data <= "10000000";  -- CD = 80
      when "000011001110" =>  data <= "00010000";  -- CE = 10
      when "000011001111" =>  data <= "11111011";  -- CF = FB
      when "000011010000" =>  data <= "10111101";  -- D0 = BD
      when "000011010001" =>  data <= "10101001";  -- D1 = A9
      when "000011010010" =>  data <= "11110000";  -- D2 = F0
      when "000011010011" =>  data <= "10001101";  -- D3 = 8D
      when "000011010100" =>  data <= "00011001";  -- D4 = 19
      when "000011010101" =>  data <= "10000000";  -- D5 = 80
      when "000011010110" =>  data <= "11101000";  -- D6 = E8
      when "000011010111" =>  data <= "10111101";  -- D7 = BD
      when "000011011000" =>  data <= "10101001";  -- D8 = A9
      when "000011011001" =>  data <= "11110000";  -- D9 = F0
      when "000011011010" =>  data <= "10001101";  -- DA = 8D
      when "000011011011" =>  data <= "00011010";  -- DB = 1A
      when "000011011100" =>  data <= "10000000";  -- DC = 80
      when "000011011101" =>  data <= "11101000";  -- DD = E8
      when "000011011110" =>  data <= "10101001";  -- DE = A9
      when "000011011111" =>  data <= "10000000";  -- DF = 80
      when "000011100000" =>  data <= "10001101";  -- E0 = 8D
      when "000011100001" =>  data <= "00011000";  -- E1 = 18
      when "000011100010" =>  data <= "10000000";  -- E2 = 80
      when "000011100011" =>  data <= "11100000";  -- E3 = E0
      when "000011100100" =>  data <= "00100000";  -- E4 = 20
      when "000011100101" =>  data <= "11010000";  -- E5 = D0
      when "000011100110" =>  data <= "11100100";  -- E6 = E4
      when "000011100111" =>  data <= "01100000";  -- E7 = 60
      when "000011101000" =>  data <= "10101101";  -- E8 = AD
      when "000011101001" =>  data <= "00010010";  -- E9 = 12
      when "000011101010" =>  data <= "10000000";  -- EA = 80
      when "000011101011" =>  data <= "00101001";  -- EB = 29
      when "000011101100" =>  data <= "11000000";  -- EC = C0
      when "000011101101" =>  data <= "11001001";  -- ED = C9
      when "000011101110" =>  data <= "11000000";  -- EE = C0
      when "000011101111" =>  data <= "11110000";  -- EF = F0
      when "000011110000" =>  data <= "00000001";  -- F0 = 1
      when "000011110001" =>  data <= "01100000";  -- F1 = 60
      when "000011110010" =>  data <= "10100010";  -- F2 = A2
      when "000011110011" =>  data <= "00000000";  -- F3 = 0
      when "000011110100" =>  data <= "10111101";  -- F4 = BD
      when "000011110101" =>  data <= "00000000";  -- F5 = 0
      when "000011110110" =>  data <= "11110001";  -- F6 = F1
      when "000011110111" =>  data <= "10011101";  -- F7 = 9D
      when "000011111000" =>  data <= "00000000";  -- F8 = 0
      when "000011111001" =>  data <= "00000010";  -- F9 = 2
      when "000011111010" =>  data <= "11101000";  -- FA = E8
      when "000011111011" =>  data <= "11010000";  -- FB = D0
      when "000011111100" =>  data <= "11110111";  -- FC = F7
      when "000011111101" =>  data <= "01001100";  -- FD = 4C
      when "000011111110" =>  data <= "00000000";  -- FE = 0
      when "000011111111" =>  data <= "00000010";  -- FF = 2
      when "000100000000" =>  data <= "10101001";  -- 100 = A9
      when "000100000001" =>  data <= "10000000";  -- 101 = 80
      when "000100000010" =>  data <= "10001101";  -- 102 = 8D
      when "000100000011" =>  data <= "00000100";  -- 103 = 4
      when "000100000100" =>  data <= "10000000";  -- 104 = 80
      when "000100000101" =>  data <= "10100000";  -- 105 = A0
      when "000100000110" =>  data <= "00000000";  -- 106 = 0
      when "000100000111" =>  data <= "10101110";  -- 107 = AE
      when "000100001000" =>  data <= "00010011";  -- 108 = 13
      when "000100001001" =>  data <= "10000000";  -- 109 = 80
      when "000100001010" =>  data <= "00010000";  -- 10A = 10
      when "000100001011" =>  data <= "11111011";  -- 10B = FB
      when "000100001100" =>  data <= "10101101";  -- 10C = AD
      when "000100001101" =>  data <= "00010101";  -- 10D = 15
      when "000100001110" =>  data <= "10000000";  -- 10E = 80
      when "000100001111" =>  data <= "10001110";  -- 10F = 8E
      when "000100010000" =>  data <= "00010100";  -- 110 = 14
      when "000100010001" =>  data <= "10000000";  -- 111 = 80
      when "000100010010" =>  data <= "10011001";  -- 112 = 99
      when "000100010011" =>  data <= "00000000";  -- 113 = 0
      when "000100010100" =>  data <= "00000000";  -- 114 = 0
      when "000100010101" =>  data <= "11001000";  -- 115 = C8
      when "000100010110" =>  data <= "11000000";  -- 116 = C0
      when "000100010111" =>  data <= "00001000";  -- 117 = 8
      when "000100011000" =>  data <= "11010000";  -- 118 = D0
      when "000100011001" =>  data <= "11101101";  -- 119 = ED
      when "000100011010" =>  data <= "10100101";  -- 11A = A5
      when "000100011011" =>  data <= "00000000";  -- 11B = 0
      when "000100011100" =>  data <= "00100101";  -- 11C = 25
      when "000100011101" =>  data <= "00000001";  -- 11D = 1
      when "000100011110" =>  data <= "00100101";  -- 11E = 25
      when "000100011111" =>  data <= "00000010";  -- 11F = 2
      when "000100100000" =>  data <= "00100101";  -- 120 = 25
      when "000100100001" =>  data <= "00000011";  -- 121 = 3
      when "000100100010" =>  data <= "11001001";  -- 122 = C9
      when "000100100011" =>  data <= "11111111";  -- 123 = FF
      when "000100100100" =>  data <= "11010000";  -- 124 = D0
      when "000100100101" =>  data <= "00000011";  -- 125 = 3
      when "000100100110" =>  data <= "01101100";  -- 126 = 6C
      when "000100100111" =>  data <= "11111100";  -- 127 = FC
      when "000100101000" =>  data <= "11111111";  -- 128 = FF
      when "000100101001" =>  data <= "10100101";  -- 129 = A5
      when "000100101010" =>  data <= "00000000";  -- 12A = 0
      when "000100101011" =>  data <= "10001101";  -- 12B = 8D
      when "000100101100" =>  data <= "00100000";  -- 12C = 20
      when "000100101101" =>  data <= "10000000";  -- 12D = 80
      when "000100101110" =>  data <= "10100101";  -- 12E = A5
      when "000100101111" =>  data <= "00000001";  -- 12F = 1
      when "000100110000" =>  data <= "10001101";  -- 130 = 8D
      when "000100110001" =>  data <= "00100001";  -- 131 = 21
      when "000100110010" =>  data <= "10000000";  -- 132 = 80
      when "000100110011" =>  data <= "10100101";  -- 133 = A5
      when "000100110100" =>  data <= "00000010";  -- 134 = 2
      when "000100110101" =>  data <= "10001101";  -- 135 = 8D
      when "000100110110" =>  data <= "00100010";  -- 136 = 22
      when "000100110111" =>  data <= "10000000";  -- 137 = 80
      when "000100111000" =>  data <= "10100101";  -- 138 = A5
      when "000100111001" =>  data <= "00000011";  -- 139 = 3
      when "000100111010" =>  data <= "10001101";  -- 13A = 8D
      when "000100111011" =>  data <= "00100011";  -- 13B = 23
      when "000100111100" =>  data <= "10000000";  -- 13C = 80
      when "000100111101" =>  data <= "10101101";  -- 13D = AD
      when "000100111110" =>  data <= "00010011";  -- 13E = 13
      when "000100111111" =>  data <= "10000000";  -- 13F = 80
      when "000101000000" =>  data <= "00010000";  -- 140 = 10
      when "000101000001" =>  data <= "11111011";  -- 141 = FB
      when "000101000010" =>  data <= "10101110";  -- 142 = AE
      when "000101000011" =>  data <= "00010101";  -- 143 = 15
      when "000101000100" =>  data <= "10000000";  -- 144 = 80
      when "000101000101" =>  data <= "10001101";  -- 145 = 8D
      when "000101000110" =>  data <= "00010100";  -- 146 = 14
      when "000101000111" =>  data <= "10000000";  -- 147 = 80
      when "000101001000" =>  data <= "10001110";  -- 148 = 8E
      when "000101001001" =>  data <= "00000101";  -- 149 = 5
      when "000101001010" =>  data <= "10000000";  -- 14A = 80
      when "000101001011" =>  data <= "11100110";  -- 14B = E6
      when "000101001100" =>  data <= "00000011";  -- 14C = 3
      when "000101001101" =>  data <= "11010000";  -- 14D = D0
      when "000101001110" =>  data <= "00001010";  -- 14E = A
      when "000101001111" =>  data <= "11100110";  -- 14F = E6
      when "000101010000" =>  data <= "00000010";  -- 150 = 2
      when "000101010001" =>  data <= "11010000";  -- 151 = D0
      when "000101010010" =>  data <= "00000110";  -- 152 = 6
      when "000101010011" =>  data <= "11100110";  -- 153 = E6
      when "000101010100" =>  data <= "00000001";  -- 154 = 1
      when "000101010101" =>  data <= "11010000";  -- 155 = D0
      when "000101010110" =>  data <= "00000010";  -- 156 = 2
      when "000101010111" =>  data <= "11100110";  -- 157 = E6
      when "000101011000" =>  data <= "00000000";  -- 158 = 0
      when "000101011001" =>  data <= "11000110";  -- 159 = C6
      when "000101011010" =>  data <= "00000111";  -- 15A = 7
      when "000101011011" =>  data <= "11010000";  -- 15B = D0
      when "000101011100" =>  data <= "00001010";  -- 15C = A
      when "000101011101" =>  data <= "11000110";  -- 15D = C6
      when "000101011110" =>  data <= "00000110";  -- 15E = 6
      when "000101011111" =>  data <= "11010000";  -- 15F = D0
      when "000101100000" =>  data <= "00000110";  -- 160 = 6
      when "000101100001" =>  data <= "11000110";  -- 161 = C6
      when "000101100010" =>  data <= "00000101";  -- 162 = 5
      when "000101100011" =>  data <= "11010000";  -- 163 = D0
      when "000101100100" =>  data <= "00000010";  -- 164 = 2
      when "000101100101" =>  data <= "11000110";  -- 165 = C6
      when "000101100110" =>  data <= "00000100";  -- 166 = 4
      when "000101100111" =>  data <= "10100101";  -- 167 = A5
      when "000101101000" =>  data <= "00000100";  -- 168 = 4
      when "000101101001" =>  data <= "00000101";  -- 169 = 5
      when "000101101010" =>  data <= "00000101";  -- 16A = 5
      when "000101101011" =>  data <= "00000101";  -- 16B = 5
      when "000101101100" =>  data <= "00000110";  -- 16C = 6
      when "000101101101" =>  data <= "00000101";  -- 16D = 5
      when "000101101110" =>  data <= "00000111";  -- 16E = 7
      when "000101101111" =>  data <= "11010000";  -- 16F = D0
      when "000101110000" =>  data <= "10111000";  -- 170 = B8
      when "000101110001" =>  data <= "11110000";  -- 171 = F0
      when "000101110010" =>  data <= "10001101";  -- 172 = 8D
      when "000101110011" =>  data <= "10100000";  -- 173 = A0
      when "000101110100" =>  data <= "00000111";  -- 174 = 7
      when "000101110101" =>  data <= "10100010";  -- 175 = A2
      when "000101110110" =>  data <= "00000011";  -- 176 = 3
      when "000101110111" =>  data <= "10110001";  -- 177 = B1
      when "000101111000" =>  data <= "10001111";  -- 178 = 8F
      when "000101111001" =>  data <= "10011101";  -- 179 = 9D
      when "000101111010" =>  data <= "00001010";  -- 17A = A
      when "000101111011" =>  data <= "10000000";  -- 17B = 80
      when "000101111100" =>  data <= "10010101";  -- 17C = 95
      when "000101111101" =>  data <= "10000101";  -- 17D = 85
      when "000101111110" =>  data <= "10001000";  -- 17E = 88
      when "000101111111" =>  data <= "11001010";  -- 17F = CA
      when "000110000000" =>  data <= "00010000";  -- 180 = 10
      when "000110000001" =>  data <= "11110101";  -- 181 = F5
      when "000110000010" =>  data <= "10100000";  -- 182 = A0
      when "000110000011" =>  data <= "00000011";  -- 183 = 3
      when "000110000100" =>  data <= "00011000";  -- 184 = 18
      when "000110000101" =>  data <= "10110001";  -- 185 = B1
      when "000110000110" =>  data <= "10001111";  -- 186 = 8F
      when "000110000111" =>  data <= "01111001";  -- 187 = 79
      when "000110001000" =>  data <= "10000101";  -- 188 = 85
      when "000110001001" =>  data <= "00000000";  -- 189 = 0
      when "000110001010" =>  data <= "10011001";  -- 18A = 99
      when "000110001011" =>  data <= "00001110";  -- 18B = E
      when "000110001100" =>  data <= "10000000";  -- 18C = 80
      when "000110001101" =>  data <= "10001000";  -- 18D = 88
      when "000110001110" =>  data <= "00010000";  -- 18E = 10
      when "000110001111" =>  data <= "11110101";  -- 18F = F5
      when "000110010000" =>  data <= "10101001";  -- 190 = A9
      when "000110010001" =>  data <= "00100000";  -- 191 = 20
      when "000110010010" =>  data <= "10001101";  -- 192 = 8D
      when "000110010011" =>  data <= "00000100";  -- 193 = 4
      when "000110010100" =>  data <= "10000000";  -- 194 = 80
      when "000110010101" =>  data <= "10101001";  -- 195 = A9
      when "000110010110" =>  data <= "10000000";  -- 196 = 80
      when "000110010111" =>  data <= "10001101";  -- 197 = 8D
      when "000110011000" =>  data <= "00000110";  -- 198 = 6
      when "000110011001" =>  data <= "10000000";  -- 199 = 80
      when "000110011010" =>  data <= "01001100";  -- 19A = 4C
      when "000110011011" =>  data <= "00010101";  -- 19B = 15
      when "000110011100" =>  data <= "11110010";  -- 19C = F2
      when "000110011101" =>  data <= "10101001";  -- 19D = A9
      when "000110011110" =>  data <= "00100000";  -- 19E = 20
      when "000110011111" =>  data <= "10001101";  -- 19F = 8D
      when "000110100000" =>  data <= "00000100";  -- 1A0 = 4
      when "000110100001" =>  data <= "10000000";  -- 1A1 = 80
      when "000110100010" =>  data <= "10101001";  -- 1A2 = A9
      when "000110100011" =>  data <= "01000000";  -- 1A3 = 40
      when "000110100100" =>  data <= "10001101";  -- 1A4 = 8D
      when "000110100101" =>  data <= "00000110";  -- 1A5 = 6
      when "000110100110" =>  data <= "10000000";  -- 1A6 = 80
      when "000110100111" =>  data <= "01100000";  -- 1A7 = 60
      when "000110101000" =>  data <= "10101001";  -- 1A8 = A9
      when "000110101001" =>  data <= "00010000";  -- 1A9 = 10
      when "000110101010" =>  data <= "10001101";  -- 1AA = 8D
      when "000110101011" =>  data <= "00000100";  -- 1AB = 4
      when "000110101100" =>  data <= "10000000";  -- 1AC = 80
      when "000110101101" =>  data <= "01100000";  -- 1AD = 60
      when "000110101110" =>  data <= "00101001";  -- 1AE = 29
      when "000110101111" =>  data <= "00100000";  -- 1AF = 20
      when "000110110000" =>  data <= "11110000";  -- 1B0 = F0
      when "000110110001" =>  data <= "00101100";  -- 1B1 = 2C
      when "000110110010" =>  data <= "10100101";  -- 1B2 = A5
      when "000110110011" =>  data <= "10001111";  -- 1B3 = 8F
      when "000110110100" =>  data <= "11000101";  -- 1B4 = C5
      when "000110110101" =>  data <= "10000000";  -- 1B5 = 80
      when "000110110110" =>  data <= "11010000";  -- 1B6 = D0
      when "000110110111" =>  data <= "00010110";  -- 1B7 = 16
      when "000110111000" =>  data <= "10100101";  -- 1B8 = A5
      when "000110111001" =>  data <= "10010000";  -- 1B9 = 90
      when "000110111010" =>  data <= "11000101";  -- 1BA = C5
      when "000110111011" =>  data <= "10000001";  -- 1BB = 81
      when "000110111100" =>  data <= "11010000";  -- 1BC = D0
      when "000110111101" =>  data <= "00010000";  -- 1BD = 10
      when "000110111110" =>  data <= "10100101";  -- 1BE = A5
      when "000110111111" =>  data <= "10000010";  -- 1BF = 82
      when "000111000000" =>  data <= "00111000";  -- 1C0 = 38
      when "000111000001" =>  data <= "11101001";  -- 1C1 = E9
      when "000111000010" =>  data <= "00100000";  -- 1C2 = 20
      when "000111000011" =>  data <= "10000101";  -- 1C3 = 85
      when "000111000100" =>  data <= "10001111";  -- 1C4 = 8F
      when "000111000101" =>  data <= "10100101";  -- 1C5 = A5
      when "000111000110" =>  data <= "10000011";  -- 1C6 = 83
      when "000111000111" =>  data <= "11101001";  -- 1C7 = E9
      when "000111001000" =>  data <= "00000000";  -- 1C8 = 0
      when "000111001001" =>  data <= "10000101";  -- 1C9 = 85
      when "000111001010" =>  data <= "10010000";  -- 1CA = 90
      when "000111001011" =>  data <= "01001100";  -- 1CB = 4C
      when "000111001100" =>  data <= "11111111";  -- 1CC = FF
      when "000111001101" =>  data <= "11110001";  -- 1CD = F1
      when "000111001110" =>  data <= "10100101";  -- 1CE = A5
      when "000111001111" =>  data <= "10001111";  -- 1CF = 8F
      when "000111010000" =>  data <= "00111000";  -- 1D0 = 38
      when "000111010001" =>  data <= "11101001";  -- 1D1 = E9
      when "000111010010" =>  data <= "00100000";  -- 1D2 = 20
      when "000111010011" =>  data <= "10000101";  -- 1D3 = 85
      when "000111010100" =>  data <= "10001111";  -- 1D4 = 8F
      when "000111010101" =>  data <= "10100101";  -- 1D5 = A5
      when "000111010110" =>  data <= "10010000";  -- 1D6 = 90
      when "000111010111" =>  data <= "11101001";  -- 1D7 = E9
      when "000111011000" =>  data <= "00000000";  -- 1D8 = 0
      when "000111011001" =>  data <= "10000101";  -- 1D9 = 85
      when "000111011010" =>  data <= "10010000";  -- 1DA = 90
      when "000111011011" =>  data <= "01001100";  -- 1DB = 4C
      when "000111011100" =>  data <= "11111111";  -- 1DC = FF
      when "000111011101" =>  data <= "11110001";  -- 1DD = F1
      when "000111011110" =>  data <= "10100101";  -- 1DE = A5
      when "000111011111" =>  data <= "10001111";  -- 1DF = 8F
      when "000111100000" =>  data <= "00011000";  -- 1E0 = 18
      when "000111100001" =>  data <= "01101001";  -- 1E1 = 69
      when "000111100010" =>  data <= "00100000";  -- 1E2 = 20
      when "000111100011" =>  data <= "10000101";  -- 1E3 = 85
      when "000111100100" =>  data <= "10001111";  -- 1E4 = 8F
      when "000111100101" =>  data <= "10100101";  -- 1E5 = A5
      when "000111100110" =>  data <= "10010000";  -- 1E6 = 90
      when "000111100111" =>  data <= "01101001";  -- 1E7 = 69
      when "000111101000" =>  data <= "00000000";  -- 1E8 = 0
      when "000111101001" =>  data <= "10000101";  -- 1E9 = 85
      when "000111101010" =>  data <= "10010000";  -- 1EA = 90
      when "000111101011" =>  data <= "10100101";  -- 1EB = A5
      when "000111101100" =>  data <= "10001111";  -- 1EC = 8F
      when "000111101101" =>  data <= "11000101";  -- 1ED = C5
      when "000111101110" =>  data <= "10000010";  -- 1EE = 82
      when "000111101111" =>  data <= "11010000";  -- 1EF = D0
      when "000111110000" =>  data <= "00001110";  -- 1F0 = E
      when "000111110001" =>  data <= "10100101";  -- 1F1 = A5
      when "000111110010" =>  data <= "10010000";  -- 1F2 = 90
      when "000111110011" =>  data <= "11000101";  -- 1F3 = C5
      when "000111110100" =>  data <= "10000011";  -- 1F4 = 83
      when "000111110101" =>  data <= "11010000";  -- 1F5 = D0
      when "000111110110" =>  data <= "00001000";  -- 1F6 = 8
      when "000111110111" =>  data <= "10100110";  -- 1F7 = A6
      when "000111111000" =>  data <= "10000000";  -- 1F8 = 80
      when "000111111001" =>  data <= "10100100";  -- 1F9 = A4
      when "000111111010" =>  data <= "10000001";  -- 1FA = 81
      when "000111111011" =>  data <= "10000110";  -- 1FB = 86
      when "000111111100" =>  data <= "10001111";  -- 1FC = 8F
      when "000111111101" =>  data <= "10000100";  -- 1FD = 84
      when "000111111110" =>  data <= "10010000";  -- 1FE = 90
      when "000111111111" =>  data <= "00100000";  -- 1FF = 20
      when "001000000000" =>  data <= "10100100";  -- 200 = A4
      when "001000000001" =>  data <= "11110010";  -- 201 = F2
      when "001000000010" =>  data <= "10100101";  -- 202 = A5
      when "001000000011" =>  data <= "10001110";  -- 203 = 8E
      when "001000000100" =>  data <= "11001001";  -- 204 = C9
      when "001000000101" =>  data <= "00000001";  -- 205 = 1
      when "001000000110" =>  data <= "11010000";  -- 206 = D0
      when "001000000111" =>  data <= "00000110";  -- 207 = 6
      when "001000001000" =>  data <= "00100000";  -- 208 = 20
      when "001000001001" =>  data <= "10101000";  -- 209 = A8
      when "001000001010" =>  data <= "11110001";  -- 20A = F1
      when "001000001011" =>  data <= "01001100";  -- 20B = 4C
      when "001000001100" =>  data <= "01110011";  -- 20C = 73
      when "001000001101" =>  data <= "11110001";  -- 20D = F1
      when "001000001110" =>  data <= "01100000";  -- 20E = 60
      when "001000001111" =>  data <= "01100000";  -- 20F = 60
      when "001000010000" =>  data <= "10101001";  -- 210 = A9
      when "001000010001" =>  data <= "00001100";  -- 211 = C
      when "001000010010" =>  data <= "01001100";  -- 212 = 4C
      when "001000010011" =>  data <= "11101001";  -- 213 = E9
      when "001000010100" =>  data <= "11110100";  -- 214 = F4
      when "001000010101" =>  data <= "10101001";  -- 215 = A9
      when "001000010110" =>  data <= "00010000";  -- 216 = 10
      when "001000010111" =>  data <= "00100000";  -- 217 = 20
      when "001000011000" =>  data <= "11101001";  -- 218 = E9
      when "001000011001" =>  data <= "11110100";  -- 219 = F4
      when "001000011010" =>  data <= "10101001";  -- 21A = A9
      when "001000011011" =>  data <= "01000111";  -- 21B = 47
      when "001000011100" =>  data <= "00100000";  -- 21C = 20
      when "001000011101" =>  data <= "11101001";  -- 21D = E9
      when "001000011110" =>  data <= "11110100";  -- 21E = F4
      when "001000011111" =>  data <= "10100010";  -- 21F = A2
      when "001000100000" =>  data <= "00001000";  -- 220 = 8
      when "001000100001" =>  data <= "10101001";  -- 221 = A9
      when "001000100010" =>  data <= "00100000";  -- 222 = 20
      when "001000100011" =>  data <= "00100000";  -- 223 = 20
      when "001000100100" =>  data <= "11101001";  -- 224 = E9
      when "001000100101" =>  data <= "11110100";  -- 225 = F4
      when "001000100110" =>  data <= "11001010";  -- 226 = CA
      when "001000100111" =>  data <= "00010000";  -- 227 = 10
      when "001000101000" =>  data <= "11111000";  -- 228 = F8
      when "001000101001" =>  data <= "01001100";  -- 229 = 4C
      when "001000101010" =>  data <= "01000111";  -- 22A = 47
      when "001000101011" =>  data <= "11110010";  -- 22B = F2
      when "001000101100" =>  data <= "10101001";  -- 22C = A9
      when "001000101101" =>  data <= "00100000";  -- 22D = 20
      when "001000101110" =>  data <= "10001101";  -- 22E = 8D
      when "001000101111" =>  data <= "00000110";  -- 22F = 6
      when "001000110000" =>  data <= "10000000";  -- 230 = 80
      when "001000110001" =>  data <= "10101101";  -- 231 = AD
      when "001000110010" =>  data <= "00000111";  -- 232 = 7
      when "001000110011" =>  data <= "10000000";  -- 233 = 80
      when "001000110100" =>  data <= "11000101";  -- 234 = C5
      when "001000110101" =>  data <= "10010001";  -- 235 = 91
      when "001000110110" =>  data <= "11010000";  -- 236 = D0
      when "001000110111" =>  data <= "00001111";  -- 237 = F
      when "001000111000" =>  data <= "10101101";  -- 238 = AD
      when "001000111001" =>  data <= "00001000";  -- 239 = 8
      when "001000111010" =>  data <= "10000000";  -- 23A = 80
      when "001000111011" =>  data <= "11000101";  -- 23B = C5
      when "001000111100" =>  data <= "10010010";  -- 23C = 92
      when "001000111101" =>  data <= "11010000";  -- 23D = D0
      when "001000111110" =>  data <= "00001000";  -- 23E = 8
      when "001000111111" =>  data <= "10101101";  -- 23F = AD
      when "001001000000" =>  data <= "00001001";  -- 240 = 9
      when "001001000001" =>  data <= "10000000";  -- 241 = 80
      when "001001000010" =>  data <= "11000101";  -- 242 = C5
      when "001001000011" =>  data <= "10010011";  -- 243 = 93
      when "001001000100" =>  data <= "11010000";  -- 244 = D0
      when "001001000101" =>  data <= "00000001";  -- 245 = 1
      when "001001000110" =>  data <= "01100000";  -- 246 = 60
      when "001001000111" =>  data <= "10101001";  -- 247 = A9
      when "001001001000" =>  data <= "00010000";  -- 248 = 10
      when "001001001001" =>  data <= "00100000";  -- 249 = 20
      when "001001001010" =>  data <= "11101001";  -- 24A = E9
      when "001001001011" =>  data <= "11110100";  -- 24B = F4
      when "001001001100" =>  data <= "10101001";  -- 24C = A9
      when "001001001101" =>  data <= "01000111";  -- 24D = 47
      when "001001001110" =>  data <= "00100000";  -- 24E = 20
      when "001001001111" =>  data <= "11101001";  -- 24F = E9
      when "001001010000" =>  data <= "11110100";  -- 250 = F4
      when "001001010001" =>  data <= "10101101";  -- 251 = AD
      when "001001010010" =>  data <= "00000111";  -- 252 = 7
      when "001001010011" =>  data <= "10000000";  -- 253 = 80
      when "001001010100" =>  data <= "11110000";  -- 254 = F0
      when "001001010101" =>  data <= "00001000";  -- 255 = 8
      when "001001010110" =>  data <= "00100000";  -- 256 = 20
      when "001001010111" =>  data <= "01111111";  -- 257 = 7F
      when "001001011000" =>  data <= "11110010";  -- 258 = F2
      when "001001011001" =>  data <= "10101001";  -- 259 = A9
      when "001001011010" =>  data <= "00111010";  -- 25A = 3A
      when "001001011011" =>  data <= "00100000";  -- 25B = 20
      when "001001011100" =>  data <= "11101001";  -- 25C = E9
      when "001001011101" =>  data <= "11110100";  -- 25D = F4
      when "001001011110" =>  data <= "10101101";  -- 25E = AD
      when "001001011111" =>  data <= "00001000";  -- 25F = 8
      when "001001100000" =>  data <= "10000000";  -- 260 = 80
      when "001001100001" =>  data <= "00100000";  -- 261 = 20
      when "001001100010" =>  data <= "01111111";  -- 262 = 7F
      when "001001100011" =>  data <= "11110010";  -- 263 = F2
      when "001001100100" =>  data <= "10101001";  -- 264 = A9
      when "001001100101" =>  data <= "00111010";  -- 265 = 3A
      when "001001100110" =>  data <= "00100000";  -- 266 = 20
      when "001001100111" =>  data <= "11101001";  -- 267 = E9
      when "001001101000" =>  data <= "11110100";  -- 268 = F4
      when "001001101001" =>  data <= "10101101";  -- 269 = AD
      when "001001101010" =>  data <= "00001001";  -- 26A = 9
      when "001001101011" =>  data <= "10000000";  -- 26B = 80
      when "001001101100" =>  data <= "00100000";  -- 26C = 20
      when "001001101101" =>  data <= "01111111";  -- 26D = 7F
      when "001001101110" =>  data <= "11110010";  -- 26E = F2
      when "001001101111" =>  data <= "10101101";  -- 26F = AD
      when "001001110000" =>  data <= "00000111";  -- 270 = 7
      when "001001110001" =>  data <= "10000000";  -- 271 = 80
      when "001001110010" =>  data <= "10000101";  -- 272 = 85
      when "001001110011" =>  data <= "10010001";  -- 273 = 91
      when "001001110100" =>  data <= "10101101";  -- 274 = AD
      when "001001110101" =>  data <= "00001000";  -- 275 = 8
      when "001001110110" =>  data <= "10000000";  -- 276 = 80
      when "001001110111" =>  data <= "10000101";  -- 277 = 85
      when "001001111000" =>  data <= "10010010";  -- 278 = 92
      when "001001111001" =>  data <= "10101101";  -- 279 = AD
      when "001001111010" =>  data <= "00001001";  -- 27A = 9
      when "001001111011" =>  data <= "10000000";  -- 27B = 80
      when "001001111100" =>  data <= "10000101";  -- 27C = 85
      when "001001111101" =>  data <= "10010011";  -- 27D = 93
      when "001001111110" =>  data <= "01100000";  -- 27E = 60
      when "001001111111" =>  data <= "01001000";  -- 27F = 48
      when "001010000000" =>  data <= "01001010";  -- 280 = 4A
      when "001010000001" =>  data <= "01001010";  -- 281 = 4A
      when "001010000010" =>  data <= "01001010";  -- 282 = 4A
      when "001010000011" =>  data <= "01001010";  -- 283 = 4A
      when "001010000100" =>  data <= "00011000";  -- 284 = 18
      when "001010000101" =>  data <= "01101001";  -- 285 = 69
      when "001010000110" =>  data <= "00110000";  -- 286 = 30
      when "001010000111" =>  data <= "00100000";  -- 287 = 20
      when "001010001000" =>  data <= "11101001";  -- 288 = E9
      when "001010001001" =>  data <= "11110100";  -- 289 = F4
      when "001010001010" =>  data <= "01101000";  -- 28A = 68
      when "001010001011" =>  data <= "00101001";  -- 28B = 29
      when "001010001100" =>  data <= "00001111";  -- 28C = F
      when "001010001101" =>  data <= "00011000";  -- 28D = 18
      when "001010001110" =>  data <= "01101001";  -- 28E = 69
      when "001010001111" =>  data <= "00110000";  -- 28F = 30
      when "001010010000" =>  data <= "01001100";  -- 290 = 4C
      when "001010010001" =>  data <= "11101001";  -- 291 = E9
      when "001010010010" =>  data <= "11110100";  -- 292 = F4
      when "001010010011" =>  data <= "00100000";  -- 293 = 20
      when "001010010100" =>  data <= "00010000";  -- 294 = 10
      when "001010010101" =>  data <= "11110010";  -- 295 = F2
      when "001010010110" =>  data <= "00100000";  -- 296 = 20
      when "001010010111" =>  data <= "11101110";  -- 297 = EE
      when "001010011000" =>  data <= "11110010";  -- 298 = F2
      when "001010011001" =>  data <= "10100101";  -- 299 = A5
      when "001010011010" =>  data <= "10000100";  -- 29A = 84
      when "001010011011" =>  data <= "00100000";  -- 29B = 20
      when "001010011100" =>  data <= "01100000";  -- 29C = 60
      when "001010011101" =>  data <= "11110011";  -- 29D = F3
      when "001010011110" =>  data <= "00100000";  -- 29E = 20
      when "001010011111" =>  data <= "11110101";  -- 29F = F5
      when "001010100000" =>  data <= "11110010";  -- 2A0 = F2
      when "001010100001" =>  data <= "01001100";  -- 2A1 = 4C
      when "001010100010" =>  data <= "11010001";  -- 2A2 = D1
      when "001010100011" =>  data <= "11110010";  -- 2A3 = F2
      when "001010100100" =>  data <= "10101001";  -- 2A4 = A9
      when "001010100101" =>  data <= "00010000";  -- 2A5 = 10
      when "001010100110" =>  data <= "00100000";  -- 2A6 = 20
      when "001010100111" =>  data <= "11101001";  -- 2A7 = E9
      when "001010101000" =>  data <= "11110100";  -- 2A8 = F4
      when "001010101001" =>  data <= "10101001";  -- 2A9 = A9
      when "001010101010" =>  data <= "01010100";  -- 2AA = 54
      when "001010101011" =>  data <= "00100000";  -- 2AB = 20
      when "001010101100" =>  data <= "11101001";  -- 2AC = E9
      when "001010101101" =>  data <= "11110100";  -- 2AD = F4
      when "001010101110" =>  data <= "10101001";  -- 2AE = A9
      when "001010101111" =>  data <= "00010101";  -- 2AF = 15
      when "001010110000" =>  data <= "10100000";  -- 2B0 = A0
      when "001010110001" =>  data <= "00001000";  -- 2B1 = 8
      when "001010110010" =>  data <= "10110001";  -- 2B2 = B1
      when "001010110011" =>  data <= "10001111";  -- 2B3 = 8F
      when "001010110100" =>  data <= "11110000";  -- 2B4 = F0
      when "001010110101" =>  data <= "00001000";  -- 2B5 = 8
      when "001010110110" =>  data <= "00100000";  -- 2B6 = 20
      when "001010110111" =>  data <= "11101001";  -- 2B7 = E9
      when "001010111000" =>  data <= "11110100";  -- 2B8 = F4
      when "001010111001" =>  data <= "11001000";  -- 2B9 = C8
      when "001010111010" =>  data <= "11000000";  -- 2BA = C0
      when "001010111011" =>  data <= "00011100";  -- 2BB = 1C
      when "001010111100" =>  data <= "11010000";  -- 2BC = D0
      when "001010111101" =>  data <= "11110100";  -- 2BD = F4
      when "001010111110" =>  data <= "01001100";  -- 2BE = 4C
      when "001010111111" =>  data <= "11010001";  -- 2BF = D1
      when "001011000000" =>  data <= "11110010";  -- 2C0 = F2
      when "001011000001" =>  data <= "10100010";  -- 2C1 = A2
      when "001011000010" =>  data <= "00000000";  -- 2C2 = 0
      when "001011000011" =>  data <= "10111101";  -- 2C3 = BD
      when "001011000100" =>  data <= "00000001";  -- 2C4 = 1
      when "001011000101" =>  data <= "00000010";  -- 2C5 = 2
      when "001011000110" =>  data <= "11110000";  -- 2C6 = F0
      when "001011000111" =>  data <= "00001000";  -- 2C7 = 8
      when "001011001000" =>  data <= "00100000";  -- 2C8 = 20
      when "001011001001" =>  data <= "11101001";  -- 2C9 = E9
      when "001011001010" =>  data <= "11110100";  -- 2CA = F4
      when "001011001011" =>  data <= "11101000";  -- 2CB = E8
      when "001011001100" =>  data <= "11100000";  -- 2CC = E0
      when "001011001101" =>  data <= "00011111";  -- 2CD = 1F
      when "001011001110" =>  data <= "11010000";  -- 2CE = D0
      when "001011001111" =>  data <= "11110011";  -- 2CF = F3
      when "001011010000" =>  data <= "01100000";  -- 2D0 = 60
      when "001011010001" =>  data <= "10100101";  -- 2D1 = A5
      when "001011010010" =>  data <= "10001110";  -- 2D2 = 8E
      when "001011010011" =>  data <= "11010000";  -- 2D3 = D0
      when "001011010100" =>  data <= "00000111";  -- 2D4 = 7
      when "001011010101" =>  data <= "10100010";  -- 2D5 = A2
      when "001011010110" =>  data <= "00000001";  -- 2D6 = 1
      when "001011010111" =>  data <= "10100000";  -- 2D7 = A0
      when "001011011000" =>  data <= "11110101";  -- 2D8 = F5
      when "001011011001" =>  data <= "01001100";  -- 2D9 = 4C
      when "001011011010" =>  data <= "01001111";  -- 2DA = 4F
      when "001011011011" =>  data <= "11110011";  -- 2DB = F3
      when "001011011100" =>  data <= "11001001";  -- 2DC = C9
      when "001011011101" =>  data <= "00000001";  -- 2DD = 1
      when "001011011110" =>  data <= "11010000";  -- 2DE = D0
      when "001011011111" =>  data <= "00000111";  -- 2DF = 7
      when "001011100000" =>  data <= "10100010";  -- 2E0 = A2
      when "001011100001" =>  data <= "00001001";  -- 2E1 = 9
      when "001011100010" =>  data <= "10100000";  -- 2E2 = A0
      when "001011100011" =>  data <= "11110101";  -- 2E3 = F5
      when "001011100100" =>  data <= "01001100";  -- 2E4 = 4C
      when "001011100101" =>  data <= "01001111";  -- 2E5 = 4F
      when "001011100110" =>  data <= "11110011";  -- 2E6 = F3
      when "001011100111" =>  data <= "10100010";  -- 2E7 = A2
      when "001011101000" =>  data <= "00010001";  -- 2E8 = 11
      when "001011101001" =>  data <= "10100000";  -- 2E9 = A0
      when "001011101010" =>  data <= "11110101";  -- 2EA = F5
      when "001011101011" =>  data <= "01001100";  -- 2EB = 4C
      when "001011101100" =>  data <= "01001111";  -- 2EC = 4F
      when "001011101101" =>  data <= "11110011";  -- 2ED = F3
      when "001011101110" =>  data <= "10100010";  -- 2EE = A2
      when "001011101111" =>  data <= "00011001";  -- 2EF = 19
      when "001011110000" =>  data <= "10100000";  -- 2F0 = A0
      when "001011110001" =>  data <= "11110101";  -- 2F1 = F5
      when "001011110010" =>  data <= "01001100";  -- 2F2 = 4C
      when "001011110011" =>  data <= "01001111";  -- 2F3 = 4F
      when "001011110100" =>  data <= "11110011";  -- 2F4 = F3
      when "001011110101" =>  data <= "10100010";  -- 2F5 = A2
      when "001011110110" =>  data <= "00100010";  -- 2F6 = 22
      when "001011110111" =>  data <= "10100000";  -- 2F7 = A0
      when "001011111000" =>  data <= "11110101";  -- 2F8 = F5
      when "001011111001" =>  data <= "00100000";  -- 2F9 = 20
      when "001011111010" =>  data <= "01001111";  -- 2FA = 4F
      when "001011111011" =>  data <= "11110011";  -- 2FB = F3
      when "001011111100" =>  data <= "10100101";  -- 2FC = A5
      when "001011111101" =>  data <= "10000010";  -- 2FD = 82
      when "001011111110" =>  data <= "00111000";  -- 2FE = 38
      when "001011111111" =>  data <= "11101001";  -- 2FF = E9
      when "001100000000" =>  data <= "00100000";  -- 300 = 20
      when "001100000001" =>  data <= "10000101";  -- 301 = 85
      when "001100000010" =>  data <= "00000000";  -- 302 = 0
      when "001100000011" =>  data <= "10100101";  -- 303 = A5
      when "001100000100" =>  data <= "10000011";  -- 304 = 83
      when "001100000101" =>  data <= "11101001";  -- 305 = E9
      when "001100000110" =>  data <= "00000000";  -- 306 = 0
      when "001100000111" =>  data <= "10000101";  -- 307 = 85
      when "001100001000" =>  data <= "00000001";  -- 308 = 1
      when "001100001001" =>  data <= "10100101";  -- 309 = A5
      when "001100001010" =>  data <= "00000000";  -- 30A = 0
      when "001100001011" =>  data <= "00011000";  -- 30B = 18
      when "001100001100" =>  data <= "01101001";  -- 30C = 69
      when "001100001101" =>  data <= "00000100";  -- 30D = 4
      when "001100001110" =>  data <= "10000101";  -- 30E = 85
      when "001100001111" =>  data <= "00000010";  -- 30F = 2
      when "001100010000" =>  data <= "10100101";  -- 310 = A5
      when "001100010001" =>  data <= "00000001";  -- 311 = 1
      when "001100010010" =>  data <= "01101001";  -- 312 = 69
      when "001100010011" =>  data <= "00000000";  -- 313 = 0
      when "001100010100" =>  data <= "10000101";  -- 314 = 85
      when "001100010101" =>  data <= "00000011";  -- 315 = 3
      when "001100010110" =>  data <= "10100000";  -- 316 = A0
      when "001100010111" =>  data <= "00000011";  -- 317 = 3
      when "001100011000" =>  data <= "00011000";  -- 318 = 18
      when "001100011001" =>  data <= "10110001";  -- 319 = B1
      when "001100011010" =>  data <= "00000000";  -- 31A = 0
      when "001100011011" =>  data <= "01110001";  -- 31B = 71
      when "001100011100" =>  data <= "00000010";  -- 31C = 2
      when "001100011101" =>  data <= "10011001";  -- 31D = 99
      when "001100011110" =>  data <= "10000101";  -- 31E = 85
      when "001100011111" =>  data <= "00000000";  -- 31F = 0
      when "001100100000" =>  data <= "10001000";  -- 320 = 88
      when "001100100001" =>  data <= "00010000";  -- 321 = 10
      when "001100100010" =>  data <= "11110110";  -- 322 = F6
      when "001100100011" =>  data <= "10100101";  -- 323 = A5
      when "001100100100" =>  data <= "10000110";  -- 324 = 86
      when "001100100101" =>  data <= "00111000";  -- 325 = 38
      when "001100100110" =>  data <= "11101001";  -- 326 = E9
      when "001100100111" =>  data <= "00000001";  -- 327 = 1
      when "001100101000" =>  data <= "10000101";  -- 328 = 85
      when "001100101001" =>  data <= "10000110";  -- 329 = 86
      when "001100101010" =>  data <= "10100101";  -- 32A = A5
      when "001100101011" =>  data <= "10000101";  -- 32B = 85
      when "001100101100" =>  data <= "11101001";  -- 32C = E9
      when "001100101101" =>  data <= "00000000";  -- 32D = 0
      when "001100101110" =>  data <= "10000101";  -- 32E = 85
      when "001100101111" =>  data <= "10000101";  -- 32F = 85
      when "001100110000" =>  data <= "10100010";  -- 330 = A2
      when "001100110001" =>  data <= "00001001";  -- 331 = 9
      when "001100110010" =>  data <= "01000110";  -- 332 = 46
      when "001100110011" =>  data <= "10000101";  -- 333 = 85
      when "001100110100" =>  data <= "01100110";  -- 334 = 66
      when "001100110101" =>  data <= "10000110";  -- 335 = 86
      when "001100110110" =>  data <= "01100110";  -- 336 = 66
      when "001100110111" =>  data <= "10000111";  -- 337 = 87
      when "001100111000" =>  data <= "01100110";  -- 338 = 66
      when "001100111001" =>  data <= "10001000";  -- 339 = 88
      when "001100111010" =>  data <= "11001010";  -- 33A = CA
      when "001100111011" =>  data <= "00010000";  -- 33B = 10
      when "001100111100" =>  data <= "11110101";  -- 33C = F5
      when "001100111101" =>  data <= "00100000";  -- 33D = 20
      when "001100111110" =>  data <= "10011001";  -- 33E = 99
      when "001100111111" =>  data <= "11110011";  -- 33F = F3
      when "001101000000" =>  data <= "10101001";  -- 340 = A9
      when "001101000001" =>  data <= "00100000";  -- 341 = 20
      when "001101000010" =>  data <= "00100000";  -- 342 = 20
      when "001101000011" =>  data <= "11101001";  -- 343 = E9
      when "001101000100" =>  data <= "11110100";  -- 344 = F4
      when "001101000101" =>  data <= "10101001";  -- 345 = A9
      when "001101000110" =>  data <= "01001011";  -- 346 = 4B
      when "001101000111" =>  data <= "00100000";  -- 347 = 20
      when "001101001000" =>  data <= "11101001";  -- 348 = E9
      when "001101001001" =>  data <= "11110100";  -- 349 = F4
      when "001101001010" =>  data <= "10101001";  -- 34A = A9
      when "001101001011" =>  data <= "01000010";  -- 34B = 42
      when "001101001100" =>  data <= "01001100";  -- 34C = 4C
      when "001101001101" =>  data <= "11101001";  -- 34D = E9
      when "001101001110" =>  data <= "11110100";  -- 34E = F4
      when "001101001111" =>  data <= "10000110";  -- 34F = 86
      when "001101010000" =>  data <= "00000000";  -- 350 = 0
      when "001101010001" =>  data <= "10000100";  -- 351 = 84
      when "001101010010" =>  data <= "00000001";  -- 352 = 1
      when "001101010011" =>  data <= "10100000";  -- 353 = A0
      when "001101010100" =>  data <= "00000000";  -- 354 = 0
      when "001101010101" =>  data <= "10110001";  -- 355 = B1
      when "001101010110" =>  data <= "00000000";  -- 356 = 0
      when "001101010111" =>  data <= "11110000";  -- 357 = F0
      when "001101011000" =>  data <= "00000110";  -- 358 = 6
      when "001101011001" =>  data <= "00100000";  -- 359 = 20
      when "001101011010" =>  data <= "11101001";  -- 35A = E9
      when "001101011011" =>  data <= "11110100";  -- 35B = F4
      when "001101011100" =>  data <= "11001000";  -- 35C = C8
      when "001101011101" =>  data <= "11010000";  -- 35D = D0
      when "001101011110" =>  data <= "11110110";  -- 35E = F6
      when "001101011111" =>  data <= "01100000";  -- 35F = 60
      when "001101100000" =>  data <= "10100010";  -- 360 = A2
      when "001101100001" =>  data <= "00000000";  -- 361 = 0
      when "001101100010" =>  data <= "10100000";  -- 362 = A0
      when "001101100011" =>  data <= "00000000";  -- 363 = 0
      when "001101100100" =>  data <= "11001001";  -- 364 = C9
      when "001101100101" =>  data <= "01100100";  -- 365 = 64
      when "001101100110" =>  data <= "10010000";  -- 366 = 90
      when "001101100111" =>  data <= "00000111";  -- 367 = 7
      when "001101101000" =>  data <= "11101000";  -- 368 = E8
      when "001101101001" =>  data <= "00111000";  -- 369 = 38
      when "001101101010" =>  data <= "11101001";  -- 36A = E9
      when "001101101011" =>  data <= "01100100";  -- 36B = 64
      when "001101101100" =>  data <= "01001100";  -- 36C = 4C
      when "001101101101" =>  data <= "01100100";  -- 36D = 64
      when "001101101110" =>  data <= "11110011";  -- 36E = F3
      when "001101101111" =>  data <= "11001001";  -- 36F = C9
      when "001101110000" =>  data <= "00001010";  -- 370 = A
      when "001101110001" =>  data <= "10010000";  -- 371 = 90
      when "001101110010" =>  data <= "00000111";  -- 372 = 7
      when "001101110011" =>  data <= "11001000";  -- 373 = C8
      when "001101110100" =>  data <= "00111000";  -- 374 = 38
      when "001101110101" =>  data <= "11101001";  -- 375 = E9
      when "001101110110" =>  data <= "00001010";  -- 376 = A
      when "001101110111" =>  data <= "01001100";  -- 377 = 4C
      when "001101111000" =>  data <= "01101111";  -- 378 = 6F
      when "001101111001" =>  data <= "11110011";  -- 379 = F3
      when "001101111010" =>  data <= "01001000";  -- 37A = 48
      when "001101111011" =>  data <= "10001010";  -- 37B = 8A
      when "001101111100" =>  data <= "11110000";  -- 37C = F0
      when "001101111101" =>  data <= "00000110";  -- 37D = 6
      when "001101111110" =>  data <= "00011000";  -- 37E = 18
      when "001101111111" =>  data <= "01101001";  -- 37F = 69
      when "001110000000" =>  data <= "00110000";  -- 380 = 30
      when "001110000001" =>  data <= "00100000";  -- 381 = 20
      when "001110000010" =>  data <= "11101001";  -- 382 = E9
      when "001110000011" =>  data <= "11110100";  -- 383 = F4
      when "001110000100" =>  data <= "10011000";  -- 384 = 98
      when "001110000101" =>  data <= "11100000";  -- 385 = E0
      when "001110000110" =>  data <= "00000000";  -- 386 = 0
      when "001110000111" =>  data <= "11010000";  -- 387 = D0
      when "001110001000" =>  data <= "00000011";  -- 388 = 3
      when "001110001001" =>  data <= "10011000";  -- 389 = 98
      when "001110001010" =>  data <= "11110000";  -- 38A = F0
      when "001110001011" =>  data <= "00000110";  -- 38B = 6
      when "001110001100" =>  data <= "00011000";  -- 38C = 18
      when "001110001101" =>  data <= "01101001";  -- 38D = 69
      when "001110001110" =>  data <= "00110000";  -- 38E = 30
      when "001110001111" =>  data <= "00100000";  -- 38F = 20
      when "001110010000" =>  data <= "11101001";  -- 390 = E9
      when "001110010001" =>  data <= "11110100";  -- 391 = F4
      when "001110010010" =>  data <= "01101000";  -- 392 = 68
      when "001110010011" =>  data <= "00011000";  -- 393 = 18
      when "001110010100" =>  data <= "01101001";  -- 394 = 69
      when "001110010101" =>  data <= "00110000";  -- 395 = 30
      when "001110010110" =>  data <= "01001100";  -- 396 = 4C
      when "001110010111" =>  data <= "11101001";  -- 397 = E9
      when "001110011000" =>  data <= "11110100";  -- 398 = F4
      when "001110011001" =>  data <= "00100000";  -- 399 = 20
      when "001110011010" =>  data <= "10110001";  -- 39A = B1
      when "001110011011" =>  data <= "11110100";  -- 39B = F4
      when "001110011100" =>  data <= "00100000";  -- 39C = 20
      when "001110011101" =>  data <= "11010001";  -- 39D = D1
      when "001110011110" =>  data <= "11110011";  -- 39E = F3
      when "001110011111" =>  data <= "10101001";  -- 39F = A9
      when "001110100000" =>  data <= "00000000";  -- 3A0 = 0
      when "001110100001" =>  data <= "10000101";  -- 3A1 = 85
      when "001110100010" =>  data <= "00000000";  -- 3A2 = 0
      when "001110100011" =>  data <= "10100000";  -- 3A3 = A0
      when "001110100100" =>  data <= "00000011";  -- 3A4 = 3
      when "001110100101" =>  data <= "10111001";  -- 3A5 = B9
      when "001110100110" =>  data <= "10001001";  -- 3A6 = 89
      when "001110100111" =>  data <= "00000000";  -- 3A7 = 0
      when "001110101000" =>  data <= "01001010";  -- 3A8 = 4A
      when "001110101001" =>  data <= "01001010";  -- 3A9 = 4A
      when "001110101010" =>  data <= "01001010";  -- 3AA = 4A
      when "001110101011" =>  data <= "01001010";  -- 3AB = 4A
      when "001110101100" =>  data <= "11010000";  -- 3AC = D0
      when "001110101101" =>  data <= "00000100";  -- 3AD = 4
      when "001110101110" =>  data <= "10100110";  -- 3AE = A6
      when "001110101111" =>  data <= "00000000";  -- 3AF = 0
      when "001110110000" =>  data <= "11110000";  -- 3B0 = F0
      when "001110110001" =>  data <= "00001000";  -- 3B1 = 8
      when "001110110010" =>  data <= "11100110";  -- 3B2 = E6
      when "001110110011" =>  data <= "00000000";  -- 3B3 = 0
      when "001110110100" =>  data <= "00011000";  -- 3B4 = 18
      when "001110110101" =>  data <= "01101001";  -- 3B5 = 69
      when "001110110110" =>  data <= "00110000";  -- 3B6 = 30
      when "001110110111" =>  data <= "00100000";  -- 3B7 = 20
      when "001110111000" =>  data <= "11101001";  -- 3B8 = E9
      when "001110111001" =>  data <= "11110100";  -- 3B9 = F4
      when "001110111010" =>  data <= "10111001";  -- 3BA = B9
      when "001110111011" =>  data <= "10001001";  -- 3BB = 89
      when "001110111100" =>  data <= "00000000";  -- 3BC = 0
      when "001110111101" =>  data <= "00101001";  -- 3BD = 29
      when "001110111110" =>  data <= "00001111";  -- 3BE = F
      when "001110111111" =>  data <= "11010000";  -- 3BF = D0
      when "001111000000" =>  data <= "00000100";  -- 3C0 = 4
      when "001111000001" =>  data <= "10100110";  -- 3C1 = A6
      when "001111000010" =>  data <= "00000000";  -- 3C2 = 0
      when "001111000011" =>  data <= "11110000";  -- 3C3 = F0
      when "001111000100" =>  data <= "00001000";  -- 3C4 = 8
      when "001111000101" =>  data <= "11100110";  -- 3C5 = E6
      when "001111000110" =>  data <= "00000000";  -- 3C6 = 0
      when "001111000111" =>  data <= "00011000";  -- 3C7 = 18
      when "001111001000" =>  data <= "01101001";  -- 3C8 = 69
      when "001111001001" =>  data <= "00110000";  -- 3C9 = 30
      when "001111001010" =>  data <= "00100000";  -- 3CA = 20
      when "001111001011" =>  data <= "11101001";  -- 3CB = E9
      when "001111001100" =>  data <= "11110100";  -- 3CC = F4
      when "001111001101" =>  data <= "10001000";  -- 3CD = 88
      when "001111001110" =>  data <= "00010000";  -- 3CE = 10
      when "001111001111" =>  data <= "11010101";  -- 3CF = D5
      when "001111010000" =>  data <= "01100000";  -- 3D0 = 60
      when "001111010001" =>  data <= "10101001";  -- 3D1 = A9
      when "001111010010" =>  data <= "00000000";  -- 3D2 = 0
      when "001111010011" =>  data <= "10000101";  -- 3D3 = 85
      when "001111010100" =>  data <= "10001001";  -- 3D4 = 89
      when "001111010101" =>  data <= "10000101";  -- 3D5 = 85
      when "001111010110" =>  data <= "10001010";  -- 3D6 = 8A
      when "001111010111" =>  data <= "10000101";  -- 3D7 = 85
      when "001111011000" =>  data <= "10001011";  -- 3D8 = 8B
      when "001111011001" =>  data <= "10000101";  -- 3D9 = 85
      when "001111011010" =>  data <= "10001100";  -- 3DA = 8C
      when "001111011011" =>  data <= "10100101";  -- 3DB = A5
      when "001111011100" =>  data <= "10001000";  -- 3DC = 88
      when "001111011101" =>  data <= "00101001";  -- 3DD = 29
      when "001111011110" =>  data <= "11110000";  -- 3DE = F0
      when "001111011111" =>  data <= "01001010";  -- 3DF = 4A
      when "001111100000" =>  data <= "01001010";  -- 3E0 = 4A
      when "001111100001" =>  data <= "01001010";  -- 3E1 = 4A
      when "001111100010" =>  data <= "01001010";  -- 3E2 = 4A
      when "001111100011" =>  data <= "10000101";  -- 3E3 = 85
      when "001111100100" =>  data <= "10001101";  -- 3E4 = 8D
      when "001111100101" =>  data <= "10100101";  -- 3E5 = A5
      when "001111100110" =>  data <= "10001000";  -- 3E6 = 88
      when "001111100111" =>  data <= "00101001";  -- 3E7 = 29
      when "001111101000" =>  data <= "00001111";  -- 3E8 = F
      when "001111101001" =>  data <= "10000101";  -- 3E9 = 85
      when "001111101010" =>  data <= "10001000";  -- 3EA = 88
      when "001111101011" =>  data <= "10100000";  -- 3EB = A0
      when "001111101100" =>  data <= "00000000";  -- 3EC = 0
      when "001111101101" =>  data <= "10100010";  -- 3ED = A2
      when "001111101110" =>  data <= "00000000";  -- 3EE = 0
      when "001111101111" =>  data <= "10100101";  -- 3EF = A5
      when "001111110000" =>  data <= "10000101";  -- 3F0 = 85
      when "001111110001" =>  data <= "00111000";  -- 3F1 = 38
      when "001111110010" =>  data <= "11111001";  -- 3F2 = F9
      when "001111110011" =>  data <= "01010001";  -- 3F3 = 51
      when "001111110100" =>  data <= "11110100";  -- 3F4 = F4
      when "001111110101" =>  data <= "10000101";  -- 3F5 = 85
      when "001111110110" =>  data <= "10000101";  -- 3F6 = 85
      when "001111110111" =>  data <= "10100101";  -- 3F7 = A5
      when "001111111000" =>  data <= "10000110";  -- 3F8 = 86
      when "001111111001" =>  data <= "11001000";  -- 3F9 = C8
      when "001111111010" =>  data <= "11111001";  -- 3FA = F9
      when "001111111011" =>  data <= "01010001";  -- 3FB = 51
      when "001111111100" =>  data <= "11110100";  -- 3FC = F4
      when "001111111101" =>  data <= "10000101";  -- 3FD = 85
      when "001111111110" =>  data <= "10000110";  -- 3FE = 86
      when "001111111111" =>  data <= "10100101";  -- 3FF = A5
      when "010000000000" =>  data <= "10000111";  -- 400 = 87
      when "010000000001" =>  data <= "11001000";  -- 401 = C8
      when "010000000010" =>  data <= "11111001";  -- 402 = F9
      when "010000000011" =>  data <= "01010001";  -- 403 = 51
      when "010000000100" =>  data <= "11110100";  -- 404 = F4
      when "010000000101" =>  data <= "10000101";  -- 405 = 85
      when "010000000110" =>  data <= "10000111";  -- 406 = 87
      when "010000000111" =>  data <= "10100101";  -- 407 = A5
      when "010000001000" =>  data <= "10001000";  -- 408 = 88
      when "010000001001" =>  data <= "11001000";  -- 409 = C8
      when "010000001010" =>  data <= "11111001";  -- 40A = F9
      when "010000001011" =>  data <= "01010001";  -- 40B = 51
      when "010000001100" =>  data <= "11110100";  -- 40C = F4
      when "010000001101" =>  data <= "10010000";  -- 40D = 90
      when "010000001110" =>  data <= "00001001";  -- 40E = 9
      when "010000001111" =>  data <= "10000101";  -- 40F = 85
      when "010000010000" =>  data <= "10001000";  -- 410 = 88
      when "010000010001" =>  data <= "10001000";  -- 411 = 88
      when "010000010010" =>  data <= "10001000";  -- 412 = 88
      when "010000010011" =>  data <= "10001000";  -- 413 = 88
      when "010000010100" =>  data <= "11101000";  -- 414 = E8
      when "010000010101" =>  data <= "01001100";  -- 415 = 4C
      when "010000010110" =>  data <= "11101111";  -- 416 = EF
      when "010000010111" =>  data <= "11110011";  -- 417 = F3
      when "010000011000" =>  data <= "10001000";  -- 418 = 88
      when "010000011001" =>  data <= "10001000";  -- 419 = 88
      when "010000011010" =>  data <= "10001000";  -- 41A = 88
      when "010000011011" =>  data <= "10100101";  -- 41B = A5
      when "010000011100" =>  data <= "10000101";  -- 41C = 85
      when "010000011101" =>  data <= "01111001";  -- 41D = 79
      when "010000011110" =>  data <= "01010001";  -- 41E = 51
      when "010000011111" =>  data <= "11110100";  -- 41F = F4
      when "010000100000" =>  data <= "10000101";  -- 420 = 85
      when "010000100001" =>  data <= "10000101";  -- 421 = 85
      when "010000100010" =>  data <= "10100101";  -- 422 = A5
      when "010000100011" =>  data <= "10000110";  -- 423 = 86
      when "010000100100" =>  data <= "11001000";  -- 424 = C8
      when "010000100101" =>  data <= "01111001";  -- 425 = 79
      when "010000100110" =>  data <= "01010001";  -- 426 = 51
      when "010000100111" =>  data <= "11110100";  -- 427 = F4
      when "010000101000" =>  data <= "10000101";  -- 428 = 85
      when "010000101001" =>  data <= "10000110";  -- 429 = 86
      when "010000101010" =>  data <= "10100101";  -- 42A = A5
      when "010000101011" =>  data <= "10000111";  -- 42B = 87
      when "010000101100" =>  data <= "11001000";  -- 42C = C8
      when "010000101101" =>  data <= "01111001";  -- 42D = 79
      when "010000101110" =>  data <= "01010001";  -- 42E = 51
      when "010000101111" =>  data <= "11110100";  -- 42F = F4
      when "010000110000" =>  data <= "10000101";  -- 430 = 85
      when "010000110001" =>  data <= "10000111";  -- 431 = 87
      when "010000110010" =>  data <= "10001010";  -- 432 = 8A
      when "010000110011" =>  data <= "00100000";  -- 433 = 20
      when "010000110100" =>  data <= "00111110";  -- 434 = 3E
      when "010000110101" =>  data <= "11110100";  -- 435 = F4
      when "010000110110" =>  data <= "11001000";  -- 436 = C8
      when "010000110111" =>  data <= "11001000";  -- 437 = C8
      when "010000111000" =>  data <= "11000000";  -- 438 = C0
      when "010000111001" =>  data <= "00100000";  -- 439 = 20
      when "010000111010" =>  data <= "10010000";  -- 43A = 90
      when "010000111011" =>  data <= "10110001";  -- 43B = B1
      when "010000111100" =>  data <= "10100101";  -- 43C = A5
      when "010000111101" =>  data <= "10000101";  -- 43D = 85
      when "010000111110" =>  data <= "00001010";  -- 43E = A
      when "010000111111" =>  data <= "00001010";  -- 43F = A
      when "010001000000" =>  data <= "00001010";  -- 440 = A
      when "010001000001" =>  data <= "00001010";  -- 441 = A
      when "010001000010" =>  data <= "10100010";  -- 442 = A2
      when "010001000011" =>  data <= "00000100";  -- 443 = 4
      when "010001000100" =>  data <= "00001010";  -- 444 = A
      when "010001000101" =>  data <= "00100110";  -- 445 = 26
      when "010001000110" =>  data <= "10001001";  -- 446 = 89
      when "010001000111" =>  data <= "00100110";  -- 447 = 26
      when "010001001000" =>  data <= "10001010";  -- 448 = 8A
      when "010001001001" =>  data <= "00100110";  -- 449 = 26
      when "010001001010" =>  data <= "10001011";  -- 44A = 8B
      when "010001001011" =>  data <= "00100110";  -- 44B = 26
      when "010001001100" =>  data <= "10001100";  -- 44C = 8C
      when "010001001101" =>  data <= "11001010";  -- 44D = CA
      when "010001001110" =>  data <= "11010000";  -- 44E = D0
      when "010001001111" =>  data <= "11110100";  -- 44F = F4
      when "010001010000" =>  data <= "01100000";  -- 450 = 60
      when "010001010001" =>  data <= "00000000";  -- 451 = 0
      when "010001010010" =>  data <= "11100001";  -- 452 = E1
      when "010001010011" =>  data <= "11110101";  -- 453 = F5
      when "010001010100" =>  data <= "00000101";  -- 454 = 5
      when "010001010101" =>  data <= "10000000";  -- 455 = 80
      when "010001010110" =>  data <= "10010110";  -- 456 = 96
      when "010001010111" =>  data <= "10011000";  -- 457 = 98
      when "010001011000" =>  data <= "00000000";  -- 458 = 0
      when "010001011001" =>  data <= "01000000";  -- 459 = 40
      when "010001011010" =>  data <= "01000010";  -- 45A = 42
      when "010001011011" =>  data <= "00001111";  -- 45B = F
      when "010001011100" =>  data <= "00000000";  -- 45C = 0
      when "010001011101" =>  data <= "10100000";  -- 45D = A0
      when "010001011110" =>  data <= "10000110";  -- 45E = 86
      when "010001011111" =>  data <= "00000001";  -- 45F = 1
      when "010001100000" =>  data <= "00000000";  -- 460 = 0
      when "010001100001" =>  data <= "00010000";  -- 461 = 10
      when "010001100010" =>  data <= "00100111";  -- 462 = 27
      when "010001100011" =>  data <= "00000000";  -- 463 = 0
      when "010001100100" =>  data <= "00000000";  -- 464 = 0
      when "010001100101" =>  data <= "11101000";  -- 465 = E8
      when "010001100110" =>  data <= "00000011";  -- 466 = 3
      when "010001100111" =>  data <= "00000000";  -- 467 = 0
      when "010001101000" =>  data <= "00000000";  -- 468 = 0
      when "010001101001" =>  data <= "01100100";  -- 469 = 64
      when "010001101010" =>  data <= "00000000";  -- 46A = 0
      when "010001101011" =>  data <= "00000000";  -- 46B = 0
      when "010001101100" =>  data <= "00000000";  -- 46C = 0
      when "010001101101" =>  data <= "00001010";  -- 46D = A
      when "010001101110" =>  data <= "00000000";  -- 46E = 0
      when "010001101111" =>  data <= "00000000";  -- 46F = 0
      when "010001110000" =>  data <= "00000000";  -- 470 = 0
      when "010001110001" =>  data <= "10000110";  -- 471 = 86
      when "010001110010" =>  data <= "00000000";  -- 472 = 0
      when "010001110011" =>  data <= "10000100";  -- 473 = 84
      when "010001110100" =>  data <= "00000001";  -- 474 = 1
      when "010001110101" =>  data <= "10000101";  -- 475 = 85
      when "010001110110" =>  data <= "00000010";  -- 476 = 2
      when "010001110111" =>  data <= "10101001";  -- 477 = A9
      when "010001111000" =>  data <= "00010000";  -- 478 = 10
      when "010001111001" =>  data <= "00100000";  -- 479 = 20
      when "010001111010" =>  data <= "11101001";  -- 47A = E9
      when "010001111011" =>  data <= "11110100";  -- 47B = F4
      when "010001111100" =>  data <= "10101001";  -- 47C = A9
      when "010001111101" =>  data <= "01000000";  -- 47D = 40
      when "010001111110" =>  data <= "00100000";  -- 47E = 20
      when "010001111111" =>  data <= "11101001";  -- 47F = E9
      when "010010000000" =>  data <= "11110100";  -- 480 = F4
      when "010010000001" =>  data <= "10100000";  -- 481 = A0
      when "010010000010" =>  data <= "00000000";  -- 482 = 0
      when "010010000011" =>  data <= "10110001";  -- 483 = B1
      when "010010000100" =>  data <= "00000000";  -- 484 = 0
      when "010010000101" =>  data <= "01001010";  -- 485 = 4A
      when "010010000110" =>  data <= "01001010";  -- 486 = 4A
      when "010010000111" =>  data <= "01001010";  -- 487 = 4A
      when "010010001000" =>  data <= "01001010";  -- 488 = 4A
      when "010010001001" =>  data <= "10101010";  -- 489 = AA
      when "010010001010" =>  data <= "10111101";  -- 48A = BD
      when "010010001011" =>  data <= "10100001";  -- 48B = A1
      when "010010001100" =>  data <= "11110100";  -- 48C = F4
      when "010010001101" =>  data <= "00100000";  -- 48D = 20
      when "010010001110" =>  data <= "11101001";  -- 48E = E9
      when "010010001111" =>  data <= "11110100";  -- 48F = F4
      when "010010010000" =>  data <= "10110001";  -- 490 = B1
      when "010010010001" =>  data <= "00000000";  -- 491 = 0
      when "010010010010" =>  data <= "00101001";  -- 492 = 29
      when "010010010011" =>  data <= "00001111";  -- 493 = F
      when "010010010100" =>  data <= "10101010";  -- 494 = AA
      when "010010010101" =>  data <= "10111101";  -- 495 = BD
      when "010010010110" =>  data <= "10100001";  -- 496 = A1
      when "010010010111" =>  data <= "11110100";  -- 497 = F4
      when "010010011000" =>  data <= "00100000";  -- 498 = 20
      when "010010011001" =>  data <= "11101001";  -- 499 = E9
      when "010010011010" =>  data <= "11110100";  -- 49A = F4
      when "010010011011" =>  data <= "11001000";  -- 49B = C8
      when "010010011100" =>  data <= "11000100";  -- 49C = C4
      when "010010011101" =>  data <= "00000010";  -- 49D = 2
      when "010010011110" =>  data <= "11010000";  -- 49E = D0
      when "010010011111" =>  data <= "11100011";  -- 49F = E3
      when "010010100000" =>  data <= "01100000";  -- 4A0 = 60
      when "010010100001" =>  data <= "00110000";  -- 4A1 = 30
      when "010010100010" =>  data <= "00110001";  -- 4A2 = 31
      when "010010100011" =>  data <= "00110010";  -- 4A3 = 32
      when "010010100100" =>  data <= "00110011";  -- 4A4 = 33
      when "010010100101" =>  data <= "00110100";  -- 4A5 = 34
      when "010010100110" =>  data <= "00110101";  -- 4A6 = 35
      when "010010100111" =>  data <= "00110110";  -- 4A7 = 36
      when "010010101000" =>  data <= "00110111";  -- 4A8 = 37
      when "010010101001" =>  data <= "00111000";  -- 4A9 = 38
      when "010010101010" =>  data <= "00111001";  -- 4AA = 39
      when "010010101011" =>  data <= "01000001";  -- 4AB = 41
      when "010010101100" =>  data <= "01000010";  -- 4AC = 42
      when "010010101101" =>  data <= "01000011";  -- 4AD = 43
      when "010010101110" =>  data <= "01000100";  -- 4AE = 44
      when "010010101111" =>  data <= "01000101";  -- 4AF = 45
      when "010010110000" =>  data <= "01000110";  -- 4B0 = 46
      when "010010110001" =>  data <= "10100010";  -- 4B1 = A2
      when "010010110010" =>  data <= "00000011";  -- 4B2 = 3
      when "010010110011" =>  data <= "10110101";  -- 4B3 = B5
      when "010010110100" =>  data <= "10000101";  -- 4B4 = 85
      when "010010110101" =>  data <= "01001000";  -- 4B5 = 48
      when "010010110110" =>  data <= "11001010";  -- 4B6 = CA
      when "010010110111" =>  data <= "00010000";  -- 4B7 = 10
      when "010010111000" =>  data <= "11111010";  -- 4B8 = FA
      when "010010111001" =>  data <= "10100010";  -- 4B9 = A2
      when "010010111010" =>  data <= "00000011";  -- 4BA = 3
      when "010010111011" =>  data <= "01101000";  -- 4BB = 68
      when "010010111100" =>  data <= "10010101";  -- 4BC = 95
      when "010010111101" =>  data <= "10000101";  -- 4BD = 85
      when "010010111110" =>  data <= "11001010";  -- 4BE = CA
      when "010010111111" =>  data <= "00010000";  -- 4BF = 10
      when "010011000000" =>  data <= "11111010";  -- 4C0 = FA
      when "010011000001" =>  data <= "01100000";  -- 4C1 = 60
      when "010011000010" =>  data <= "10100101";  -- 4C2 = A5
      when "010011000011" =>  data <= "10000100";  -- 4C3 = 84
      when "010011000100" =>  data <= "11001001";  -- 4C4 = C9
      when "010011000101" =>  data <= "00000000";  -- 4C5 = 0
      when "010011000110" =>  data <= "11010000";  -- 4C6 = D0
      when "010011000111" =>  data <= "00000001";  -- 4C7 = 1
      when "010011001000" =>  data <= "01100000";  -- 4C8 = 60
      when "010011001001" =>  data <= "10100010";  -- 4C9 = A2
      when "010011001010" =>  data <= "01000000";  -- 4CA = 40
      when "010011001011" =>  data <= "10100000";  -- 4CB = A0
      when "010011001100" =>  data <= "00000100";  -- 4CC = 4
      when "010011001101" =>  data <= "10000110";  -- 4CD = 86
      when "010011001110" =>  data <= "10000000";  -- 4CE = 80
      when "010011001111" =>  data <= "10000100";  -- 4CF = 84
      when "010011010000" =>  data <= "10000001";  -- 4D0 = 81
      when "010011010001" =>  data <= "10000110";  -- 4D1 = 86
      when "010011010010" =>  data <= "10001111";  -- 4D2 = 8F
      when "010011010011" =>  data <= "10000100";  -- 4D3 = 84
      when "010011010100" =>  data <= "10010000";  -- 4D4 = 90
      when "010011010101" =>  data <= "10000110";  -- 4D5 = 86
      when "010011010110" =>  data <= "10000010";  -- 4D6 = 82
      when "010011010111" =>  data <= "10000100";  -- 4D7 = 84
      when "010011011000" =>  data <= "10000011";  -- 4D8 = 83
      when "010011011001" =>  data <= "10101010";  -- 4D9 = AA
      when "010011011010" =>  data <= "10100101";  -- 4DA = A5
      when "010011011011" =>  data <= "10000010";  -- 4DB = 82
      when "010011011100" =>  data <= "00011000";  -- 4DC = 18
      when "010011011101" =>  data <= "01101001";  -- 4DD = 69
      when "010011011110" =>  data <= "00100000";  -- 4DE = 20
      when "010011011111" =>  data <= "10000101";  -- 4DF = 85
      when "010011100000" =>  data <= "10000010";  -- 4E0 = 82
      when "010011100001" =>  data <= "10010000";  -- 4E1 = 90
      when "010011100010" =>  data <= "00000010";  -- 4E2 = 2
      when "010011100011" =>  data <= "11100110";  -- 4E3 = E6
      when "010011100100" =>  data <= "10000011";  -- 4E4 = 83
      when "010011100101" =>  data <= "11001010";  -- 4E5 = CA
      when "010011100110" =>  data <= "11010000";  -- 4E6 = D0
      when "010011100111" =>  data <= "11110010";  -- 4E7 = F2
      when "010011101000" =>  data <= "01100000";  -- 4E8 = 60
      when "010011101001" =>  data <= "01001000";  -- 4E9 = 48
      when "010011101010" =>  data <= "10101101";  -- 4EA = AD
      when "010011101011" =>  data <= "00000000";  -- 4EB = 0
      when "010011101100" =>  data <= "10000000";  -- 4EC = 80
      when "010011101101" =>  data <= "11110000";  -- 4ED = F0
      when "010011101110" =>  data <= "11111011";  -- 4EE = FB
      when "010011101111" =>  data <= "01101000";  -- 4EF = 68
      when "010011110000" =>  data <= "10001101";  -- 4F0 = 8D
      when "010011110001" =>  data <= "00000010";  -- 4F1 = 2
      when "010011110010" =>  data <= "10000000";  -- 4F2 = 80
      when "010011110011" =>  data <= "10101001";  -- 4F3 = A9
      when "010011110100" =>  data <= "10000000";  -- 4F4 = 80
      when "010011110101" =>  data <= "10001101";  -- 4F5 = 8D
      when "010011110110" =>  data <= "00000001";  -- 4F6 = 1
      when "010011110111" =>  data <= "10000000";  -- 4F7 = 80
      when "010011111000" =>  data <= "01100000";  -- 4F8 = 60
      when "010011111001" =>  data <= "01000000";  -- 4F9 = 40
      when "010011111010" =>  data <= "01000000";  -- 4FA = 40
      when "010011111011" =>  data <= "01001100";  -- 4FB = 4C
      when "010011111100" =>  data <= "11111001";  -- 4FC = F9
      when "010011111101" =>  data <= "11110100";  -- 4FD = F4
      when "010011111110" =>  data <= "01001100";  -- 4FE = 4C
      when "010011111111" =>  data <= "11111010";  -- 4FF = FA
      when "010100000000" =>  data <= "11110100";  -- 500 = F4
      when "010100000001" =>  data <= "00010000";  -- 501 = 10
      when "010100000010" =>  data <= "10001011";  -- 502 = 8B
      when "010100000011" =>  data <= "01110011";  -- 503 = 73
      when "010100000100" =>  data <= "01110100";  -- 504 = 74
      when "010100000101" =>  data <= "01101111";  -- 505 = 6F
      when "010100000110" =>  data <= "01110000";  -- 506 = 70
      when "010100000111" =>  data <= "00100000";  -- 507 = 20
      when "010100001000" =>  data <= "00000000";  -- 508 = 0
      when "010100001001" =>  data <= "00010000";  -- 509 = 10
      when "010100001010" =>  data <= "10001011";  -- 50A = 8B
      when "010100001011" =>  data <= "01110000";  -- 50B = 70
      when "010100001100" =>  data <= "01101100";  -- 50C = 6C
      when "010100001101" =>  data <= "01100001";  -- 50D = 61
      when "010100001110" =>  data <= "01111001";  -- 50E = 79
      when "010100001111" =>  data <= "00100000";  -- 50F = 20
      when "010100010000" =>  data <= "00000000";  -- 510 = 0
      when "010100010001" =>  data <= "00010000";  -- 511 = 10
      when "010100010010" =>  data <= "10001011";  -- 512 = 8B
      when "010100010011" =>  data <= "01110000";  -- 513 = 70
      when "010100010100" =>  data <= "01100001";  -- 514 = 61
      when "010100010101" =>  data <= "01110101";  -- 515 = 75
      when "010100010110" =>  data <= "01110011";  -- 516 = 73
      when "010100010111" =>  data <= "01100101";  -- 517 = 65
      when "010100011000" =>  data <= "00000000";  -- 518 = 0
      when "010100011001" =>  data <= "00001101";  -- 519 = D
      when "010100011010" =>  data <= "01010011";  -- 51A = 53
      when "010100011011" =>  data <= "01101111";  -- 51B = 6F
      when "010100011100" =>  data <= "01101110";  -- 51C = 6E
      when "010100011101" =>  data <= "01100111";  -- 51D = 67
      when "010100011110" =>  data <= "01110011";  -- 51E = 73
      when "010100011111" =>  data <= "00111010";  -- 51F = 3A
      when "010100100000" =>  data <= "00100000";  -- 520 = 20
      when "010100100001" =>  data <= "00000000";  -- 521 = 0
      when "010100100010" =>  data <= "00001101";  -- 522 = D
      when "010100100011" =>  data <= "01010011";  -- 523 = 53
      when "010100100100" =>  data <= "01110100";  -- 524 = 74
      when "010100100101" =>  data <= "01101111";  -- 525 = 6F
      when "010100100110" =>  data <= "01110010";  -- 526 = 72
      when "010100100111" =>  data <= "01100001";  -- 527 = 61
      when "010100101000" =>  data <= "01100111";  -- 528 = 67
      when "010100101001" =>  data <= "01100101";  -- 529 = 65
      when "010100101010" =>  data <= "00111010";  -- 52A = 3A
      when "010100101011" =>  data <= "00100000";  -- 52B = 20
      when "010100101100" =>  data <= "00000000";  -- 52C = 0
      when "010100101101" =>  data <= "11111111";  -- 52D = FF
      when "010100101110" =>  data <= "11111111";  -- 52E = FF
      when "010100101111" =>  data <= "11111111";  -- 52F = FF
      when "010100110000" =>  data <= "11111111";  -- 530 = FF
      when "010100110001" =>  data <= "11111111";  -- 531 = FF
      when "010100110010" =>  data <= "11111111";  -- 532 = FF
      when "010100110011" =>  data <= "11111111";  -- 533 = FF
      when "010100110100" =>  data <= "11111111";  -- 534 = FF
      when "010100110101" =>  data <= "11111111";  -- 535 = FF
      when "010100110110" =>  data <= "11111111";  -- 536 = FF
      when "010100110111" =>  data <= "11111111";  -- 537 = FF
      when "010100111000" =>  data <= "11111111";  -- 538 = FF
      when "010100111001" =>  data <= "11111111";  -- 539 = FF
      when "010100111010" =>  data <= "11111111";  -- 53A = FF
      when "010100111011" =>  data <= "11111111";  -- 53B = FF
      when "010100111100" =>  data <= "11111111";  -- 53C = FF
      when "010100111101" =>  data <= "11111111";  -- 53D = FF
      when "010100111110" =>  data <= "11111111";  -- 53E = FF
      when "010100111111" =>  data <= "11111111";  -- 53F = FF
      when "010101000000" =>  data <= "11111111";  -- 540 = FF
      when "010101000001" =>  data <= "11111111";  -- 541 = FF
      when "010101000010" =>  data <= "11111111";  -- 542 = FF
      when "010101000011" =>  data <= "11111111";  -- 543 = FF
      when "010101000100" =>  data <= "11111111";  -- 544 = FF
      when "010101000101" =>  data <= "11111111";  -- 545 = FF
      when "010101000110" =>  data <= "11111111";  -- 546 = FF
      when "010101000111" =>  data <= "11111111";  -- 547 = FF
      when "010101001000" =>  data <= "11111111";  -- 548 = FF
      when "010101001001" =>  data <= "11111111";  -- 549 = FF
      when "010101001010" =>  data <= "11111111";  -- 54A = FF
      when "010101001011" =>  data <= "11111111";  -- 54B = FF
      when "010101001100" =>  data <= "11111111";  -- 54C = FF
      when "010101001101" =>  data <= "11111111";  -- 54D = FF
      when "010101001110" =>  data <= "11111111";  -- 54E = FF
      when "010101001111" =>  data <= "11111111";  -- 54F = FF
      when "010101010000" =>  data <= "11111111";  -- 550 = FF
      when "010101010001" =>  data <= "11111111";  -- 551 = FF
      when "010101010010" =>  data <= "11111111";  -- 552 = FF
      when "010101010011" =>  data <= "11111111";  -- 553 = FF
      when "010101010100" =>  data <= "11111111";  -- 554 = FF
      when "010101010101" =>  data <= "11111111";  -- 555 = FF
      when "010101010110" =>  data <= "11111111";  -- 556 = FF
      when "010101010111" =>  data <= "11111111";  -- 557 = FF
      when "010101011000" =>  data <= "11111111";  -- 558 = FF
      when "010101011001" =>  data <= "11111111";  -- 559 = FF
      when "010101011010" =>  data <= "11111111";  -- 55A = FF
      when "010101011011" =>  data <= "11111111";  -- 55B = FF
      when "010101011100" =>  data <= "11111111";  -- 55C = FF
      when "010101011101" =>  data <= "11111111";  -- 55D = FF
      when "010101011110" =>  data <= "11111111";  -- 55E = FF
      when "010101011111" =>  data <= "11111111";  -- 55F = FF
      when "010101100000" =>  data <= "11111111";  -- 560 = FF
      when "010101100001" =>  data <= "11111111";  -- 561 = FF
      when "010101100010" =>  data <= "11111111";  -- 562 = FF
      when "010101100011" =>  data <= "11111111";  -- 563 = FF
      when "010101100100" =>  data <= "11111111";  -- 564 = FF
      when "010101100101" =>  data <= "11111111";  -- 565 = FF
      when "010101100110" =>  data <= "11111111";  -- 566 = FF
      when "010101100111" =>  data <= "11111111";  -- 567 = FF
      when "010101101000" =>  data <= "11111111";  -- 568 = FF
      when "010101101001" =>  data <= "11111111";  -- 569 = FF
      when "010101101010" =>  data <= "11111111";  -- 56A = FF
      when "010101101011" =>  data <= "11111111";  -- 56B = FF
      when "010101101100" =>  data <= "11111111";  -- 56C = FF
      when "010101101101" =>  data <= "11111111";  -- 56D = FF
      when "010101101110" =>  data <= "11111111";  -- 56E = FF
      when "010101101111" =>  data <= "11111111";  -- 56F = FF
      when "010101110000" =>  data <= "11111111";  -- 570 = FF
      when "010101110001" =>  data <= "11111111";  -- 571 = FF
      when "010101110010" =>  data <= "11111111";  -- 572 = FF
      when "010101110011" =>  data <= "11111111";  -- 573 = FF
      when "010101110100" =>  data <= "11111111";  -- 574 = FF
      when "010101110101" =>  data <= "11111111";  -- 575 = FF
      when "010101110110" =>  data <= "11111111";  -- 576 = FF
      when "010101110111" =>  data <= "11111111";  -- 577 = FF
      when "010101111000" =>  data <= "11111111";  -- 578 = FF
      when "010101111001" =>  data <= "11111111";  -- 579 = FF
      when "010101111010" =>  data <= "11111111";  -- 57A = FF
      when "010101111011" =>  data <= "11111111";  -- 57B = FF
      when "010101111100" =>  data <= "11111111";  -- 57C = FF
      when "010101111101" =>  data <= "11111111";  -- 57D = FF
      when "010101111110" =>  data <= "11111111";  -- 57E = FF
      when "010101111111" =>  data <= "11111111";  -- 57F = FF
      when "010110000000" =>  data <= "11111111";  -- 580 = FF
      when "010110000001" =>  data <= "11111111";  -- 581 = FF
      when "010110000010" =>  data <= "11111111";  -- 582 = FF
      when "010110000011" =>  data <= "11111111";  -- 583 = FF
      when "010110000100" =>  data <= "11111111";  -- 584 = FF
      when "010110000101" =>  data <= "11111111";  -- 585 = FF
      when "010110000110" =>  data <= "11111111";  -- 586 = FF
      when "010110000111" =>  data <= "11111111";  -- 587 = FF
      when "010110001000" =>  data <= "11111111";  -- 588 = FF
      when "010110001001" =>  data <= "11111111";  -- 589 = FF
      when "010110001010" =>  data <= "11111111";  -- 58A = FF
      when "010110001011" =>  data <= "11111111";  -- 58B = FF
      when "010110001100" =>  data <= "11111111";  -- 58C = FF
      when "010110001101" =>  data <= "11111111";  -- 58D = FF
      when "010110001110" =>  data <= "11111111";  -- 58E = FF
      when "010110001111" =>  data <= "11111111";  -- 58F = FF
      when "010110010000" =>  data <= "11111111";  -- 590 = FF
      when "010110010001" =>  data <= "11111111";  -- 591 = FF
      when "010110010010" =>  data <= "11111111";  -- 592 = FF
      when "010110010011" =>  data <= "11111111";  -- 593 = FF
      when "010110010100" =>  data <= "11111111";  -- 594 = FF
      when "010110010101" =>  data <= "11111111";  -- 595 = FF
      when "010110010110" =>  data <= "11111111";  -- 596 = FF
      when "010110010111" =>  data <= "11111111";  -- 597 = FF
      when "010110011000" =>  data <= "11111111";  -- 598 = FF
      when "010110011001" =>  data <= "11111111";  -- 599 = FF
      when "010110011010" =>  data <= "11111111";  -- 59A = FF
      when "010110011011" =>  data <= "11111111";  -- 59B = FF
      when "010110011100" =>  data <= "11111111";  -- 59C = FF
      when "010110011101" =>  data <= "11111111";  -- 59D = FF
      when "010110011110" =>  data <= "11111111";  -- 59E = FF
      when "010110011111" =>  data <= "11111111";  -- 59F = FF
      when "010110100000" =>  data <= "11111111";  -- 5A0 = FF
      when "010110100001" =>  data <= "11111111";  -- 5A1 = FF
      when "010110100010" =>  data <= "11111111";  -- 5A2 = FF
      when "010110100011" =>  data <= "11111111";  -- 5A3 = FF
      when "010110100100" =>  data <= "11111111";  -- 5A4 = FF
      when "010110100101" =>  data <= "11111111";  -- 5A5 = FF
      when "010110100110" =>  data <= "11111111";  -- 5A6 = FF
      when "010110100111" =>  data <= "11111111";  -- 5A7 = FF
      when "010110101000" =>  data <= "11111111";  -- 5A8 = FF
      when "010110101001" =>  data <= "11111111";  -- 5A9 = FF
      when "010110101010" =>  data <= "11111111";  -- 5AA = FF
      when "010110101011" =>  data <= "11111111";  -- 5AB = FF
      when "010110101100" =>  data <= "11111111";  -- 5AC = FF
      when "010110101101" =>  data <= "11111111";  -- 5AD = FF
      when "010110101110" =>  data <= "11111111";  -- 5AE = FF
      when "010110101111" =>  data <= "11111111";  -- 5AF = FF
      when "010110110000" =>  data <= "11111111";  -- 5B0 = FF
      when "010110110001" =>  data <= "11111111";  -- 5B1 = FF
      when "010110110010" =>  data <= "11111111";  -- 5B2 = FF
      when "010110110011" =>  data <= "11111111";  -- 5B3 = FF
      when "010110110100" =>  data <= "11111111";  -- 5B4 = FF
      when "010110110101" =>  data <= "11111111";  -- 5B5 = FF
      when "010110110110" =>  data <= "11111111";  -- 5B6 = FF
      when "010110110111" =>  data <= "11111111";  -- 5B7 = FF
      when "010110111000" =>  data <= "11111111";  -- 5B8 = FF
      when "010110111001" =>  data <= "11111111";  -- 5B9 = FF
      when "010110111010" =>  data <= "11111111";  -- 5BA = FF
      when "010110111011" =>  data <= "11111111";  -- 5BB = FF
      when "010110111100" =>  data <= "11111111";  -- 5BC = FF
      when "010110111101" =>  data <= "11111111";  -- 5BD = FF
      when "010110111110" =>  data <= "11111111";  -- 5BE = FF
      when "010110111111" =>  data <= "11111111";  -- 5BF = FF
      when "010111000000" =>  data <= "11111111";  -- 5C0 = FF
      when "010111000001" =>  data <= "11111111";  -- 5C1 = FF
      when "010111000010" =>  data <= "11111111";  -- 5C2 = FF
      when "010111000011" =>  data <= "11111111";  -- 5C3 = FF
      when "010111000100" =>  data <= "11111111";  -- 5C4 = FF
      when "010111000101" =>  data <= "11111111";  -- 5C5 = FF
      when "010111000110" =>  data <= "11111111";  -- 5C6 = FF
      when "010111000111" =>  data <= "11111111";  -- 5C7 = FF
      when "010111001000" =>  data <= "11111111";  -- 5C8 = FF
      when "010111001001" =>  data <= "11111111";  -- 5C9 = FF
      when "010111001010" =>  data <= "11111111";  -- 5CA = FF
      when "010111001011" =>  data <= "11111111";  -- 5CB = FF
      when "010111001100" =>  data <= "11111111";  -- 5CC = FF
      when "010111001101" =>  data <= "11111111";  -- 5CD = FF
      when "010111001110" =>  data <= "11111111";  -- 5CE = FF
      when "010111001111" =>  data <= "11111111";  -- 5CF = FF
      when "010111010000" =>  data <= "11111111";  -- 5D0 = FF
      when "010111010001" =>  data <= "11111111";  -- 5D1 = FF
      when "010111010010" =>  data <= "11111111";  -- 5D2 = FF
      when "010111010011" =>  data <= "11111111";  -- 5D3 = FF
      when "010111010100" =>  data <= "11111111";  -- 5D4 = FF
      when "010111010101" =>  data <= "11111111";  -- 5D5 = FF
      when "010111010110" =>  data <= "11111111";  -- 5D6 = FF
      when "010111010111" =>  data <= "11111111";  -- 5D7 = FF
      when "010111011000" =>  data <= "11111111";  -- 5D8 = FF
      when "010111011001" =>  data <= "11111111";  -- 5D9 = FF
      when "010111011010" =>  data <= "11111111";  -- 5DA = FF
      when "010111011011" =>  data <= "11111111";  -- 5DB = FF
      when "010111011100" =>  data <= "11111111";  -- 5DC = FF
      when "010111011101" =>  data <= "11111111";  -- 5DD = FF
      when "010111011110" =>  data <= "11111111";  -- 5DE = FF
      when "010111011111" =>  data <= "11111111";  -- 5DF = FF
      when "010111100000" =>  data <= "11111111";  -- 5E0 = FF
      when "010111100001" =>  data <= "11111111";  -- 5E1 = FF
      when "010111100010" =>  data <= "11111111";  -- 5E2 = FF
      when "010111100011" =>  data <= "11111111";  -- 5E3 = FF
      when "010111100100" =>  data <= "11111111";  -- 5E4 = FF
      when "010111100101" =>  data <= "11111111";  -- 5E5 = FF
      when "010111100110" =>  data <= "11111111";  -- 5E6 = FF
      when "010111100111" =>  data <= "11111111";  -- 5E7 = FF
      when "010111101000" =>  data <= "11111111";  -- 5E8 = FF
      when "010111101001" =>  data <= "11111111";  -- 5E9 = FF
      when "010111101010" =>  data <= "11111111";  -- 5EA = FF
      when "010111101011" =>  data <= "11111111";  -- 5EB = FF
      when "010111101100" =>  data <= "11111111";  -- 5EC = FF
      when "010111101101" =>  data <= "11111111";  -- 5ED = FF
      when "010111101110" =>  data <= "11111111";  -- 5EE = FF
      when "010111101111" =>  data <= "11111111";  -- 5EF = FF
      when "010111110000" =>  data <= "11111111";  -- 5F0 = FF
      when "010111110001" =>  data <= "11111111";  -- 5F1 = FF
      when "010111110010" =>  data <= "11111111";  -- 5F2 = FF
      when "010111110011" =>  data <= "11111111";  -- 5F3 = FF
      when "010111110100" =>  data <= "11111111";  -- 5F4 = FF
      when "010111110101" =>  data <= "11111111";  -- 5F5 = FF
      when "010111110110" =>  data <= "11111111";  -- 5F6 = FF
      when "010111110111" =>  data <= "11111111";  -- 5F7 = FF
      when "010111111000" =>  data <= "11111111";  -- 5F8 = FF
      when "010111111001" =>  data <= "11111111";  -- 5F9 = FF
      when "010111111010" =>  data <= "11111111";  -- 5FA = FF
      when "010111111011" =>  data <= "11111111";  -- 5FB = FF
      when "010111111100" =>  data <= "11111111";  -- 5FC = FF
      when "010111111101" =>  data <= "11111111";  -- 5FD = FF
      when "010111111110" =>  data <= "11111111";  -- 5FE = FF
      when "010111111111" =>  data <= "11111111";  -- 5FF = FF
      when "011000000000" =>  data <= "11111111";  -- 600 = FF
      when "011000000001" =>  data <= "11111111";  -- 601 = FF
      when "011000000010" =>  data <= "11111111";  -- 602 = FF
      when "011000000011" =>  data <= "11111111";  -- 603 = FF
      when "011000000100" =>  data <= "11111111";  -- 604 = FF
      when "011000000101" =>  data <= "11111111";  -- 605 = FF
      when "011000000110" =>  data <= "11111111";  -- 606 = FF
      when "011000000111" =>  data <= "11111111";  -- 607 = FF
      when "011000001000" =>  data <= "11111111";  -- 608 = FF
      when "011000001001" =>  data <= "11111111";  -- 609 = FF
      when "011000001010" =>  data <= "11111111";  -- 60A = FF
      when "011000001011" =>  data <= "11111111";  -- 60B = FF
      when "011000001100" =>  data <= "11111111";  -- 60C = FF
      when "011000001101" =>  data <= "11111111";  -- 60D = FF
      when "011000001110" =>  data <= "11111111";  -- 60E = FF
      when "011000001111" =>  data <= "11111111";  -- 60F = FF
      when "011000010000" =>  data <= "11111111";  -- 610 = FF
      when "011000010001" =>  data <= "11111111";  -- 611 = FF
      when "011000010010" =>  data <= "11111111";  -- 612 = FF
      when "011000010011" =>  data <= "11111111";  -- 613 = FF
      when "011000010100" =>  data <= "11111111";  -- 614 = FF
      when "011000010101" =>  data <= "11111111";  -- 615 = FF
      when "011000010110" =>  data <= "11111111";  -- 616 = FF
      when "011000010111" =>  data <= "11111111";  -- 617 = FF
      when "011000011000" =>  data <= "11111111";  -- 618 = FF
      when "011000011001" =>  data <= "11111111";  -- 619 = FF
      when "011000011010" =>  data <= "11111111";  -- 61A = FF
      when "011000011011" =>  data <= "11111111";  -- 61B = FF
      when "011000011100" =>  data <= "11111111";  -- 61C = FF
      when "011000011101" =>  data <= "11111111";  -- 61D = FF
      when "011000011110" =>  data <= "11111111";  -- 61E = FF
      when "011000011111" =>  data <= "11111111";  -- 61F = FF
      when "011000100000" =>  data <= "11111111";  -- 620 = FF
      when "011000100001" =>  data <= "11111111";  -- 621 = FF
      when "011000100010" =>  data <= "11111111";  -- 622 = FF
      when "011000100011" =>  data <= "11111111";  -- 623 = FF
      when "011000100100" =>  data <= "11111111";  -- 624 = FF
      when "011000100101" =>  data <= "11111111";  -- 625 = FF
      when "011000100110" =>  data <= "11111111";  -- 626 = FF
      when "011000100111" =>  data <= "11111111";  -- 627 = FF
      when "011000101000" =>  data <= "11111111";  -- 628 = FF
      when "011000101001" =>  data <= "11111111";  -- 629 = FF
      when "011000101010" =>  data <= "11111111";  -- 62A = FF
      when "011000101011" =>  data <= "11111111";  -- 62B = FF
      when "011000101100" =>  data <= "11111111";  -- 62C = FF
      when "011000101101" =>  data <= "11111111";  -- 62D = FF
      when "011000101110" =>  data <= "11111111";  -- 62E = FF
      when "011000101111" =>  data <= "11111111";  -- 62F = FF
      when "011000110000" =>  data <= "11111111";  -- 630 = FF
      when "011000110001" =>  data <= "11111111";  -- 631 = FF
      when "011000110010" =>  data <= "11111111";  -- 632 = FF
      when "011000110011" =>  data <= "11111111";  -- 633 = FF
      when "011000110100" =>  data <= "11111111";  -- 634 = FF
      when "011000110101" =>  data <= "11111111";  -- 635 = FF
      when "011000110110" =>  data <= "11111111";  -- 636 = FF
      when "011000110111" =>  data <= "11111111";  -- 637 = FF
      when "011000111000" =>  data <= "11111111";  -- 638 = FF
      when "011000111001" =>  data <= "11111111";  -- 639 = FF
      when "011000111010" =>  data <= "11111111";  -- 63A = FF
      when "011000111011" =>  data <= "11111111";  -- 63B = FF
      when "011000111100" =>  data <= "11111111";  -- 63C = FF
      when "011000111101" =>  data <= "11111111";  -- 63D = FF
      when "011000111110" =>  data <= "11111111";  -- 63E = FF
      when "011000111111" =>  data <= "11111111";  -- 63F = FF
      when "011001000000" =>  data <= "11111111";  -- 640 = FF
      when "011001000001" =>  data <= "11111111";  -- 641 = FF
      when "011001000010" =>  data <= "11111111";  -- 642 = FF
      when "011001000011" =>  data <= "11111111";  -- 643 = FF
      when "011001000100" =>  data <= "11111111";  -- 644 = FF
      when "011001000101" =>  data <= "11111111";  -- 645 = FF
      when "011001000110" =>  data <= "11111111";  -- 646 = FF
      when "011001000111" =>  data <= "11111111";  -- 647 = FF
      when "011001001000" =>  data <= "11111111";  -- 648 = FF
      when "011001001001" =>  data <= "11111111";  -- 649 = FF
      when "011001001010" =>  data <= "11111111";  -- 64A = FF
      when "011001001011" =>  data <= "11111111";  -- 64B = FF
      when "011001001100" =>  data <= "11111111";  -- 64C = FF
      when "011001001101" =>  data <= "11111111";  -- 64D = FF
      when "011001001110" =>  data <= "11111111";  -- 64E = FF
      when "011001001111" =>  data <= "11111111";  -- 64F = FF
      when "011001010000" =>  data <= "11111111";  -- 650 = FF
      when "011001010001" =>  data <= "11111111";  -- 651 = FF
      when "011001010010" =>  data <= "11111111";  -- 652 = FF
      when "011001010011" =>  data <= "11111111";  -- 653 = FF
      when "011001010100" =>  data <= "11111111";  -- 654 = FF
      when "011001010101" =>  data <= "11111111";  -- 655 = FF
      when "011001010110" =>  data <= "11111111";  -- 656 = FF
      when "011001010111" =>  data <= "11111111";  -- 657 = FF
      when "011001011000" =>  data <= "11111111";  -- 658 = FF
      when "011001011001" =>  data <= "11111111";  -- 659 = FF
      when "011001011010" =>  data <= "11111111";  -- 65A = FF
      when "011001011011" =>  data <= "11111111";  -- 65B = FF
      when "011001011100" =>  data <= "11111111";  -- 65C = FF
      when "011001011101" =>  data <= "11111111";  -- 65D = FF
      when "011001011110" =>  data <= "11111111";  -- 65E = FF
      when "011001011111" =>  data <= "11111111";  -- 65F = FF
      when "011001100000" =>  data <= "11111111";  -- 660 = FF
      when "011001100001" =>  data <= "11111111";  -- 661 = FF
      when "011001100010" =>  data <= "11111111";  -- 662 = FF
      when "011001100011" =>  data <= "11111111";  -- 663 = FF
      when "011001100100" =>  data <= "11111111";  -- 664 = FF
      when "011001100101" =>  data <= "11111111";  -- 665 = FF
      when "011001100110" =>  data <= "11111111";  -- 666 = FF
      when "011001100111" =>  data <= "11111111";  -- 667 = FF
      when "011001101000" =>  data <= "11111111";  -- 668 = FF
      when "011001101001" =>  data <= "11111111";  -- 669 = FF
      when "011001101010" =>  data <= "11111111";  -- 66A = FF
      when "011001101011" =>  data <= "11111111";  -- 66B = FF
      when "011001101100" =>  data <= "11111111";  -- 66C = FF
      when "011001101101" =>  data <= "11111111";  -- 66D = FF
      when "011001101110" =>  data <= "11111111";  -- 66E = FF
      when "011001101111" =>  data <= "11111111";  -- 66F = FF
      when "011001110000" =>  data <= "11111111";  -- 670 = FF
      when "011001110001" =>  data <= "11111111";  -- 671 = FF
      when "011001110010" =>  data <= "11111111";  -- 672 = FF
      when "011001110011" =>  data <= "11111111";  -- 673 = FF
      when "011001110100" =>  data <= "11111111";  -- 674 = FF
      when "011001110101" =>  data <= "11111111";  -- 675 = FF
      when "011001110110" =>  data <= "11111111";  -- 676 = FF
      when "011001110111" =>  data <= "11111111";  -- 677 = FF
      when "011001111000" =>  data <= "11111111";  -- 678 = FF
      when "011001111001" =>  data <= "11111111";  -- 679 = FF
      when "011001111010" =>  data <= "11111111";  -- 67A = FF
      when "011001111011" =>  data <= "11111111";  -- 67B = FF
      when "011001111100" =>  data <= "11111111";  -- 67C = FF
      when "011001111101" =>  data <= "11111111";  -- 67D = FF
      when "011001111110" =>  data <= "11111111";  -- 67E = FF
      when "011001111111" =>  data <= "11111111";  -- 67F = FF
      when "011010000000" =>  data <= "11111111";  -- 680 = FF
      when "011010000001" =>  data <= "11111111";  -- 681 = FF
      when "011010000010" =>  data <= "11111111";  -- 682 = FF
      when "011010000011" =>  data <= "11111111";  -- 683 = FF
      when "011010000100" =>  data <= "11111111";  -- 684 = FF
      when "011010000101" =>  data <= "11111111";  -- 685 = FF
      when "011010000110" =>  data <= "11111111";  -- 686 = FF
      when "011010000111" =>  data <= "11111111";  -- 687 = FF
      when "011010001000" =>  data <= "11111111";  -- 688 = FF
      when "011010001001" =>  data <= "11111111";  -- 689 = FF
      when "011010001010" =>  data <= "11111111";  -- 68A = FF
      when "011010001011" =>  data <= "11111111";  -- 68B = FF
      when "011010001100" =>  data <= "11111111";  -- 68C = FF
      when "011010001101" =>  data <= "11111111";  -- 68D = FF
      when "011010001110" =>  data <= "11111111";  -- 68E = FF
      when "011010001111" =>  data <= "11111111";  -- 68F = FF
      when "011010010000" =>  data <= "11111111";  -- 690 = FF
      when "011010010001" =>  data <= "11111111";  -- 691 = FF
      when "011010010010" =>  data <= "11111111";  -- 692 = FF
      when "011010010011" =>  data <= "11111111";  -- 693 = FF
      when "011010010100" =>  data <= "11111111";  -- 694 = FF
      when "011010010101" =>  data <= "11111111";  -- 695 = FF
      when "011010010110" =>  data <= "11111111";  -- 696 = FF
      when "011010010111" =>  data <= "11111111";  -- 697 = FF
      when "011010011000" =>  data <= "11111111";  -- 698 = FF
      when "011010011001" =>  data <= "11111111";  -- 699 = FF
      when "011010011010" =>  data <= "11111111";  -- 69A = FF
      when "011010011011" =>  data <= "11111111";  -- 69B = FF
      when "011010011100" =>  data <= "11111111";  -- 69C = FF
      when "011010011101" =>  data <= "11111111";  -- 69D = FF
      when "011010011110" =>  data <= "11111111";  -- 69E = FF
      when "011010011111" =>  data <= "11111111";  -- 69F = FF
      when "011010100000" =>  data <= "11111111";  -- 6A0 = FF
      when "011010100001" =>  data <= "11111111";  -- 6A1 = FF
      when "011010100010" =>  data <= "11111111";  -- 6A2 = FF
      when "011010100011" =>  data <= "11111111";  -- 6A3 = FF
      when "011010100100" =>  data <= "11111111";  -- 6A4 = FF
      when "011010100101" =>  data <= "11111111";  -- 6A5 = FF
      when "011010100110" =>  data <= "11111111";  -- 6A6 = FF
      when "011010100111" =>  data <= "11111111";  -- 6A7 = FF
      when "011010101000" =>  data <= "11111111";  -- 6A8 = FF
      when "011010101001" =>  data <= "11111111";  -- 6A9 = FF
      when "011010101010" =>  data <= "11111111";  -- 6AA = FF
      when "011010101011" =>  data <= "11111111";  -- 6AB = FF
      when "011010101100" =>  data <= "11111111";  -- 6AC = FF
      when "011010101101" =>  data <= "11111111";  -- 6AD = FF
      when "011010101110" =>  data <= "11111111";  -- 6AE = FF
      when "011010101111" =>  data <= "11111111";  -- 6AF = FF
      when "011010110000" =>  data <= "11111111";  -- 6B0 = FF
      when "011010110001" =>  data <= "11111111";  -- 6B1 = FF
      when "011010110010" =>  data <= "11111111";  -- 6B2 = FF
      when "011010110011" =>  data <= "11111111";  -- 6B3 = FF
      when "011010110100" =>  data <= "11111111";  -- 6B4 = FF
      when "011010110101" =>  data <= "11111111";  -- 6B5 = FF
      when "011010110110" =>  data <= "11111111";  -- 6B6 = FF
      when "011010110111" =>  data <= "11111111";  -- 6B7 = FF
      when "011010111000" =>  data <= "11111111";  -- 6B8 = FF
      when "011010111001" =>  data <= "11111111";  -- 6B9 = FF
      when "011010111010" =>  data <= "11111111";  -- 6BA = FF
      when "011010111011" =>  data <= "11111111";  -- 6BB = FF
      when "011010111100" =>  data <= "11111111";  -- 6BC = FF
      when "011010111101" =>  data <= "11111111";  -- 6BD = FF
      when "011010111110" =>  data <= "11111111";  -- 6BE = FF
      when "011010111111" =>  data <= "11111111";  -- 6BF = FF
      when "011011000000" =>  data <= "11111111";  -- 6C0 = FF
      when "011011000001" =>  data <= "11111111";  -- 6C1 = FF
      when "011011000010" =>  data <= "11111111";  -- 6C2 = FF
      when "011011000011" =>  data <= "11111111";  -- 6C3 = FF
      when "011011000100" =>  data <= "11111111";  -- 6C4 = FF
      when "011011000101" =>  data <= "11111111";  -- 6C5 = FF
      when "011011000110" =>  data <= "11111111";  -- 6C6 = FF
      when "011011000111" =>  data <= "11111111";  -- 6C7 = FF
      when "011011001000" =>  data <= "11111111";  -- 6C8 = FF
      when "011011001001" =>  data <= "11111111";  -- 6C9 = FF
      when "011011001010" =>  data <= "11111111";  -- 6CA = FF
      when "011011001011" =>  data <= "11111111";  -- 6CB = FF
      when "011011001100" =>  data <= "11111111";  -- 6CC = FF
      when "011011001101" =>  data <= "11111111";  -- 6CD = FF
      when "011011001110" =>  data <= "11111111";  -- 6CE = FF
      when "011011001111" =>  data <= "11111111";  -- 6CF = FF
      when "011011010000" =>  data <= "11111111";  -- 6D0 = FF
      when "011011010001" =>  data <= "11111111";  -- 6D1 = FF
      when "011011010010" =>  data <= "11111111";  -- 6D2 = FF
      when "011011010011" =>  data <= "11111111";  -- 6D3 = FF
      when "011011010100" =>  data <= "11111111";  -- 6D4 = FF
      when "011011010101" =>  data <= "11111111";  -- 6D5 = FF
      when "011011010110" =>  data <= "11111111";  -- 6D6 = FF
      when "011011010111" =>  data <= "11111111";  -- 6D7 = FF
      when "011011011000" =>  data <= "11111111";  -- 6D8 = FF
      when "011011011001" =>  data <= "11111111";  -- 6D9 = FF
      when "011011011010" =>  data <= "11111111";  -- 6DA = FF
      when "011011011011" =>  data <= "11111111";  -- 6DB = FF
      when "011011011100" =>  data <= "11111111";  -- 6DC = FF
      when "011011011101" =>  data <= "11111111";  -- 6DD = FF
      when "011011011110" =>  data <= "11111111";  -- 6DE = FF
      when "011011011111" =>  data <= "11111111";  -- 6DF = FF
      when "011011100000" =>  data <= "11111111";  -- 6E0 = FF
      when "011011100001" =>  data <= "11111111";  -- 6E1 = FF
      when "011011100010" =>  data <= "11111111";  -- 6E2 = FF
      when "011011100011" =>  data <= "11111111";  -- 6E3 = FF
      when "011011100100" =>  data <= "11111111";  -- 6E4 = FF
      when "011011100101" =>  data <= "11111111";  -- 6E5 = FF
      when "011011100110" =>  data <= "11111111";  -- 6E6 = FF
      when "011011100111" =>  data <= "11111111";  -- 6E7 = FF
      when "011011101000" =>  data <= "11111111";  -- 6E8 = FF
      when "011011101001" =>  data <= "11111111";  -- 6E9 = FF
      when "011011101010" =>  data <= "11111111";  -- 6EA = FF
      when "011011101011" =>  data <= "11111111";  -- 6EB = FF
      when "011011101100" =>  data <= "11111111";  -- 6EC = FF
      when "011011101101" =>  data <= "11111111";  -- 6ED = FF
      when "011011101110" =>  data <= "11111111";  -- 6EE = FF
      when "011011101111" =>  data <= "11111111";  -- 6EF = FF
      when "011011110000" =>  data <= "11111111";  -- 6F0 = FF
      when "011011110001" =>  data <= "11111111";  -- 6F1 = FF
      when "011011110010" =>  data <= "11111111";  -- 6F2 = FF
      when "011011110011" =>  data <= "11111111";  -- 6F3 = FF
      when "011011110100" =>  data <= "11111111";  -- 6F4 = FF
      when "011011110101" =>  data <= "11111111";  -- 6F5 = FF
      when "011011110110" =>  data <= "11111111";  -- 6F6 = FF
      when "011011110111" =>  data <= "11111111";  -- 6F7 = FF
      when "011011111000" =>  data <= "11111111";  -- 6F8 = FF
      when "011011111001" =>  data <= "11111111";  -- 6F9 = FF
      when "011011111010" =>  data <= "11111111";  -- 6FA = FF
      when "011011111011" =>  data <= "11111111";  -- 6FB = FF
      when "011011111100" =>  data <= "11111111";  -- 6FC = FF
      when "011011111101" =>  data <= "11111111";  -- 6FD = FF
      when "011011111110" =>  data <= "11111111";  -- 6FE = FF
      when "011011111111" =>  data <= "11111111";  -- 6FF = FF
      when "011100000000" =>  data <= "11111111";  -- 700 = FF
      when "011100000001" =>  data <= "11111111";  -- 701 = FF
      when "011100000010" =>  data <= "11111111";  -- 702 = FF
      when "011100000011" =>  data <= "11111111";  -- 703 = FF
      when "011100000100" =>  data <= "11111111";  -- 704 = FF
      when "011100000101" =>  data <= "11111111";  -- 705 = FF
      when "011100000110" =>  data <= "11111111";  -- 706 = FF
      when "011100000111" =>  data <= "11111111";  -- 707 = FF
      when "011100001000" =>  data <= "11111111";  -- 708 = FF
      when "011100001001" =>  data <= "11111111";  -- 709 = FF
      when "011100001010" =>  data <= "11111111";  -- 70A = FF
      when "011100001011" =>  data <= "11111111";  -- 70B = FF
      when "011100001100" =>  data <= "11111111";  -- 70C = FF
      when "011100001101" =>  data <= "11111111";  -- 70D = FF
      when "011100001110" =>  data <= "11111111";  -- 70E = FF
      when "011100001111" =>  data <= "11111111";  -- 70F = FF
      when "011100010000" =>  data <= "11111111";  -- 710 = FF
      when "011100010001" =>  data <= "11111111";  -- 711 = FF
      when "011100010010" =>  data <= "11111111";  -- 712 = FF
      when "011100010011" =>  data <= "11111111";  -- 713 = FF
      when "011100010100" =>  data <= "11111111";  -- 714 = FF
      when "011100010101" =>  data <= "11111111";  -- 715 = FF
      when "011100010110" =>  data <= "11111111";  -- 716 = FF
      when "011100010111" =>  data <= "11111111";  -- 717 = FF
      when "011100011000" =>  data <= "11111111";  -- 718 = FF
      when "011100011001" =>  data <= "11111111";  -- 719 = FF
      when "011100011010" =>  data <= "11111111";  -- 71A = FF
      when "011100011011" =>  data <= "11111111";  -- 71B = FF
      when "011100011100" =>  data <= "11111111";  -- 71C = FF
      when "011100011101" =>  data <= "11111111";  -- 71D = FF
      when "011100011110" =>  data <= "11111111";  -- 71E = FF
      when "011100011111" =>  data <= "11111111";  -- 71F = FF
      when "011100100000" =>  data <= "11111111";  -- 720 = FF
      when "011100100001" =>  data <= "11111111";  -- 721 = FF
      when "011100100010" =>  data <= "11111111";  -- 722 = FF
      when "011100100011" =>  data <= "11111111";  -- 723 = FF
      when "011100100100" =>  data <= "11111111";  -- 724 = FF
      when "011100100101" =>  data <= "11111111";  -- 725 = FF
      when "011100100110" =>  data <= "11111111";  -- 726 = FF
      when "011100100111" =>  data <= "11111111";  -- 727 = FF
      when "011100101000" =>  data <= "11111111";  -- 728 = FF
      when "011100101001" =>  data <= "11111111";  -- 729 = FF
      when "011100101010" =>  data <= "11111111";  -- 72A = FF
      when "011100101011" =>  data <= "11111111";  -- 72B = FF
      when "011100101100" =>  data <= "11111111";  -- 72C = FF
      when "011100101101" =>  data <= "11111111";  -- 72D = FF
      when "011100101110" =>  data <= "11111111";  -- 72E = FF
      when "011100101111" =>  data <= "11111111";  -- 72F = FF
      when "011100110000" =>  data <= "11111111";  -- 730 = FF
      when "011100110001" =>  data <= "11111111";  -- 731 = FF
      when "011100110010" =>  data <= "11111111";  -- 732 = FF
      when "011100110011" =>  data <= "11111111";  -- 733 = FF
      when "011100110100" =>  data <= "11111111";  -- 734 = FF
      when "011100110101" =>  data <= "11111111";  -- 735 = FF
      when "011100110110" =>  data <= "11111111";  -- 736 = FF
      when "011100110111" =>  data <= "11111111";  -- 737 = FF
      when "011100111000" =>  data <= "11111111";  -- 738 = FF
      when "011100111001" =>  data <= "11111111";  -- 739 = FF
      when "011100111010" =>  data <= "11111111";  -- 73A = FF
      when "011100111011" =>  data <= "11111111";  -- 73B = FF
      when "011100111100" =>  data <= "11111111";  -- 73C = FF
      when "011100111101" =>  data <= "11111111";  -- 73D = FF
      when "011100111110" =>  data <= "11111111";  -- 73E = FF
      when "011100111111" =>  data <= "11111111";  -- 73F = FF
      when "011101000000" =>  data <= "11111111";  -- 740 = FF
      when "011101000001" =>  data <= "11111111";  -- 741 = FF
      when "011101000010" =>  data <= "11111111";  -- 742 = FF
      when "011101000011" =>  data <= "11111111";  -- 743 = FF
      when "011101000100" =>  data <= "11111111";  -- 744 = FF
      when "011101000101" =>  data <= "11111111";  -- 745 = FF
      when "011101000110" =>  data <= "11111111";  -- 746 = FF
      when "011101000111" =>  data <= "11111111";  -- 747 = FF
      when "011101001000" =>  data <= "11111111";  -- 748 = FF
      when "011101001001" =>  data <= "11111111";  -- 749 = FF
      when "011101001010" =>  data <= "11111111";  -- 74A = FF
      when "011101001011" =>  data <= "11111111";  -- 74B = FF
      when "011101001100" =>  data <= "11111111";  -- 74C = FF
      when "011101001101" =>  data <= "11111111";  -- 74D = FF
      when "011101001110" =>  data <= "11111111";  -- 74E = FF
      when "011101001111" =>  data <= "11111111";  -- 74F = FF
      when "011101010000" =>  data <= "11111111";  -- 750 = FF
      when "011101010001" =>  data <= "11111111";  -- 751 = FF
      when "011101010010" =>  data <= "11111111";  -- 752 = FF
      when "011101010011" =>  data <= "11111111";  -- 753 = FF
      when "011101010100" =>  data <= "11111111";  -- 754 = FF
      when "011101010101" =>  data <= "11111111";  -- 755 = FF
      when "011101010110" =>  data <= "11111111";  -- 756 = FF
      when "011101010111" =>  data <= "11111111";  -- 757 = FF
      when "011101011000" =>  data <= "11111111";  -- 758 = FF
      when "011101011001" =>  data <= "11111111";  -- 759 = FF
      when "011101011010" =>  data <= "11111111";  -- 75A = FF
      when "011101011011" =>  data <= "11111111";  -- 75B = FF
      when "011101011100" =>  data <= "11111111";  -- 75C = FF
      when "011101011101" =>  data <= "11111111";  -- 75D = FF
      when "011101011110" =>  data <= "11111111";  -- 75E = FF
      when "011101011111" =>  data <= "11111111";  -- 75F = FF
      when "011101100000" =>  data <= "11111111";  -- 760 = FF
      when "011101100001" =>  data <= "11111111";  -- 761 = FF
      when "011101100010" =>  data <= "11111111";  -- 762 = FF
      when "011101100011" =>  data <= "11111111";  -- 763 = FF
      when "011101100100" =>  data <= "11111111";  -- 764 = FF
      when "011101100101" =>  data <= "11111111";  -- 765 = FF
      when "011101100110" =>  data <= "11111111";  -- 766 = FF
      when "011101100111" =>  data <= "11111111";  -- 767 = FF
      when "011101101000" =>  data <= "11111111";  -- 768 = FF
      when "011101101001" =>  data <= "11111111";  -- 769 = FF
      when "011101101010" =>  data <= "11111111";  -- 76A = FF
      when "011101101011" =>  data <= "11111111";  -- 76B = FF
      when "011101101100" =>  data <= "11111111";  -- 76C = FF
      when "011101101101" =>  data <= "11111111";  -- 76D = FF
      when "011101101110" =>  data <= "11111111";  -- 76E = FF
      when "011101101111" =>  data <= "11111111";  -- 76F = FF
      when "011101110000" =>  data <= "11111111";  -- 770 = FF
      when "011101110001" =>  data <= "11111111";  -- 771 = FF
      when "011101110010" =>  data <= "11111111";  -- 772 = FF
      when "011101110011" =>  data <= "11111111";  -- 773 = FF
      when "011101110100" =>  data <= "11111111";  -- 774 = FF
      when "011101110101" =>  data <= "11111111";  -- 775 = FF
      when "011101110110" =>  data <= "11111111";  -- 776 = FF
      when "011101110111" =>  data <= "11111111";  -- 777 = FF
      when "011101111000" =>  data <= "11111111";  -- 778 = FF
      when "011101111001" =>  data <= "11111111";  -- 779 = FF
      when "011101111010" =>  data <= "11111111";  -- 77A = FF
      when "011101111011" =>  data <= "11111111";  -- 77B = FF
      when "011101111100" =>  data <= "11111111";  -- 77C = FF
      when "011101111101" =>  data <= "11111111";  -- 77D = FF
      when "011101111110" =>  data <= "11111111";  -- 77E = FF
      when "011101111111" =>  data <= "11111111";  -- 77F = FF
      when "011110000000" =>  data <= "11111111";  -- 780 = FF
      when "011110000001" =>  data <= "11111111";  -- 781 = FF
      when "011110000010" =>  data <= "11111111";  -- 782 = FF
      when "011110000011" =>  data <= "11111111";  -- 783 = FF
      when "011110000100" =>  data <= "11111111";  -- 784 = FF
      when "011110000101" =>  data <= "11111111";  -- 785 = FF
      when "011110000110" =>  data <= "11111111";  -- 786 = FF
      when "011110000111" =>  data <= "11111111";  -- 787 = FF
      when "011110001000" =>  data <= "11111111";  -- 788 = FF
      when "011110001001" =>  data <= "11111111";  -- 789 = FF
      when "011110001010" =>  data <= "11111111";  -- 78A = FF
      when "011110001011" =>  data <= "11111111";  -- 78B = FF
      when "011110001100" =>  data <= "11111111";  -- 78C = FF
      when "011110001101" =>  data <= "11111111";  -- 78D = FF
      when "011110001110" =>  data <= "11111111";  -- 78E = FF
      when "011110001111" =>  data <= "11111111";  -- 78F = FF
      when "011110010000" =>  data <= "11111111";  -- 790 = FF
      when "011110010001" =>  data <= "11111111";  -- 791 = FF
      when "011110010010" =>  data <= "11111111";  -- 792 = FF
      when "011110010011" =>  data <= "11111111";  -- 793 = FF
      when "011110010100" =>  data <= "11111111";  -- 794 = FF
      when "011110010101" =>  data <= "11111111";  -- 795 = FF
      when "011110010110" =>  data <= "11111111";  -- 796 = FF
      when "011110010111" =>  data <= "11111111";  -- 797 = FF
      when "011110011000" =>  data <= "11111111";  -- 798 = FF
      when "011110011001" =>  data <= "11111111";  -- 799 = FF
      when "011110011010" =>  data <= "11111111";  -- 79A = FF
      when "011110011011" =>  data <= "11111111";  -- 79B = FF
      when "011110011100" =>  data <= "11111111";  -- 79C = FF
      when "011110011101" =>  data <= "11111111";  -- 79D = FF
      when "011110011110" =>  data <= "11111111";  -- 79E = FF
      when "011110011111" =>  data <= "11111111";  -- 79F = FF
      when "011110100000" =>  data <= "11111111";  -- 7A0 = FF
      when "011110100001" =>  data <= "11111111";  -- 7A1 = FF
      when "011110100010" =>  data <= "11111111";  -- 7A2 = FF
      when "011110100011" =>  data <= "11111111";  -- 7A3 = FF
      when "011110100100" =>  data <= "11111111";  -- 7A4 = FF
      when "011110100101" =>  data <= "11111111";  -- 7A5 = FF
      when "011110100110" =>  data <= "11111111";  -- 7A6 = FF
      when "011110100111" =>  data <= "11111111";  -- 7A7 = FF
      when "011110101000" =>  data <= "11111111";  -- 7A8 = FF
      when "011110101001" =>  data <= "11111111";  -- 7A9 = FF
      when "011110101010" =>  data <= "11111111";  -- 7AA = FF
      when "011110101011" =>  data <= "11111111";  -- 7AB = FF
      when "011110101100" =>  data <= "11111111";  -- 7AC = FF
      when "011110101101" =>  data <= "11111111";  -- 7AD = FF
      when "011110101110" =>  data <= "11111111";  -- 7AE = FF
      when "011110101111" =>  data <= "11111111";  -- 7AF = FF
      when "011110110000" =>  data <= "11111111";  -- 7B0 = FF
      when "011110110001" =>  data <= "11111111";  -- 7B1 = FF
      when "011110110010" =>  data <= "11111111";  -- 7B2 = FF
      when "011110110011" =>  data <= "11111111";  -- 7B3 = FF
      when "011110110100" =>  data <= "11111111";  -- 7B4 = FF
      when "011110110101" =>  data <= "11111111";  -- 7B5 = FF
      when "011110110110" =>  data <= "11111111";  -- 7B6 = FF
      when "011110110111" =>  data <= "11111111";  -- 7B7 = FF
      when "011110111000" =>  data <= "11111111";  -- 7B8 = FF
      when "011110111001" =>  data <= "11111111";  -- 7B9 = FF
      when "011110111010" =>  data <= "11111111";  -- 7BA = FF
      when "011110111011" =>  data <= "11111111";  -- 7BB = FF
      when "011110111100" =>  data <= "11111111";  -- 7BC = FF
      when "011110111101" =>  data <= "11111111";  -- 7BD = FF
      when "011110111110" =>  data <= "11111111";  -- 7BE = FF
      when "011110111111" =>  data <= "11111111";  -- 7BF = FF
      when "011111000000" =>  data <= "11111111";  -- 7C0 = FF
      when "011111000001" =>  data <= "11111111";  -- 7C1 = FF
      when "011111000010" =>  data <= "11111111";  -- 7C2 = FF
      when "011111000011" =>  data <= "11111111";  -- 7C3 = FF
      when "011111000100" =>  data <= "11111111";  -- 7C4 = FF
      when "011111000101" =>  data <= "11111111";  -- 7C5 = FF
      when "011111000110" =>  data <= "11111111";  -- 7C6 = FF
      when "011111000111" =>  data <= "11111111";  -- 7C7 = FF
      when "011111001000" =>  data <= "11111111";  -- 7C8 = FF
      when "011111001001" =>  data <= "11111111";  -- 7C9 = FF
      when "011111001010" =>  data <= "11111111";  -- 7CA = FF
      when "011111001011" =>  data <= "11111111";  -- 7CB = FF
      when "011111001100" =>  data <= "11111111";  -- 7CC = FF
      when "011111001101" =>  data <= "11111111";  -- 7CD = FF
      when "011111001110" =>  data <= "11111111";  -- 7CE = FF
      when "011111001111" =>  data <= "11111111";  -- 7CF = FF
      when "011111010000" =>  data <= "11111111";  -- 7D0 = FF
      when "011111010001" =>  data <= "11111111";  -- 7D1 = FF
      when "011111010010" =>  data <= "11111111";  -- 7D2 = FF
      when "011111010011" =>  data <= "11111111";  -- 7D3 = FF
      when "011111010100" =>  data <= "11111111";  -- 7D4 = FF
      when "011111010101" =>  data <= "11111111";  -- 7D5 = FF
      when "011111010110" =>  data <= "11111111";  -- 7D6 = FF
      when "011111010111" =>  data <= "11111111";  -- 7D7 = FF
      when "011111011000" =>  data <= "11111111";  -- 7D8 = FF
      when "011111011001" =>  data <= "11111111";  -- 7D9 = FF
      when "011111011010" =>  data <= "11111111";  -- 7DA = FF
      when "011111011011" =>  data <= "11111111";  -- 7DB = FF
      when "011111011100" =>  data <= "11111111";  -- 7DC = FF
      when "011111011101" =>  data <= "11111111";  -- 7DD = FF
      when "011111011110" =>  data <= "11111111";  -- 7DE = FF
      when "011111011111" =>  data <= "11111111";  -- 7DF = FF
      when "011111100000" =>  data <= "11111111";  -- 7E0 = FF
      when "011111100001" =>  data <= "11111111";  -- 7E1 = FF
      when "011111100010" =>  data <= "11111111";  -- 7E2 = FF
      when "011111100011" =>  data <= "11111111";  -- 7E3 = FF
      when "011111100100" =>  data <= "11111111";  -- 7E4 = FF
      when "011111100101" =>  data <= "11111111";  -- 7E5 = FF
      when "011111100110" =>  data <= "11111111";  -- 7E6 = FF
      when "011111100111" =>  data <= "11111111";  -- 7E7 = FF
      when "011111101000" =>  data <= "11111111";  -- 7E8 = FF
      when "011111101001" =>  data <= "11111111";  -- 7E9 = FF
      when "011111101010" =>  data <= "11111111";  -- 7EA = FF
      when "011111101011" =>  data <= "11111111";  -- 7EB = FF
      when "011111101100" =>  data <= "11111111";  -- 7EC = FF
      when "011111101101" =>  data <= "11111111";  -- 7ED = FF
      when "011111101110" =>  data <= "11111111";  -- 7EE = FF
      when "011111101111" =>  data <= "11111111";  -- 7EF = FF
      when "011111110000" =>  data <= "11111111";  -- 7F0 = FF
      when "011111110001" =>  data <= "11111111";  -- 7F1 = FF
      when "011111110010" =>  data <= "11111111";  -- 7F2 = FF
      when "011111110011" =>  data <= "11111111";  -- 7F3 = FF
      when "011111110100" =>  data <= "11111111";  -- 7F4 = FF
      when "011111110101" =>  data <= "11111111";  -- 7F5 = FF
      when "011111110110" =>  data <= "11111111";  -- 7F6 = FF
      when "011111110111" =>  data <= "11111111";  -- 7F7 = FF
      when "011111111000" =>  data <= "11111111";  -- 7F8 = FF
      when "011111111001" =>  data <= "11111111";  -- 7F9 = FF
      when "011111111010" =>  data <= "11111111";  -- 7FA = FF
      when "011111111011" =>  data <= "11111111";  -- 7FB = FF
      when "011111111100" =>  data <= "11111111";  -- 7FC = FF
      when "011111111101" =>  data <= "11111111";  -- 7FD = FF
      when "011111111110" =>  data <= "11111111";  -- 7FE = FF
      when "011111111111" =>  data <= "11111111";  -- 7FF = FF
      when "100000000000" =>  data <= "11111111";  -- 800 = FF
      when "100000000001" =>  data <= "11111111";  -- 801 = FF
      when "100000000010" =>  data <= "11111111";  -- 802 = FF
      when "100000000011" =>  data <= "11111111";  -- 803 = FF
      when "100000000100" =>  data <= "11111111";  -- 804 = FF
      when "100000000101" =>  data <= "11111111";  -- 805 = FF
      when "100000000110" =>  data <= "11111111";  -- 806 = FF
      when "100000000111" =>  data <= "11111111";  -- 807 = FF
      when "100000001000" =>  data <= "11111111";  -- 808 = FF
      when "100000001001" =>  data <= "11111111";  -- 809 = FF
      when "100000001010" =>  data <= "11111111";  -- 80A = FF
      when "100000001011" =>  data <= "11111111";  -- 80B = FF
      when "100000001100" =>  data <= "11111111";  -- 80C = FF
      when "100000001101" =>  data <= "11111111";  -- 80D = FF
      when "100000001110" =>  data <= "11111111";  -- 80E = FF
      when "100000001111" =>  data <= "11111111";  -- 80F = FF
      when "100000010000" =>  data <= "11111111";  -- 810 = FF
      when "100000010001" =>  data <= "11111111";  -- 811 = FF
      when "100000010010" =>  data <= "11111111";  -- 812 = FF
      when "100000010011" =>  data <= "11111111";  -- 813 = FF
      when "100000010100" =>  data <= "11111111";  -- 814 = FF
      when "100000010101" =>  data <= "11111111";  -- 815 = FF
      when "100000010110" =>  data <= "11111111";  -- 816 = FF
      when "100000010111" =>  data <= "11111111";  -- 817 = FF
      when "100000011000" =>  data <= "11111111";  -- 818 = FF
      when "100000011001" =>  data <= "11111111";  -- 819 = FF
      when "100000011010" =>  data <= "11111111";  -- 81A = FF
      when "100000011011" =>  data <= "11111111";  -- 81B = FF
      when "100000011100" =>  data <= "11111111";  -- 81C = FF
      when "100000011101" =>  data <= "11111111";  -- 81D = FF
      when "100000011110" =>  data <= "11111111";  -- 81E = FF
      when "100000011111" =>  data <= "11111111";  -- 81F = FF
      when "100000100000" =>  data <= "11111111";  -- 820 = FF
      when "100000100001" =>  data <= "11111111";  -- 821 = FF
      when "100000100010" =>  data <= "11111111";  -- 822 = FF
      when "100000100011" =>  data <= "11111111";  -- 823 = FF
      when "100000100100" =>  data <= "11111111";  -- 824 = FF
      when "100000100101" =>  data <= "11111111";  -- 825 = FF
      when "100000100110" =>  data <= "11111111";  -- 826 = FF
      when "100000100111" =>  data <= "11111111";  -- 827 = FF
      when "100000101000" =>  data <= "11111111";  -- 828 = FF
      when "100000101001" =>  data <= "11111111";  -- 829 = FF
      when "100000101010" =>  data <= "11111111";  -- 82A = FF
      when "100000101011" =>  data <= "11111111";  -- 82B = FF
      when "100000101100" =>  data <= "11111111";  -- 82C = FF
      when "100000101101" =>  data <= "11111111";  -- 82D = FF
      when "100000101110" =>  data <= "11111111";  -- 82E = FF
      when "100000101111" =>  data <= "11111111";  -- 82F = FF
      when "100000110000" =>  data <= "11111111";  -- 830 = FF
      when "100000110001" =>  data <= "11111111";  -- 831 = FF
      when "100000110010" =>  data <= "11111111";  -- 832 = FF
      when "100000110011" =>  data <= "11111111";  -- 833 = FF
      when "100000110100" =>  data <= "11111111";  -- 834 = FF
      when "100000110101" =>  data <= "11111111";  -- 835 = FF
      when "100000110110" =>  data <= "11111111";  -- 836 = FF
      when "100000110111" =>  data <= "11111111";  -- 837 = FF
      when "100000111000" =>  data <= "11111111";  -- 838 = FF
      when "100000111001" =>  data <= "11111111";  -- 839 = FF
      when "100000111010" =>  data <= "11111111";  -- 83A = FF
      when "100000111011" =>  data <= "11111111";  -- 83B = FF
      when "100000111100" =>  data <= "11111111";  -- 83C = FF
      when "100000111101" =>  data <= "11111111";  -- 83D = FF
      when "100000111110" =>  data <= "11111111";  -- 83E = FF
      when "100000111111" =>  data <= "11111111";  -- 83F = FF
      when "100001000000" =>  data <= "11111111";  -- 840 = FF
      when "100001000001" =>  data <= "11111111";  -- 841 = FF
      when "100001000010" =>  data <= "11111111";  -- 842 = FF
      when "100001000011" =>  data <= "11111111";  -- 843 = FF
      when "100001000100" =>  data <= "11111111";  -- 844 = FF
      when "100001000101" =>  data <= "11111111";  -- 845 = FF
      when "100001000110" =>  data <= "11111111";  -- 846 = FF
      when "100001000111" =>  data <= "11111111";  -- 847 = FF
      when "100001001000" =>  data <= "11111111";  -- 848 = FF
      when "100001001001" =>  data <= "11111111";  -- 849 = FF
      when "100001001010" =>  data <= "11111111";  -- 84A = FF
      when "100001001011" =>  data <= "11111111";  -- 84B = FF
      when "100001001100" =>  data <= "11111111";  -- 84C = FF
      when "100001001101" =>  data <= "11111111";  -- 84D = FF
      when "100001001110" =>  data <= "11111111";  -- 84E = FF
      when "100001001111" =>  data <= "11111111";  -- 84F = FF
      when "100001010000" =>  data <= "11111111";  -- 850 = FF
      when "100001010001" =>  data <= "11111111";  -- 851 = FF
      when "100001010010" =>  data <= "11111111";  -- 852 = FF
      when "100001010011" =>  data <= "11111111";  -- 853 = FF
      when "100001010100" =>  data <= "11111111";  -- 854 = FF
      when "100001010101" =>  data <= "11111111";  -- 855 = FF
      when "100001010110" =>  data <= "11111111";  -- 856 = FF
      when "100001010111" =>  data <= "11111111";  -- 857 = FF
      when "100001011000" =>  data <= "11111111";  -- 858 = FF
      when "100001011001" =>  data <= "11111111";  -- 859 = FF
      when "100001011010" =>  data <= "11111111";  -- 85A = FF
      when "100001011011" =>  data <= "11111111";  -- 85B = FF
      when "100001011100" =>  data <= "11111111";  -- 85C = FF
      when "100001011101" =>  data <= "11111111";  -- 85D = FF
      when "100001011110" =>  data <= "11111111";  -- 85E = FF
      when "100001011111" =>  data <= "11111111";  -- 85F = FF
      when "100001100000" =>  data <= "11111111";  -- 860 = FF
      when "100001100001" =>  data <= "11111111";  -- 861 = FF
      when "100001100010" =>  data <= "11111111";  -- 862 = FF
      when "100001100011" =>  data <= "11111111";  -- 863 = FF
      when "100001100100" =>  data <= "11111111";  -- 864 = FF
      when "100001100101" =>  data <= "11111111";  -- 865 = FF
      when "100001100110" =>  data <= "11111111";  -- 866 = FF
      when "100001100111" =>  data <= "11111111";  -- 867 = FF
      when "100001101000" =>  data <= "11111111";  -- 868 = FF
      when "100001101001" =>  data <= "11111111";  -- 869 = FF
      when "100001101010" =>  data <= "11111111";  -- 86A = FF
      when "100001101011" =>  data <= "11111111";  -- 86B = FF
      when "100001101100" =>  data <= "11111111";  -- 86C = FF
      when "100001101101" =>  data <= "11111111";  -- 86D = FF
      when "100001101110" =>  data <= "11111111";  -- 86E = FF
      when "100001101111" =>  data <= "11111111";  -- 86F = FF
      when "100001110000" =>  data <= "11111111";  -- 870 = FF
      when "100001110001" =>  data <= "11111111";  -- 871 = FF
      when "100001110010" =>  data <= "11111111";  -- 872 = FF
      when "100001110011" =>  data <= "11111111";  -- 873 = FF
      when "100001110100" =>  data <= "11111111";  -- 874 = FF
      when "100001110101" =>  data <= "11111111";  -- 875 = FF
      when "100001110110" =>  data <= "11111111";  -- 876 = FF
      when "100001110111" =>  data <= "11111111";  -- 877 = FF
      when "100001111000" =>  data <= "11111111";  -- 878 = FF
      when "100001111001" =>  data <= "11111111";  -- 879 = FF
      when "100001111010" =>  data <= "11111111";  -- 87A = FF
      when "100001111011" =>  data <= "11111111";  -- 87B = FF
      when "100001111100" =>  data <= "11111111";  -- 87C = FF
      when "100001111101" =>  data <= "11111111";  -- 87D = FF
      when "100001111110" =>  data <= "11111111";  -- 87E = FF
      when "100001111111" =>  data <= "11111111";  -- 87F = FF
      when "100010000000" =>  data <= "11111111";  -- 880 = FF
      when "100010000001" =>  data <= "11111111";  -- 881 = FF
      when "100010000010" =>  data <= "11111111";  -- 882 = FF
      when "100010000011" =>  data <= "11111111";  -- 883 = FF
      when "100010000100" =>  data <= "11111111";  -- 884 = FF
      when "100010000101" =>  data <= "11111111";  -- 885 = FF
      when "100010000110" =>  data <= "11111111";  -- 886 = FF
      when "100010000111" =>  data <= "11111111";  -- 887 = FF
      when "100010001000" =>  data <= "11111111";  -- 888 = FF
      when "100010001001" =>  data <= "11111111";  -- 889 = FF
      when "100010001010" =>  data <= "11111111";  -- 88A = FF
      when "100010001011" =>  data <= "11111111";  -- 88B = FF
      when "100010001100" =>  data <= "11111111";  -- 88C = FF
      when "100010001101" =>  data <= "11111111";  -- 88D = FF
      when "100010001110" =>  data <= "11111111";  -- 88E = FF
      when "100010001111" =>  data <= "11111111";  -- 88F = FF
      when "100010010000" =>  data <= "11111111";  -- 890 = FF
      when "100010010001" =>  data <= "11111111";  -- 891 = FF
      when "100010010010" =>  data <= "11111111";  -- 892 = FF
      when "100010010011" =>  data <= "11111111";  -- 893 = FF
      when "100010010100" =>  data <= "11111111";  -- 894 = FF
      when "100010010101" =>  data <= "11111111";  -- 895 = FF
      when "100010010110" =>  data <= "11111111";  -- 896 = FF
      when "100010010111" =>  data <= "11111111";  -- 897 = FF
      when "100010011000" =>  data <= "11111111";  -- 898 = FF
      when "100010011001" =>  data <= "11111111";  -- 899 = FF
      when "100010011010" =>  data <= "11111111";  -- 89A = FF
      when "100010011011" =>  data <= "11111111";  -- 89B = FF
      when "100010011100" =>  data <= "11111111";  -- 89C = FF
      when "100010011101" =>  data <= "11111111";  -- 89D = FF
      when "100010011110" =>  data <= "11111111";  -- 89E = FF
      when "100010011111" =>  data <= "11111111";  -- 89F = FF
      when "100010100000" =>  data <= "11111111";  -- 8A0 = FF
      when "100010100001" =>  data <= "11111111";  -- 8A1 = FF
      when "100010100010" =>  data <= "11111111";  -- 8A2 = FF
      when "100010100011" =>  data <= "11111111";  -- 8A3 = FF
      when "100010100100" =>  data <= "11111111";  -- 8A4 = FF
      when "100010100101" =>  data <= "11111111";  -- 8A5 = FF
      when "100010100110" =>  data <= "11111111";  -- 8A6 = FF
      when "100010100111" =>  data <= "11111111";  -- 8A7 = FF
      when "100010101000" =>  data <= "11111111";  -- 8A8 = FF
      when "100010101001" =>  data <= "11111111";  -- 8A9 = FF
      when "100010101010" =>  data <= "11111111";  -- 8AA = FF
      when "100010101011" =>  data <= "11111111";  -- 8AB = FF
      when "100010101100" =>  data <= "11111111";  -- 8AC = FF
      when "100010101101" =>  data <= "11111111";  -- 8AD = FF
      when "100010101110" =>  data <= "11111111";  -- 8AE = FF
      when "100010101111" =>  data <= "11111111";  -- 8AF = FF
      when "100010110000" =>  data <= "11111111";  -- 8B0 = FF
      when "100010110001" =>  data <= "11111111";  -- 8B1 = FF
      when "100010110010" =>  data <= "11111111";  -- 8B2 = FF
      when "100010110011" =>  data <= "11111111";  -- 8B3 = FF
      when "100010110100" =>  data <= "11111111";  -- 8B4 = FF
      when "100010110101" =>  data <= "11111111";  -- 8B5 = FF
      when "100010110110" =>  data <= "11111111";  -- 8B6 = FF
      when "100010110111" =>  data <= "11111111";  -- 8B7 = FF
      when "100010111000" =>  data <= "11111111";  -- 8B8 = FF
      when "100010111001" =>  data <= "11111111";  -- 8B9 = FF
      when "100010111010" =>  data <= "11111111";  -- 8BA = FF
      when "100010111011" =>  data <= "11111111";  -- 8BB = FF
      when "100010111100" =>  data <= "11111111";  -- 8BC = FF
      when "100010111101" =>  data <= "11111111";  -- 8BD = FF
      when "100010111110" =>  data <= "11111111";  -- 8BE = FF
      when "100010111111" =>  data <= "11111111";  -- 8BF = FF
      when "100011000000" =>  data <= "11111111";  -- 8C0 = FF
      when "100011000001" =>  data <= "11111111";  -- 8C1 = FF
      when "100011000010" =>  data <= "11111111";  -- 8C2 = FF
      when "100011000011" =>  data <= "11111111";  -- 8C3 = FF
      when "100011000100" =>  data <= "11111111";  -- 8C4 = FF
      when "100011000101" =>  data <= "11111111";  -- 8C5 = FF
      when "100011000110" =>  data <= "11111111";  -- 8C6 = FF
      when "100011000111" =>  data <= "11111111";  -- 8C7 = FF
      when "100011001000" =>  data <= "11111111";  -- 8C8 = FF
      when "100011001001" =>  data <= "11111111";  -- 8C9 = FF
      when "100011001010" =>  data <= "11111111";  -- 8CA = FF
      when "100011001011" =>  data <= "11111111";  -- 8CB = FF
      when "100011001100" =>  data <= "11111111";  -- 8CC = FF
      when "100011001101" =>  data <= "11111111";  -- 8CD = FF
      when "100011001110" =>  data <= "11111111";  -- 8CE = FF
      when "100011001111" =>  data <= "11111111";  -- 8CF = FF
      when "100011010000" =>  data <= "11111111";  -- 8D0 = FF
      when "100011010001" =>  data <= "11111111";  -- 8D1 = FF
      when "100011010010" =>  data <= "11111111";  -- 8D2 = FF
      when "100011010011" =>  data <= "11111111";  -- 8D3 = FF
      when "100011010100" =>  data <= "11111111";  -- 8D4 = FF
      when "100011010101" =>  data <= "11111111";  -- 8D5 = FF
      when "100011010110" =>  data <= "11111111";  -- 8D6 = FF
      when "100011010111" =>  data <= "11111111";  -- 8D7 = FF
      when "100011011000" =>  data <= "11111111";  -- 8D8 = FF
      when "100011011001" =>  data <= "11111111";  -- 8D9 = FF
      when "100011011010" =>  data <= "11111111";  -- 8DA = FF
      when "100011011011" =>  data <= "11111111";  -- 8DB = FF
      when "100011011100" =>  data <= "11111111";  -- 8DC = FF
      when "100011011101" =>  data <= "11111111";  -- 8DD = FF
      when "100011011110" =>  data <= "11111111";  -- 8DE = FF
      when "100011011111" =>  data <= "11111111";  -- 8DF = FF
      when "100011100000" =>  data <= "11111111";  -- 8E0 = FF
      when "100011100001" =>  data <= "11111111";  -- 8E1 = FF
      when "100011100010" =>  data <= "11111111";  -- 8E2 = FF
      when "100011100011" =>  data <= "11111111";  -- 8E3 = FF
      when "100011100100" =>  data <= "11111111";  -- 8E4 = FF
      when "100011100101" =>  data <= "11111111";  -- 8E5 = FF
      when "100011100110" =>  data <= "11111111";  -- 8E6 = FF
      when "100011100111" =>  data <= "11111111";  -- 8E7 = FF
      when "100011101000" =>  data <= "11111111";  -- 8E8 = FF
      when "100011101001" =>  data <= "11111111";  -- 8E9 = FF
      when "100011101010" =>  data <= "11111111";  -- 8EA = FF
      when "100011101011" =>  data <= "11111111";  -- 8EB = FF
      when "100011101100" =>  data <= "11111111";  -- 8EC = FF
      when "100011101101" =>  data <= "11111111";  -- 8ED = FF
      when "100011101110" =>  data <= "11111111";  -- 8EE = FF
      when "100011101111" =>  data <= "11111111";  -- 8EF = FF
      when "100011110000" =>  data <= "11111111";  -- 8F0 = FF
      when "100011110001" =>  data <= "11111111";  -- 8F1 = FF
      when "100011110010" =>  data <= "11111111";  -- 8F2 = FF
      when "100011110011" =>  data <= "11111111";  -- 8F3 = FF
      when "100011110100" =>  data <= "11111111";  -- 8F4 = FF
      when "100011110101" =>  data <= "11111111";  -- 8F5 = FF
      when "100011110110" =>  data <= "11111111";  -- 8F6 = FF
      when "100011110111" =>  data <= "11111111";  -- 8F7 = FF
      when "100011111000" =>  data <= "11111111";  -- 8F8 = FF
      when "100011111001" =>  data <= "11111111";  -- 8F9 = FF
      when "100011111010" =>  data <= "11111111";  -- 8FA = FF
      when "100011111011" =>  data <= "11111111";  -- 8FB = FF
      when "100011111100" =>  data <= "11111111";  -- 8FC = FF
      when "100011111101" =>  data <= "11111111";  -- 8FD = FF
      when "100011111110" =>  data <= "11111111";  -- 8FE = FF
      when "100011111111" =>  data <= "11111111";  -- 8FF = FF
      when "100100000000" =>  data <= "11111111";  -- 900 = FF
      when "100100000001" =>  data <= "11111111";  -- 901 = FF
      when "100100000010" =>  data <= "11111111";  -- 902 = FF
      when "100100000011" =>  data <= "11111111";  -- 903 = FF
      when "100100000100" =>  data <= "11111111";  -- 904 = FF
      when "100100000101" =>  data <= "11111111";  -- 905 = FF
      when "100100000110" =>  data <= "11111111";  -- 906 = FF
      when "100100000111" =>  data <= "11111111";  -- 907 = FF
      when "100100001000" =>  data <= "11111111";  -- 908 = FF
      when "100100001001" =>  data <= "11111111";  -- 909 = FF
      when "100100001010" =>  data <= "11111111";  -- 90A = FF
      when "100100001011" =>  data <= "11111111";  -- 90B = FF
      when "100100001100" =>  data <= "11111111";  -- 90C = FF
      when "100100001101" =>  data <= "11111111";  -- 90D = FF
      when "100100001110" =>  data <= "11111111";  -- 90E = FF
      when "100100001111" =>  data <= "11111111";  -- 90F = FF
      when "100100010000" =>  data <= "11111111";  -- 910 = FF
      when "100100010001" =>  data <= "11111111";  -- 911 = FF
      when "100100010010" =>  data <= "11111111";  -- 912 = FF
      when "100100010011" =>  data <= "11111111";  -- 913 = FF
      when "100100010100" =>  data <= "11111111";  -- 914 = FF
      when "100100010101" =>  data <= "11111111";  -- 915 = FF
      when "100100010110" =>  data <= "11111111";  -- 916 = FF
      when "100100010111" =>  data <= "11111111";  -- 917 = FF
      when "100100011000" =>  data <= "11111111";  -- 918 = FF
      when "100100011001" =>  data <= "11111111";  -- 919 = FF
      when "100100011010" =>  data <= "11111111";  -- 91A = FF
      when "100100011011" =>  data <= "11111111";  -- 91B = FF
      when "100100011100" =>  data <= "11111111";  -- 91C = FF
      when "100100011101" =>  data <= "11111111";  -- 91D = FF
      when "100100011110" =>  data <= "11111111";  -- 91E = FF
      when "100100011111" =>  data <= "11111111";  -- 91F = FF
      when "100100100000" =>  data <= "11111111";  -- 920 = FF
      when "100100100001" =>  data <= "11111111";  -- 921 = FF
      when "100100100010" =>  data <= "11111111";  -- 922 = FF
      when "100100100011" =>  data <= "11111111";  -- 923 = FF
      when "100100100100" =>  data <= "11111111";  -- 924 = FF
      when "100100100101" =>  data <= "11111111";  -- 925 = FF
      when "100100100110" =>  data <= "11111111";  -- 926 = FF
      when "100100100111" =>  data <= "11111111";  -- 927 = FF
      when "100100101000" =>  data <= "11111111";  -- 928 = FF
      when "100100101001" =>  data <= "11111111";  -- 929 = FF
      when "100100101010" =>  data <= "11111111";  -- 92A = FF
      when "100100101011" =>  data <= "11111111";  -- 92B = FF
      when "100100101100" =>  data <= "11111111";  -- 92C = FF
      when "100100101101" =>  data <= "11111111";  -- 92D = FF
      when "100100101110" =>  data <= "11111111";  -- 92E = FF
      when "100100101111" =>  data <= "11111111";  -- 92F = FF
      when "100100110000" =>  data <= "11111111";  -- 930 = FF
      when "100100110001" =>  data <= "11111111";  -- 931 = FF
      when "100100110010" =>  data <= "11111111";  -- 932 = FF
      when "100100110011" =>  data <= "11111111";  -- 933 = FF
      when "100100110100" =>  data <= "11111111";  -- 934 = FF
      when "100100110101" =>  data <= "11111111";  -- 935 = FF
      when "100100110110" =>  data <= "11111111";  -- 936 = FF
      when "100100110111" =>  data <= "11111111";  -- 937 = FF
      when "100100111000" =>  data <= "11111111";  -- 938 = FF
      when "100100111001" =>  data <= "11111111";  -- 939 = FF
      when "100100111010" =>  data <= "11111111";  -- 93A = FF
      when "100100111011" =>  data <= "11111111";  -- 93B = FF
      when "100100111100" =>  data <= "11111111";  -- 93C = FF
      when "100100111101" =>  data <= "11111111";  -- 93D = FF
      when "100100111110" =>  data <= "11111111";  -- 93E = FF
      when "100100111111" =>  data <= "11111111";  -- 93F = FF
      when "100101000000" =>  data <= "11111111";  -- 940 = FF
      when "100101000001" =>  data <= "11111111";  -- 941 = FF
      when "100101000010" =>  data <= "11111111";  -- 942 = FF
      when "100101000011" =>  data <= "11111111";  -- 943 = FF
      when "100101000100" =>  data <= "11111111";  -- 944 = FF
      when "100101000101" =>  data <= "11111111";  -- 945 = FF
      when "100101000110" =>  data <= "11111111";  -- 946 = FF
      when "100101000111" =>  data <= "11111111";  -- 947 = FF
      when "100101001000" =>  data <= "11111111";  -- 948 = FF
      when "100101001001" =>  data <= "11111111";  -- 949 = FF
      when "100101001010" =>  data <= "11111111";  -- 94A = FF
      when "100101001011" =>  data <= "11111111";  -- 94B = FF
      when "100101001100" =>  data <= "11111111";  -- 94C = FF
      when "100101001101" =>  data <= "11111111";  -- 94D = FF
      when "100101001110" =>  data <= "11111111";  -- 94E = FF
      when "100101001111" =>  data <= "11111111";  -- 94F = FF
      when "100101010000" =>  data <= "11111111";  -- 950 = FF
      when "100101010001" =>  data <= "11111111";  -- 951 = FF
      when "100101010010" =>  data <= "11111111";  -- 952 = FF
      when "100101010011" =>  data <= "11111111";  -- 953 = FF
      when "100101010100" =>  data <= "11111111";  -- 954 = FF
      when "100101010101" =>  data <= "11111111";  -- 955 = FF
      when "100101010110" =>  data <= "11111111";  -- 956 = FF
      when "100101010111" =>  data <= "11111111";  -- 957 = FF
      when "100101011000" =>  data <= "11111111";  -- 958 = FF
      when "100101011001" =>  data <= "11111111";  -- 959 = FF
      when "100101011010" =>  data <= "11111111";  -- 95A = FF
      when "100101011011" =>  data <= "11111111";  -- 95B = FF
      when "100101011100" =>  data <= "11111111";  -- 95C = FF
      when "100101011101" =>  data <= "11111111";  -- 95D = FF
      when "100101011110" =>  data <= "11111111";  -- 95E = FF
      when "100101011111" =>  data <= "11111111";  -- 95F = FF
      when "100101100000" =>  data <= "11111111";  -- 960 = FF
      when "100101100001" =>  data <= "11111111";  -- 961 = FF
      when "100101100010" =>  data <= "11111111";  -- 962 = FF
      when "100101100011" =>  data <= "11111111";  -- 963 = FF
      when "100101100100" =>  data <= "11111111";  -- 964 = FF
      when "100101100101" =>  data <= "11111111";  -- 965 = FF
      when "100101100110" =>  data <= "11111111";  -- 966 = FF
      when "100101100111" =>  data <= "11111111";  -- 967 = FF
      when "100101101000" =>  data <= "11111111";  -- 968 = FF
      when "100101101001" =>  data <= "11111111";  -- 969 = FF
      when "100101101010" =>  data <= "11111111";  -- 96A = FF
      when "100101101011" =>  data <= "11111111";  -- 96B = FF
      when "100101101100" =>  data <= "11111111";  -- 96C = FF
      when "100101101101" =>  data <= "11111111";  -- 96D = FF
      when "100101101110" =>  data <= "11111111";  -- 96E = FF
      when "100101101111" =>  data <= "11111111";  -- 96F = FF
      when "100101110000" =>  data <= "11111111";  -- 970 = FF
      when "100101110001" =>  data <= "11111111";  -- 971 = FF
      when "100101110010" =>  data <= "11111111";  -- 972 = FF
      when "100101110011" =>  data <= "11111111";  -- 973 = FF
      when "100101110100" =>  data <= "11111111";  -- 974 = FF
      when "100101110101" =>  data <= "11111111";  -- 975 = FF
      when "100101110110" =>  data <= "11111111";  -- 976 = FF
      when "100101110111" =>  data <= "11111111";  -- 977 = FF
      when "100101111000" =>  data <= "11111111";  -- 978 = FF
      when "100101111001" =>  data <= "11111111";  -- 979 = FF
      when "100101111010" =>  data <= "11111111";  -- 97A = FF
      when "100101111011" =>  data <= "11111111";  -- 97B = FF
      when "100101111100" =>  data <= "11111111";  -- 97C = FF
      when "100101111101" =>  data <= "11111111";  -- 97D = FF
      when "100101111110" =>  data <= "11111111";  -- 97E = FF
      when "100101111111" =>  data <= "11111111";  -- 97F = FF
      when "100110000000" =>  data <= "11111111";  -- 980 = FF
      when "100110000001" =>  data <= "11111111";  -- 981 = FF
      when "100110000010" =>  data <= "11111111";  -- 982 = FF
      when "100110000011" =>  data <= "11111111";  -- 983 = FF
      when "100110000100" =>  data <= "11111111";  -- 984 = FF
      when "100110000101" =>  data <= "11111111";  -- 985 = FF
      when "100110000110" =>  data <= "11111111";  -- 986 = FF
      when "100110000111" =>  data <= "11111111";  -- 987 = FF
      when "100110001000" =>  data <= "11111111";  -- 988 = FF
      when "100110001001" =>  data <= "11111111";  -- 989 = FF
      when "100110001010" =>  data <= "11111111";  -- 98A = FF
      when "100110001011" =>  data <= "11111111";  -- 98B = FF
      when "100110001100" =>  data <= "11111111";  -- 98C = FF
      when "100110001101" =>  data <= "11111111";  -- 98D = FF
      when "100110001110" =>  data <= "11111111";  -- 98E = FF
      when "100110001111" =>  data <= "11111111";  -- 98F = FF
      when "100110010000" =>  data <= "11111111";  -- 990 = FF
      when "100110010001" =>  data <= "11111111";  -- 991 = FF
      when "100110010010" =>  data <= "11111111";  -- 992 = FF
      when "100110010011" =>  data <= "11111111";  -- 993 = FF
      when "100110010100" =>  data <= "11111111";  -- 994 = FF
      when "100110010101" =>  data <= "11111111";  -- 995 = FF
      when "100110010110" =>  data <= "11111111";  -- 996 = FF
      when "100110010111" =>  data <= "11111111";  -- 997 = FF
      when "100110011000" =>  data <= "11111111";  -- 998 = FF
      when "100110011001" =>  data <= "11111111";  -- 999 = FF
      when "100110011010" =>  data <= "11111111";  -- 99A = FF
      when "100110011011" =>  data <= "11111111";  -- 99B = FF
      when "100110011100" =>  data <= "11111111";  -- 99C = FF
      when "100110011101" =>  data <= "11111111";  -- 99D = FF
      when "100110011110" =>  data <= "11111111";  -- 99E = FF
      when "100110011111" =>  data <= "11111111";  -- 99F = FF
      when "100110100000" =>  data <= "11111111";  -- 9A0 = FF
      when "100110100001" =>  data <= "11111111";  -- 9A1 = FF
      when "100110100010" =>  data <= "11111111";  -- 9A2 = FF
      when "100110100011" =>  data <= "11111111";  -- 9A3 = FF
      when "100110100100" =>  data <= "11111111";  -- 9A4 = FF
      when "100110100101" =>  data <= "11111111";  -- 9A5 = FF
      when "100110100110" =>  data <= "11111111";  -- 9A6 = FF
      when "100110100111" =>  data <= "11111111";  -- 9A7 = FF
      when "100110101000" =>  data <= "11111111";  -- 9A8 = FF
      when "100110101001" =>  data <= "11111111";  -- 9A9 = FF
      when "100110101010" =>  data <= "11111111";  -- 9AA = FF
      when "100110101011" =>  data <= "11111111";  -- 9AB = FF
      when "100110101100" =>  data <= "11111111";  -- 9AC = FF
      when "100110101101" =>  data <= "11111111";  -- 9AD = FF
      when "100110101110" =>  data <= "11111111";  -- 9AE = FF
      when "100110101111" =>  data <= "11111111";  -- 9AF = FF
      when "100110110000" =>  data <= "11111111";  -- 9B0 = FF
      when "100110110001" =>  data <= "11111111";  -- 9B1 = FF
      when "100110110010" =>  data <= "11111111";  -- 9B2 = FF
      when "100110110011" =>  data <= "11111111";  -- 9B3 = FF
      when "100110110100" =>  data <= "11111111";  -- 9B4 = FF
      when "100110110101" =>  data <= "11111111";  -- 9B5 = FF
      when "100110110110" =>  data <= "11111111";  -- 9B6 = FF
      when "100110110111" =>  data <= "11111111";  -- 9B7 = FF
      when "100110111000" =>  data <= "11111111";  -- 9B8 = FF
      when "100110111001" =>  data <= "11111111";  -- 9B9 = FF
      when "100110111010" =>  data <= "11111111";  -- 9BA = FF
      when "100110111011" =>  data <= "11111111";  -- 9BB = FF
      when "100110111100" =>  data <= "11111111";  -- 9BC = FF
      when "100110111101" =>  data <= "11111111";  -- 9BD = FF
      when "100110111110" =>  data <= "11111111";  -- 9BE = FF
      when "100110111111" =>  data <= "11111111";  -- 9BF = FF
      when "100111000000" =>  data <= "11111111";  -- 9C0 = FF
      when "100111000001" =>  data <= "11111111";  -- 9C1 = FF
      when "100111000010" =>  data <= "11111111";  -- 9C2 = FF
      when "100111000011" =>  data <= "11111111";  -- 9C3 = FF
      when "100111000100" =>  data <= "11111111";  -- 9C4 = FF
      when "100111000101" =>  data <= "11111111";  -- 9C5 = FF
      when "100111000110" =>  data <= "11111111";  -- 9C6 = FF
      when "100111000111" =>  data <= "11111111";  -- 9C7 = FF
      when "100111001000" =>  data <= "11111111";  -- 9C8 = FF
      when "100111001001" =>  data <= "11111111";  -- 9C9 = FF
      when "100111001010" =>  data <= "11111111";  -- 9CA = FF
      when "100111001011" =>  data <= "11111111";  -- 9CB = FF
      when "100111001100" =>  data <= "11111111";  -- 9CC = FF
      when "100111001101" =>  data <= "11111111";  -- 9CD = FF
      when "100111001110" =>  data <= "11111111";  -- 9CE = FF
      when "100111001111" =>  data <= "11111111";  -- 9CF = FF
      when "100111010000" =>  data <= "11111111";  -- 9D0 = FF
      when "100111010001" =>  data <= "11111111";  -- 9D1 = FF
      when "100111010010" =>  data <= "11111111";  -- 9D2 = FF
      when "100111010011" =>  data <= "11111111";  -- 9D3 = FF
      when "100111010100" =>  data <= "11111111";  -- 9D4 = FF
      when "100111010101" =>  data <= "11111111";  -- 9D5 = FF
      when "100111010110" =>  data <= "11111111";  -- 9D6 = FF
      when "100111010111" =>  data <= "11111111";  -- 9D7 = FF
      when "100111011000" =>  data <= "11111111";  -- 9D8 = FF
      when "100111011001" =>  data <= "11111111";  -- 9D9 = FF
      when "100111011010" =>  data <= "11111111";  -- 9DA = FF
      when "100111011011" =>  data <= "11111111";  -- 9DB = FF
      when "100111011100" =>  data <= "11111111";  -- 9DC = FF
      when "100111011101" =>  data <= "11111111";  -- 9DD = FF
      when "100111011110" =>  data <= "11111111";  -- 9DE = FF
      when "100111011111" =>  data <= "11111111";  -- 9DF = FF
      when "100111100000" =>  data <= "11111111";  -- 9E0 = FF
      when "100111100001" =>  data <= "11111111";  -- 9E1 = FF
      when "100111100010" =>  data <= "11111111";  -- 9E2 = FF
      when "100111100011" =>  data <= "11111111";  -- 9E3 = FF
      when "100111100100" =>  data <= "11111111";  -- 9E4 = FF
      when "100111100101" =>  data <= "11111111";  -- 9E5 = FF
      when "100111100110" =>  data <= "11111111";  -- 9E6 = FF
      when "100111100111" =>  data <= "11111111";  -- 9E7 = FF
      when "100111101000" =>  data <= "11111111";  -- 9E8 = FF
      when "100111101001" =>  data <= "11111111";  -- 9E9 = FF
      when "100111101010" =>  data <= "11111111";  -- 9EA = FF
      when "100111101011" =>  data <= "11111111";  -- 9EB = FF
      when "100111101100" =>  data <= "11111111";  -- 9EC = FF
      when "100111101101" =>  data <= "11111111";  -- 9ED = FF
      when "100111101110" =>  data <= "11111111";  -- 9EE = FF
      when "100111101111" =>  data <= "11111111";  -- 9EF = FF
      when "100111110000" =>  data <= "11111111";  -- 9F0 = FF
      when "100111110001" =>  data <= "11111111";  -- 9F1 = FF
      when "100111110010" =>  data <= "11111111";  -- 9F2 = FF
      when "100111110011" =>  data <= "11111111";  -- 9F3 = FF
      when "100111110100" =>  data <= "11111111";  -- 9F4 = FF
      when "100111110101" =>  data <= "11111111";  -- 9F5 = FF
      when "100111110110" =>  data <= "11111111";  -- 9F6 = FF
      when "100111110111" =>  data <= "11111111";  -- 9F7 = FF
      when "100111111000" =>  data <= "11111111";  -- 9F8 = FF
      when "100111111001" =>  data <= "11111111";  -- 9F9 = FF
      when "100111111010" =>  data <= "11111111";  -- 9FA = FF
      when "100111111011" =>  data <= "11111111";  -- 9FB = FF
      when "100111111100" =>  data <= "11111111";  -- 9FC = FF
      when "100111111101" =>  data <= "11111111";  -- 9FD = FF
      when "100111111110" =>  data <= "11111111";  -- 9FE = FF
      when "100111111111" =>  data <= "11111111";  -- 9FF = FF
      when "101000000000" =>  data <= "11111111";  -- A00 = FF
      when "101000000001" =>  data <= "11111111";  -- A01 = FF
      when "101000000010" =>  data <= "11111111";  -- A02 = FF
      when "101000000011" =>  data <= "11111111";  -- A03 = FF
      when "101000000100" =>  data <= "11111111";  -- A04 = FF
      when "101000000101" =>  data <= "11111111";  -- A05 = FF
      when "101000000110" =>  data <= "11111111";  -- A06 = FF
      when "101000000111" =>  data <= "11111111";  -- A07 = FF
      when "101000001000" =>  data <= "11111111";  -- A08 = FF
      when "101000001001" =>  data <= "11111111";  -- A09 = FF
      when "101000001010" =>  data <= "11111111";  -- A0A = FF
      when "101000001011" =>  data <= "11111111";  -- A0B = FF
      when "101000001100" =>  data <= "11111111";  -- A0C = FF
      when "101000001101" =>  data <= "11111111";  -- A0D = FF
      when "101000001110" =>  data <= "11111111";  -- A0E = FF
      when "101000001111" =>  data <= "11111111";  -- A0F = FF
      when "101000010000" =>  data <= "11111111";  -- A10 = FF
      when "101000010001" =>  data <= "11111111";  -- A11 = FF
      when "101000010010" =>  data <= "11111111";  -- A12 = FF
      when "101000010011" =>  data <= "11111111";  -- A13 = FF
      when "101000010100" =>  data <= "11111111";  -- A14 = FF
      when "101000010101" =>  data <= "11111111";  -- A15 = FF
      when "101000010110" =>  data <= "11111111";  -- A16 = FF
      when "101000010111" =>  data <= "11111111";  -- A17 = FF
      when "101000011000" =>  data <= "11111111";  -- A18 = FF
      when "101000011001" =>  data <= "11111111";  -- A19 = FF
      when "101000011010" =>  data <= "11111111";  -- A1A = FF
      when "101000011011" =>  data <= "11111111";  -- A1B = FF
      when "101000011100" =>  data <= "11111111";  -- A1C = FF
      when "101000011101" =>  data <= "11111111";  -- A1D = FF
      when "101000011110" =>  data <= "11111111";  -- A1E = FF
      when "101000011111" =>  data <= "11111111";  -- A1F = FF
      when "101000100000" =>  data <= "11111111";  -- A20 = FF
      when "101000100001" =>  data <= "11111111";  -- A21 = FF
      when "101000100010" =>  data <= "11111111";  -- A22 = FF
      when "101000100011" =>  data <= "11111111";  -- A23 = FF
      when "101000100100" =>  data <= "11111111";  -- A24 = FF
      when "101000100101" =>  data <= "11111111";  -- A25 = FF
      when "101000100110" =>  data <= "11111111";  -- A26 = FF
      when "101000100111" =>  data <= "11111111";  -- A27 = FF
      when "101000101000" =>  data <= "11111111";  -- A28 = FF
      when "101000101001" =>  data <= "11111111";  -- A29 = FF
      when "101000101010" =>  data <= "11111111";  -- A2A = FF
      when "101000101011" =>  data <= "11111111";  -- A2B = FF
      when "101000101100" =>  data <= "11111111";  -- A2C = FF
      when "101000101101" =>  data <= "11111111";  -- A2D = FF
      when "101000101110" =>  data <= "11111111";  -- A2E = FF
      when "101000101111" =>  data <= "11111111";  -- A2F = FF
      when "101000110000" =>  data <= "11111111";  -- A30 = FF
      when "101000110001" =>  data <= "11111111";  -- A31 = FF
      when "101000110010" =>  data <= "11111111";  -- A32 = FF
      when "101000110011" =>  data <= "11111111";  -- A33 = FF
      when "101000110100" =>  data <= "11111111";  -- A34 = FF
      when "101000110101" =>  data <= "11111111";  -- A35 = FF
      when "101000110110" =>  data <= "11111111";  -- A36 = FF
      when "101000110111" =>  data <= "11111111";  -- A37 = FF
      when "101000111000" =>  data <= "11111111";  -- A38 = FF
      when "101000111001" =>  data <= "11111111";  -- A39 = FF
      when "101000111010" =>  data <= "11111111";  -- A3A = FF
      when "101000111011" =>  data <= "11111111";  -- A3B = FF
      when "101000111100" =>  data <= "11111111";  -- A3C = FF
      when "101000111101" =>  data <= "11111111";  -- A3D = FF
      when "101000111110" =>  data <= "11111111";  -- A3E = FF
      when "101000111111" =>  data <= "11111111";  -- A3F = FF
      when "101001000000" =>  data <= "11111111";  -- A40 = FF
      when "101001000001" =>  data <= "11111111";  -- A41 = FF
      when "101001000010" =>  data <= "11111111";  -- A42 = FF
      when "101001000011" =>  data <= "11111111";  -- A43 = FF
      when "101001000100" =>  data <= "11111111";  -- A44 = FF
      when "101001000101" =>  data <= "11111111";  -- A45 = FF
      when "101001000110" =>  data <= "11111111";  -- A46 = FF
      when "101001000111" =>  data <= "11111111";  -- A47 = FF
      when "101001001000" =>  data <= "11111111";  -- A48 = FF
      when "101001001001" =>  data <= "11111111";  -- A49 = FF
      when "101001001010" =>  data <= "11111111";  -- A4A = FF
      when "101001001011" =>  data <= "11111111";  -- A4B = FF
      when "101001001100" =>  data <= "11111111";  -- A4C = FF
      when "101001001101" =>  data <= "11111111";  -- A4D = FF
      when "101001001110" =>  data <= "11111111";  -- A4E = FF
      when "101001001111" =>  data <= "11111111";  -- A4F = FF
      when "101001010000" =>  data <= "11111111";  -- A50 = FF
      when "101001010001" =>  data <= "11111111";  -- A51 = FF
      when "101001010010" =>  data <= "11111111";  -- A52 = FF
      when "101001010011" =>  data <= "11111111";  -- A53 = FF
      when "101001010100" =>  data <= "11111111";  -- A54 = FF
      when "101001010101" =>  data <= "11111111";  -- A55 = FF
      when "101001010110" =>  data <= "11111111";  -- A56 = FF
      when "101001010111" =>  data <= "11111111";  -- A57 = FF
      when "101001011000" =>  data <= "11111111";  -- A58 = FF
      when "101001011001" =>  data <= "11111111";  -- A59 = FF
      when "101001011010" =>  data <= "11111111";  -- A5A = FF
      when "101001011011" =>  data <= "11111111";  -- A5B = FF
      when "101001011100" =>  data <= "11111111";  -- A5C = FF
      when "101001011101" =>  data <= "11111111";  -- A5D = FF
      when "101001011110" =>  data <= "11111111";  -- A5E = FF
      when "101001011111" =>  data <= "11111111";  -- A5F = FF
      when "101001100000" =>  data <= "11111111";  -- A60 = FF
      when "101001100001" =>  data <= "11111111";  -- A61 = FF
      when "101001100010" =>  data <= "11111111";  -- A62 = FF
      when "101001100011" =>  data <= "11111111";  -- A63 = FF
      when "101001100100" =>  data <= "11111111";  -- A64 = FF
      when "101001100101" =>  data <= "11111111";  -- A65 = FF
      when "101001100110" =>  data <= "11111111";  -- A66 = FF
      when "101001100111" =>  data <= "11111111";  -- A67 = FF
      when "101001101000" =>  data <= "11111111";  -- A68 = FF
      when "101001101001" =>  data <= "11111111";  -- A69 = FF
      when "101001101010" =>  data <= "11111111";  -- A6A = FF
      when "101001101011" =>  data <= "11111111";  -- A6B = FF
      when "101001101100" =>  data <= "11111111";  -- A6C = FF
      when "101001101101" =>  data <= "11111111";  -- A6D = FF
      when "101001101110" =>  data <= "11111111";  -- A6E = FF
      when "101001101111" =>  data <= "11111111";  -- A6F = FF
      when "101001110000" =>  data <= "11111111";  -- A70 = FF
      when "101001110001" =>  data <= "11111111";  -- A71 = FF
      when "101001110010" =>  data <= "11111111";  -- A72 = FF
      when "101001110011" =>  data <= "11111111";  -- A73 = FF
      when "101001110100" =>  data <= "11111111";  -- A74 = FF
      when "101001110101" =>  data <= "11111111";  -- A75 = FF
      when "101001110110" =>  data <= "11111111";  -- A76 = FF
      when "101001110111" =>  data <= "11111111";  -- A77 = FF
      when "101001111000" =>  data <= "11111111";  -- A78 = FF
      when "101001111001" =>  data <= "11111111";  -- A79 = FF
      when "101001111010" =>  data <= "11111111";  -- A7A = FF
      when "101001111011" =>  data <= "11111111";  -- A7B = FF
      when "101001111100" =>  data <= "11111111";  -- A7C = FF
      when "101001111101" =>  data <= "11111111";  -- A7D = FF
      when "101001111110" =>  data <= "11111111";  -- A7E = FF
      when "101001111111" =>  data <= "11111111";  -- A7F = FF
      when "101010000000" =>  data <= "11111111";  -- A80 = FF
      when "101010000001" =>  data <= "11111111";  -- A81 = FF
      when "101010000010" =>  data <= "11111111";  -- A82 = FF
      when "101010000011" =>  data <= "11111111";  -- A83 = FF
      when "101010000100" =>  data <= "11111111";  -- A84 = FF
      when "101010000101" =>  data <= "11111111";  -- A85 = FF
      when "101010000110" =>  data <= "11111111";  -- A86 = FF
      when "101010000111" =>  data <= "11111111";  -- A87 = FF
      when "101010001000" =>  data <= "11111111";  -- A88 = FF
      when "101010001001" =>  data <= "11111111";  -- A89 = FF
      when "101010001010" =>  data <= "11111111";  -- A8A = FF
      when "101010001011" =>  data <= "11111111";  -- A8B = FF
      when "101010001100" =>  data <= "11111111";  -- A8C = FF
      when "101010001101" =>  data <= "11111111";  -- A8D = FF
      when "101010001110" =>  data <= "11111111";  -- A8E = FF
      when "101010001111" =>  data <= "11111111";  -- A8F = FF
      when "101010010000" =>  data <= "11111111";  -- A90 = FF
      when "101010010001" =>  data <= "11111111";  -- A91 = FF
      when "101010010010" =>  data <= "11111111";  -- A92 = FF
      when "101010010011" =>  data <= "11111111";  -- A93 = FF
      when "101010010100" =>  data <= "11111111";  -- A94 = FF
      when "101010010101" =>  data <= "11111111";  -- A95 = FF
      when "101010010110" =>  data <= "11111111";  -- A96 = FF
      when "101010010111" =>  data <= "11111111";  -- A97 = FF
      when "101010011000" =>  data <= "11111111";  -- A98 = FF
      when "101010011001" =>  data <= "11111111";  -- A99 = FF
      when "101010011010" =>  data <= "11111111";  -- A9A = FF
      when "101010011011" =>  data <= "11111111";  -- A9B = FF
      when "101010011100" =>  data <= "11111111";  -- A9C = FF
      when "101010011101" =>  data <= "11111111";  -- A9D = FF
      when "101010011110" =>  data <= "11111111";  -- A9E = FF
      when "101010011111" =>  data <= "11111111";  -- A9F = FF
      when "101010100000" =>  data <= "11111111";  -- AA0 = FF
      when "101010100001" =>  data <= "11111111";  -- AA1 = FF
      when "101010100010" =>  data <= "11111111";  -- AA2 = FF
      when "101010100011" =>  data <= "11111111";  -- AA3 = FF
      when "101010100100" =>  data <= "11111111";  -- AA4 = FF
      when "101010100101" =>  data <= "11111111";  -- AA5 = FF
      when "101010100110" =>  data <= "11111111";  -- AA6 = FF
      when "101010100111" =>  data <= "11111111";  -- AA7 = FF
      when "101010101000" =>  data <= "11111111";  -- AA8 = FF
      when "101010101001" =>  data <= "11111111";  -- AA9 = FF
      when "101010101010" =>  data <= "11111111";  -- AAA = FF
      when "101010101011" =>  data <= "11111111";  -- AAB = FF
      when "101010101100" =>  data <= "11111111";  -- AAC = FF
      when "101010101101" =>  data <= "11111111";  -- AAD = FF
      when "101010101110" =>  data <= "11111111";  -- AAE = FF
      when "101010101111" =>  data <= "11111111";  -- AAF = FF
      when "101010110000" =>  data <= "11111111";  -- AB0 = FF
      when "101010110001" =>  data <= "11111111";  -- AB1 = FF
      when "101010110010" =>  data <= "11111111";  -- AB2 = FF
      when "101010110011" =>  data <= "11111111";  -- AB3 = FF
      when "101010110100" =>  data <= "11111111";  -- AB4 = FF
      when "101010110101" =>  data <= "11111111";  -- AB5 = FF
      when "101010110110" =>  data <= "11111111";  -- AB6 = FF
      when "101010110111" =>  data <= "11111111";  -- AB7 = FF
      when "101010111000" =>  data <= "11111111";  -- AB8 = FF
      when "101010111001" =>  data <= "11111111";  -- AB9 = FF
      when "101010111010" =>  data <= "11111111";  -- ABA = FF
      when "101010111011" =>  data <= "11111111";  -- ABB = FF
      when "101010111100" =>  data <= "11111111";  -- ABC = FF
      when "101010111101" =>  data <= "11111111";  -- ABD = FF
      when "101010111110" =>  data <= "11111111";  -- ABE = FF
      when "101010111111" =>  data <= "11111111";  -- ABF = FF
      when "101011000000" =>  data <= "11111111";  -- AC0 = FF
      when "101011000001" =>  data <= "11111111";  -- AC1 = FF
      when "101011000010" =>  data <= "11111111";  -- AC2 = FF
      when "101011000011" =>  data <= "11111111";  -- AC3 = FF
      when "101011000100" =>  data <= "11111111";  -- AC4 = FF
      when "101011000101" =>  data <= "11111111";  -- AC5 = FF
      when "101011000110" =>  data <= "11111111";  -- AC6 = FF
      when "101011000111" =>  data <= "11111111";  -- AC7 = FF
      when "101011001000" =>  data <= "11111111";  -- AC8 = FF
      when "101011001001" =>  data <= "11111111";  -- AC9 = FF
      when "101011001010" =>  data <= "11111111";  -- ACA = FF
      when "101011001011" =>  data <= "11111111";  -- ACB = FF
      when "101011001100" =>  data <= "11111111";  -- ACC = FF
      when "101011001101" =>  data <= "11111111";  -- ACD = FF
      when "101011001110" =>  data <= "11111111";  -- ACE = FF
      when "101011001111" =>  data <= "11111111";  -- ACF = FF
      when "101011010000" =>  data <= "11111111";  -- AD0 = FF
      when "101011010001" =>  data <= "11111111";  -- AD1 = FF
      when "101011010010" =>  data <= "11111111";  -- AD2 = FF
      when "101011010011" =>  data <= "11111111";  -- AD3 = FF
      when "101011010100" =>  data <= "11111111";  -- AD4 = FF
      when "101011010101" =>  data <= "11111111";  -- AD5 = FF
      when "101011010110" =>  data <= "11111111";  -- AD6 = FF
      when "101011010111" =>  data <= "11111111";  -- AD7 = FF
      when "101011011000" =>  data <= "11111111";  -- AD8 = FF
      when "101011011001" =>  data <= "11111111";  -- AD9 = FF
      when "101011011010" =>  data <= "11111111";  -- ADA = FF
      when "101011011011" =>  data <= "11111111";  -- ADB = FF
      when "101011011100" =>  data <= "11111111";  -- ADC = FF
      when "101011011101" =>  data <= "11111111";  -- ADD = FF
      when "101011011110" =>  data <= "11111111";  -- ADE = FF
      when "101011011111" =>  data <= "11111111";  -- ADF = FF
      when "101011100000" =>  data <= "11111111";  -- AE0 = FF
      when "101011100001" =>  data <= "11111111";  -- AE1 = FF
      when "101011100010" =>  data <= "11111111";  -- AE2 = FF
      when "101011100011" =>  data <= "11111111";  -- AE3 = FF
      when "101011100100" =>  data <= "11111111";  -- AE4 = FF
      when "101011100101" =>  data <= "11111111";  -- AE5 = FF
      when "101011100110" =>  data <= "11111111";  -- AE6 = FF
      when "101011100111" =>  data <= "11111111";  -- AE7 = FF
      when "101011101000" =>  data <= "11111111";  -- AE8 = FF
      when "101011101001" =>  data <= "11111111";  -- AE9 = FF
      when "101011101010" =>  data <= "11111111";  -- AEA = FF
      when "101011101011" =>  data <= "11111111";  -- AEB = FF
      when "101011101100" =>  data <= "11111111";  -- AEC = FF
      when "101011101101" =>  data <= "11111111";  -- AED = FF
      when "101011101110" =>  data <= "11111111";  -- AEE = FF
      when "101011101111" =>  data <= "11111111";  -- AEF = FF
      when "101011110000" =>  data <= "11111111";  -- AF0 = FF
      when "101011110001" =>  data <= "11111111";  -- AF1 = FF
      when "101011110010" =>  data <= "11111111";  -- AF2 = FF
      when "101011110011" =>  data <= "11111111";  -- AF3 = FF
      when "101011110100" =>  data <= "11111111";  -- AF4 = FF
      when "101011110101" =>  data <= "11111111";  -- AF5 = FF
      when "101011110110" =>  data <= "11111111";  -- AF6 = FF
      when "101011110111" =>  data <= "11111111";  -- AF7 = FF
      when "101011111000" =>  data <= "11111111";  -- AF8 = FF
      when "101011111001" =>  data <= "11111111";  -- AF9 = FF
      when "101011111010" =>  data <= "11111111";  -- AFA = FF
      when "101011111011" =>  data <= "11111111";  -- AFB = FF
      when "101011111100" =>  data <= "11111111";  -- AFC = FF
      when "101011111101" =>  data <= "11111111";  -- AFD = FF
      when "101011111110" =>  data <= "11111111";  -- AFE = FF
      when "101011111111" =>  data <= "11111111";  -- AFF = FF
      when "101100000000" =>  data <= "11111111";  -- B00 = FF
      when "101100000001" =>  data <= "11111111";  -- B01 = FF
      when "101100000010" =>  data <= "11111111";  -- B02 = FF
      when "101100000011" =>  data <= "11111111";  -- B03 = FF
      when "101100000100" =>  data <= "11111111";  -- B04 = FF
      when "101100000101" =>  data <= "11111111";  -- B05 = FF
      when "101100000110" =>  data <= "11111111";  -- B06 = FF
      when "101100000111" =>  data <= "11111111";  -- B07 = FF
      when "101100001000" =>  data <= "11111111";  -- B08 = FF
      when "101100001001" =>  data <= "11111111";  -- B09 = FF
      when "101100001010" =>  data <= "11111111";  -- B0A = FF
      when "101100001011" =>  data <= "11111111";  -- B0B = FF
      when "101100001100" =>  data <= "11111111";  -- B0C = FF
      when "101100001101" =>  data <= "11111111";  -- B0D = FF
      when "101100001110" =>  data <= "11111111";  -- B0E = FF
      when "101100001111" =>  data <= "11111111";  -- B0F = FF
      when "101100010000" =>  data <= "11111111";  -- B10 = FF
      when "101100010001" =>  data <= "11111111";  -- B11 = FF
      when "101100010010" =>  data <= "11111111";  -- B12 = FF
      when "101100010011" =>  data <= "11111111";  -- B13 = FF
      when "101100010100" =>  data <= "11111111";  -- B14 = FF
      when "101100010101" =>  data <= "11111111";  -- B15 = FF
      when "101100010110" =>  data <= "11111111";  -- B16 = FF
      when "101100010111" =>  data <= "11111111";  -- B17 = FF
      when "101100011000" =>  data <= "11111111";  -- B18 = FF
      when "101100011001" =>  data <= "11111111";  -- B19 = FF
      when "101100011010" =>  data <= "11111111";  -- B1A = FF
      when "101100011011" =>  data <= "11111111";  -- B1B = FF
      when "101100011100" =>  data <= "11111111";  -- B1C = FF
      when "101100011101" =>  data <= "11111111";  -- B1D = FF
      when "101100011110" =>  data <= "11111111";  -- B1E = FF
      when "101100011111" =>  data <= "11111111";  -- B1F = FF
      when "101100100000" =>  data <= "11111111";  -- B20 = FF
      when "101100100001" =>  data <= "11111111";  -- B21 = FF
      when "101100100010" =>  data <= "11111111";  -- B22 = FF
      when "101100100011" =>  data <= "11111111";  -- B23 = FF
      when "101100100100" =>  data <= "11111111";  -- B24 = FF
      when "101100100101" =>  data <= "11111111";  -- B25 = FF
      when "101100100110" =>  data <= "11111111";  -- B26 = FF
      when "101100100111" =>  data <= "11111111";  -- B27 = FF
      when "101100101000" =>  data <= "11111111";  -- B28 = FF
      when "101100101001" =>  data <= "11111111";  -- B29 = FF
      when "101100101010" =>  data <= "11111111";  -- B2A = FF
      when "101100101011" =>  data <= "11111111";  -- B2B = FF
      when "101100101100" =>  data <= "11111111";  -- B2C = FF
      when "101100101101" =>  data <= "11111111";  -- B2D = FF
      when "101100101110" =>  data <= "11111111";  -- B2E = FF
      when "101100101111" =>  data <= "11111111";  -- B2F = FF
      when "101100110000" =>  data <= "11111111";  -- B30 = FF
      when "101100110001" =>  data <= "11111111";  -- B31 = FF
      when "101100110010" =>  data <= "11111111";  -- B32 = FF
      when "101100110011" =>  data <= "11111111";  -- B33 = FF
      when "101100110100" =>  data <= "11111111";  -- B34 = FF
      when "101100110101" =>  data <= "11111111";  -- B35 = FF
      when "101100110110" =>  data <= "11111111";  -- B36 = FF
      when "101100110111" =>  data <= "11111111";  -- B37 = FF
      when "101100111000" =>  data <= "11111111";  -- B38 = FF
      when "101100111001" =>  data <= "11111111";  -- B39 = FF
      when "101100111010" =>  data <= "11111111";  -- B3A = FF
      when "101100111011" =>  data <= "11111111";  -- B3B = FF
      when "101100111100" =>  data <= "11111111";  -- B3C = FF
      when "101100111101" =>  data <= "11111111";  -- B3D = FF
      when "101100111110" =>  data <= "11111111";  -- B3E = FF
      when "101100111111" =>  data <= "11111111";  -- B3F = FF
      when "101101000000" =>  data <= "11111111";  -- B40 = FF
      when "101101000001" =>  data <= "11111111";  -- B41 = FF
      when "101101000010" =>  data <= "11111111";  -- B42 = FF
      when "101101000011" =>  data <= "11111111";  -- B43 = FF
      when "101101000100" =>  data <= "11111111";  -- B44 = FF
      when "101101000101" =>  data <= "11111111";  -- B45 = FF
      when "101101000110" =>  data <= "11111111";  -- B46 = FF
      when "101101000111" =>  data <= "11111111";  -- B47 = FF
      when "101101001000" =>  data <= "11111111";  -- B48 = FF
      when "101101001001" =>  data <= "11111111";  -- B49 = FF
      when "101101001010" =>  data <= "11111111";  -- B4A = FF
      when "101101001011" =>  data <= "11111111";  -- B4B = FF
      when "101101001100" =>  data <= "11111111";  -- B4C = FF
      when "101101001101" =>  data <= "11111111";  -- B4D = FF
      when "101101001110" =>  data <= "11111111";  -- B4E = FF
      when "101101001111" =>  data <= "11111111";  -- B4F = FF
      when "101101010000" =>  data <= "11111111";  -- B50 = FF
      when "101101010001" =>  data <= "11111111";  -- B51 = FF
      when "101101010010" =>  data <= "11111111";  -- B52 = FF
      when "101101010011" =>  data <= "11111111";  -- B53 = FF
      when "101101010100" =>  data <= "11111111";  -- B54 = FF
      when "101101010101" =>  data <= "11111111";  -- B55 = FF
      when "101101010110" =>  data <= "11111111";  -- B56 = FF
      when "101101010111" =>  data <= "11111111";  -- B57 = FF
      when "101101011000" =>  data <= "11111111";  -- B58 = FF
      when "101101011001" =>  data <= "11111111";  -- B59 = FF
      when "101101011010" =>  data <= "11111111";  -- B5A = FF
      when "101101011011" =>  data <= "11111111";  -- B5B = FF
      when "101101011100" =>  data <= "11111111";  -- B5C = FF
      when "101101011101" =>  data <= "11111111";  -- B5D = FF
      when "101101011110" =>  data <= "11111111";  -- B5E = FF
      when "101101011111" =>  data <= "11111111";  -- B5F = FF
      when "101101100000" =>  data <= "11111111";  -- B60 = FF
      when "101101100001" =>  data <= "11111111";  -- B61 = FF
      when "101101100010" =>  data <= "11111111";  -- B62 = FF
      when "101101100011" =>  data <= "11111111";  -- B63 = FF
      when "101101100100" =>  data <= "11111111";  -- B64 = FF
      when "101101100101" =>  data <= "11111111";  -- B65 = FF
      when "101101100110" =>  data <= "11111111";  -- B66 = FF
      when "101101100111" =>  data <= "11111111";  -- B67 = FF
      when "101101101000" =>  data <= "11111111";  -- B68 = FF
      when "101101101001" =>  data <= "11111111";  -- B69 = FF
      when "101101101010" =>  data <= "11111111";  -- B6A = FF
      when "101101101011" =>  data <= "11111111";  -- B6B = FF
      when "101101101100" =>  data <= "11111111";  -- B6C = FF
      when "101101101101" =>  data <= "11111111";  -- B6D = FF
      when "101101101110" =>  data <= "11111111";  -- B6E = FF
      when "101101101111" =>  data <= "11111111";  -- B6F = FF
      when "101101110000" =>  data <= "11111111";  -- B70 = FF
      when "101101110001" =>  data <= "11111111";  -- B71 = FF
      when "101101110010" =>  data <= "11111111";  -- B72 = FF
      when "101101110011" =>  data <= "11111111";  -- B73 = FF
      when "101101110100" =>  data <= "11111111";  -- B74 = FF
      when "101101110101" =>  data <= "11111111";  -- B75 = FF
      when "101101110110" =>  data <= "11111111";  -- B76 = FF
      when "101101110111" =>  data <= "11111111";  -- B77 = FF
      when "101101111000" =>  data <= "11111111";  -- B78 = FF
      when "101101111001" =>  data <= "11111111";  -- B79 = FF
      when "101101111010" =>  data <= "11111111";  -- B7A = FF
      when "101101111011" =>  data <= "11111111";  -- B7B = FF
      when "101101111100" =>  data <= "11111111";  -- B7C = FF
      when "101101111101" =>  data <= "11111111";  -- B7D = FF
      when "101101111110" =>  data <= "11111111";  -- B7E = FF
      when "101101111111" =>  data <= "11111111";  -- B7F = FF
      when "101110000000" =>  data <= "11111111";  -- B80 = FF
      when "101110000001" =>  data <= "11111111";  -- B81 = FF
      when "101110000010" =>  data <= "11111111";  -- B82 = FF
      when "101110000011" =>  data <= "11111111";  -- B83 = FF
      when "101110000100" =>  data <= "11111111";  -- B84 = FF
      when "101110000101" =>  data <= "11111111";  -- B85 = FF
      when "101110000110" =>  data <= "11111111";  -- B86 = FF
      when "101110000111" =>  data <= "11111111";  -- B87 = FF
      when "101110001000" =>  data <= "11111111";  -- B88 = FF
      when "101110001001" =>  data <= "11111111";  -- B89 = FF
      when "101110001010" =>  data <= "11111111";  -- B8A = FF
      when "101110001011" =>  data <= "11111111";  -- B8B = FF
      when "101110001100" =>  data <= "11111111";  -- B8C = FF
      when "101110001101" =>  data <= "11111111";  -- B8D = FF
      when "101110001110" =>  data <= "11111111";  -- B8E = FF
      when "101110001111" =>  data <= "11111111";  -- B8F = FF
      when "101110010000" =>  data <= "11111111";  -- B90 = FF
      when "101110010001" =>  data <= "11111111";  -- B91 = FF
      when "101110010010" =>  data <= "11111111";  -- B92 = FF
      when "101110010011" =>  data <= "11111111";  -- B93 = FF
      when "101110010100" =>  data <= "11111111";  -- B94 = FF
      when "101110010101" =>  data <= "11111111";  -- B95 = FF
      when "101110010110" =>  data <= "11111111";  -- B96 = FF
      when "101110010111" =>  data <= "11111111";  -- B97 = FF
      when "101110011000" =>  data <= "11111111";  -- B98 = FF
      when "101110011001" =>  data <= "11111111";  -- B99 = FF
      when "101110011010" =>  data <= "11111111";  -- B9A = FF
      when "101110011011" =>  data <= "11111111";  -- B9B = FF
      when "101110011100" =>  data <= "11111111";  -- B9C = FF
      when "101110011101" =>  data <= "11111111";  -- B9D = FF
      when "101110011110" =>  data <= "11111111";  -- B9E = FF
      when "101110011111" =>  data <= "11111111";  -- B9F = FF
      when "101110100000" =>  data <= "11111111";  -- BA0 = FF
      when "101110100001" =>  data <= "11111111";  -- BA1 = FF
      when "101110100010" =>  data <= "11111111";  -- BA2 = FF
      when "101110100011" =>  data <= "11111111";  -- BA3 = FF
      when "101110100100" =>  data <= "11111111";  -- BA4 = FF
      when "101110100101" =>  data <= "11111111";  -- BA5 = FF
      when "101110100110" =>  data <= "11111111";  -- BA6 = FF
      when "101110100111" =>  data <= "11111111";  -- BA7 = FF
      when "101110101000" =>  data <= "11111111";  -- BA8 = FF
      when "101110101001" =>  data <= "11111111";  -- BA9 = FF
      when "101110101010" =>  data <= "11111111";  -- BAA = FF
      when "101110101011" =>  data <= "11111111";  -- BAB = FF
      when "101110101100" =>  data <= "11111111";  -- BAC = FF
      when "101110101101" =>  data <= "11111111";  -- BAD = FF
      when "101110101110" =>  data <= "11111111";  -- BAE = FF
      when "101110101111" =>  data <= "11111111";  -- BAF = FF
      when "101110110000" =>  data <= "11111111";  -- BB0 = FF
      when "101110110001" =>  data <= "11111111";  -- BB1 = FF
      when "101110110010" =>  data <= "11111111";  -- BB2 = FF
      when "101110110011" =>  data <= "11111111";  -- BB3 = FF
      when "101110110100" =>  data <= "11111111";  -- BB4 = FF
      when "101110110101" =>  data <= "11111111";  -- BB5 = FF
      when "101110110110" =>  data <= "11111111";  -- BB6 = FF
      when "101110110111" =>  data <= "11111111";  -- BB7 = FF
      when "101110111000" =>  data <= "11111111";  -- BB8 = FF
      when "101110111001" =>  data <= "11111111";  -- BB9 = FF
      when "101110111010" =>  data <= "11111111";  -- BBA = FF
      when "101110111011" =>  data <= "11111111";  -- BBB = FF
      when "101110111100" =>  data <= "11111111";  -- BBC = FF
      when "101110111101" =>  data <= "11111111";  -- BBD = FF
      when "101110111110" =>  data <= "11111111";  -- BBE = FF
      when "101110111111" =>  data <= "11111111";  -- BBF = FF
      when "101111000000" =>  data <= "11111111";  -- BC0 = FF
      when "101111000001" =>  data <= "11111111";  -- BC1 = FF
      when "101111000010" =>  data <= "11111111";  -- BC2 = FF
      when "101111000011" =>  data <= "11111111";  -- BC3 = FF
      when "101111000100" =>  data <= "11111111";  -- BC4 = FF
      when "101111000101" =>  data <= "11111111";  -- BC5 = FF
      when "101111000110" =>  data <= "11111111";  -- BC6 = FF
      when "101111000111" =>  data <= "11111111";  -- BC7 = FF
      when "101111001000" =>  data <= "11111111";  -- BC8 = FF
      when "101111001001" =>  data <= "11111111";  -- BC9 = FF
      when "101111001010" =>  data <= "11111111";  -- BCA = FF
      when "101111001011" =>  data <= "11111111";  -- BCB = FF
      when "101111001100" =>  data <= "11111111";  -- BCC = FF
      when "101111001101" =>  data <= "11111111";  -- BCD = FF
      when "101111001110" =>  data <= "11111111";  -- BCE = FF
      when "101111001111" =>  data <= "11111111";  -- BCF = FF
      when "101111010000" =>  data <= "11111111";  -- BD0 = FF
      when "101111010001" =>  data <= "11111111";  -- BD1 = FF
      when "101111010010" =>  data <= "11111111";  -- BD2 = FF
      when "101111010011" =>  data <= "11111111";  -- BD3 = FF
      when "101111010100" =>  data <= "11111111";  -- BD4 = FF
      when "101111010101" =>  data <= "11111111";  -- BD5 = FF
      when "101111010110" =>  data <= "11111111";  -- BD6 = FF
      when "101111010111" =>  data <= "11111111";  -- BD7 = FF
      when "101111011000" =>  data <= "11111111";  -- BD8 = FF
      when "101111011001" =>  data <= "11111111";  -- BD9 = FF
      when "101111011010" =>  data <= "11111111";  -- BDA = FF
      when "101111011011" =>  data <= "11111111";  -- BDB = FF
      when "101111011100" =>  data <= "11111111";  -- BDC = FF
      when "101111011101" =>  data <= "11111111";  -- BDD = FF
      when "101111011110" =>  data <= "11111111";  -- BDE = FF
      when "101111011111" =>  data <= "11111111";  -- BDF = FF
      when "101111100000" =>  data <= "11111111";  -- BE0 = FF
      when "101111100001" =>  data <= "11111111";  -- BE1 = FF
      when "101111100010" =>  data <= "11111111";  -- BE2 = FF
      when "101111100011" =>  data <= "11111111";  -- BE3 = FF
      when "101111100100" =>  data <= "11111111";  -- BE4 = FF
      when "101111100101" =>  data <= "11111111";  -- BE5 = FF
      when "101111100110" =>  data <= "11111111";  -- BE6 = FF
      when "101111100111" =>  data <= "11111111";  -- BE7 = FF
      when "101111101000" =>  data <= "11111111";  -- BE8 = FF
      when "101111101001" =>  data <= "11111111";  -- BE9 = FF
      when "101111101010" =>  data <= "11111111";  -- BEA = FF
      when "101111101011" =>  data <= "11111111";  -- BEB = FF
      when "101111101100" =>  data <= "11111111";  -- BEC = FF
      when "101111101101" =>  data <= "11111111";  -- BED = FF
      when "101111101110" =>  data <= "11111111";  -- BEE = FF
      when "101111101111" =>  data <= "11111111";  -- BEF = FF
      when "101111110000" =>  data <= "11111111";  -- BF0 = FF
      when "101111110001" =>  data <= "11111111";  -- BF1 = FF
      when "101111110010" =>  data <= "11111111";  -- BF2 = FF
      when "101111110011" =>  data <= "11111111";  -- BF3 = FF
      when "101111110100" =>  data <= "11111111";  -- BF4 = FF
      when "101111110101" =>  data <= "11111111";  -- BF5 = FF
      when "101111110110" =>  data <= "11111111";  -- BF6 = FF
      when "101111110111" =>  data <= "11111111";  -- BF7 = FF
      when "101111111000" =>  data <= "11111111";  -- BF8 = FF
      when "101111111001" =>  data <= "11111111";  -- BF9 = FF
      when "101111111010" =>  data <= "11111111";  -- BFA = FF
      when "101111111011" =>  data <= "11111111";  -- BFB = FF
      when "101111111100" =>  data <= "11111111";  -- BFC = FF
      when "101111111101" =>  data <= "11111111";  -- BFD = FF
      when "101111111110" =>  data <= "11111111";  -- BFE = FF
      when "101111111111" =>  data <= "11111111";  -- BFF = FF
      when "110000000000" =>  data <= "11111111";  -- C00 = FF
      when "110000000001" =>  data <= "11111111";  -- C01 = FF
      when "110000000010" =>  data <= "11111111";  -- C02 = FF
      when "110000000011" =>  data <= "11111111";  -- C03 = FF
      when "110000000100" =>  data <= "11111111";  -- C04 = FF
      when "110000000101" =>  data <= "11111111";  -- C05 = FF
      when "110000000110" =>  data <= "11111111";  -- C06 = FF
      when "110000000111" =>  data <= "11111111";  -- C07 = FF
      when "110000001000" =>  data <= "11111111";  -- C08 = FF
      when "110000001001" =>  data <= "11111111";  -- C09 = FF
      when "110000001010" =>  data <= "11111111";  -- C0A = FF
      when "110000001011" =>  data <= "11111111";  -- C0B = FF
      when "110000001100" =>  data <= "11111111";  -- C0C = FF
      when "110000001101" =>  data <= "11111111";  -- C0D = FF
      when "110000001110" =>  data <= "11111111";  -- C0E = FF
      when "110000001111" =>  data <= "11111111";  -- C0F = FF
      when "110000010000" =>  data <= "11111111";  -- C10 = FF
      when "110000010001" =>  data <= "11111111";  -- C11 = FF
      when "110000010010" =>  data <= "11111111";  -- C12 = FF
      when "110000010011" =>  data <= "11111111";  -- C13 = FF
      when "110000010100" =>  data <= "11111111";  -- C14 = FF
      when "110000010101" =>  data <= "11111111";  -- C15 = FF
      when "110000010110" =>  data <= "11111111";  -- C16 = FF
      when "110000010111" =>  data <= "11111111";  -- C17 = FF
      when "110000011000" =>  data <= "11111111";  -- C18 = FF
      when "110000011001" =>  data <= "11111111";  -- C19 = FF
      when "110000011010" =>  data <= "11111111";  -- C1A = FF
      when "110000011011" =>  data <= "11111111";  -- C1B = FF
      when "110000011100" =>  data <= "11111111";  -- C1C = FF
      when "110000011101" =>  data <= "11111111";  -- C1D = FF
      when "110000011110" =>  data <= "11111111";  -- C1E = FF
      when "110000011111" =>  data <= "11111111";  -- C1F = FF
      when "110000100000" =>  data <= "11111111";  -- C20 = FF
      when "110000100001" =>  data <= "11111111";  -- C21 = FF
      when "110000100010" =>  data <= "11111111";  -- C22 = FF
      when "110000100011" =>  data <= "11111111";  -- C23 = FF
      when "110000100100" =>  data <= "11111111";  -- C24 = FF
      when "110000100101" =>  data <= "11111111";  -- C25 = FF
      when "110000100110" =>  data <= "11111111";  -- C26 = FF
      when "110000100111" =>  data <= "11111111";  -- C27 = FF
      when "110000101000" =>  data <= "11111111";  -- C28 = FF
      when "110000101001" =>  data <= "11111111";  -- C29 = FF
      when "110000101010" =>  data <= "11111111";  -- C2A = FF
      when "110000101011" =>  data <= "11111111";  -- C2B = FF
      when "110000101100" =>  data <= "11111111";  -- C2C = FF
      when "110000101101" =>  data <= "11111111";  -- C2D = FF
      when "110000101110" =>  data <= "11111111";  -- C2E = FF
      when "110000101111" =>  data <= "11111111";  -- C2F = FF
      when "110000110000" =>  data <= "11111111";  -- C30 = FF
      when "110000110001" =>  data <= "11111111";  -- C31 = FF
      when "110000110010" =>  data <= "11111111";  -- C32 = FF
      when "110000110011" =>  data <= "11111111";  -- C33 = FF
      when "110000110100" =>  data <= "11111111";  -- C34 = FF
      when "110000110101" =>  data <= "11111111";  -- C35 = FF
      when "110000110110" =>  data <= "11111111";  -- C36 = FF
      when "110000110111" =>  data <= "11111111";  -- C37 = FF
      when "110000111000" =>  data <= "11111111";  -- C38 = FF
      when "110000111001" =>  data <= "11111111";  -- C39 = FF
      when "110000111010" =>  data <= "11111111";  -- C3A = FF
      when "110000111011" =>  data <= "11111111";  -- C3B = FF
      when "110000111100" =>  data <= "11111111";  -- C3C = FF
      when "110000111101" =>  data <= "11111111";  -- C3D = FF
      when "110000111110" =>  data <= "11111111";  -- C3E = FF
      when "110000111111" =>  data <= "11111111";  -- C3F = FF
      when "110001000000" =>  data <= "11111111";  -- C40 = FF
      when "110001000001" =>  data <= "11111111";  -- C41 = FF
      when "110001000010" =>  data <= "11111111";  -- C42 = FF
      when "110001000011" =>  data <= "11111111";  -- C43 = FF
      when "110001000100" =>  data <= "11111111";  -- C44 = FF
      when "110001000101" =>  data <= "11111111";  -- C45 = FF
      when "110001000110" =>  data <= "11111111";  -- C46 = FF
      when "110001000111" =>  data <= "11111111";  -- C47 = FF
      when "110001001000" =>  data <= "11111111";  -- C48 = FF
      when "110001001001" =>  data <= "11111111";  -- C49 = FF
      when "110001001010" =>  data <= "11111111";  -- C4A = FF
      when "110001001011" =>  data <= "11111111";  -- C4B = FF
      when "110001001100" =>  data <= "11111111";  -- C4C = FF
      when "110001001101" =>  data <= "11111111";  -- C4D = FF
      when "110001001110" =>  data <= "11111111";  -- C4E = FF
      when "110001001111" =>  data <= "11111111";  -- C4F = FF
      when "110001010000" =>  data <= "11111111";  -- C50 = FF
      when "110001010001" =>  data <= "11111111";  -- C51 = FF
      when "110001010010" =>  data <= "11111111";  -- C52 = FF
      when "110001010011" =>  data <= "11111111";  -- C53 = FF
      when "110001010100" =>  data <= "11111111";  -- C54 = FF
      when "110001010101" =>  data <= "11111111";  -- C55 = FF
      when "110001010110" =>  data <= "11111111";  -- C56 = FF
      when "110001010111" =>  data <= "11111111";  -- C57 = FF
      when "110001011000" =>  data <= "11111111";  -- C58 = FF
      when "110001011001" =>  data <= "11111111";  -- C59 = FF
      when "110001011010" =>  data <= "11111111";  -- C5A = FF
      when "110001011011" =>  data <= "11111111";  -- C5B = FF
      when "110001011100" =>  data <= "11111111";  -- C5C = FF
      when "110001011101" =>  data <= "11111111";  -- C5D = FF
      when "110001011110" =>  data <= "11111111";  -- C5E = FF
      when "110001011111" =>  data <= "11111111";  -- C5F = FF
      when "110001100000" =>  data <= "11111111";  -- C60 = FF
      when "110001100001" =>  data <= "11111111";  -- C61 = FF
      when "110001100010" =>  data <= "11111111";  -- C62 = FF
      when "110001100011" =>  data <= "11111111";  -- C63 = FF
      when "110001100100" =>  data <= "11111111";  -- C64 = FF
      when "110001100101" =>  data <= "11111111";  -- C65 = FF
      when "110001100110" =>  data <= "11111111";  -- C66 = FF
      when "110001100111" =>  data <= "11111111";  -- C67 = FF
      when "110001101000" =>  data <= "11111111";  -- C68 = FF
      when "110001101001" =>  data <= "11111111";  -- C69 = FF
      when "110001101010" =>  data <= "11111111";  -- C6A = FF
      when "110001101011" =>  data <= "11111111";  -- C6B = FF
      when "110001101100" =>  data <= "11111111";  -- C6C = FF
      when "110001101101" =>  data <= "11111111";  -- C6D = FF
      when "110001101110" =>  data <= "11111111";  -- C6E = FF
      when "110001101111" =>  data <= "11111111";  -- C6F = FF
      when "110001110000" =>  data <= "11111111";  -- C70 = FF
      when "110001110001" =>  data <= "11111111";  -- C71 = FF
      when "110001110010" =>  data <= "11111111";  -- C72 = FF
      when "110001110011" =>  data <= "11111111";  -- C73 = FF
      when "110001110100" =>  data <= "11111111";  -- C74 = FF
      when "110001110101" =>  data <= "11111111";  -- C75 = FF
      when "110001110110" =>  data <= "11111111";  -- C76 = FF
      when "110001110111" =>  data <= "11111111";  -- C77 = FF
      when "110001111000" =>  data <= "11111111";  -- C78 = FF
      when "110001111001" =>  data <= "11111111";  -- C79 = FF
      when "110001111010" =>  data <= "11111111";  -- C7A = FF
      when "110001111011" =>  data <= "11111111";  -- C7B = FF
      when "110001111100" =>  data <= "11111111";  -- C7C = FF
      when "110001111101" =>  data <= "11111111";  -- C7D = FF
      when "110001111110" =>  data <= "11111111";  -- C7E = FF
      when "110001111111" =>  data <= "11111111";  -- C7F = FF
      when "110010000000" =>  data <= "11111111";  -- C80 = FF
      when "110010000001" =>  data <= "11111111";  -- C81 = FF
      when "110010000010" =>  data <= "11111111";  -- C82 = FF
      when "110010000011" =>  data <= "11111111";  -- C83 = FF
      when "110010000100" =>  data <= "11111111";  -- C84 = FF
      when "110010000101" =>  data <= "11111111";  -- C85 = FF
      when "110010000110" =>  data <= "11111111";  -- C86 = FF
      when "110010000111" =>  data <= "11111111";  -- C87 = FF
      when "110010001000" =>  data <= "11111111";  -- C88 = FF
      when "110010001001" =>  data <= "11111111";  -- C89 = FF
      when "110010001010" =>  data <= "11111111";  -- C8A = FF
      when "110010001011" =>  data <= "11111111";  -- C8B = FF
      when "110010001100" =>  data <= "11111111";  -- C8C = FF
      when "110010001101" =>  data <= "11111111";  -- C8D = FF
      when "110010001110" =>  data <= "11111111";  -- C8E = FF
      when "110010001111" =>  data <= "11111111";  -- C8F = FF
      when "110010010000" =>  data <= "11111111";  -- C90 = FF
      when "110010010001" =>  data <= "11111111";  -- C91 = FF
      when "110010010010" =>  data <= "11111111";  -- C92 = FF
      when "110010010011" =>  data <= "11111111";  -- C93 = FF
      when "110010010100" =>  data <= "11111111";  -- C94 = FF
      when "110010010101" =>  data <= "11111111";  -- C95 = FF
      when "110010010110" =>  data <= "11111111";  -- C96 = FF
      when "110010010111" =>  data <= "11111111";  -- C97 = FF
      when "110010011000" =>  data <= "11111111";  -- C98 = FF
      when "110010011001" =>  data <= "11111111";  -- C99 = FF
      when "110010011010" =>  data <= "11111111";  -- C9A = FF
      when "110010011011" =>  data <= "11111111";  -- C9B = FF
      when "110010011100" =>  data <= "11111111";  -- C9C = FF
      when "110010011101" =>  data <= "11111111";  -- C9D = FF
      when "110010011110" =>  data <= "11111111";  -- C9E = FF
      when "110010011111" =>  data <= "11111111";  -- C9F = FF
      when "110010100000" =>  data <= "11111111";  -- CA0 = FF
      when "110010100001" =>  data <= "11111111";  -- CA1 = FF
      when "110010100010" =>  data <= "11111111";  -- CA2 = FF
      when "110010100011" =>  data <= "11111111";  -- CA3 = FF
      when "110010100100" =>  data <= "11111111";  -- CA4 = FF
      when "110010100101" =>  data <= "11111111";  -- CA5 = FF
      when "110010100110" =>  data <= "11111111";  -- CA6 = FF
      when "110010100111" =>  data <= "11111111";  -- CA7 = FF
      when "110010101000" =>  data <= "11111111";  -- CA8 = FF
      when "110010101001" =>  data <= "11111111";  -- CA9 = FF
      when "110010101010" =>  data <= "11111111";  -- CAA = FF
      when "110010101011" =>  data <= "11111111";  -- CAB = FF
      when "110010101100" =>  data <= "11111111";  -- CAC = FF
      when "110010101101" =>  data <= "11111111";  -- CAD = FF
      when "110010101110" =>  data <= "11111111";  -- CAE = FF
      when "110010101111" =>  data <= "11111111";  -- CAF = FF
      when "110010110000" =>  data <= "11111111";  -- CB0 = FF
      when "110010110001" =>  data <= "11111111";  -- CB1 = FF
      when "110010110010" =>  data <= "11111111";  -- CB2 = FF
      when "110010110011" =>  data <= "11111111";  -- CB3 = FF
      when "110010110100" =>  data <= "11111111";  -- CB4 = FF
      when "110010110101" =>  data <= "11111111";  -- CB5 = FF
      when "110010110110" =>  data <= "11111111";  -- CB6 = FF
      when "110010110111" =>  data <= "11111111";  -- CB7 = FF
      when "110010111000" =>  data <= "11111111";  -- CB8 = FF
      when "110010111001" =>  data <= "11111111";  -- CB9 = FF
      when "110010111010" =>  data <= "11111111";  -- CBA = FF
      when "110010111011" =>  data <= "11111111";  -- CBB = FF
      when "110010111100" =>  data <= "11111111";  -- CBC = FF
      when "110010111101" =>  data <= "11111111";  -- CBD = FF
      when "110010111110" =>  data <= "11111111";  -- CBE = FF
      when "110010111111" =>  data <= "11111111";  -- CBF = FF
      when "110011000000" =>  data <= "11111111";  -- CC0 = FF
      when "110011000001" =>  data <= "11111111";  -- CC1 = FF
      when "110011000010" =>  data <= "11111111";  -- CC2 = FF
      when "110011000011" =>  data <= "11111111";  -- CC3 = FF
      when "110011000100" =>  data <= "11111111";  -- CC4 = FF
      when "110011000101" =>  data <= "11111111";  -- CC5 = FF
      when "110011000110" =>  data <= "11111111";  -- CC6 = FF
      when "110011000111" =>  data <= "11111111";  -- CC7 = FF
      when "110011001000" =>  data <= "11111111";  -- CC8 = FF
      when "110011001001" =>  data <= "11111111";  -- CC9 = FF
      when "110011001010" =>  data <= "11111111";  -- CCA = FF
      when "110011001011" =>  data <= "11111111";  -- CCB = FF
      when "110011001100" =>  data <= "11111111";  -- CCC = FF
      when "110011001101" =>  data <= "11111111";  -- CCD = FF
      when "110011001110" =>  data <= "11111111";  -- CCE = FF
      when "110011001111" =>  data <= "11111111";  -- CCF = FF
      when "110011010000" =>  data <= "11111111";  -- CD0 = FF
      when "110011010001" =>  data <= "11111111";  -- CD1 = FF
      when "110011010010" =>  data <= "11111111";  -- CD2 = FF
      when "110011010011" =>  data <= "11111111";  -- CD3 = FF
      when "110011010100" =>  data <= "11111111";  -- CD4 = FF
      when "110011010101" =>  data <= "11111111";  -- CD5 = FF
      when "110011010110" =>  data <= "11111111";  -- CD6 = FF
      when "110011010111" =>  data <= "11111111";  -- CD7 = FF
      when "110011011000" =>  data <= "11111111";  -- CD8 = FF
      when "110011011001" =>  data <= "11111111";  -- CD9 = FF
      when "110011011010" =>  data <= "11111111";  -- CDA = FF
      when "110011011011" =>  data <= "11111111";  -- CDB = FF
      when "110011011100" =>  data <= "11111111";  -- CDC = FF
      when "110011011101" =>  data <= "11111111";  -- CDD = FF
      when "110011011110" =>  data <= "11111111";  -- CDE = FF
      when "110011011111" =>  data <= "11111111";  -- CDF = FF
      when "110011100000" =>  data <= "11111111";  -- CE0 = FF
      when "110011100001" =>  data <= "11111111";  -- CE1 = FF
      when "110011100010" =>  data <= "11111111";  -- CE2 = FF
      when "110011100011" =>  data <= "11111111";  -- CE3 = FF
      when "110011100100" =>  data <= "11111111";  -- CE4 = FF
      when "110011100101" =>  data <= "11111111";  -- CE5 = FF
      when "110011100110" =>  data <= "11111111";  -- CE6 = FF
      when "110011100111" =>  data <= "11111111";  -- CE7 = FF
      when "110011101000" =>  data <= "11111111";  -- CE8 = FF
      when "110011101001" =>  data <= "11111111";  -- CE9 = FF
      when "110011101010" =>  data <= "11111111";  -- CEA = FF
      when "110011101011" =>  data <= "11111111";  -- CEB = FF
      when "110011101100" =>  data <= "11111111";  -- CEC = FF
      when "110011101101" =>  data <= "11111111";  -- CED = FF
      when "110011101110" =>  data <= "11111111";  -- CEE = FF
      when "110011101111" =>  data <= "11111111";  -- CEF = FF
      when "110011110000" =>  data <= "11111111";  -- CF0 = FF
      when "110011110001" =>  data <= "11111111";  -- CF1 = FF
      when "110011110010" =>  data <= "11111111";  -- CF2 = FF
      when "110011110011" =>  data <= "11111111";  -- CF3 = FF
      when "110011110100" =>  data <= "11111111";  -- CF4 = FF
      when "110011110101" =>  data <= "11111111";  -- CF5 = FF
      when "110011110110" =>  data <= "11111111";  -- CF6 = FF
      when "110011110111" =>  data <= "11111111";  -- CF7 = FF
      when "110011111000" =>  data <= "11111111";  -- CF8 = FF
      when "110011111001" =>  data <= "11111111";  -- CF9 = FF
      when "110011111010" =>  data <= "11111111";  -- CFA = FF
      when "110011111011" =>  data <= "11111111";  -- CFB = FF
      when "110011111100" =>  data <= "11111111";  -- CFC = FF
      when "110011111101" =>  data <= "11111111";  -- CFD = FF
      when "110011111110" =>  data <= "11111111";  -- CFE = FF
      when "110011111111" =>  data <= "11111111";  -- CFF = FF
      when "110100000000" =>  data <= "11111111";  -- D00 = FF
      when "110100000001" =>  data <= "11111111";  -- D01 = FF
      when "110100000010" =>  data <= "11111111";  -- D02 = FF
      when "110100000011" =>  data <= "11111111";  -- D03 = FF
      when "110100000100" =>  data <= "11111111";  -- D04 = FF
      when "110100000101" =>  data <= "11111111";  -- D05 = FF
      when "110100000110" =>  data <= "11111111";  -- D06 = FF
      when "110100000111" =>  data <= "11111111";  -- D07 = FF
      when "110100001000" =>  data <= "11111111";  -- D08 = FF
      when "110100001001" =>  data <= "11111111";  -- D09 = FF
      when "110100001010" =>  data <= "11111111";  -- D0A = FF
      when "110100001011" =>  data <= "11111111";  -- D0B = FF
      when "110100001100" =>  data <= "11111111";  -- D0C = FF
      when "110100001101" =>  data <= "11111111";  -- D0D = FF
      when "110100001110" =>  data <= "11111111";  -- D0E = FF
      when "110100001111" =>  data <= "11111111";  -- D0F = FF
      when "110100010000" =>  data <= "11111111";  -- D10 = FF
      when "110100010001" =>  data <= "11111111";  -- D11 = FF
      when "110100010010" =>  data <= "11111111";  -- D12 = FF
      when "110100010011" =>  data <= "11111111";  -- D13 = FF
      when "110100010100" =>  data <= "11111111";  -- D14 = FF
      when "110100010101" =>  data <= "11111111";  -- D15 = FF
      when "110100010110" =>  data <= "11111111";  -- D16 = FF
      when "110100010111" =>  data <= "11111111";  -- D17 = FF
      when "110100011000" =>  data <= "11111111";  -- D18 = FF
      when "110100011001" =>  data <= "11111111";  -- D19 = FF
      when "110100011010" =>  data <= "11111111";  -- D1A = FF
      when "110100011011" =>  data <= "11111111";  -- D1B = FF
      when "110100011100" =>  data <= "11111111";  -- D1C = FF
      when "110100011101" =>  data <= "11111111";  -- D1D = FF
      when "110100011110" =>  data <= "11111111";  -- D1E = FF
      when "110100011111" =>  data <= "11111111";  -- D1F = FF
      when "110100100000" =>  data <= "11111111";  -- D20 = FF
      when "110100100001" =>  data <= "11111111";  -- D21 = FF
      when "110100100010" =>  data <= "11111111";  -- D22 = FF
      when "110100100011" =>  data <= "11111111";  -- D23 = FF
      when "110100100100" =>  data <= "11111111";  -- D24 = FF
      when "110100100101" =>  data <= "11111111";  -- D25 = FF
      when "110100100110" =>  data <= "11111111";  -- D26 = FF
      when "110100100111" =>  data <= "11111111";  -- D27 = FF
      when "110100101000" =>  data <= "11111111";  -- D28 = FF
      when "110100101001" =>  data <= "11111111";  -- D29 = FF
      when "110100101010" =>  data <= "11111111";  -- D2A = FF
      when "110100101011" =>  data <= "11111111";  -- D2B = FF
      when "110100101100" =>  data <= "11111111";  -- D2C = FF
      when "110100101101" =>  data <= "11111111";  -- D2D = FF
      when "110100101110" =>  data <= "11111111";  -- D2E = FF
      when "110100101111" =>  data <= "11111111";  -- D2F = FF
      when "110100110000" =>  data <= "11111111";  -- D30 = FF
      when "110100110001" =>  data <= "11111111";  -- D31 = FF
      when "110100110010" =>  data <= "11111111";  -- D32 = FF
      when "110100110011" =>  data <= "11111111";  -- D33 = FF
      when "110100110100" =>  data <= "11111111";  -- D34 = FF
      when "110100110101" =>  data <= "11111111";  -- D35 = FF
      when "110100110110" =>  data <= "11111111";  -- D36 = FF
      when "110100110111" =>  data <= "11111111";  -- D37 = FF
      when "110100111000" =>  data <= "11111111";  -- D38 = FF
      when "110100111001" =>  data <= "11111111";  -- D39 = FF
      when "110100111010" =>  data <= "11111111";  -- D3A = FF
      when "110100111011" =>  data <= "11111111";  -- D3B = FF
      when "110100111100" =>  data <= "11111111";  -- D3C = FF
      when "110100111101" =>  data <= "11111111";  -- D3D = FF
      when "110100111110" =>  data <= "11111111";  -- D3E = FF
      when "110100111111" =>  data <= "11111111";  -- D3F = FF
      when "110101000000" =>  data <= "11111111";  -- D40 = FF
      when "110101000001" =>  data <= "11111111";  -- D41 = FF
      when "110101000010" =>  data <= "11111111";  -- D42 = FF
      when "110101000011" =>  data <= "11111111";  -- D43 = FF
      when "110101000100" =>  data <= "11111111";  -- D44 = FF
      when "110101000101" =>  data <= "11111111";  -- D45 = FF
      when "110101000110" =>  data <= "11111111";  -- D46 = FF
      when "110101000111" =>  data <= "11111111";  -- D47 = FF
      when "110101001000" =>  data <= "11111111";  -- D48 = FF
      when "110101001001" =>  data <= "11111111";  -- D49 = FF
      when "110101001010" =>  data <= "11111111";  -- D4A = FF
      when "110101001011" =>  data <= "11111111";  -- D4B = FF
      when "110101001100" =>  data <= "11111111";  -- D4C = FF
      when "110101001101" =>  data <= "11111111";  -- D4D = FF
      when "110101001110" =>  data <= "11111111";  -- D4E = FF
      when "110101001111" =>  data <= "11111111";  -- D4F = FF
      when "110101010000" =>  data <= "11111111";  -- D50 = FF
      when "110101010001" =>  data <= "11111111";  -- D51 = FF
      when "110101010010" =>  data <= "11111111";  -- D52 = FF
      when "110101010011" =>  data <= "11111111";  -- D53 = FF
      when "110101010100" =>  data <= "11111111";  -- D54 = FF
      when "110101010101" =>  data <= "11111111";  -- D55 = FF
      when "110101010110" =>  data <= "11111111";  -- D56 = FF
      when "110101010111" =>  data <= "11111111";  -- D57 = FF
      when "110101011000" =>  data <= "11111111";  -- D58 = FF
      when "110101011001" =>  data <= "11111111";  -- D59 = FF
      when "110101011010" =>  data <= "11111111";  -- D5A = FF
      when "110101011011" =>  data <= "11111111";  -- D5B = FF
      when "110101011100" =>  data <= "11111111";  -- D5C = FF
      when "110101011101" =>  data <= "11111111";  -- D5D = FF
      when "110101011110" =>  data <= "11111111";  -- D5E = FF
      when "110101011111" =>  data <= "11111111";  -- D5F = FF
      when "110101100000" =>  data <= "11111111";  -- D60 = FF
      when "110101100001" =>  data <= "11111111";  -- D61 = FF
      when "110101100010" =>  data <= "11111111";  -- D62 = FF
      when "110101100011" =>  data <= "11111111";  -- D63 = FF
      when "110101100100" =>  data <= "11111111";  -- D64 = FF
      when "110101100101" =>  data <= "11111111";  -- D65 = FF
      when "110101100110" =>  data <= "11111111";  -- D66 = FF
      when "110101100111" =>  data <= "11111111";  -- D67 = FF
      when "110101101000" =>  data <= "11111111";  -- D68 = FF
      when "110101101001" =>  data <= "11111111";  -- D69 = FF
      when "110101101010" =>  data <= "11111111";  -- D6A = FF
      when "110101101011" =>  data <= "11111111";  -- D6B = FF
      when "110101101100" =>  data <= "11111111";  -- D6C = FF
      when "110101101101" =>  data <= "11111111";  -- D6D = FF
      when "110101101110" =>  data <= "11111111";  -- D6E = FF
      when "110101101111" =>  data <= "11111111";  -- D6F = FF
      when "110101110000" =>  data <= "11111111";  -- D70 = FF
      when "110101110001" =>  data <= "11111111";  -- D71 = FF
      when "110101110010" =>  data <= "11111111";  -- D72 = FF
      when "110101110011" =>  data <= "11111111";  -- D73 = FF
      when "110101110100" =>  data <= "11111111";  -- D74 = FF
      when "110101110101" =>  data <= "11111111";  -- D75 = FF
      when "110101110110" =>  data <= "11111111";  -- D76 = FF
      when "110101110111" =>  data <= "11111111";  -- D77 = FF
      when "110101111000" =>  data <= "11111111";  -- D78 = FF
      when "110101111001" =>  data <= "11111111";  -- D79 = FF
      when "110101111010" =>  data <= "11111111";  -- D7A = FF
      when "110101111011" =>  data <= "11111111";  -- D7B = FF
      when "110101111100" =>  data <= "11111111";  -- D7C = FF
      when "110101111101" =>  data <= "11111111";  -- D7D = FF
      when "110101111110" =>  data <= "11111111";  -- D7E = FF
      when "110101111111" =>  data <= "11111111";  -- D7F = FF
      when "110110000000" =>  data <= "11111111";  -- D80 = FF
      when "110110000001" =>  data <= "11111111";  -- D81 = FF
      when "110110000010" =>  data <= "11111111";  -- D82 = FF
      when "110110000011" =>  data <= "11111111";  -- D83 = FF
      when "110110000100" =>  data <= "11111111";  -- D84 = FF
      when "110110000101" =>  data <= "11111111";  -- D85 = FF
      when "110110000110" =>  data <= "11111111";  -- D86 = FF
      when "110110000111" =>  data <= "11111111";  -- D87 = FF
      when "110110001000" =>  data <= "11111111";  -- D88 = FF
      when "110110001001" =>  data <= "11111111";  -- D89 = FF
      when "110110001010" =>  data <= "11111111";  -- D8A = FF
      when "110110001011" =>  data <= "11111111";  -- D8B = FF
      when "110110001100" =>  data <= "11111111";  -- D8C = FF
      when "110110001101" =>  data <= "11111111";  -- D8D = FF
      when "110110001110" =>  data <= "11111111";  -- D8E = FF
      when "110110001111" =>  data <= "11111111";  -- D8F = FF
      when "110110010000" =>  data <= "11111111";  -- D90 = FF
      when "110110010001" =>  data <= "11111111";  -- D91 = FF
      when "110110010010" =>  data <= "11111111";  -- D92 = FF
      when "110110010011" =>  data <= "11111111";  -- D93 = FF
      when "110110010100" =>  data <= "11111111";  -- D94 = FF
      when "110110010101" =>  data <= "11111111";  -- D95 = FF
      when "110110010110" =>  data <= "11111111";  -- D96 = FF
      when "110110010111" =>  data <= "11111111";  -- D97 = FF
      when "110110011000" =>  data <= "11111111";  -- D98 = FF
      when "110110011001" =>  data <= "11111111";  -- D99 = FF
      when "110110011010" =>  data <= "11111111";  -- D9A = FF
      when "110110011011" =>  data <= "11111111";  -- D9B = FF
      when "110110011100" =>  data <= "11111111";  -- D9C = FF
      when "110110011101" =>  data <= "11111111";  -- D9D = FF
      when "110110011110" =>  data <= "11111111";  -- D9E = FF
      when "110110011111" =>  data <= "11111111";  -- D9F = FF
      when "110110100000" =>  data <= "11111111";  -- DA0 = FF
      when "110110100001" =>  data <= "11111111";  -- DA1 = FF
      when "110110100010" =>  data <= "11111111";  -- DA2 = FF
      when "110110100011" =>  data <= "11111111";  -- DA3 = FF
      when "110110100100" =>  data <= "11111111";  -- DA4 = FF
      when "110110100101" =>  data <= "11111111";  -- DA5 = FF
      when "110110100110" =>  data <= "11111111";  -- DA6 = FF
      when "110110100111" =>  data <= "11111111";  -- DA7 = FF
      when "110110101000" =>  data <= "11111111";  -- DA8 = FF
      when "110110101001" =>  data <= "11111111";  -- DA9 = FF
      when "110110101010" =>  data <= "11111111";  -- DAA = FF
      when "110110101011" =>  data <= "11111111";  -- DAB = FF
      when "110110101100" =>  data <= "11111111";  -- DAC = FF
      when "110110101101" =>  data <= "11111111";  -- DAD = FF
      when "110110101110" =>  data <= "11111111";  -- DAE = FF
      when "110110101111" =>  data <= "11111111";  -- DAF = FF
      when "110110110000" =>  data <= "11111111";  -- DB0 = FF
      when "110110110001" =>  data <= "11111111";  -- DB1 = FF
      when "110110110010" =>  data <= "11111111";  -- DB2 = FF
      when "110110110011" =>  data <= "11111111";  -- DB3 = FF
      when "110110110100" =>  data <= "11111111";  -- DB4 = FF
      when "110110110101" =>  data <= "11111111";  -- DB5 = FF
      when "110110110110" =>  data <= "11111111";  -- DB6 = FF
      when "110110110111" =>  data <= "11111111";  -- DB7 = FF
      when "110110111000" =>  data <= "11111111";  -- DB8 = FF
      when "110110111001" =>  data <= "11111111";  -- DB9 = FF
      when "110110111010" =>  data <= "11111111";  -- DBA = FF
      when "110110111011" =>  data <= "11111111";  -- DBB = FF
      when "110110111100" =>  data <= "11111111";  -- DBC = FF
      when "110110111101" =>  data <= "11111111";  -- DBD = FF
      when "110110111110" =>  data <= "11111111";  -- DBE = FF
      when "110110111111" =>  data <= "11111111";  -- DBF = FF
      when "110111000000" =>  data <= "11111111";  -- DC0 = FF
      when "110111000001" =>  data <= "11111111";  -- DC1 = FF
      when "110111000010" =>  data <= "11111111";  -- DC2 = FF
      when "110111000011" =>  data <= "11111111";  -- DC3 = FF
      when "110111000100" =>  data <= "11111111";  -- DC4 = FF
      when "110111000101" =>  data <= "11111111";  -- DC5 = FF
      when "110111000110" =>  data <= "11111111";  -- DC6 = FF
      when "110111000111" =>  data <= "11111111";  -- DC7 = FF
      when "110111001000" =>  data <= "11111111";  -- DC8 = FF
      when "110111001001" =>  data <= "11111111";  -- DC9 = FF
      when "110111001010" =>  data <= "11111111";  -- DCA = FF
      when "110111001011" =>  data <= "11111111";  -- DCB = FF
      when "110111001100" =>  data <= "11111111";  -- DCC = FF
      when "110111001101" =>  data <= "11111111";  -- DCD = FF
      when "110111001110" =>  data <= "11111111";  -- DCE = FF
      when "110111001111" =>  data <= "11111111";  -- DCF = FF
      when "110111010000" =>  data <= "11111111";  -- DD0 = FF
      when "110111010001" =>  data <= "11111111";  -- DD1 = FF
      when "110111010010" =>  data <= "11111111";  -- DD2 = FF
      when "110111010011" =>  data <= "11111111";  -- DD3 = FF
      when "110111010100" =>  data <= "11111111";  -- DD4 = FF
      when "110111010101" =>  data <= "11111111";  -- DD5 = FF
      when "110111010110" =>  data <= "11111111";  -- DD6 = FF
      when "110111010111" =>  data <= "11111111";  -- DD7 = FF
      when "110111011000" =>  data <= "11111111";  -- DD8 = FF
      when "110111011001" =>  data <= "11111111";  -- DD9 = FF
      when "110111011010" =>  data <= "11111111";  -- DDA = FF
      when "110111011011" =>  data <= "11111111";  -- DDB = FF
      when "110111011100" =>  data <= "11111111";  -- DDC = FF
      when "110111011101" =>  data <= "11111111";  -- DDD = FF
      when "110111011110" =>  data <= "11111111";  -- DDE = FF
      when "110111011111" =>  data <= "11111111";  -- DDF = FF
      when "110111100000" =>  data <= "11111111";  -- DE0 = FF
      when "110111100001" =>  data <= "11111111";  -- DE1 = FF
      when "110111100010" =>  data <= "11111111";  -- DE2 = FF
      when "110111100011" =>  data <= "11111111";  -- DE3 = FF
      when "110111100100" =>  data <= "11111111";  -- DE4 = FF
      when "110111100101" =>  data <= "11111111";  -- DE5 = FF
      when "110111100110" =>  data <= "11111111";  -- DE6 = FF
      when "110111100111" =>  data <= "11111111";  -- DE7 = FF
      when "110111101000" =>  data <= "11111111";  -- DE8 = FF
      when "110111101001" =>  data <= "11111111";  -- DE9 = FF
      when "110111101010" =>  data <= "11111111";  -- DEA = FF
      when "110111101011" =>  data <= "11111111";  -- DEB = FF
      when "110111101100" =>  data <= "11111111";  -- DEC = FF
      when "110111101101" =>  data <= "11111111";  -- DED = FF
      when "110111101110" =>  data <= "11111111";  -- DEE = FF
      when "110111101111" =>  data <= "11111111";  -- DEF = FF
      when "110111110000" =>  data <= "11111111";  -- DF0 = FF
      when "110111110001" =>  data <= "11111111";  -- DF1 = FF
      when "110111110010" =>  data <= "11111111";  -- DF2 = FF
      when "110111110011" =>  data <= "11111111";  -- DF3 = FF
      when "110111110100" =>  data <= "11111111";  -- DF4 = FF
      when "110111110101" =>  data <= "11111111";  -- DF5 = FF
      when "110111110110" =>  data <= "11111111";  -- DF6 = FF
      when "110111110111" =>  data <= "11111111";  -- DF7 = FF
      when "110111111000" =>  data <= "11111111";  -- DF8 = FF
      when "110111111001" =>  data <= "11111111";  -- DF9 = FF
      when "110111111010" =>  data <= "11111111";  -- DFA = FF
      when "110111111011" =>  data <= "11111111";  -- DFB = FF
      when "110111111100" =>  data <= "11111111";  -- DFC = FF
      when "110111111101" =>  data <= "11111111";  -- DFD = FF
      when "110111111110" =>  data <= "11111111";  -- DFE = FF
      when "110111111111" =>  data <= "11111111";  -- DFF = FF
      when "111000000000" =>  data <= "11111111";  -- E00 = FF
      when "111000000001" =>  data <= "11111111";  -- E01 = FF
      when "111000000010" =>  data <= "11111111";  -- E02 = FF
      when "111000000011" =>  data <= "11111111";  -- E03 = FF
      when "111000000100" =>  data <= "11111111";  -- E04 = FF
      when "111000000101" =>  data <= "11111111";  -- E05 = FF
      when "111000000110" =>  data <= "11111111";  -- E06 = FF
      when "111000000111" =>  data <= "11111111";  -- E07 = FF
      when "111000001000" =>  data <= "11111111";  -- E08 = FF
      when "111000001001" =>  data <= "11111111";  -- E09 = FF
      when "111000001010" =>  data <= "11111111";  -- E0A = FF
      when "111000001011" =>  data <= "11111111";  -- E0B = FF
      when "111000001100" =>  data <= "11111111";  -- E0C = FF
      when "111000001101" =>  data <= "11111111";  -- E0D = FF
      when "111000001110" =>  data <= "11111111";  -- E0E = FF
      when "111000001111" =>  data <= "11111111";  -- E0F = FF
      when "111000010000" =>  data <= "11111111";  -- E10 = FF
      when "111000010001" =>  data <= "11111111";  -- E11 = FF
      when "111000010010" =>  data <= "11111111";  -- E12 = FF
      when "111000010011" =>  data <= "11111111";  -- E13 = FF
      when "111000010100" =>  data <= "11111111";  -- E14 = FF
      when "111000010101" =>  data <= "11111111";  -- E15 = FF
      when "111000010110" =>  data <= "11111111";  -- E16 = FF
      when "111000010111" =>  data <= "11111111";  -- E17 = FF
      when "111000011000" =>  data <= "11111111";  -- E18 = FF
      when "111000011001" =>  data <= "11111111";  -- E19 = FF
      when "111000011010" =>  data <= "11111111";  -- E1A = FF
      when "111000011011" =>  data <= "11111111";  -- E1B = FF
      when "111000011100" =>  data <= "11111111";  -- E1C = FF
      when "111000011101" =>  data <= "11111111";  -- E1D = FF
      when "111000011110" =>  data <= "11111111";  -- E1E = FF
      when "111000011111" =>  data <= "11111111";  -- E1F = FF
      when "111000100000" =>  data <= "11111111";  -- E20 = FF
      when "111000100001" =>  data <= "11111111";  -- E21 = FF
      when "111000100010" =>  data <= "11111111";  -- E22 = FF
      when "111000100011" =>  data <= "11111111";  -- E23 = FF
      when "111000100100" =>  data <= "11111111";  -- E24 = FF
      when "111000100101" =>  data <= "11111111";  -- E25 = FF
      when "111000100110" =>  data <= "11111111";  -- E26 = FF
      when "111000100111" =>  data <= "11111111";  -- E27 = FF
      when "111000101000" =>  data <= "11111111";  -- E28 = FF
      when "111000101001" =>  data <= "11111111";  -- E29 = FF
      when "111000101010" =>  data <= "11111111";  -- E2A = FF
      when "111000101011" =>  data <= "11111111";  -- E2B = FF
      when "111000101100" =>  data <= "11111111";  -- E2C = FF
      when "111000101101" =>  data <= "11111111";  -- E2D = FF
      when "111000101110" =>  data <= "11111111";  -- E2E = FF
      when "111000101111" =>  data <= "11111111";  -- E2F = FF
      when "111000110000" =>  data <= "11111111";  -- E30 = FF
      when "111000110001" =>  data <= "11111111";  -- E31 = FF
      when "111000110010" =>  data <= "11111111";  -- E32 = FF
      when "111000110011" =>  data <= "11111111";  -- E33 = FF
      when "111000110100" =>  data <= "11111111";  -- E34 = FF
      when "111000110101" =>  data <= "11111111";  -- E35 = FF
      when "111000110110" =>  data <= "11111111";  -- E36 = FF
      when "111000110111" =>  data <= "11111111";  -- E37 = FF
      when "111000111000" =>  data <= "11111111";  -- E38 = FF
      when "111000111001" =>  data <= "11111111";  -- E39 = FF
      when "111000111010" =>  data <= "11111111";  -- E3A = FF
      when "111000111011" =>  data <= "11111111";  -- E3B = FF
      when "111000111100" =>  data <= "11111111";  -- E3C = FF
      when "111000111101" =>  data <= "11111111";  -- E3D = FF
      when "111000111110" =>  data <= "11111111";  -- E3E = FF
      when "111000111111" =>  data <= "11111111";  -- E3F = FF
      when "111001000000" =>  data <= "11111111";  -- E40 = FF
      when "111001000001" =>  data <= "11111111";  -- E41 = FF
      when "111001000010" =>  data <= "11111111";  -- E42 = FF
      when "111001000011" =>  data <= "11111111";  -- E43 = FF
      when "111001000100" =>  data <= "11111111";  -- E44 = FF
      when "111001000101" =>  data <= "11111111";  -- E45 = FF
      when "111001000110" =>  data <= "11111111";  -- E46 = FF
      when "111001000111" =>  data <= "11111111";  -- E47 = FF
      when "111001001000" =>  data <= "11111111";  -- E48 = FF
      when "111001001001" =>  data <= "11111111";  -- E49 = FF
      when "111001001010" =>  data <= "11111111";  -- E4A = FF
      when "111001001011" =>  data <= "11111111";  -- E4B = FF
      when "111001001100" =>  data <= "11111111";  -- E4C = FF
      when "111001001101" =>  data <= "11111111";  -- E4D = FF
      when "111001001110" =>  data <= "11111111";  -- E4E = FF
      when "111001001111" =>  data <= "11111111";  -- E4F = FF
      when "111001010000" =>  data <= "11111111";  -- E50 = FF
      when "111001010001" =>  data <= "11111111";  -- E51 = FF
      when "111001010010" =>  data <= "11111111";  -- E52 = FF
      when "111001010011" =>  data <= "11111111";  -- E53 = FF
      when "111001010100" =>  data <= "11111111";  -- E54 = FF
      when "111001010101" =>  data <= "11111111";  -- E55 = FF
      when "111001010110" =>  data <= "11111111";  -- E56 = FF
      when "111001010111" =>  data <= "11111111";  -- E57 = FF
      when "111001011000" =>  data <= "11111111";  -- E58 = FF
      when "111001011001" =>  data <= "11111111";  -- E59 = FF
      when "111001011010" =>  data <= "11111111";  -- E5A = FF
      when "111001011011" =>  data <= "11111111";  -- E5B = FF
      when "111001011100" =>  data <= "11111111";  -- E5C = FF
      when "111001011101" =>  data <= "11111111";  -- E5D = FF
      when "111001011110" =>  data <= "11111111";  -- E5E = FF
      when "111001011111" =>  data <= "11111111";  -- E5F = FF
      when "111001100000" =>  data <= "11111111";  -- E60 = FF
      when "111001100001" =>  data <= "11111111";  -- E61 = FF
      when "111001100010" =>  data <= "11111111";  -- E62 = FF
      when "111001100011" =>  data <= "11111111";  -- E63 = FF
      when "111001100100" =>  data <= "11111111";  -- E64 = FF
      when "111001100101" =>  data <= "11111111";  -- E65 = FF
      when "111001100110" =>  data <= "11111111";  -- E66 = FF
      when "111001100111" =>  data <= "11111111";  -- E67 = FF
      when "111001101000" =>  data <= "11111111";  -- E68 = FF
      when "111001101001" =>  data <= "11111111";  -- E69 = FF
      when "111001101010" =>  data <= "11111111";  -- E6A = FF
      when "111001101011" =>  data <= "11111111";  -- E6B = FF
      when "111001101100" =>  data <= "11111111";  -- E6C = FF
      when "111001101101" =>  data <= "11111111";  -- E6D = FF
      when "111001101110" =>  data <= "11111111";  -- E6E = FF
      when "111001101111" =>  data <= "11111111";  -- E6F = FF
      when "111001110000" =>  data <= "11111111";  -- E70 = FF
      when "111001110001" =>  data <= "11111111";  -- E71 = FF
      when "111001110010" =>  data <= "11111111";  -- E72 = FF
      when "111001110011" =>  data <= "11111111";  -- E73 = FF
      when "111001110100" =>  data <= "11111111";  -- E74 = FF
      when "111001110101" =>  data <= "11111111";  -- E75 = FF
      when "111001110110" =>  data <= "11111111";  -- E76 = FF
      when "111001110111" =>  data <= "11111111";  -- E77 = FF
      when "111001111000" =>  data <= "11111111";  -- E78 = FF
      when "111001111001" =>  data <= "11111111";  -- E79 = FF
      when "111001111010" =>  data <= "11111111";  -- E7A = FF
      when "111001111011" =>  data <= "11111111";  -- E7B = FF
      when "111001111100" =>  data <= "11111111";  -- E7C = FF
      when "111001111101" =>  data <= "11111111";  -- E7D = FF
      when "111001111110" =>  data <= "11111111";  -- E7E = FF
      when "111001111111" =>  data <= "11111111";  -- E7F = FF
      when "111010000000" =>  data <= "11111111";  -- E80 = FF
      when "111010000001" =>  data <= "11111111";  -- E81 = FF
      when "111010000010" =>  data <= "11111111";  -- E82 = FF
      when "111010000011" =>  data <= "11111111";  -- E83 = FF
      when "111010000100" =>  data <= "11111111";  -- E84 = FF
      when "111010000101" =>  data <= "11111111";  -- E85 = FF
      when "111010000110" =>  data <= "11111111";  -- E86 = FF
      when "111010000111" =>  data <= "11111111";  -- E87 = FF
      when "111010001000" =>  data <= "11111111";  -- E88 = FF
      when "111010001001" =>  data <= "11111111";  -- E89 = FF
      when "111010001010" =>  data <= "11111111";  -- E8A = FF
      when "111010001011" =>  data <= "11111111";  -- E8B = FF
      when "111010001100" =>  data <= "11111111";  -- E8C = FF
      when "111010001101" =>  data <= "11111111";  -- E8D = FF
      when "111010001110" =>  data <= "11111111";  -- E8E = FF
      when "111010001111" =>  data <= "11111111";  -- E8F = FF
      when "111010010000" =>  data <= "11111111";  -- E90 = FF
      when "111010010001" =>  data <= "11111111";  -- E91 = FF
      when "111010010010" =>  data <= "11111111";  -- E92 = FF
      when "111010010011" =>  data <= "11111111";  -- E93 = FF
      when "111010010100" =>  data <= "11111111";  -- E94 = FF
      when "111010010101" =>  data <= "11111111";  -- E95 = FF
      when "111010010110" =>  data <= "11111111";  -- E96 = FF
      when "111010010111" =>  data <= "11111111";  -- E97 = FF
      when "111010011000" =>  data <= "11111111";  -- E98 = FF
      when "111010011001" =>  data <= "11111111";  -- E99 = FF
      when "111010011010" =>  data <= "11111111";  -- E9A = FF
      when "111010011011" =>  data <= "11111111";  -- E9B = FF
      when "111010011100" =>  data <= "11111111";  -- E9C = FF
      when "111010011101" =>  data <= "11111111";  -- E9D = FF
      when "111010011110" =>  data <= "11111111";  -- E9E = FF
      when "111010011111" =>  data <= "11111111";  -- E9F = FF
      when "111010100000" =>  data <= "11111111";  -- EA0 = FF
      when "111010100001" =>  data <= "11111111";  -- EA1 = FF
      when "111010100010" =>  data <= "11111111";  -- EA2 = FF
      when "111010100011" =>  data <= "11111111";  -- EA3 = FF
      when "111010100100" =>  data <= "11111111";  -- EA4 = FF
      when "111010100101" =>  data <= "11111111";  -- EA5 = FF
      when "111010100110" =>  data <= "11111111";  -- EA6 = FF
      when "111010100111" =>  data <= "11111111";  -- EA7 = FF
      when "111010101000" =>  data <= "11111111";  -- EA8 = FF
      when "111010101001" =>  data <= "11111111";  -- EA9 = FF
      when "111010101010" =>  data <= "11111111";  -- EAA = FF
      when "111010101011" =>  data <= "11111111";  -- EAB = FF
      when "111010101100" =>  data <= "11111111";  -- EAC = FF
      when "111010101101" =>  data <= "11111111";  -- EAD = FF
      when "111010101110" =>  data <= "11111111";  -- EAE = FF
      when "111010101111" =>  data <= "11111111";  -- EAF = FF
      when "111010110000" =>  data <= "11111111";  -- EB0 = FF
      when "111010110001" =>  data <= "11111111";  -- EB1 = FF
      when "111010110010" =>  data <= "11111111";  -- EB2 = FF
      when "111010110011" =>  data <= "11111111";  -- EB3 = FF
      when "111010110100" =>  data <= "11111111";  -- EB4 = FF
      when "111010110101" =>  data <= "11111111";  -- EB5 = FF
      when "111010110110" =>  data <= "11111111";  -- EB6 = FF
      when "111010110111" =>  data <= "11111111";  -- EB7 = FF
      when "111010111000" =>  data <= "11111111";  -- EB8 = FF
      when "111010111001" =>  data <= "11111111";  -- EB9 = FF
      when "111010111010" =>  data <= "11111111";  -- EBA = FF
      when "111010111011" =>  data <= "11111111";  -- EBB = FF
      when "111010111100" =>  data <= "11111111";  -- EBC = FF
      when "111010111101" =>  data <= "11111111";  -- EBD = FF
      when "111010111110" =>  data <= "11111111";  -- EBE = FF
      when "111010111111" =>  data <= "11111111";  -- EBF = FF
      when "111011000000" =>  data <= "11111111";  -- EC0 = FF
      when "111011000001" =>  data <= "11111111";  -- EC1 = FF
      when "111011000010" =>  data <= "11111111";  -- EC2 = FF
      when "111011000011" =>  data <= "11111111";  -- EC3 = FF
      when "111011000100" =>  data <= "11111111";  -- EC4 = FF
      when "111011000101" =>  data <= "11111111";  -- EC5 = FF
      when "111011000110" =>  data <= "11111111";  -- EC6 = FF
      when "111011000111" =>  data <= "11111111";  -- EC7 = FF
      when "111011001000" =>  data <= "11111111";  -- EC8 = FF
      when "111011001001" =>  data <= "11111111";  -- EC9 = FF
      when "111011001010" =>  data <= "11111111";  -- ECA = FF
      when "111011001011" =>  data <= "11111111";  -- ECB = FF
      when "111011001100" =>  data <= "11111111";  -- ECC = FF
      when "111011001101" =>  data <= "11111111";  -- ECD = FF
      when "111011001110" =>  data <= "11111111";  -- ECE = FF
      when "111011001111" =>  data <= "11111111";  -- ECF = FF
      when "111011010000" =>  data <= "11111111";  -- ED0 = FF
      when "111011010001" =>  data <= "11111111";  -- ED1 = FF
      when "111011010010" =>  data <= "11111111";  -- ED2 = FF
      when "111011010011" =>  data <= "11111111";  -- ED3 = FF
      when "111011010100" =>  data <= "11111111";  -- ED4 = FF
      when "111011010101" =>  data <= "11111111";  -- ED5 = FF
      when "111011010110" =>  data <= "11111111";  -- ED6 = FF
      when "111011010111" =>  data <= "11111111";  -- ED7 = FF
      when "111011011000" =>  data <= "11111111";  -- ED8 = FF
      when "111011011001" =>  data <= "11111111";  -- ED9 = FF
      when "111011011010" =>  data <= "11111111";  -- EDA = FF
      when "111011011011" =>  data <= "11111111";  -- EDB = FF
      when "111011011100" =>  data <= "11111111";  -- EDC = FF
      when "111011011101" =>  data <= "11111111";  -- EDD = FF
      when "111011011110" =>  data <= "11111111";  -- EDE = FF
      when "111011011111" =>  data <= "11111111";  -- EDF = FF
      when "111011100000" =>  data <= "11111111";  -- EE0 = FF
      when "111011100001" =>  data <= "11111111";  -- EE1 = FF
      when "111011100010" =>  data <= "11111111";  -- EE2 = FF
      when "111011100011" =>  data <= "11111111";  -- EE3 = FF
      when "111011100100" =>  data <= "11111111";  -- EE4 = FF
      when "111011100101" =>  data <= "11111111";  -- EE5 = FF
      when "111011100110" =>  data <= "11111111";  -- EE6 = FF
      when "111011100111" =>  data <= "11111111";  -- EE7 = FF
      when "111011101000" =>  data <= "11111111";  -- EE8 = FF
      when "111011101001" =>  data <= "11111111";  -- EE9 = FF
      when "111011101010" =>  data <= "11111111";  -- EEA = FF
      when "111011101011" =>  data <= "11111111";  -- EEB = FF
      when "111011101100" =>  data <= "11111111";  -- EEC = FF
      when "111011101101" =>  data <= "11111111";  -- EED = FF
      when "111011101110" =>  data <= "11111111";  -- EEE = FF
      when "111011101111" =>  data <= "11111111";  -- EEF = FF
      when "111011110000" =>  data <= "11111111";  -- EF0 = FF
      when "111011110001" =>  data <= "11111111";  -- EF1 = FF
      when "111011110010" =>  data <= "11111111";  -- EF2 = FF
      when "111011110011" =>  data <= "11111111";  -- EF3 = FF
      when "111011110100" =>  data <= "11111111";  -- EF4 = FF
      when "111011110101" =>  data <= "11111111";  -- EF5 = FF
      when "111011110110" =>  data <= "11111111";  -- EF6 = FF
      when "111011110111" =>  data <= "11111111";  -- EF7 = FF
      when "111011111000" =>  data <= "11111111";  -- EF8 = FF
      when "111011111001" =>  data <= "11111111";  -- EF9 = FF
      when "111011111010" =>  data <= "11111111";  -- EFA = FF
      when "111011111011" =>  data <= "11111111";  -- EFB = FF
      when "111011111100" =>  data <= "11111111";  -- EFC = FF
      when "111011111101" =>  data <= "11111111";  -- EFD = FF
      when "111011111110" =>  data <= "11111111";  -- EFE = FF
      when "111011111111" =>  data <= "11111111";  -- EFF = FF
      when "111100000000" =>  data <= "11111111";  -- F00 = FF
      when "111100000001" =>  data <= "11111111";  -- F01 = FF
      when "111100000010" =>  data <= "11111111";  -- F02 = FF
      when "111100000011" =>  data <= "11111111";  -- F03 = FF
      when "111100000100" =>  data <= "11111111";  -- F04 = FF
      when "111100000101" =>  data <= "11111111";  -- F05 = FF
      when "111100000110" =>  data <= "11111111";  -- F06 = FF
      when "111100000111" =>  data <= "11111111";  -- F07 = FF
      when "111100001000" =>  data <= "11111111";  -- F08 = FF
      when "111100001001" =>  data <= "11111111";  -- F09 = FF
      when "111100001010" =>  data <= "11111111";  -- F0A = FF
      when "111100001011" =>  data <= "11111111";  -- F0B = FF
      when "111100001100" =>  data <= "11111111";  -- F0C = FF
      when "111100001101" =>  data <= "11111111";  -- F0D = FF
      when "111100001110" =>  data <= "11111111";  -- F0E = FF
      when "111100001111" =>  data <= "11111111";  -- F0F = FF
      when "111100010000" =>  data <= "11111111";  -- F10 = FF
      when "111100010001" =>  data <= "11111111";  -- F11 = FF
      when "111100010010" =>  data <= "11111111";  -- F12 = FF
      when "111100010011" =>  data <= "11111111";  -- F13 = FF
      when "111100010100" =>  data <= "11111111";  -- F14 = FF
      when "111100010101" =>  data <= "11111111";  -- F15 = FF
      when "111100010110" =>  data <= "11111111";  -- F16 = FF
      when "111100010111" =>  data <= "11111111";  -- F17 = FF
      when "111100011000" =>  data <= "11111111";  -- F18 = FF
      when "111100011001" =>  data <= "11111111";  -- F19 = FF
      when "111100011010" =>  data <= "11111111";  -- F1A = FF
      when "111100011011" =>  data <= "11111111";  -- F1B = FF
      when "111100011100" =>  data <= "11111111";  -- F1C = FF
      when "111100011101" =>  data <= "11111111";  -- F1D = FF
      when "111100011110" =>  data <= "11111111";  -- F1E = FF
      when "111100011111" =>  data <= "11111111";  -- F1F = FF
      when "111100100000" =>  data <= "11111111";  -- F20 = FF
      when "111100100001" =>  data <= "11111111";  -- F21 = FF
      when "111100100010" =>  data <= "11111111";  -- F22 = FF
      when "111100100011" =>  data <= "11111111";  -- F23 = FF
      when "111100100100" =>  data <= "11111111";  -- F24 = FF
      when "111100100101" =>  data <= "11111111";  -- F25 = FF
      when "111100100110" =>  data <= "11111111";  -- F26 = FF
      when "111100100111" =>  data <= "11111111";  -- F27 = FF
      when "111100101000" =>  data <= "11111111";  -- F28 = FF
      when "111100101001" =>  data <= "11111111";  -- F29 = FF
      when "111100101010" =>  data <= "11111111";  -- F2A = FF
      when "111100101011" =>  data <= "11111111";  -- F2B = FF
      when "111100101100" =>  data <= "11111111";  -- F2C = FF
      when "111100101101" =>  data <= "11111111";  -- F2D = FF
      when "111100101110" =>  data <= "11111111";  -- F2E = FF
      when "111100101111" =>  data <= "11111111";  -- F2F = FF
      when "111100110000" =>  data <= "11111111";  -- F30 = FF
      when "111100110001" =>  data <= "11111111";  -- F31 = FF
      when "111100110010" =>  data <= "11111111";  -- F32 = FF
      when "111100110011" =>  data <= "11111111";  -- F33 = FF
      when "111100110100" =>  data <= "11111111";  -- F34 = FF
      when "111100110101" =>  data <= "11111111";  -- F35 = FF
      when "111100110110" =>  data <= "11111111";  -- F36 = FF
      when "111100110111" =>  data <= "11111111";  -- F37 = FF
      when "111100111000" =>  data <= "11111111";  -- F38 = FF
      when "111100111001" =>  data <= "11111111";  -- F39 = FF
      when "111100111010" =>  data <= "11111111";  -- F3A = FF
      when "111100111011" =>  data <= "11111111";  -- F3B = FF
      when "111100111100" =>  data <= "11111111";  -- F3C = FF
      when "111100111101" =>  data <= "11111111";  -- F3D = FF
      when "111100111110" =>  data <= "11111111";  -- F3E = FF
      when "111100111111" =>  data <= "11111111";  -- F3F = FF
      when "111101000000" =>  data <= "11111111";  -- F40 = FF
      when "111101000001" =>  data <= "11111111";  -- F41 = FF
      when "111101000010" =>  data <= "11111111";  -- F42 = FF
      when "111101000011" =>  data <= "11111111";  -- F43 = FF
      when "111101000100" =>  data <= "11111111";  -- F44 = FF
      when "111101000101" =>  data <= "11111111";  -- F45 = FF
      when "111101000110" =>  data <= "11111111";  -- F46 = FF
      when "111101000111" =>  data <= "11111111";  -- F47 = FF
      when "111101001000" =>  data <= "11111111";  -- F48 = FF
      when "111101001001" =>  data <= "11111111";  -- F49 = FF
      when "111101001010" =>  data <= "11111111";  -- F4A = FF
      when "111101001011" =>  data <= "11111111";  -- F4B = FF
      when "111101001100" =>  data <= "11111111";  -- F4C = FF
      when "111101001101" =>  data <= "11111111";  -- F4D = FF
      when "111101001110" =>  data <= "11111111";  -- F4E = FF
      when "111101001111" =>  data <= "11111111";  -- F4F = FF
      when "111101010000" =>  data <= "11111111";  -- F50 = FF
      when "111101010001" =>  data <= "11111111";  -- F51 = FF
      when "111101010010" =>  data <= "11111111";  -- F52 = FF
      when "111101010011" =>  data <= "11111111";  -- F53 = FF
      when "111101010100" =>  data <= "11111111";  -- F54 = FF
      when "111101010101" =>  data <= "11111111";  -- F55 = FF
      when "111101010110" =>  data <= "11111111";  -- F56 = FF
      when "111101010111" =>  data <= "11111111";  -- F57 = FF
      when "111101011000" =>  data <= "11111111";  -- F58 = FF
      when "111101011001" =>  data <= "11111111";  -- F59 = FF
      when "111101011010" =>  data <= "11111111";  -- F5A = FF
      when "111101011011" =>  data <= "11111111";  -- F5B = FF
      when "111101011100" =>  data <= "11111111";  -- F5C = FF
      when "111101011101" =>  data <= "11111111";  -- F5D = FF
      when "111101011110" =>  data <= "11111111";  -- F5E = FF
      when "111101011111" =>  data <= "11111111";  -- F5F = FF
      when "111101100000" =>  data <= "11111111";  -- F60 = FF
      when "111101100001" =>  data <= "11111111";  -- F61 = FF
      when "111101100010" =>  data <= "11111111";  -- F62 = FF
      when "111101100011" =>  data <= "11111111";  -- F63 = FF
      when "111101100100" =>  data <= "11111111";  -- F64 = FF
      when "111101100101" =>  data <= "11111111";  -- F65 = FF
      when "111101100110" =>  data <= "11111111";  -- F66 = FF
      when "111101100111" =>  data <= "11111111";  -- F67 = FF
      when "111101101000" =>  data <= "11111111";  -- F68 = FF
      when "111101101001" =>  data <= "11111111";  -- F69 = FF
      when "111101101010" =>  data <= "11111111";  -- F6A = FF
      when "111101101011" =>  data <= "11111111";  -- F6B = FF
      when "111101101100" =>  data <= "11111111";  -- F6C = FF
      when "111101101101" =>  data <= "11111111";  -- F6D = FF
      when "111101101110" =>  data <= "11111111";  -- F6E = FF
      when "111101101111" =>  data <= "11111111";  -- F6F = FF
      when "111101110000" =>  data <= "11111111";  -- F70 = FF
      when "111101110001" =>  data <= "11111111";  -- F71 = FF
      when "111101110010" =>  data <= "11111111";  -- F72 = FF
      when "111101110011" =>  data <= "11111111";  -- F73 = FF
      when "111101110100" =>  data <= "11111111";  -- F74 = FF
      when "111101110101" =>  data <= "11111111";  -- F75 = FF
      when "111101110110" =>  data <= "11111111";  -- F76 = FF
      when "111101110111" =>  data <= "11111111";  -- F77 = FF
      when "111101111000" =>  data <= "11111111";  -- F78 = FF
      when "111101111001" =>  data <= "11111111";  -- F79 = FF
      when "111101111010" =>  data <= "11111111";  -- F7A = FF
      when "111101111011" =>  data <= "11111111";  -- F7B = FF
      when "111101111100" =>  data <= "11111111";  -- F7C = FF
      when "111101111101" =>  data <= "11111111";  -- F7D = FF
      when "111101111110" =>  data <= "11111111";  -- F7E = FF
      when "111101111111" =>  data <= "11111111";  -- F7F = FF
      when "111110000000" =>  data <= "11111111";  -- F80 = FF
      when "111110000001" =>  data <= "11111111";  -- F81 = FF
      when "111110000010" =>  data <= "11111111";  -- F82 = FF
      when "111110000011" =>  data <= "11111111";  -- F83 = FF
      when "111110000100" =>  data <= "11111111";  -- F84 = FF
      when "111110000101" =>  data <= "11111111";  -- F85 = FF
      when "111110000110" =>  data <= "11111111";  -- F86 = FF
      when "111110000111" =>  data <= "11111111";  -- F87 = FF
      when "111110001000" =>  data <= "11111111";  -- F88 = FF
      when "111110001001" =>  data <= "11111111";  -- F89 = FF
      when "111110001010" =>  data <= "11111111";  -- F8A = FF
      when "111110001011" =>  data <= "11111111";  -- F8B = FF
      when "111110001100" =>  data <= "11111111";  -- F8C = FF
      when "111110001101" =>  data <= "11111111";  -- F8D = FF
      when "111110001110" =>  data <= "11111111";  -- F8E = FF
      when "111110001111" =>  data <= "11111111";  -- F8F = FF
      when "111110010000" =>  data <= "11111111";  -- F90 = FF
      when "111110010001" =>  data <= "11111111";  -- F91 = FF
      when "111110010010" =>  data <= "11111111";  -- F92 = FF
      when "111110010011" =>  data <= "11111111";  -- F93 = FF
      when "111110010100" =>  data <= "11111111";  -- F94 = FF
      when "111110010101" =>  data <= "11111111";  -- F95 = FF
      when "111110010110" =>  data <= "11111111";  -- F96 = FF
      when "111110010111" =>  data <= "11111111";  -- F97 = FF
      when "111110011000" =>  data <= "11111111";  -- F98 = FF
      when "111110011001" =>  data <= "11111111";  -- F99 = FF
      when "111110011010" =>  data <= "11111111";  -- F9A = FF
      when "111110011011" =>  data <= "11111111";  -- F9B = FF
      when "111110011100" =>  data <= "11111111";  -- F9C = FF
      when "111110011101" =>  data <= "11111111";  -- F9D = FF
      when "111110011110" =>  data <= "11111111";  -- F9E = FF
      when "111110011111" =>  data <= "11111111";  -- F9F = FF
      when "111110100000" =>  data <= "11111111";  -- FA0 = FF
      when "111110100001" =>  data <= "11111111";  -- FA1 = FF
      when "111110100010" =>  data <= "11111111";  -- FA2 = FF
      when "111110100011" =>  data <= "11111111";  -- FA3 = FF
      when "111110100100" =>  data <= "11111111";  -- FA4 = FF
      when "111110100101" =>  data <= "11111111";  -- FA5 = FF
      when "111110100110" =>  data <= "11111111";  -- FA6 = FF
      when "111110100111" =>  data <= "11111111";  -- FA7 = FF
      when "111110101000" =>  data <= "11111111";  -- FA8 = FF
      when "111110101001" =>  data <= "11111111";  -- FA9 = FF
      when "111110101010" =>  data <= "11111111";  -- FAA = FF
      when "111110101011" =>  data <= "11111111";  -- FAB = FF
      when "111110101100" =>  data <= "11111111";  -- FAC = FF
      when "111110101101" =>  data <= "11111111";  -- FAD = FF
      when "111110101110" =>  data <= "11111111";  -- FAE = FF
      when "111110101111" =>  data <= "11111111";  -- FAF = FF
      when "111110110000" =>  data <= "11111111";  -- FB0 = FF
      when "111110110001" =>  data <= "11111111";  -- FB1 = FF
      when "111110110010" =>  data <= "11111111";  -- FB2 = FF
      when "111110110011" =>  data <= "11111111";  -- FB3 = FF
      when "111110110100" =>  data <= "11111111";  -- FB4 = FF
      when "111110110101" =>  data <= "11111111";  -- FB5 = FF
      when "111110110110" =>  data <= "11111111";  -- FB6 = FF
      when "111110110111" =>  data <= "11111111";  -- FB7 = FF
      when "111110111000" =>  data <= "11111111";  -- FB8 = FF
      when "111110111001" =>  data <= "11111111";  -- FB9 = FF
      when "111110111010" =>  data <= "11111111";  -- FBA = FF
      when "111110111011" =>  data <= "11111111";  -- FBB = FF
      when "111110111100" =>  data <= "11111111";  -- FBC = FF
      when "111110111101" =>  data <= "11111111";  -- FBD = FF
      when "111110111110" =>  data <= "11111111";  -- FBE = FF
      when "111110111111" =>  data <= "11111111";  -- FBF = FF
      when "111111000000" =>  data <= "11111111";  -- FC0 = FF
      when "111111000001" =>  data <= "11111111";  -- FC1 = FF
      when "111111000010" =>  data <= "11111111";  -- FC2 = FF
      when "111111000011" =>  data <= "11111111";  -- FC3 = FF
      when "111111000100" =>  data <= "11111111";  -- FC4 = FF
      when "111111000101" =>  data <= "11111111";  -- FC5 = FF
      when "111111000110" =>  data <= "11111111";  -- FC6 = FF
      when "111111000111" =>  data <= "11111111";  -- FC7 = FF
      when "111111001000" =>  data <= "11111111";  -- FC8 = FF
      when "111111001001" =>  data <= "11111111";  -- FC9 = FF
      when "111111001010" =>  data <= "11111111";  -- FCA = FF
      when "111111001011" =>  data <= "11111111";  -- FCB = FF
      when "111111001100" =>  data <= "11111111";  -- FCC = FF
      when "111111001101" =>  data <= "11111111";  -- FCD = FF
      when "111111001110" =>  data <= "11111111";  -- FCE = FF
      when "111111001111" =>  data <= "11111111";  -- FCF = FF
      when "111111010000" =>  data <= "11111111";  -- FD0 = FF
      when "111111010001" =>  data <= "11111111";  -- FD1 = FF
      when "111111010010" =>  data <= "11111111";  -- FD2 = FF
      when "111111010011" =>  data <= "11111111";  -- FD3 = FF
      when "111111010100" =>  data <= "11111111";  -- FD4 = FF
      when "111111010101" =>  data <= "11111111";  -- FD5 = FF
      when "111111010110" =>  data <= "11111111";  -- FD6 = FF
      when "111111010111" =>  data <= "11111111";  -- FD7 = FF
      when "111111011000" =>  data <= "11111111";  -- FD8 = FF
      when "111111011001" =>  data <= "11111111";  -- FD9 = FF
      when "111111011010" =>  data <= "11111111";  -- FDA = FF
      when "111111011011" =>  data <= "11111111";  -- FDB = FF
      when "111111011100" =>  data <= "11111111";  -- FDC = FF
      when "111111011101" =>  data <= "11111111";  -- FDD = FF
      when "111111011110" =>  data <= "11111111";  -- FDE = FF
      when "111111011111" =>  data <= "11111111";  -- FDF = FF
      when "111111100000" =>  data <= "11111111";  -- FE0 = FF
      when "111111100001" =>  data <= "11111111";  -- FE1 = FF
      when "111111100010" =>  data <= "11111111";  -- FE2 = FF
      when "111111100011" =>  data <= "11111111";  -- FE3 = FF
      when "111111100100" =>  data <= "11111111";  -- FE4 = FF
      when "111111100101" =>  data <= "11111111";  -- FE5 = FF
      when "111111100110" =>  data <= "11111111";  -- FE6 = FF
      when "111111100111" =>  data <= "11111111";  -- FE7 = FF
      when "111111101000" =>  data <= "11111111";  -- FE8 = FF
      when "111111101001" =>  data <= "11111111";  -- FE9 = FF
      when "111111101010" =>  data <= "11111111";  -- FEA = FF
      when "111111101011" =>  data <= "11111111";  -- FEB = FF
      when "111111101100" =>  data <= "11111111";  -- FEC = FF
      when "111111101101" =>  data <= "11111111";  -- FED = FF
      when "111111101110" =>  data <= "11111111";  -- FEE = FF
      when "111111101111" =>  data <= "11111111";  -- FEF = FF
      when "111111110000" =>  data <= "11111111";  -- FF0 = FF
      when "111111110001" =>  data <= "11111111";  -- FF1 = FF
      when "111111110010" =>  data <= "11111111";  -- FF2 = FF
      when "111111110011" =>  data <= "11111111";  -- FF3 = FF
      when "111111110100" =>  data <= "11111111";  -- FF4 = FF
      when "111111110101" =>  data <= "11111111";  -- FF5 = FF
      when "111111110110" =>  data <= "11111111";  -- FF6 = FF
      when "111111110111" =>  data <= "11111111";  -- FF7 = FF
      when "111111111000" =>  data <= "11111111";  -- FF8 = FF
      when "111111111001" =>  data <= "11111111";  -- FF9 = FF
      when "111111111010" =>  data <= "11111110";  -- FFA = FE
      when "111111111011" =>  data <= "11110100";  -- FFB = F4
      when "111111111100" =>  data <= "00000000";  -- FFC = 0
      when "111111111101" =>  data <= "11110000";  -- FFD = F0
      when "111111111110" =>  data <= "11111011";  -- FFE = FB
      when "111111111111" =>  data <= "11110100";  -- FFF = F4
      --end_of_rom
      when others =>  data <= "00000000";
    end case;
  end process;

end JESUS;

------------------------------------------------------------------------------
------------------------------------------------------------------------------

